# FileName: atetool-update-v2.ps1
#=============================================================================
# Script Name: [ATE PLANNING TOOL update script]
# Created: [5/4/2013]
# Version: [2.0]
# Author: [Martin Kotlar]
# Email: [martin.kotlar@honeywell.com]
# Company: [Honeywell]  
# Arguments: None
# =============================================================================
# Purpose: This script is to be used to updated ATE Planning Tool to last
# version in the subversion repository
# =============================================================================

#=============================================================================
# FUNCTION LISTINGS
#=============================================================================
# Function: Decrypt-SecureString
# Arguments: String $sstr Secure string input texts
# =============================================================================
# Purpose: Decrypts secure string into plain text.
# =============================================================================
function Decrypt-SecureString {
    Param(
        [Parameter(ValueFromPipeline=$true,Mandatory=$true,Position=0)]
        [System.Security.SecureString]
        $sstr
    )
    
    $marshal = [System.Runtime.InteropServices.Marshal]
    $ptr = $marshal::SecureStringToBSTR( $sstr )
    $str = $marshal::PtrToStringBSTR( $ptr )
    $marshal::ZeroFreeBSTR( $ptr )
    return $str
} 

#=============================================================================
# Function: Replace-FileContent
# Arguments: 
#  String $inputFile Path to the input file to read/parse
#  String $outputFile Path to the output file to write replaced texts
#  String $pattern String for pattern matching (for String -match method)
#  String $replace String to replace pattern with
# =============================================================================
# Purpose: Reads $inputFile and replaces strings matching $pattern with $replace
#         string and writes it into $outputFile.
# =============================================================================
Function Replace-FileContent
{
    Param(
      [Parameter(Mandatory=$True,Position=1)]
       [string]$inputFile,
    	
       [Parameter(Mandatory=$True,Position=2)]
       [string]$outputFile,
    
       [Parameter(Mandatory=$True,Position=3)]
       [string]$pattern,
    
       [Parameter(Mandatory=$True,Position=4)]
       [string]$replace
    )
    Get-Content -Path $inputFile | ForEach-Object { 
        if ($_ -match $pattern)
        {
            $_ -replace $pattern, $replace
        }
        else
        {
            $_
        }
    } | Set-Content -Path $outputFile
}

#=============================================================================
# Function: Ask-YesNoQuestion
# Arguments: 
#  String $question Text to show as question (without ?)
#  String $foregroundcolor Color to use for the question (same values possible as for Write-Host cmdlet)
# =============================================================================
# Purpose: Ask user a Yes/No question. Returns "y" or "n" Question can be colored. 
# =============================================================================
Function Ask-YesNoQuestion
{
    Param(
       [Parameter(Mandatory=$True,Position=1)]
       [string]$Question,
    
       [Parameter(Position=2)]
       [string]$ForegroundColor
    )
    $my_confirm = "Undefined"
    do {
        Write-Host ($Question + " [y/n]? ") -ForegroundColor $ForegroundColor -NoNewLine 
        $my_confirm = Read-Host  
    } until (($my_confirm -eq "y") -or ($my_confirm -eq "n"))
    return $my_confirm
}

#=============================================================================
# Function: AskView-FileContent
# Arguments: 
#  String $path Path to the file to view
#  String $question Text to show as question (without ending ?)
#  String $foregroundcolor Color to use for the question (same values possible as for Write-Host cmdlet)
# =============================================================================
# Purpose: Ask whether user wants to see the file content and if confirmed
#           shows the content of the file using 'more' command. 
# =============================================================================
Function AskView-FileContent
{
    Param(
      [Parameter(Mandatory=$True,Position=1)]
       [string]$Path,
    	
       [Parameter(Mandatory=$True,Position=2)]
       [string]$Question,
    
       [Parameter(Position=3)]
       [string]$ForegroundColor="Gray"
    )
    $my_confirm = "Undefined"
    do {
        Write-Host ($Question + " (use q to quit) [y/n]? ") -ForegroundColor $ForegroundColor -NoNewLine 
        $my_confirm = Read-Host  
    } until (($my_confirm -eq "y") -or ($my_confirm -eq "n"))
    if ($my_confirm -eq "y")
    { 
        echo "------  START OF FILE -----"
        cat $Path | more 
        echo "------  END OF FILE -----"
    }
}

#=============================================================================
# Function: Set-PathWritable
# Arguments: 
#  String $path Path to the directory to set 
#  String $user Name of the user/group to set the path writable (Modify permission)
#  Boolean $cleanUp Whether to clean up inherited permissions
# =============================================================================
# Purpose: Adds Modify access rule to the list of ACL of the file/dir on the path. 
# =============================================================================
Function Set-PathWritable
{
    Param(
      [Parameter(Mandatory=$True)]
       [string]$path,
    	
       [Parameter(Mandatory=$True)]
       [string]$user,
    
       [Parameter(Mandatory=$False)]
       [bool]$cleanUp = $False
    )
    
    $acl = Get-Acl -Path $path
    if ($cleanUp) { $acl.SetAccessRuleProtection($True, $False) }    
    $rule = New-Object System.Security.AccessControl.FileSystemAccessRule($user, "Modify", "ContainerInherit, ObjectInherit", "None", "Allow")
    $acl.AddAccessRule($rule)
    Set-Acl -Path $path -AclObject $acl
}

#=============================================================================
# Function: Get-IPAddress
# Arguments: 
#  String $computer Name of the computer (e.g. LocalHost) 
# =============================================================================
# Purpose: Returns array IP addresses of this computer. If not found returns
#  empty array. 
# =============================================================================
Function Get-IPAddress
{
    Param(
        [Parameter(Mandatory=$True,Position=1)]
        [string]$computer
    )
    
    # Get Networking Adapter Configuration 
    $IPConfigSet = Get-WmiObject -filter "IPEnabled='True'" -ComputerName $computer Win32_NetworkAdapterConfiguration
    $IPAddresses = @()       
    # Iterate and get IP address 
    foreach ($IPConfig in $IPConfigSet) {
       if ($Ipconfig.IPaddress) { 
           foreach ($addr in $IPConfig.IPAddress) { 
                #"IP Address   : {0}" -f  $addr; 
                $IPAddresses = $IPAddresses + $addr         
          } 
       } 
    }
    return $IPAddresses
}

#=============================================================================
# Function: Pause
# Arguments: 
#  String $Message Optional message to write to output before waiting for input 
# =============================================================================
# Purpose: Simulates "pause" comand in old windows shell. 
# =============================================================================
function Pause ($Message="Press any key to continue...")
{
    Write-Host -NoNewLine $Message
    $null = $Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
    Write-Host ""
}

#=============================================================================
# CONFIGURATION SECTION
#=============================================================================
# ATE Planning Tool repository URL
$my_svn_base_url = 'https://aterepos.honeywell.com/svn/ATE_Tool/'
# IIS document root
$my_iis_base_dir = "C:\inetpub\wwwroot\"
# ATE Planning Tool application subdirectory in the IIS document root
$my_target_dir="hon-planning-tool"
# ATE Planning Tool MySQL database name
$my_db="tool"
# Path to the subversion command-line tool executable
$my_svn_exec = 'c:\Program Files\TortoiseSVN\bin\svn.exe'
# Directory to backup previous version in
$my_backup_base = 'C:\Backup\ate-planning-tool\app\'
# User for the IIS server to be able to write to directories
$my_iis_usr = "IUSR"   
#$my_iis_usr = "IIS_IUSRS"
# setup of IP address mappings to deployed versions of ATE Planning Tool app
$machine_version_map = @{}
$machine_version_map["158.138.219.242"] = @{ "machine" = "Production machine"; "svn_dir" = "tags/1.17.3/src"; "svn_info" = "production version 1.17.3"; "dns" = "ate-apps.honeywell.com" }; 
$machine_version_map["165.195.211.234"]  = @{ "machine" = "QA/testing machine"; "svn_dir" = "branches/1.17.2.1-bugfixing/src"; "svn_info" = "bugfixing version 1.8.x"; "dns" = "ate-apps-test.honeywell.com" }; 
$machine_version_map["165.195.211.235"] = @{ "machine" = "Development machine"; "svn_dir" = "trunk/src"; "svn_info" = "development version"; "dns" = "ate-apps-dev.honeywell.com" }; 
# colors for outputted text
$my_color_info = "Yellow"
$my_color_success = "Green"
$my_color_fail = "Red"

#=============================================================================
# Script start
#=============================================================================

# detection of the IP address of the machine 
$addresses = Get-IPAddress "Localhost"
$choice = $null;
foreach ($key in $machine_version_map.keys) {
    if ($addresses -contains $key) { $choice = $machine_version_map[$key]; break; }
}
                                                            
Write-Host "======================================================"  -foregroundcolor "Blue"
Write-Host "ATE Planning Tool automatic update from SVN repository"  -foregroundcolor "White"
Write-Host "======================================================"  -foregroundcolor "Blue"
Write-Host "To stop the script at any time: Ctrl+C"  -foregroundcolor "Magenta" 

if ($choice -eq $null) {
    Write-Host "Unknown machine. Failed automatic selection of application version for exporting from SVN which is based on IP Address. Check mapping of IP address to version in the update script. Exiting." -foregroundcolor $my_color_fail
    exit 1
}
$my_svn_dir=$choice["svn_dir"]
Write-Host ($choice["machine"] + " was detected based on your IP address.") -foregroundcolor $my_color_info
Write-Host ("Selected SVN directory to export: " + $choice["svn_info"] + " (" + $my_svn_dir + ")") -foregroundcolor $my_color_info

$my_target_path=$my_iis_base_dir + $my_target_dir
$my_temp_target=$my_target_path + "-temp-" + (Get-Random)
$my_svn_url=$my_svn_base_url + $my_svn_dir

Write-Host "Target directory is: $my_target_path" -foregroundcolor $my_color_info
Write-Host "Temporary directory is: $my_temp_target" -foregroundcolor $my_color_info

do {
    echo ""
    Write-Host "Querying SVN at $my_svn_url" -foregroundcolor $my_color_info
    Write-Host "Input SVN repository credentials: "
    $my_svn_user = Read-Host -Prompt "Username (email)"
    $my_svn_pass = Read-Host -Prompt "Password (not outputted)" -AsSecureString
    
    Write-Host "Detailed information for SVN repository:" -foregroundcolor $my_color_info
    & $my_svn_exec @("info","--non-interactive","--trust-server-cert","--no-auth-cache","--username","$my_svn_user","--password",(Decrypt-SecureString $my_svn_pass),"$my_svn_url")    
}
until ($LastExitCode -eq 0)

$my_start = Ask-YesNoQuestion "Start operation" "Cyan" 
if ($my_start -eq "n") { echo "Stopping script."; exit 1; }

Write-Host "Exporting repository to $my_temp_target" -foregroundcolor $my_color_info
& $my_svn_exec @("export","--non-interactive","--trust-server-cert","--no-auth-cache","--username","$my_svn_user","--password",(Decrypt-SecureString $my_svn_pass),"$my_svn_url", "$my_temp_target")    

& $my_svn_exec @("info","--non-interactive","--trust-server-cert","--no-auth-cache","--username","$my_svn_user","--password",(Decrypt-SecureString $my_svn_pass),"$my_svn_url") > $my_temp_target\svn-export-info.txt     
$revision = cat $my_temp_target\svn-export-info.txt | Where { $_ -match "^Last Changed Rev:" }
$revision = $revision -replace "Last Changed Rev: ", ""
Write-Host "Last applicable SVN Revision is $revision." -foregroundcolor $my_color_info

Write-Host "Creating new directories..." -foregroundcolor $my_color_info
New-Item -Type directory -Path $my_temp_target\backup | out-null

Write-Host "Updating permission to writable directories to user $my_iis_usr ..." -foregroundcolor $my_color_info
Set-PathWritable -Path $my_temp_target\log -User $my_iis_usr
Set-PathWritable -Path $my_temp_target\temp -User $my_iis_usr
Set-PathWritable -Path $my_temp_target\backup -User $my_iis_usr
Set-PathWritable -Path $my_temp_target\data\actual -User $my_iis_usr

if (Test-Path $my_target_path\public)
{
    Write-Host "Copying content of public directory..." -foregroundcolor $my_color_info
	Copy-Item $my_target_path\public -Destination $my_temp_target -Force -Recurse
}
if (Test-Path $my_target_path\data\actual)
{
    Write-Host "Copying content of data/actual directory..." -foregroundcolor $my_color_info
	Copy-Item $my_target_path\data\actual -Destination $my_temp_target\data -Force -Recurse
}

Write-Host "Changing database name in configuration file in app (Nette-based)" -foregroundcolor $my_color_info
Replace-FileContent -inputFile $my_temp_target\app\config.neon -outputFile $my_temp_target\app\config.neon.new -pattern "database: tool" -replace "database: $my_db"
Rename-Item $my_temp_target\app\config.neon $my_temp_target\app\config.neon.orig
Rename-Item $my_temp_target\app\config.neon.new $my_temp_target\app\config.neon
AskView-FileContent -Path $my_temp_target\app\config.neon -Question "View app config.neon" "Cyan" 

Write-Host "Updating revision to $revision in index.php file" -foregroundcolor $my_color_info 
Replace-FileContent -inputFile $my_temp_target\index.php -outputFile $my_temp_target\index.php.new -pattern "define\('APPLICATION_VERSION_REV', ''\);" -replace "define('APPLICATION_VERSION_REV', '$revision');"
echo "Updated lines:"
cat $my_temp_target\index.php.new | Where-Object { $_ -match "APPLICATION_VERSION_REV" }
Rename-Item $my_temp_target\index.php $my_temp_target\index.php.orig
Rename-Item $my_temp_target\index.php.new $my_temp_target\index.php
AskView-FileContent -Path $my_temp_target\index.php -Question "View index.php" "Cyan"

# if there is MAINTENANCE file in the application root (it is stopped on purpose) so preserve it
if (Test-Path $my_target_path\MAINTENANCE)
{
    Write-Host "Detected MAINTENANCE. Preserving it in deployed version!" -foregroundcolor $my_color_info
    Copy-Item $my_target_path\MAINTENANCE $my_temp_target         
} 
 
echo ""
Write-Host "Ready to switch to new version ($my_target_path). Previous will be backed up." -foregroundcolor $my_color_info
$my_confirm = Ask-YesNoQuestion "Perform switch" "Cyan" 
if ($my_confirm -eq "n")
{
	Write-Host "Changes were not applied!" -foregroundcolor $my_color_fail
    Write-Host "Cleaning up temporary directory..." -foregroundcolor $my_color_info
	Remove-Item -Path $my_temp_target -Force -Recurse
	echo "Stopping script."
	exit 1
}

# stop IIS
$svc_name = "W3SVC"
$svc = Get-Service $svc_name
"IIS ($svc_name) is now " + $svc.Status
if ($svc.Status -ne "Stopped")
{
  echo "Stopping service..."
  Stop-Service $svc_name
  Start-Sleep 5 # wait for 5 second
  $svc = Get-Service $svc_name
  "IIS ($svc_name) is now " + $svc.Status
}

$my_backup_path=$my_backup_base + $my_target_dir + "-backup-" + (Get-Date -UFormat "%Y-%m-%d--%H-%M-%S")
Write-Host "Backing-up original directory to:" -foregroundcolor $my_color_info
echo $my_backup_path
Write-Host "and switching to new version." -foregroundcolor $my_color_info
if (Test-Path $my_target_path)
{
    Move-Item $my_target_path $my_backup_path
}
Rename-Item $my_temp_target $my_target_path

echo "Starting $svc_name service..."
Start-Service $svc_name
Start-Sleep 5 # wait for 5 second
$svc = Get-Service $svc_name
"$svc_name is now " + $svc.Status


Write-Host "Finished." -foregroundcolor $my_color_success
echo "" 
echo "You might want to test whether application is running:"
echo ("http://" + $choice["dns"] + "/hon-planning-tool/")
echo "" 

Pause

exit