<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package components/WBS
 */

/**
 * WBSControl provides displaying of WBS overview
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class WBSControl extends Control
{
    /** @var Project */
    private $project;
    /** @var WorkPackage */
    private $workpackage;
    /** @var int Unrolled year */
    private $year;
	/** @var string Name of the presenter we'll redirect to,
	 * if current TaskPlan doesn't exist. */
	private $prevPresenter = '';
    
    /**
     * @param Project|WorkPackage $control_item
     * @param int $year
     */
    public function __construct($control_item, $year = NULL, $prevPresenter = '')
    {
        parent::__construct();
        
        if ($control_item instanceof Project)
            $this->project = $control_item;
        elseif ($control_item instanceof WorkPackage)
            $this->workpackage = $control_item;
        else
        {
            $this->project = new Project();
            $this->workpackage = new WorkPackage();
        }
        
        $this->year = ($year) ? $year : date('Y');
		$this->prevPresenter = $prevPresenter;
    }
    
    /**
     * Shows WBS for project
     */
    public function renderProject()
    {
		if ($this->presenter->isAjax()) // is ajax request
		{
			$template = $this->template;
			$template->setFile(__DIR__ . '/WBSControl.latte');

			$available_years = $this->presenter->context->projectRepository->findProjectYears($this->project->ID);
			if (!count($available_years))
				$this->flashMessage("There are no available data for time line part of Work Breakdown Structure.", 'info');

			$setting = $this->presenter->context->userSettingRepository->find($this->presenter->getUser()->getId());
			$wbs_tree = $this->presenter->context->workPackageRepository->getProjectWBSTree($this->project->ID);
			$nodes = $this->getNodes($wbs_tree->getTreeIterator(), $available_years, $setting);

			$template->year = $this->year;
			$template->project = $this->project;
			$template->setting = $setting;
			$this->template->wbs_tree = $nodes;
			$this->template->available_years = $available_years;
			$template->prevPresenter = $this->prevPresenter;

			$template->render();
		}
    }
    
    /**
     * Shows WBS for workpackage
     */
    public function renderWorkpackage()
    {
		if ($this->presenter->isAjax()) // is ajax request
		{
			$template = $this->template;
			$template->setFile(__DIR__ . '/WBSControl.latte');

			$this->project = $this->presenter->context->projectRepository->find($this->workpackage->ProjectID);
			if (!$this->project)
				$this->project = new Project();

			$available_years = $this->presenter->context->projectRepository->findProjectYears($this->project->ID);
			if (!count($available_years))
				$this->flashMessage("There are no available data for time line part of Work Breakdown Structure.", 'info');

			$setting = $this->presenter->context->userSettingRepository->find($this->presenter->getUser()->getId());
			$wbs_tree = $this->presenter->context->workPackageRepository->getProjectWBSTree($this->project->ID);

			$main_wp_node = new WBSTree();
			foreach ($wbs_tree->getTreeIterator() as $node)
			{
				if ($node->getData()->ID == $this->workpackage->ID)
				{
					$main_wp_node->addItem($node);
					break;
				}
			}

			$nodes = $this->getNodes($main_wp_node->getTreeIterator(), $available_years, $setting);

			$template->year = $this->year;
			$template->project = $this->project;
			$template->setting = $setting;
			$this->template->wbs_tree = $nodes;
			$this->template->available_years = $available_years;
			$template->prevPresenter = $this->prevPresenter;

			$template->render();
		}
    }
    
    public function getNodes($tree, $available_years, $setting)
    {        
        $nodes = $wp_filter = $duration = array();
        $work_centers = $this->presenter->context->costCenterRepository->findAllCostCenters();
		$project_eac_reporter = new ProjectEACReporter($this->presenter->context, $this->project, $this->year);
        
        foreach ($tree as $node)
		{
			if (($node->getWBSType() == WBSTreeElementItem::WBS_TYPE_TASK) && (substr($node->getName(), 0, 7) != "NOT USE"))
			{
                $skip = FALSE;
                
                // Hide non-labor tasks
                if ($setting->WBSShowLabourTasksOnly && !$node->getData()->isLabourTask())
                    $skip = TRUE;

                // Hide tasks with no effort (No assignment, no actuals and no plan)
				if ($setting->WBSHideTasksWithoutEffort)
				{
					$task_plan = $this->presenter->context->taskPlanRepository->findTaskYearPlan($node->getData()->ID, $this->year); 
                    // Task is not planned but possibly has some work/costs for this year
					if ($task_plan === NULL)
						$task_plan = new TaskYearPlan($this->year);

                    $sum = 0;
					$assignments_cost = $actuals = array();
                    
					if($node->getData()->isLabourTask())
					{
						$actuals = $project_eac_reporter->getTaskLabourEACHoursMonthTotals($node->getData());
					}
					else
					{
						$assignments_cost = $this->presenter->context->assignmentsCostRepository->findTaskYearAssignments($node->getData()->ID, $this->year);
						$actuals = $this->presenter->context->costCenterRepository->findActualCostsYearSummary($node->getData()->ID, $this->year, 'USD');
					}
                    
					for ($i = 1; $i <= 12; $i++)
					{
						if ($node->getData()->isLabourTask())
						{
                            // Plan
							$sum += abs($task_plan->getMonthPlan($i)->Hours);
                            // Actuals
							$sum += abs($actuals[$i]);
						}
						else
						{
							if (isset($assignments_cost[$i-1]))
							{
                                // Assignment
								$sum += abs($assignments_cost[$i-1]->Cost);
							}
                            // Plan
							$sum += abs($task_plan->getMonthPlan($i)->Budget);
                            // Actuals
							$sum += abs($actuals[$i]);
						}
					}
                    
					if ($sum == 0)
						$skip = TRUE;
				}
                
                $task_plan_years = $actuals_years = $actuals_cost_years = array();
                
				$task_plan_years = $this->presenter->context->taskPlanRepository->findTaskYears($node->getData()->ID);
                if ($node->getData()->isLabourTask())
				    $actuals_years = $this->presenter->context->actualsRepository->findTaskYears($node->getData()->SAPID);
                else
                    $actuals_cost_years = $this->presenter->context->actualsCostRepository->findTaskYears($node->getData()->ID);
                
                $item = new stdClass();
                $item->node = $node;
                $item->workCenter = (isset($work_centers[$node->getData()->CostCenterID])) ? $work_centers[$node->getData()->CostCenterID]->Name : '-';
                $item->workCenterID = (isset($work_centers[$node->getData()->CostCenterID])) ? $work_centers[$node->getData()->CostCenterID]->ID : NULL;
                $item->isTRV = (substr($node->getName(), 0, 3) == "TRV") ? TRUE : FALSE;
                $item->duration = array_intersect(
                        $available_years,
                        array_unique(array_merge($task_plan_years, $actuals_years, $actuals_cost_years))
                );
                
                $duration[$item->node->getSAPID()] = $item->duration;
                
                // Need to count workpackage duration for all tasks in workpackage,
                // but not all tasks should be visible according to setting
                if ($skip)
                    continue;
                
                $wp_filter[] = $item->node->getSAPID();
				$nodes[] = $item;
                
			}
            elseif (($node->getWBSType() == WBSTreeElementItem::WBS_TYPE_WORK_PACKAGE) && ($node->getSAPID() != NULL) && (substr($node->getName(), 0, 7) != "NOT USE"))
			{
                // Hide all workpackages
                if ($setting->WBSHideAllWP)
                    continue;
                
                $item = new stdClass();
                $item->node = $node;
                $item->duration = array();
                
                $duration[$item->node->getSAPID()] = $item->duration;
				$wp_filter[] = $item->node->getSAPID();
				$nodes[] = $item;
			}
		}

        $reversedNodes = array_reverse($nodes, TRUE);
		// This is where we agregate Tasks into a bar of displayed years for a Work Package
		// Traverse the tree from the bottom (Tasks only) to the top (only one WP - the upper one)
		// the upper one is not displayed and its name is the name of the Project
        foreach ($reversedNodes as $key => $node)
        {
            if ($node->node->getWBSType() == WBSTreeElementItem::WBS_TYPE_WORK_PACKAGE)
            {
                $hide = TRUE;
                
                foreach ($node->node->getChildren()->getIterator() as $child)
                {
                    // Hide all empty workpackages according to filtering of wbs above
					// meaning hide all without Task or sub Work Package
                    if (in_array($child->getSAPID(), $wp_filter))
                        $hide = FALSE;
                        
                    // Count workpackage duration as unification of its task durations
                    if (!$setting->WBSHideAllWP && array_key_exists($child->getSAPID(), $duration))
					{
                        $node->duration = array_merge($node->duration, $duration[$child->getSAPID()]);
						// We need to save the unified duration of a WP so we can propagate it to one level upper WPs
						// with each iteration
						$duration[$node->node->Data->getSAPID()] = array_unique(array_merge($duration[$node->node->Data->getSAPID()], $node->duration), SORT_NUMERIC);		
					}
                }
                
                if ($hide)
				{
					unset($nodes[$key]);
					unset($wp_filter[$key]);
				}
            }
        }
        
        return $nodes;
    }
	
    /**
     * Load values to wbs component
     */
    public function handleAjaxCall()
    {
		$this->presenter->invalidateControl('wbsComponent');
    }
}