<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter started by cron process to run iASAP actuals import.
 * Authentication is done by url (known token).
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class IASAPActualsAutoImportPresenter extends TokenAccessPresenter
{
	/* authentication token setup */
	const AUTH_TOKEN_NAME = 'token',
		AUTH_TOKEN_KEY = 'iASAP.actuals.credentials.cronToken';
	
	/**
     * Service for iASAP importing.
     * @var IASAPActualsImportServicePO
     */
	private $iASAPActualsImportService;
	
    /**
     * Returns http parameter name with authentication token.
     * 
     * @return string
     */
	protected function getAuthTokenName()
	{
		return self::AUTH_TOKEN_NAME;
	}

    /**
     * Returns http parameter name with authentication token.
	 * 
     * @return string
     */
	protected function getAuthTokenSettingsKey()
	{
		return self::AUTH_TOKEN_KEY;
	}
	
	/**
     * Run method 
     */
    public function renderRun()
    {
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'iasap_actuals_import-'.$logs_suffix.'.html';
		$batch_id = 'iasap_actuals-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "IASAP ACTUALS") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "IASAP ACTUALS"));
		
        $import_result = IASAPActualsImportServicePO::RESULT_ERROR;
		$import_ex_error = '';
		
		// find all PO numbers in current year
		$all_task_po_numbers = $this->context->taskPONumberRepository->findByYear((int) date('Y'));
		$po_numbers = array();
		foreach ($all_task_po_numbers as $po_number)
			$po_numbers[] = $po_number->PONumber;
		
        $this->iASAPActualsImportService = new IASAPActualsImportServicePO($this->context, $po_numbers, $multilog);
        
        try
        {
			$import_result = $this->iASAPActualsImportService->runImport();
            switch ($import_result)
			{
				case IASAPActualsImportServicePO::RESULT_SUCCESS:
					$multilog->log('iASAP PO actuals import was successful.', Logger::INFO);
					break;
				case IASAPActualsImportServicePO::RESULT_NO_DATA:
					$multilog->log('iASAP PO actuals import was successful but no actuals was found.', Logger::INFO);
					break;
				case IASAPActualsImportServicePO::RESULT_WARNING:
					$multilog->log('iASAP PO actuals import was successful with warnings.', Logger::WARNING);
					break;
				case IASAPActualsImportServicePO::RESULT_ERROR:
					$multilog->log('iASAP PO actuals import failed!', Logger::ERROR);
					break;
				default: break;
			}
        }
        catch (IASAPActualsImportPOServiceException $e)
        {
			$import_ex_error = $e->getMessage();
			$multilog->log('iASAP actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('iASAP actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
		}

		// gets all placeholder task for update
		$tasksLabour = $tasksCost = array();
		$allTasks = $this->context->taskRepository->findAllPlaceholderTasksForImport(Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP);
		foreach($allTasks as $task)
		{
			if($task->isLabourTask())
				$tasksLabour[$task->SAPID] = $task;
			else
				$tasksCost[$task->SAPID] = $task;
		}

		$iASAPActualsImportService = new IASAPActualsImportServiceSAPID($this->context, $tasksLabour, $multilog);
		try
		{
			$import_labour_result = $iASAPActualsImportService->runLaborImport();
			switch ($import_labour_result)
			{
				case iASAPActualsImportServiceSAPID::RESULT_SUCCESS:
					$multilog->log('iASAP actuals labour import was successful.', Logger::INFO);
					break;
				case iASAPActualsImportServiceSAPID::RESULT_WARNING:
					$multilog->log('iASAP actuals labour import was successful with warnings.', Logger::WARNING);
					break;
				case iASAPActualsImportServiceSAPID::RESULT_ERROR:
					$multilog->log('iASAP actuals labour import failed!', Logger::ERROR);
					break;
				case iASAPActualsImportServiceSAPID::RESULT_NO_DATA:
					$multilog->log('iASAP actuals labour import was successful but no actuals was found.', Logger::INFO);
					break;
				default: break;
			}
		}
		catch (IASAPActualsImportSAPIDServiceException $e)
		{
			$import_result = FALSE;
			$import_ex_error = $e->getMessage();
			$multilog->log('iASAP actuals labour import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('iASAP actuals labour import failed: ' . $e->getMessage(), Debugger::ERROR);
			$this->flashMessage('Actuals labour imported unsuccessfully.', 'error');
		}
		
		$iASAPActualsImportService = new IASAPActualsImportServiceSAPID($this->context, $tasksCost, $multilog);
		try
		{
			$import_cost_result = $iASAPActualsImportService->runCostImport();
			switch ($import_cost_result)
			{
				case iASAPActualsImportServiceSAPID::RESULT_SUCCESS:
					$multilog->log('iASAP actuals cost import was successful.', Logger::INFO);
					break;
				case iASAPActualsImportServiceSAPID::RESULT_WARNING:
					$multilog->log('iASAP actuals cost import was successful with warnings.', Logger::WARNING);
					break;
				case iASAPActualsImportServiceSAPID::RESULT_ERROR:
					$multilog->log('iASAP actuals cost import failed!', Logger::ERROR);
					break;
				case iASAPActualsImportServiceSAPID::RESULT_NO_DATA:
					$multilog->log('iASAP actuals cost import was successful but no actuals was found.', Logger::INFO);
					break;
				default: break;
			}
		}
		catch (IASAPActualsImportSAPIDServiceException $e)
		{
			$import_result = FALSE;
			$import_ex_error = $e->getMessage();
			$multilog->log('iASAP actuals cost import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('iASAP actuals cost import failed: ' . $e->getMessage(), Debugger::ERROR);
			$this->flashMessage('Actuals cost imported unsuccessfully.', 'error');
		}
		
		if ($import_result == IASAPActualsImportServicePO::RESULT_ERROR || 
				$import_labour_result == iASAPActualsImportServiceSAPID::RESULT_ERROR || 
				$import_cost_result == iASAPActualsImportServiceSAPID::RESULT_ERROR)
		{
			if (!$this->context->globalSetting->exists('mail.adminList'))
				throw new InvalidStateException("Undefined list of email addresses for notifiying of iASAP actuals import failure!");
			$mail = $this->context->nette->createMail();
			$mail->setFrom($this->context->parameters['mailer']['from']);
			$emails = explode(',', $this->context->globalSetting->get('mail.adminList'));
			$valid_notify_emails = 0;
			foreach ($emails as $email)
			{
				$notify_email = trim($email);
				if (Validators::isEmail($notify_email))
				{
					$mail->addTo($notify_email);
					$valid_notify_emails++;
				}
				else
					Debugger::log('Invalid email address specified: '.$notify_email, Debugger::WARNING);
			}
			if ($valid_notify_emails)
			{
				$mail->setSubject('HON Planning Tool - iASAP actuals import failed @ ' . date('Y-m-d H:i:s'))
					->setBody($import_ex_error)
					->addAttachment($filelog_path);
				$mail->send();
			}
			else
				Debugger::log('Cannot notify about iASAP actuals import failure! No valid email addresses specified.', Debugger::WARNING);
		}	
		
		$filelog = NULL; // clean up (save file)
        $this->template->lastUpdated    = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_IASAP);
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
    }
}
