<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presents interface for importing data from iASAP.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class IASAPActualsImportPresenter extends SecuredPresenter
{
    /**
     * Service for iASAP importing.
     * @var IASAPActualsImportServicePO
     */
	private $iASAPActualsImportService;
    
    /**
     * Path to logfile
     * @var string
     */
    private $logfileName = 'iasap_actuals_import.html';
    
    /**
     * Nette default startup method 
     */
    public function startup()
    {
        parent::startup();
        
        if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_VIEW))
        {
            $this->flashMessage('You don\'t have permission to use iASAP actuals import. You were redirected.');
            $this->redirect('Dashboard:');
        }
        
    }
    
    /**
     * Nette default render method 
     */
	public function renderDefault()
	{
        $this->template->lastUpdated = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_IASAP);
        $this->template->maxExecTime = IASAPActualsImportServicePO::MAXIMUM_EXECUTION_TIME;
		$lock = $this->context->createLock(IASAPActualsImportServicePO::LOCK_KEY);
		$this->template->running = $lock->locked();
	}
	
    /**
     * Nette sapIdImport render method 
     */
	public function renderSapIdImport()
	{
        $this->template->maxExecTime = IASAPActualsImportServiceSAPID::MAXIMUM_EXECUTION_TIME;
		$lock = $this->context->createLock(IASAPActualsImportServiceSAPID::LOCK_KEY);
		$this->template->running = $lock->locked();
	}
    
    /**
     * Run method 
     */
    public function renderRunPO()
    {
		if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_RUN))
        {
            $this->flashMessage('You don\'t have permission to use iASAP actuals import. You were redirected.');
            $this->redirect('Dashboard:');
        }
		
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'iasap_actuals_import-'.$logs_suffix.'.html';
		$batch_id = 'iasap_actuals-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "IASAP ACTUALS") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "IASAP ACTUALS"));

        $import_result = FALSE;
		$import_ex_error = '';
		
		// find all PO numbers in current year
		$all_task_po_numbers = $this->context->taskPONumberRepository->findByYear((int) date('Y'));
		$po_numbers = array();
		foreach ($all_task_po_numbers as $po_number)
			$po_numbers[] = $po_number->PONumber;

        $this->iASAPActualsImportService = new IASAPActualsImportServicePO($this->context, $po_numbers, $multilog);
        
        try
        {
			switch ($this->iASAPActualsImportService->runImport())
			{
				case IASAPActualsImportServicePO::RESULT_SUCCESS:
					$this->flashMessage('iASAP PO actuals import was successful.', 'success');
					break;
				case IASAPActualsImportServicePO::RESULT_NO_DATA:
					$this->flashMessage('iASAP PO actuals import was successful but no actuals was found.', 'warning');
					break;
				case IASAPActualsImportServicePO::RESULT_WARNING:
					$this->flashMessage('iASAP PO actuals import was successful with warnings.', 'warning');
					break;
				case IASAPActualsImportServicePO::RESULT_ERROR:
					$this->flashMessage('iASAP PO actuals import failed!', 'error');
					break;
				default: break;
			}
        }
        catch (IASAPActualsImportPOServiceException $e)
        {
			$this->flashMessage("iASAP actuals import failed: " . $e->getMessage(), 'error');
			$multilog->log('iASAP actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('iASAP actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
		}
        $filelog->close();
		
        $this->template->lastUpdated    = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_IASAP);
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
		$this->template->batchNumber	= $batch_id;
    }
	
/**
     * Run method 
     */
    public function renderRunSAPID()
    {
		if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_RUN))
        {
            $this->flashMessage('You don\'t have permission to use iASAP actuals import. You were redirected.');
            $this->redirect('Dashboard:');
        }
		
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'iasap_actuals_import-'.$logs_suffix.'.html';
		$batch_id = 'iasap_actuals-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
		$filelog = $this->context->createFileLogger($filelog_path, $batch_id, "IASAP ACTUALS") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "IASAP ACTUALS"));

		$import_result = FALSE;
		$import_ex_error = '';

		// gets all placeholder task for update
		$tasksLabour = $tasksCost = array();
		$allTasks = $this->context->taskRepository->findAllPlaceholderTasksForImport(Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP);
		foreach($allTasks as $task)
		{
			if($task->isLabourTask())
				$tasksLabour[$task->SAPID] = $task;
			else
				$tasksCost[$task->SAPID] = $task;
		}
		
		//Import labour tasks
		$iASAPActualsImportService = new IASAPActualsImportServiceSAPID($this->context, $tasksLabour, $multilog);
		try
		{
			switch ($iASAPActualsImportService->runLaborImport())
			{
				case iASAPActualsImportServiceSAPID::RESULT_SUCCESS:
					$this->flashMessage('iASAP actuals labour import was successful.', 'success');
					break;
				case iASAPActualsImportServiceSAPID::RESULT_WARNING:
					$this->flashMessage('iASAP actuals labour import was successful with warnings.', 'warning');
					break;
				case iASAPActualsImportServiceSAPID::RESULT_ERROR:
					$this->flashMessage('iASAP actuals labour import failed!', 'error');
					break;
				case iASAPActualsImportServiceSAPID::RESULT_NO_DATA:
					$this->flashMessage('iASAP actuals import was successful but no actuals was found.', 'warning');
					break;
				default: break;
			}
		}
		catch (IASAPActualsImportSAPIDServiceException $e)
		{
			$import_result = FALSE;
			$import_ex_error = $e->getMessage();
			$multilog->log('iASAP actuals labour import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('iASAP actuals labour import failed: ' . $e->getMessage(), Debugger::ERROR);
			$this->flashMessage('Actuals labour imported unsuccessfully.', 'error');
		}
		
		//Import cost tasks
		$iASAPActualsImportService = new IASAPActualsImportServiceSAPID($this->context, $tasksCost, $multilog);
		try
		{
			switch ($iASAPActualsImportService->runCostImport())
			{
				case iASAPActualsImportServiceSAPID::RESULT_SUCCESS:
					$this->flashMessage('iASAP actuals cost import was successful.', 'success');
					break;
				case iASAPActualsImportServiceSAPID::RESULT_WARNING:
					$this->flashMessage('iASAP actuals cost import was successful with warnings.', 'warning');
					break;
				case iASAPActualsImportServiceSAPID::RESULT_ERROR:
					$this->flashMessage('iASAP actuals cost import failed!', 'error');
					break;
				case iASAPActualsImportServiceSAPID::RESULT_NO_DATA:
					$this->flashMessage('iASAP actuals import was successful but no actuals was found.', 'warning');
					break;
				default: break;
			}
		}
		catch (IASAPActualsImportSAPIDServiceException $e)
		{
			$import_result = FALSE;
			$import_ex_error = $e->getMessage();
			$multilog->log('iASAP actuals cost import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('iASAP actuals cost import failed: ' . $e->getMessage(), Debugger::ERROR);
			$this->flashMessage('Actuals cost imported unsuccessfully.', 'error');
		}
		
		$filelog->close();
		
        $this->template->lastUpdated    = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_IASAP);
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
		$this->template->batchNumber	= $batch_id;
    }
}
