<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of project charters.
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterPresenter extends SecuredPresenter
{
	/** @var session State of number of available rows for budgets of technical groups in the form. */
	protected $session;
	/** @var string Path to storage of project charter files. */
	protected $charters_storage_dir;
	/** @var string Path to temporary storage of project charter files. */
	protected $charters_temp_storage_dir;
	/** @var string Prefix for temporary file name of charter. */
	protected $charter_temp_prefix;
	/** @var string Prefix for main directory of one project charter. */
	protected $charter_dir_prefix;
	/** @var string Prefix for directory containing one version of project charter file. */
	protected $version_dir_prefix;
	/** @var int Planning year*/
	protected $year;
	/** @const DEFAULT_TGROWS_NUMBER Default number of available rows for budgets of technical groups in the form. */
	const DEFAULT_TGROWS_NUMBER = 0;
	/** @const DEFAULT_SUBROWS_NUMBER Default number of available rows for budgets of subcontractors in the form. */
	const DEFAULT_SUBROWS_NUMBER = 0;
	/** @const If this expires, we assume the session mutex is no longer acquired.
	 *  Value is in seconds. */
	const AJAX_BEACON_TIMEOUT = 3;
	/** @var PlanningHelper Utility for managing charter's connect to the planning module. */
	private $planningHelper;
	/** @static array All rows in charter checklist (to be created for a new charter in the DB). */
	private static $checklistArray =  array('Honeywell AC', 'Other (external, customer) aircraft');

		/**
		 * Unites items starting with same letter under bookmark.
		 * @param array $array Common array: $array[key] => value
		 * @return array $array[character] => array(items starting with character)
		 */
		protected function addBookmarksToArray(array $array)
		{
			$result = array();
			// sorting items
			asort($array);
			// for each ascii character
			for ($i = 0; $i < 256; $i++)
			{
				// set current bookmark
				$bookmark = strtoupper(chr($i));
				$bookmarkItems = array();
				// if bookmark already exists go to next one
				if (in_array($bookmark, $result))
					continue;
				// search items starting with same letter as current bookmark
				foreach ($array as $key => $value)
				{
					if (strtoupper($value[0]) == $bookmark)
					{
						$bookmarkItems[$key] = $value;
					}
				}
				// something to store
				if (count($bookmarkItems))
					$result[$bookmark] = $bookmarkItems;
			}
			return $result;
		}

		/**
		 * Returns path to file.
		 * @param int $projectCharterID
		 * @param int $version File version
		 * @param string $fileName File name
		 * @return string
		 */
		protected function getPathToFile($projectCharterID, $version, $fileName)
		{
			$storage =
				'/data/actual/charters/' .
				$this->charter_dir_prefix . $projectCharterID . '/' .
				$this->version_dir_prefix . $version . '/'
			;

			return ($storage . $fileName);
		}
		
		/**
		 * Returns path to summary file.
		 * @param int $projectCharterID
		 * @param int $version File version
		 * @param string $fileName File name
		 * @return string
		 */
		protected function getPathToSummaryFile($projectCharterID, $version, $fileName)
		{
			$storage =
				'/data/actual/charters/' .
				$this->charter_dir_prefix . $projectCharterID . '/summary/' .
				$this->version_dir_prefix . $version . '/'
			;

			return ($storage . $fileName);
		}

		/**
		 * Returns path to temporary file.
		 * @param string $tmpFileName Temporary file name. Result of upload via general charter form.
		 * @return string Path to file saved in temporary storage.
		 */
		protected function getPathToTmpFile($tmpFileName)
		{
			return '/temp/charters/' . $tmpFileName;
		}

		/**
		 * Returns original file name.
		 * @param string $tmpFileName Temporary file name. Result of upload via general charter form.
		 * @return string or NULL if an error occured.
		 */
		public function getFileName($tmpFileName)
		{
			return preg_replace('/^' . $this->charter_temp_prefix . '[0-9]*_/', '', $tmpFileName);
		}

		/**
		 * Moves temporary file to destination directory.
		 * @param string $dest Destination directory.
		 * @param string $tmpFileName Temporary file name. Result of upload via general charter form.
		 * @param string $fileName New file name.
		 */
		public function moveTmpFile($dest, $tmpFileName, $fileName)
		{
			if (!is_dir($dest))
				mkdir($dest, '0777', TRUE);
			if (!rename( __DIR__ . $this->charters_temp_storage_dir . $tmpFileName,  $dest . $fileName ))
			{
				$this->flashMessage("Moving file failed.", 'error');
			}
		}

		/**
		 * Returns charter objects as array with additional informations
		 * @param $charters Array which will be supplemented by filtered items
		 * @param $filtered Array of ProjectCharter objects
		 * @return array Array indexed by ID of charter
		 */
		public function filteredChartersToArray($charters, $filtered)
		{
			foreach ($filtered as $charter)
			{
				$id = $charter->getID();
				$charters[$id]['ProjectCharter_ID'] = $id;
				$charters[$id]['Name'] = $charter->getName();
				$charters[$id]['CreationDate'] = date_format($charter->getCreationDateTime(), 'Y-m-d');
					$owner = $this->context->personRepository->find($charter->getOwner());
				$charters[$id]['Owner'] = ($owner) ? $owner->getReversedShortName() : '-';
				$charters[$id]['Owner_EID'] = ($owner) ? strtolower($owner->getID()) : '-';
				$charters[$id]['Owner_Full'] = ($owner) ? $owner->getReversedFullNameWithEID() : '-';
				$charters[$id]['SAP_ID'] = $charter->getSAP_ID();
					$lineManager = $this->context->personRepository->find($charter->getLineManager());
				$charters[$id]['LineManager'] = ($lineManager) ? $lineManager->getReversedShortName() : '-';
				$charters[$id]['LineManager_EID'] = ($lineManager) ? strtolower($lineManager->getID()) : '-';
				$charters[$id]['LineManager_Full'] = ($lineManager) ?  $lineManager->getReversedFullNameWithEID() : '-';
				$charters[$id]['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
				$charters[$id]['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
				$charters[$id]['Year'] = $charter->getYear();
				$charters[$id]['Status'] = $charter->getStatus();
				$charters[$id]['Status_Full'] = ProjectCharter::getFullStatusName($charter->getStatus());
			}
			return $charters;
		}

		/**
		 * Fills form fields.
		 * @param AppForm $form
		 * @param boolean $manager Determines fields.
		 * @param string $source Determines form component.
		 * @param ProjectCharterBudget $budgetItem Data
		 */
		public function setBudgetItemFields($form, $manager, $source, $budgetItem)
		{
			if ($manager)
			{
				$form[$source]['managerQ1']->setValue($budgetItem->getBudgetQ1());
				$form[$source]['managerQ2']->setValue($budgetItem->getBudgetQ2());
				$form[$source]['managerQ3']->setValue($budgetItem->getBudgetQ3());
				$form[$source]['managerQ4']->setValue($budgetItem->getBudgetQ4());
			}
			else
			{
				$form[$source]['ownerQ1']->setValue($budgetItem->getBudgetQ1());
				$form[$source]['ownerQ2']->setValue($budgetItem->getBudgetQ2());
				$form[$source]['ownerQ3']->setValue($budgetItem->getBudgetQ3());
				$form[$source]['ownerQ4']->setValue($budgetItem->getBudgetQ4());
			}
		}
		
		/**
		 * Restricts access to methods which requires specific combination of user role and project charter status.
		 * @param int $id Must be positive integer. It should be ID of an existing project charter.
		 * @param array $statuses Array of possible statuses for access.
		 * @param bool $isOwner Determine if currently logged in user must be in owner role or in role of line manager.
		 */
		public function secureAccess($id, $statuses, $isOwner)
		{
			$charter = $this->context->projectCharterRepository->find($id);
			if ($charter)
			{
				$user = strtolower($this->getUser()->getId());
				$watchdog = FALSE;
				
				if ($isOwner)
				{
					// must be owner
					if ($user !== strtolower($charter->getOwner()))
						$watchdog = TRUE;
				}
				else
				{
					// must be line manager
					if ($user !== strtolower($charter->getLineManager()))
						$watchdog = TRUE;
				}

				if (!in_array($charter->getStatus(), $statuses))
					$watchdog = TRUE;
				
				if ($watchdog)
				{
					$this->flashMessage('Process is not in the right phase or you have invalid credentials for manipulation with this project charter. You have been redirected.', 'warning');
					$this->redirect('Dashboard:default');
				}
			}
			else
			{
				$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
				$this->redirect('Dashboard:default');
			}			
		}

	/**
	 * Initializes common variables (e.g. paths, sessions)
	 */
	public function startup()
	{
		parent::startup();
		// initialization of session
		$this->session = $this->getSession('ProjectCharter');
		$this->session->setExpiration(0);
		if (!isset($this->session['TGRows']))
			$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
		if (!isset($this->session['SUBRows']))
			$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
		if (!isset($this->session['Form']))
			$this->session['Form'] = array();
		// initialization of path to storage of project charter files
		$this->charters_storage_dir = Environment::getConfig('charters_storage_dir');
		// initialization of path to temporary storage of project charter files
		$this->charters_temp_storage_dir = Environment::getConfig('charters_temp_storage_dir');
		// initialization of directory prefix for temporary file name of charter
		$this->charter_temp_prefix = Environment::getConfig('charter_temp_prefix');
		// initialization of directory prefixies for storage of project charter files
		$this->charter_dir_prefix = Environment::getConfig('charter_dir_prefix');
		$this->version_dir_prefix = Environment::getConfig('version_dir_prefix');

		// set up the planning year as follows:
		// If we've come from the planning module, then take year from our URL,
		// otherwise the charter already exists, so take it from the charter property
		$params = $this->request->getParameters();
		if ($this->getAction() === 'createStepII' && isset($params['year']))
		{
			$this->year = $params['year'];
		}
		else if (isset($params['id']))
		{
			// this takes care of already existing charter - give it the yeas it's been planned for
			$this->year = $this->context->projectCharterRepository->findYearByProjectCharterID($params['id']);
		}
		
		$this->planningHelper = new PlanningHelper($this->context, $this->year);
	}

	/**
	 * Returns programs belonging to customer.
	 * @param int $customer Should be customer ID.
	 * @return
	 */
	public function actionGetCustomerPrograms($customer)
	{
		$programs = $this->context->customerProgramRepository->findByCustomer($customer);
		$customerPrograms = array();
		foreach ($programs as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
		
		$this->payload->customerPrograms = $customerPrograms;
		$this->terminate();
	}
	
	/**
	 * Returns program calls belonging to customer program.
	 * @param int $program Should be customer program ID.
	 * @return
	 */
	public function actionGetCustomerProgramCalls($program)
	{
		$calls = $this->context->customerProgramCallRepository->findByProgram($program);
		$customerProgramCalls = array();
		foreach ($calls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
		
		$this->payload->customerProgramCalls = $customerProgramCalls;
		$this->terminate();
	}
	
	/**
	 * Is called when user has open an instance of Charter - Create/Edit window
	 * and so holds the mutex.
	 * @return
	 */
	public function actionUserRefreshEditingStatus($removeFlag = FALSE)
	{
		if($this->isAjax())
		{
			if($removeFlag)
			{
				$this->session->creatingOrEditingFlag = NULL;
			}
			else
			{
				// updates last timestamp
				$this->session->creatingOrEditingFlag = time();
			}
			$this->terminate();
		}
	}
	
	/**
	 * Handle for auto completion of some fields in the form.
	 */
	public function handleAutoComplete()
	{
	   $this->payload->autoComplete = array();

	   $text = trim($_GET['q']);

	   if ($text !== '')
	   {
		   $filter = new PersonFilter();
		   $filter->setFirstname($text)->setSurname($text)->setID($text);
		   $persons = $this->context->personRepository->getPersonsByOrFilter($filter);
		   foreach ($persons as $person)
		   {
			   $person_item = array($person->getReversedFullNameWithEID(), strtoupper(trim($person->ID)), $person->getReversedFullName());
			   $this->payload->autoComplete[] = $person_item;
		   }
	   }
	   $this->terminate();
	}
	
	/**
	 * Processes user's request and sends back the list of all people that could be
	 * possibly set as charter owner, while it also takes care of the filtering.
	 * 
	 * @param string $input The text that has been typed into search field by user.
	 */
	public function handlePersonList($input)
	{
		if ($this->presenter->isAjax())
		{
			$text = trim($input);
			if ($text !== '')
			{
				$filter = new PersonFilter();
				$filter->setFirstname($text)->setSurname($text)->setID($text);
				$people = $this->context->personRepository->getPersonsByOrFilter($filter);
			}
			else
			{
				$people = $this->context->personRepository->findAll();
			}

			$list = array();
			foreach ($people as $person)
			{
				if (!$person->Status || substr($person->ID, 0, 2) === 'TG')
					continue;
				
				$personItem = array('id' => strtoupper(trim($person->ID)), 'text' => $person->getReversedFullNameWithEID(), 'disabled' => false);
				$list[] = $personItem;
			}

			$this->presenter->payload->items = $list;
			$this->presenter->terminate();
		}
	}

	private function lockBudget($form)
	{
		// always lock only the PIPM part
		for ($i = 0; $i < $this->session['TGRows']; $i++)
		{
			foreach ($form['TGRow' . $i]->getComponents() as $control)
			{
				if ($control instanceof TextInput)
				{
					$control->setAttribute('readonly', 'readonly');
				}
				else if ($control instanceof SubmitButton)
					$control->setDisabled(TRUE);
			}
		}
		
		for ($i = 0; $i < $this->session['SUBRows']; $i++)
		{
			foreach ($form['SUBRow' . $i]->getComponents() as $control)
			{
				if ($control instanceof TextInput)
					$control->setAttribute('readonly', 'readonly');
				else if ($control instanceof SubmitButton)
					$control->setDisabled(TRUE);
			}
		}
		
		foreach ($form['Travel']->getComponents() as $control)
		{
			if ($control instanceof TextInput)
				$control->setAttribute('readonly', 'readonly');
		}
		
		foreach ($form['Material']->getComponents() as $control)
		{
			if ($control instanceof TextInput)
				$control->setAttribute('readonly', 'readonly');
		}
		
		// also lock budget buttons
		$form['addTGRow']->setDisabled(TRUE);
		$form['addSUBRow']->setDisabled(TRUE);
		if (isset($form['copyButtonCL']))
			$form['copyButtonCL']->setDisabled(TRUE);
	}
	
	/**
	 * Sets common content of templates
	 */
	public function beforeRender()
	{
		parent::beforeRender();
		// form tool tips
		$this->template->TIP_CHARTER_NAME = 'Charter name should be the same as name of the project.';
		$this->template->TIP_COST_SHARE = 'Cost share is percentage of costs covered by the customer.';
		$this->template->TIP_AMOUNT_HRS = 'Working hours of technical group per year quarters.<br/>For assignment of people to technical groups, please click <a href="http://orgchart.honeywell.com:8080/eGuide/servlet/eGuide" target="_blank">here</a>.';
		$this->template->TIP_SUBCONTRACTORS = 'Amounts for subcontractors per year quarters.';
		$this->template->TIP_OWNER = 'User responsible for creation of project charter.';
		$this->template->TIP_LINE_MANAGER = 'Should be cluster leader (CL) in common case.';
		$this->template->TIP_SUM_OF_TOTALS = 'Sum includes non-labor costs and labors cost which are calculated based on the labor rates corresponding to the technical groups in given year.';
		$this->template->TIP_APPROVE_DISABLED = 'Charter cannot be approved because there are differences in budget and/or charter summary and/or charter document.';
		$this->template->TIP_EC_LICENCE = 'Export control licence determines importance of given data. It usually depends on target customer.';
		$this->template->LEGEND_DIFFERENCE_I = 'Difference between PIPM and CL proposals are ';
		$this->template->LEGEND_DIFFERENCE_II = 'highlighted.';
		$this->template->LEGEND_LATEST_VERSION_I = 'Latest version of charter document is ';
		$this->template->LEGEND_LATEST_VERSION_I_BUDGET = 'Latest version of budget is ';
		$this->template->LEGEND_LATEST_VERSION_II = 'highlighted.';
		// form labels
		$this->template->percent = '%';
		$this->template->unitHRS = '(HRS)';
		$this->template->unitUSD = '(USD)';
		$this->template->LABEL_OWNER_INPUT = 'PIPM';
		$this->template->LABEL_MANAGER_INPUT = 'CL';
		// project charter statuses
		$this->template->PROJECT_CHARTER_STATUS_APPROVED = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED;
		$this->template->PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION = ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION;
		$this->template->PROJECT_CHARTER_STATUS_CANCELLED = ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED;
		$this->template->PROJECT_CHARTER_STATUS_DEFINITION = ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION;
		$this->template->PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED = ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED;
		$this->template->PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL = ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL;
		$this->template->PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL = ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL;
		// Export Control
		$all_licences = $this->context->ecLicenceRepository->findAll();
		$licences = array();
		foreach ($all_licences as $licence)
			$licences[$licence->ECLicenceID] = $licence->Name;
		$this->template->licences = $licences;
	}

	/**
	 * Shows default page with project charters overview.
	 */
	public function renderDefault()
	{
		// initialization of number of available rows for budgets of technical groups in the form
		$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
		$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
		
		$this->template->form = $this['filterCharterForm'];
		$filtered_charters = array();
		
		if ($this->isAjax()) // is ajax request
		{
			$filter = new ProjectCharterFilter();
			if (isset($this->payload->filter)) // filter is defined
			{
				$filter->Name = $this->payload->filter['Name'];
				$filter->Owner = $this->payload->filter['Owner'];
				$filter->SapID = $this->payload->filter['SAP_ID'];
				$filter->LineManager = $this->payload->filter['LineManager'];
				$filter->CustomerID = $this->payload->filter['Customer_ID'];
				$filter->ProgramID = $this->payload->filter['ProgramID'];
				$filter->CallID = $this->payload->filter['CallID'];
				$filter->ResponsibilityID = $this->payload->filter['Responsibility_ID'];
				$filter->Year = $this->payload->filter['Year'];
				$filter->Status = $this->payload->filter['Status'];
			}
			
			$this->context->formStateRepository->save('filterCharterForm', $this->template->form); // save filter state
			$filtered_charters = $this->context->projectCharterRepository->getChartersByFilter($filter);			
		}
		else
		{
			if ($this->context->formStateRepository->exists('filterCharterForm')) // exists saved filter
				$this->context->formStateRepository->load('filterCharterForm', $this->template->form);
		}
		
		$charters = array(); // charters to be shown
		foreach ($filtered_charters as $charter)
		{ 
			$item = array();
			$item['ProjectCharter_ID'] = $charter->ID;
			$item['Name'] = $charter->Name;
			$item['CreationDate'] = date_format($charter->CreationDateTime, 'Y-m-d');
				$owner = $this->context->personRepository->find($charter->Owner);
			$item['Owner'] = ($owner) ? $owner->getReversedShortName() : '-';
			$item['Owner_Full'] = ($owner) ? $owner->ReversedFullNameWithEID : '-';
			$item['SAP_ID'] = $charter->SAP_ID;
				$lineManager = $this->context->personRepository->find($charter->LineManager);
			$item['LineManager'] = ($lineManager) ? $lineManager->ReversedShortName : '-';
			$item['LineManager_Full'] = ($lineManager) ?  $lineManager->ReversedFullNameWithEID : '-';
				$customer = $this->context->customerRepository->find($charter->CustomerID);
			$item['Customer_ID'] = ($customer) ? $customer->Name : '-';
				$responsibility = $this->context->responsibilityRepository->find($charter->ResponsibilityID);
			$item['Responsibility_ID'] = ($responsibility) ? $responsibility->Name : '-';
			$item['Year'] = $charter->Year;
			$item['Status'] = ProjectCharter::getFullStatusName($charter->Status);
			$item['ECLicenceID'] = $charter->getECLicenceID();
			$charters[] = $item;
		}
		$this->template->charters = $charters;
		$this->setCustomerDependencies($this->template->form);
	}

	/**
	 * Shows charters waiting for action of currently logged user
	 */
	public function renderMyAction()
	{
		// initialization of number of available rows for budgets of technical groups in the form
		$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
		$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
		
		$this->template->form = $this['filterCharterForm'];
		
		if ($this->isAjax()) // is ajax request
		{			
			$this->context->formStateRepository->save('filterCharterFormMyAction', $this->template->form); // save filter state			
		}
		else
		{
			if ($this->context->formStateRepository->exists('filterCharterFormMyAction')) // exists saved filter
				$this->context->formStateRepository->load('filterCharterFormMyAction', $this->template->form);
		}
		
		$charters = array(); // charters to be shown
		$filtered_charters = array();
		$filter = new ProjectCharterFilter();
		if (isset($this->payload->filter)) // filter is defined
		{
			$filter->Name = $this->payload->filter['Name'];
			$filter->SapID = $this->payload->filter['SAP_ID'];
			$filter->CustomerID = $this->payload->filter['Customer_ID'];
			$filter->ProgramID = $this->payload->filter['ProgramID'];
			$filter->CallID = $this->payload->filter['CallID'];
			$filter->ResponsibilityID = $this->payload->filter['Responsibility_ID'];
			$filter->Year = $this->payload->filter['Year'];
			$filter->Status = $this->payload->filter['Status'];
		}

		// I am owner
		$filter->setOwner($this->getUser()->getId());
		$filtered_charters = $this->context->projectCharterRepository->getChartersByFilter($filter);
		$charters = $this->filteredChartersToArray($charters, $filtered_charters);

		// I am line manager
		$filter = new ProjectCharterFilter();
		$filter->setLineManager($this->getUser()->getId());
		$filter->setStatus(array(0 => ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL));
		$filtered_charters = $this->context->projectCharterRepository->getChartersByFilter($filter);
		$charters = $this->filteredChartersToArray($charters, $filtered_charters);

		//I am final approver
		$filter = new ProjectCharterFilter();
		$filter->setStatus(array(0 => ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL));
		$filtered_charters = $this->context->projectCharterRepository->getChartersByFilter($filter);
		$charters = $this->filteredChartersToArray($charters, $filtered_charters);

		$this->template->charters = $charters;
		$this->setCustomerDependencies($this->template->form);
		
		/* List of final approvers */
		$approvers = array();
		foreach($charters as $charter)
		{
			$version = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($charter['ProjectCharter_ID']);
			if($version)
			{
				$approvers[$charter['ProjectCharter_ID']] = $this->context->projectCharterApproverRepository->findAllApprover($charter['ProjectCharter_ID'], $version->Version);
			}
			$this->template->approvers = $approvers;
		}
	}

	/**
	 * Shows detail of charter.
	 * @param int $id Must be positive integer. It should be ID of an existing project charter.
	 */
	public function renderDetail($id = NULL)
	{
		// check project charter ID
		$charter = $this->context->projectCharterRepository->find($id);
		if ($charter == NULL)
		{
			$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
			$this->redirect('Dashboard:default');
		}

		$form = $this['detailCharterForm'];
		$form['ProjectCharter_ID']->setValue($id);
		$latestVersion = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($charter->ID);
		// Disable button for final approval
		if(!is_null($latestVersion))
		{
			if(count($this->context->projectCharterApproverRepository->findByApprover($charter->ID, $latestVersion->Version, $this->getUser()->getId())) > 0)
			{
				if(($charter->getStatus() !== ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL))
				$form['finalApproval']->setDisabled(TRUE);
			}
			else
			{
				$form['finalApproval']->setDisabled(TRUE);
			}
		}
		else
		{
			$form['finalApproval']->setDisabled(TRUE);
		}
		// Disable button for line manager
		if ((strtolower($this->getUser()->getId()) == strtolower($charter->getLineManager())))
		{
			if(($charter->getStatus() !== ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL))
				$form['clApprover']->setDisabled(TRUE);
		}
		else
		{
			$form['clApprover']->setDisabled(TRUE);
		}
		// Disable buttons for owner
		if ((strtolower($this->getUser()->getId()) == strtolower($charter->getOwner())))
		{
			// if current state is not in group of end states [APPROVED, CANCELLED] or in state FINAL APPROVERS DEFINITION or DISAPPROVE RECEIVED user can't restore charter
			if (!in_array($charter->getStatus(), array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED, ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED, ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION, ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED)))
				$form['restore']->setDisabled(TRUE);
			 // if current state is not in group of states where as owner is able change data [DEFINITION] user can't edit charter
			if (!($charter->getStatus() == ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION))
				$form['edit']->setDisabled(TRUE);
			if($charter->getStatus() == ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED)
				$form['restore']->setAttribute('onclick', '
					return confirm("If you choose to redefine, all the previous approvals will be discarded. Do you want to continue?");
				');
			if(($charter->getStatus() !== ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION))
				$form['finalApproversDefinition']->setDisabled(TRUE);
			if(($charter->getStatus() !== ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED))
				$form['missingApproval']->setDisabled(TRUE);
		}
		else
		{
			$form['restore']->setDisabled(TRUE);
			$form['edit']->setDisabled(TRUE);
			$form['finalApproversDefinition']->setDisabled(TRUE);
			$form['missingApproval']->setDisabled(TRUE);
		}

		// set charter basic info
		$arrCharter = array();
		$arrCharter['SAP_ID'] = $charter->getSAP_ID();
		$arrCharter['Name'] = $charter->getName();
			$owner = $this->context->personRepository->find($charter->getOwner());
		$arrCharter['Owner'] = ($owner) ? $owner->getReversedFullNameWithEID() : '-';
			$lineManager = $this->context->personRepository->find($charter->getLineManager());
		$arrCharter['LineManager'] = ($lineManager) ?  $lineManager->getReversedFullNameWithEID() : '-';
			$strapItem = $this->context->itemRepository->find($charter->StrapItemID);
		$arrCharter['StrapItem'] = ($strapItem) ? $strapItem->Name : '-';
		$arrCharter['StrapItemID'] = ($strapItem) ? $strapItem->ID : NULL;
		$arrCharter['Status'] = ProjectCharter::getFullStatusName($charter->getStatus());
		$arrCharter['CreationDateTime'] = $charter->getCreationDateTime();
		$arrCharter['CostShare'] = $charter->getCostSharePercentage();
		$arrCharter['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
			$program = $this->context->customerProgramRepository->find($charter->getProgramID());
		$arrCharter['ProgramID'] = ($program) ? $program->getProgram() : NULL;
			$call = $this->context->customerProgramCallRepository->find($charter->getCallID());
		$arrCharter['CallID'] = ($call) ? $call->getCall() : NULL;
		$arrCharter['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
		$arrCharter['Year'] = $charter->getYear();
		$arrCharter['ECLicenceID'] = $charter->getECLicenceID();


		$file = $this->context->projectCharterFileHistoryRepository->findLatestVersion($charter->getID());
		if ($file)
		{
			$this->template->filePath = $this->getPathToFile($file->getProjectCharterID(), $file->getVersion(), $file->getFileName());
			$this->template->fileName = $file->getFileName();
		}
		
		$summaryFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($charter->getID());
		if ($summaryFile)
		{
			$this->template->summaryFilePath = $this->getPathToSummaryFile($summaryFile->getProjectCharterID(), $summaryFile->getVersion(), $summaryFile->getFileName());
			$this->template->summaryFileName = $summaryFile->getFileName();
		}

		$groups = array(); $subcontractors = array(); $others = array(); $sumOfTotals = 0;
		$budget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->getID());
		if ($budget != NULL)
		{
			$budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($budget->getProjectCharterID(), $budget->getVersion());
			if ($budgetItems != NULL)
			{
				foreach ($budgetItems as $budgetItem)
				{
					if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
					{
							$orgunit = $this->context->organizationUnitRepository->find($budgetItem->getTechnicalGroup());
							if ($orgunit)
							{
								$groups[$budgetItem->getID()]['Name'] = $orgunit->getName();
								$groups[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
								$groups[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
								$groups[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
								$groups[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
								$groups[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
								$sumOfTotals += $groups[$budgetItem->getID()]['Total'] * $budgetItem->getRate();
							}
					}
					elseif ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING)
					{
						$subcontractor = $this->context->subcontractorRepository->find($budgetItem->getSubcontractor());
						if ($subcontractor)
						{
							$subcontractors[$budgetItem->getID()]['Name'] = $subcontractor->getName();
							$subcontractors[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
							$subcontractors[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
							$subcontractors[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
							$subcontractors[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
							$subcontractors[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
							$sumOfTotals += $subcontractors[$budgetItem->getID()]['Total'];
						}
					}
					else
					{
						switch ($budgetItem->getResourceType())
						{
							case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL:
								$name = 'Travel';
								break;
							case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL:
								$name = 'Material';
								break;
							default: break;
						}
						$others[$budgetItem->getID()]['Name'] = $name;
						$others[$budgetItem->getID()]['Q1'] = $budgetItem->getBudgetQ1();
						$others[$budgetItem->getID()]['Q2'] = $budgetItem->getBudgetQ2();
						$others[$budgetItem->getID()]['Q3'] = $budgetItem->getBudgetQ3();
						$others[$budgetItem->getID()]['Q4'] = $budgetItem->getBudgetQ4();
						$others[$budgetItem->getID()]['Total'] = $budgetItem->getBudgetQ1() + $budgetItem->getBudgetQ2() + $budgetItem->getBudgetQ3() + $budgetItem->getBudgetQ4();
						$sumOfTotals += $others[$budgetItem->getID()]['Total'];
					}
				}
			}
		}

		$this->template->arrCharter = $arrCharter;
		$this->template->currentCharterStatus = $charter->getStatus();
		$this->template->groups = $groups;
		$this->template->others = $others;
		$this->template->subcontractors = $subcontractors;
		$this->template->sumOfTotals = $sumOfTotals;

		/* List of final approvers */
		$version = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($id);
		if($version)
		{
			$approvers = array();
			$approvers = $this->context->projectCharterApproverRepository->findAllApprover($id, $version->Version);
			$this->template->approvers = $approvers;
		}
		
		// Checklist
		$checklistItems = $this->context->projectCharterChecklistRepository->findByProjectCharterID($charter->ID);
		foreach ($checklistItems as $item)
		{
			$tempArray = array();
			for ($i = 1; $i <= 5; $i++)
				$tempArray[$i] = ($item->NeededArray[$i]) ? 'Yes' : 'No';
			$item->NeededArray = $tempArray;
			$checklistItems[$item->Title] = $item;
		}
		if (count($checklistItems))
			$this->template->checklist = $checklistItems;	
}

	/**
	 * Shows page for creation of project charter (step I)
	 */
	public function renderCreateStepI()
	{
		$this->creatingOrEditingFlagRedirect();
		// initialization
		$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
		$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
	}

	/**
	 * Shows page for creation of project charter (step II)
	 * @param int $id Must be positive integer. It should be ID of an existing project.
	 * @param string $name Name of a new project.
	 * @param int $year Year of the intended planning.
	 */
	public function renderCreateStepII($id = NULL, $name = NULL, $year = NULL)
	{
		if (!$year)
		{
			$this->flashMessage('Planning year was not specified.', 'error');
			$this->redirect('ProjectCharter:default');
		}
		
		$this->creatingOrEditingFlagRedirect();
		
		// initialization
		$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
		$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
		
		// select form
		$form = $this['createCharterForm'];

		// pre fill form if it is empty yet.
		if (!$form['ProjectCharter_ID']->getValue())
		{
			$form['ProjectCharter_ID']->setValue('locked');
			$form['Owner']->setValue(strtoupper($this->getUser()->getId()));
			$form['Year']->setValue($year);

			// fetch project with given ID
			if ($id)
			{
				$project = $this->context->projectRepository->find($id);
				if (!$project)
				{
					$this->flashMessage('Invalid project ID. Try to choose project from existing. You have been redirected.', 'warning');
					$this->redirect('ProjectCharter:default');
				}
				$form['Name']->setValue($project->getName());
				$form['SAP_ID']->setValue($project->getSAP_ID());
				$form['Customer_ID']->setValue($project->getCustomer());
				$form['Responsibility_ID']->setValue($project->getResponsibility());
				// Charter inherits project ECO state
				$form['ECLicenceID']->setValue($project->ECLicenceID);
				$form['LineManager']->setValue($project->TM);
				$form['ProgramID']->setValue($project->ProgramID);
			}
			else if ($name)
			{
				$form['Name']->setValue($name);
			}
		}

		// preload planning data and average cost share
		$form['CostShare']->setValue(round($this->preloadPlanningData($form) * 100, 0));
		
		// lock budget
		$params = $this->request->getParameters();
		// id supplied by URL is Project ID
		$planningState = $this->context->projectPlanningCharterStateRepository->find($params['id'], $this->year);
		
		if ($planningState !== NULL && !$planningState->AtLeastOnceFinished)
			$this->lockBudget($form);
		
		// pre fill form by previous values
		if (count($this->session['Form']))
		{
			$form->setDefaults($this->session['Form']);
			$this->session['Form'] = array();
		}

		// show link to file if it was uploaded
		if ($form['TempFile']->getValue())
		{
			$this->template->currentFilePath = $this->getPathToTmpFile($form['TempFile']->getValue());
			$this->template->currentFileName = $this->getFileName($form['TempFile']->getValue());
		}
		// show link to summary file if it was uploaded
		if ($form['TempSummaryFile']->getValue())
		{
			$this->template->currentSummaryFilePath = $this->getPathToTmpFile($form['TempSummaryFile']->getValue());
			$this->template->currentSummaryFileName = $this->getFileName($form['TempSummaryFile']->getValue());
		}
		
		// set restrictions on form
		$arrTGSource = array();
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			array_push($arrTGSource, 'TGRow' . $row);
		// add rule for unique value in form
		foreach ($arrTGSource as $current)
		{
			foreach ($arrTGSource as $other)
				if ($current != $other)
					$form[$current]['TechnicalGroup']->addRule(~Form::EQUAL, 'Each technical group may be selected only once.', $form[$other]['TechnicalGroup']);
		}
		
		$arrSUBSource = array();
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			array_push($arrSUBSource, 'SUBRow' . $row);		
		// add rule for unique value in form
		foreach ($arrSUBSource as $current)
		{
			foreach ($arrSUBSource as $other)
				if ($current != $other)
					$form[$current]['Subcontractor']->addRule(~Form::EQUAL, 'Each subcontractor may be selected only once.', $form[$other]['Subcontractor']);
		}
		
		// Checklist
		$this->template->checklist = self::$checklistArray;
		
		$this->template->showHeadlineForExistingProject = $id ? TRUE : FALSE;
		$this->template->TGRows = $this->session['TGRows'];
		$this->template->SUBRows = $this->session['SUBRows'];
		$this->attachLaborRates($this->template);
		$this->setCustomerDependencies($form, TRUE);
	}

	/**
	 * Shows page for editation of project charter.
	 * @param int $id Must be positive integer. It should be ID of an existing project charter.
	 */
	public function renderEdit($id = NULL)
	{
		// check accessibility for user
		$this->secureAccess($id, array(ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION), TRUE);
		
		$this->creatingOrEditingFlagRedirect();
		
		// select form
		$form = $this['editCharterForm'];
		
		// pre fill form by previous values
		if (count($this->session['Form']))
		{
			$form->setDefaults($this->session['Form']);
			$this->session['Form'] = array();
		}

		$charterID = ($id) ? $id : $form['ProjectCharter_ID']->getValue();
		
		// fetch charter according to ID and check it
		$charterFromDB = $this->context->projectCharterRepository->find($charterID);
		if (!$charterFromDB)
		{
			$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
			$this->redirect('Dashboard:default');
		}
		
		if (!$form['ProjectCharter_ID']->getValue())
		{
			// prefill form. ID must be set at the end of the function
			$form['Name']->setValue($charterFromDB->getName());
			$form['SAP_ID']->setValue($charterFromDB->getSAP_ID());
			$form['LineManager']->setValue($charterFromDB->getLineManager());
			$form['StrapItem']->setValue($charterFromDB->getStrapItemID());
			$form['CostShare']->setValue($charterFromDB->getCostSharePercentage());
			$form['Responsibility_ID']->setValue($charterFromDB->getResponsibilityID());
			$form['Customer_ID']->setValue($charterFromDB->getCustomerID());
			$form['ProgramID']->setValue($charterFromDB->getProgramID());
			$form['CallID']->setValue($charterFromDB->getCallID());
			$form['Year']->setValue($charterFromDB->getYear());
			$form['Status']->setValue($charterFromDB->getStatus());
			$form['ECLicenceID']->setValue($charterFromDB->getECLicenceID());
			$charter = $charterFromDB;
		}
		else
		{
			$charter = new ProjectCharter();
			$charter->setID($form['ProjectCharter_ID']->getValue());
			$charter->setStatus($form['Status']->getValue());
		}
		
		// charter's owner can't be changed
		$form['Owner']->setValue($charterFromDB->getLineManager());
		$form['Owner']->setValue($charterFromDB->getOwner());

		// get data for table of file history
		$arrFileHistory = $this->context->projectCharterFileHistoryRepository->findByProjectCharterID($charter->getID());
		$arrFileInfo = array();
		foreach ($arrFileHistory as $fileHistory)
		{
			$fileInfo = array();
			$fileInfo['Version'] = $fileHistory->getVersion();
			$fileInfo['CreationDateTime'] = $fileHistory->getCreationDateTime();
			$fileInfo['FileName'] = $fileHistory->getFileName();
			$fileInfo['Path'] = $this->getPathToFile($fileHistory->getProjectCharterID(), $fileHistory->getVersion(), $fileHistory->getFileName());
				$responsible = $this->context->personRepository->find($fileHistory->getResponsible());
			$fileInfo['Responsible'] = $responsible->getReversedFullName();
			$fileInfo['Responsible_Full'] = $responsible->getReversedFullNameWithEID();
			$fileInfo['OwnerComment'] = $fileHistory->getOwnerComment();
			$fileInfo['LineManagerComment'] = $fileHistory->getLineManagerComment();
			$fileInfo['LatestVersion'] = $fileHistory->getLatestVersion();
			$arrFileInfo[] = $fileInfo;
			
			// set file version of the latest file
			if ($fileHistory->getLatestVersion())
			{
				$form['FileVersion']->setValue($fileHistory->getVersion());
				$form['OwnerFileComment']->setValue($fileHistory->getOwnerComment());
				if (!$form['LineManagerFileComment']->getValue())
					$form['LineManagerFileComment']->setValue($fileHistory->getLineManagerComment());
			}
		}
		
		// get data for table of summary file history
		$arrFileHistory = $this->context->projectCharterSummaryFileHistoryRepository->findByProjectCharterID($charter->getID());
		$arrSummaryFileInfo = array();
		foreach ($arrFileHistory as $fileHistory)
		{
			$fileInfo = array();
			$fileInfo['Version'] = $fileHistory->getVersion();
			$fileInfo['CreationDateTime'] = $fileHistory->getCreationDateTime();
			$fileInfo['FileName'] = $fileHistory->getFileName();
			$fileInfo['Path'] = $this->getPathToSummaryFile($fileHistory->getProjectCharterID(), $fileHistory->getVersion(), $fileHistory->getFileName());
				$responsible = $this->context->personRepository->find($fileHistory->getResponsible());
			$fileInfo['Responsible'] = $responsible->getReversedFullName();
			$fileInfo['Responsible_Full'] = $responsible->getReversedFullNameWithEID();
			$fileInfo['OwnerComment'] = $fileHistory->getOwnerComment();
			$fileInfo['LineManagerComment'] = $fileHistory->getLineManagerComment();
			$fileInfo['LatestVersion'] = $fileHistory->getLatestVersion();
			$arrSummaryFileInfo[] = $fileInfo;
			
			// set file version of the latest file
			if ($fileHistory->getLatestVersion())
			{
				$form['SummaryFileVersion']->setValue($fileHistory->getVersion());
				$form['OwnerSummaryFileComment']->setValue($fileHistory->getOwnerComment());
				if (!$form['LineManagerFileComment']->getValue())
					$form['LineManagerFileComment']->setValue($fileHistory->getLineManagerComment());
			}
		}
		
		// get data for table of budget history
		$arrBudgetHistory = $this->context->projectCharterBudgetHistoryRepository->findByProjectCharterID($charter->getID());
		$arrBudgetInfo = array();
		foreach ($arrBudgetHistory as $budgetHistory)
		{
			$budgetInfo = array();
			$budgetInfo['Version'] = $budgetHistory->getVersion();
			$budgetInfo['CreationDateTime'] = $budgetHistory->getCreationDateTime();
				$responsible = $this->context->personRepository->find($budgetHistory->getResponsible());
			$budgetInfo['Responsible'] = $responsible->getReversedFullName();
			$budgetInfo['Responsible_Full'] = $responsible->getReversedFullNameWithEID();
			$budgetInfo['Comment'] = $budgetHistory->getComment();
			$budgetInfo['LatestVersion'] = $budgetHistory->getLatestVersion();
			$arrBudgetInfo[] = $budgetInfo;
			
			// set file version of the latest file
			if ($budgetHistory->getLatestVersion() && (strtolower($budgetHistory->getResponsible()) == strtolower($this->getUser()->getId())))
			{
				$form['BudgetComment']->setValue($budgetHistory->getComment());
			}
		}
		$this->template->arrBudgetInfo = $arrBudgetInfo;

		if (!$form['ProjectCharter_ID']->getValue())
		{
			$latestBudget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->getID());
			$charterOwnerEqualsLM = strtolower($charter->getOwner()) === strtolower($charter->getLineManager());
			if ($latestBudget)
			{
				$form['BudgetVersion']->setValue($latestBudget->getVersion());

				$budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($latestBudget->getProjectCharterID(), $latestBudget->getVersion());
				$TGrow = $SUBrow = 0;
				foreach ($budgetItems as $budgetItem)
				{
					switch ($budgetItem->getResourceType())
					{
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL:
							$source = 'Travel' ; break;
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL:
							$source = 'Material'; break;
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING:
							$source = 'SUBRow' . $SUBrow;
							$this->addSUBRowToForm($form, $SUBrow);
							$form[$source]['Subcontractor']->setValue($budgetItem->getSubcontractor());
							$SUBrow++; break;
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS:
							$source = 'TGRow' . $TGrow;
							$this->addTGRowToForm($form, $TGrow);
							$form[$source]['TechnicalGroup']->setValue($budgetItem->getTechnicalGroup());
							$TGrow++; break;
						default: break;
					}

					$form[$source]['ProjectCharterBudget_ID']->setValue($budgetItem->getID());
					if (strtolower($latestBudget->getResponsible()) == strtolower($this->getUser()->getId()))
					{
						$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
						// charter owner is same person as selected line manager
						if ($charterOwnerEqualsLM)
							$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
					}
					else
					{
						$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
					}
				}

				$this->session['TGRows'] = $TGrow;
				$this->session['SUBRows'] = $SUBrow;

				$oldBudget = $this->context->projectCharterBudgetHistoryRepository->find($latestBudget->getProjectCharterID(), $latestBudget->getVersion() - 1);
				if ($oldBudget && !$charterOwnerEqualsLM)
				{
					$budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($oldBudget->getProjectCharterID(), $oldBudget->getVersion());
					foreach ($budgetItems as $budgetItem)
					{
						if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL)
						{
							$source = 'Travel';
							if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
								$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
							else
								$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
						}
						else if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL)
						{
							$source = 'Material';
							if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
								$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
							else
								$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
						}
						else if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
						{
							for ($row = 0; $row < $this->session['TGRows']; $row++)
							{
								$source = 'TGRow' . $row;
								if ($form[$source]['TechnicalGroup']->getValue() == $budgetItem->getTechnicalGroup())
								{
									if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
										$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
									else
										$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
									break;
								}
							}
						}
						else if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING)
						{
							for ($row = 0; $row < $this->session['SUBRows']; $row++)
							{
								$source = 'SUBRow' . $row;
								if ($form[$source]['Subcontractor']->getValue() == $budgetItem->getSubcontractor())
								{
									if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
										$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
									else
										$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
									break;
								}
							}
						}
					}
				}
			}
			// set charter ID as a lock against another autofill
			$form['ProjectCharter_ID']->setValue($charter->getID());
		}

		// lock budget - must be done after TGRows and SUBRows session sections
		// are correctly set up
		$planningState = $this->context->projectPlanningCharterStateRepository->findByProjectCharterIDAndYear($charterFromDB->ID, $this->year);
		if ($planningState !== NULL && !$planningState->AtLeastOnceFinished)
		{
			$this->lockBudget($form);
			$this->template->budgetLocked = TRUE;
		}
		
		// define list of budget items source types
		$arrBudgetSource = array('Travel', 'Material');
		// technical groups
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			array_push($arrBudgetSource, 'TGRow' . $row);
		// subcontractors
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			array_push($arrBudgetSource, 'SUBRow' . $row);
		//
		foreach ($arrBudgetSource as $source)
		{
			$form[$source]['managerQ1']->setAttribute('readonly', 'readonly');
			$form[$source]['managerQ2']->setAttribute('readonly', 'readonly');
			$form[$source]['managerQ3']->setAttribute('readonly', 'readonly');
			$form[$source]['managerQ4']->setAttribute('readonly', 'readonly');
		}

		if ($form['TempFile']->getValue())
		{
			$this->template->tempFile = $this->getPathToTmpFile($form['TempFile']->getValue());
			$this->template->tempFileName = $this->getFileName($form['TempFile']->getValue());
		}
		if ($form['TempSummaryFile']->getValue())
		{
			$this->template->tempSummaryFile = $this->getPathToTmpFile($form['TempSummaryFile']->getValue());
			$this->template->tempSummaryFileName = $this->getFileName($form['TempSummaryFile']->getValue());
		}
		
		$this->template->arrFileInfo = $arrFileInfo;
		$this->template->arrSummaryFileInfo = $arrSummaryFileInfo;
		$this->template->TGRows = $this->session['TGRows'];
		$this->template->SUBRows = $this->session['SUBRows'];
		
		$arrTGSource = array();
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			array_push($arrTGSource, 'TGRow' . $row);		
		// add rule for unique value in form
		foreach ($arrTGSource as $current)
		{
			foreach ($arrTGSource as $other)
			{
				if ($current != $other)
				{
					$form[$current]['TechnicalGroup']->addRule(~Form::EQUAL, 'Each technical group may be selected only once.', $form[$other]['TechnicalGroup']);
				}
			}
		}
		
		$arrSUBSource = array();
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			array_push($arrSUBSource, 'SUBRow' . $row);
		// add rule for unique value in form
		foreach ($arrSUBSource as $current)
		{
			foreach ($arrSUBSource as $other)
			{
				if ($current != $other)
				{
					$form[$current]['Subcontractor']->addRule(~Form::EQUAL, 'Each subcontractor may be selected only once.', $form[$other]['Subcontractor']);
				}
			}
		}
		
		$checklistItems = $this->context->projectCharterChecklistRepository->findByProjectCharterID($charter->ID);
		if (count($checklistItems))
		{
			for ($i = 0; $i < count(self::$checklistArray); $i++)
			{
				for ($j = 1; $j <= 5; $j++)
					$form['Checklist' . $i]['Needed' . $j]->setValue($checklistItems[self::$checklistArray[$i]]->NeededArray[$j]);
			}

			$this->template->checklist = self::$checklistArray;
		}
		
		$form['Owner']->setDisabled(TRUE);
		$form['LineManager']->setDisabled(TRUE);
		$this->attachLaborRates($this->template, $charter);
		$this->setCustomerDependencies($form, TRUE);
	}

	/**
	 *
	 */
	public function renderLMApproval($id = NULL)
	{
		// check accessibility for user
		$this->secureAccess($id, array(ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL), FALSE);
		
		$this->creatingOrEditingFlagRedirect();
		
		// fetch charter according to ID and check it
		$charter = $this->context->projectCharterRepository->find($id);
		if (!$charter)
		{
			$this->flashMessage('Invalid project charter ID. You have been redirected.', 'warning');
			$this->redirect('Dashboard:default');
		}
		
		// select form
		$form = $this['lMApprovalForm'];
		// pre fill form by previous values
		if (count($this->session['Form']))
		{
			$form->setDefaults($this->session['Form']);
			$this->session['Form'] = array();
		}
		
		// get base information for charter
		$charterInfo = array();
		$charterInfo['Name'] = $charter->getName();
		$charterInfo['SAP_ID'] = $charter->getSAP_ID();
		$charterInfo['Owner'] = $this->context->personRepository->find($charter->getOwner())->getReversedFullNameWithEID();
		$charterInfo['LineManager'] = $this->context->personRepository->find($charter->getLineManager())->getReversedFullNameWithEID();
			$strapItem = $this->context->itemRepository->find($charter->StrapItemID);
		$charterInfo['StrapItem'] = ($strapItem) ? $strapItem->Name : '-';
		$charterInfo['StrapItemID'] = ($strapItem) ? $strapItem->ID : NULL;
		$charterInfo['Status'] = ProjectCharter::getFullStatusName($charter->getStatus());
		$charterInfo['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
		$charterInfo['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
			$program = $this->context->customerProgramRepository->find($charter->getProgramID());
		$charterInfo['ProgramID'] = ($program) ? $program->getProgram() : NULL;
			$call = $this->context->customerProgramCallRepository->find($charter->getCallID());
		$charterInfo['CallID'] = ($call) ? $call->getCall() : NULL;
		$charterInfo['Year'] = $charter->getYear();
		$charterInfo['CostShare'] = $charter->getCostSharePercentage();
		$charterInfo['ECLicenceID'] = $charter->getECLicenceID();

		// get data for table of file history
		$arrFileHistory = $this->context->projectCharterFileHistoryRepository->findByProjectCharterID($charter->getID());
		$arrFileInfo = array();
		foreach ($arrFileHistory as $fileHistory)
		{
			$fileInfo = array();
			$fileInfo['Version'] = $fileHistory->getVersion();
			$fileInfo['CreationDateTime'] = $fileHistory->getCreationDateTime();
			$fileInfo['FileName'] = $fileHistory->getFileName();
			$fileInfo['Path'] = $this->getPathToFile($fileHistory->getProjectCharterID(), $fileHistory->getVersion(), $fileHistory->getFileName());
				$responsible = $this->context->personRepository->find($fileHistory->getResponsible());
			$fileInfo['Responsible'] = $responsible->getReversedFullName();
			$fileInfo['Responsible_Full'] = $responsible->getReversedFullNameWithEID();
			$fileInfo['OwnerComment'] = $fileHistory->getOwnerComment();
			$fileInfo['LineManagerComment'] = $fileHistory->getLineManagerComment();
			$fileInfo['LatestVersion'] = $fileHistory->getLatestVersion();
			$arrFileInfo[] = $fileInfo;
			
			// set file version of the latest file
			if ($fileHistory->getLatestVersion())
			{
				$form['FileVersion']->setValue($fileHistory->getVersion());
				$form['OwnerFileComment']->setValue($fileHistory->getOwnerComment());
				if (!$form['LineManagerFileComment']->getValue())
					$form['LineManagerFileComment']->setValue($fileHistory->getLineManagerComment());
			}
		}
		
		// get data for table of summary file history
		$arrFileHistory = $this->context->projectCharterSummaryFileHistoryRepository->findByProjectCharterID($charter->getID());
		$arrSummaryFileInfo = array();
		foreach ($arrFileHistory as $fileHistory)
		{
			$fileInfo = array();
			$fileInfo['Version'] = $fileHistory->getVersion();
			$fileInfo['CreationDateTime'] = $fileHistory->getCreationDateTime();
			$fileInfo['FileName'] = $fileHistory->getFileName();
			$fileInfo['Path'] = $this->getPathToSummaryFile($fileHistory->getProjectCharterID(), $fileHistory->getVersion(), $fileHistory->getFileName());
				$responsible = $this->context->personRepository->find($fileHistory->getResponsible());
			$fileInfo['Responsible'] = $responsible->getReversedFullName();
			$fileInfo['Responsible_Full'] = $responsible->getReversedFullNameWithEID();
			$fileInfo['OwnerComment'] = $fileHistory->getOwnerComment();
			$fileInfo['LineManagerComment'] = $fileHistory->getLineManagerComment();
			$fileInfo['LatestVersion'] = $fileHistory->getLatestVersion();
			$arrSummaryFileInfo[] = $fileInfo;
			
			// set file version of the latest file
			if ($fileHistory->getLatestVersion())
			{
				$form['SummaryFileVersion']->setValue($fileHistory->getVersion());
				$form['OwnerSummaryFileComment']->setValue($fileHistory->getOwnerComment());
				if (!$form['LineManagerSummaryFileComment']->getValue())
					$form['LineManagerSummaryFileComment']->setValue($fileHistory->getLineManagerComment());
			}
		}
		
		// get data for table of budget history
		$arrBudgetHistory = $this->context->projectCharterBudgetHistoryRepository->findByProjectCharterID($charter->getID());
		$arrBudgetInfo = array();
		foreach ($arrBudgetHistory as $budgetHistory)
		{
			$budgetInfo = array();
			$budgetInfo['Version'] = $budgetHistory->getVersion();
			$budgetInfo['CreationDateTime'] = $budgetHistory->getCreationDateTime();
				$responsible = $this->context->personRepository->find($budgetHistory->getResponsible());
			$budgetInfo['Responsible'] = $responsible->getReversedFullName();
			$budgetInfo['Responsible_Full'] = $responsible->getReversedFullNameWithEID();
			$budgetInfo['Comment'] = $budgetHistory->getComment();
			$budgetInfo['LatestVersion'] = $budgetHistory->getLatestVersion();
			$arrBudgetInfo[] = $budgetInfo;
			
			// set file version of the latest file
			if ($budgetHistory->getLatestVersion() && (strtolower($budgetHistory->getResponsible()) == strtolower($this->getUser()->getId())))
			{
				$form['BudgetComment']->setValue($budgetHistory->getComment());
			}
		}
		$this->template->arrBudgetInfo = $arrBudgetInfo;

		if (!$form['ProjectCharter_ID']->getValue())
		{
			$latestBudget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->getID());
			if ($latestBudget)
			{
				$form['BudgetVersion']->setValue($latestBudget->getVersion());
				$budgetComment = $latestBudget->getComment();
				$oldBudget = $this->context->projectCharterBudgetHistoryRepository->find($latestBudget->getProjectCharterID(), $latestBudget->getVersion() - 1);
				$charterOwnerEqualsLM = strtolower($charter->getOwner()) === strtolower($charter->getLineManager());
				
				$budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($latestBudget->getProjectCharterID(), $latestBudget->getVersion());
				$TGrow = $SUBrow = 0;
				foreach ($budgetItems as $budgetItem)
				{
					switch ($budgetItem->getResourceType())
					{
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL:
							$source = 'Travel' ; break;
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL:
							$source = 'Material'; break;
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING:
							$source = 'SUBRow' . $SUBrow;
							$this->addSUBRowToForm($form, $SUBrow);
							$form[$source]['Subcontractor']->setValue($budgetItem->getSubcontractor());
							$SUBrow++; break;
						case ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS:
							$source = 'TGRow' . $TGrow;
							$this->addTGRowToForm($form, $TGrow);
							$form[$source]['TechnicalGroup']->setValue($budgetItem->getTechnicalGroup());
							$TGrow++; break;
						default: break;
					}

					$form[$source]['ProjectCharterBudget_ID']->setValue($budgetItem->getID());
					$entry['ApproveBypassLock'] = FALSE;
					if (strtolower($latestBudget->getResponsible()) == strtolower($this->getUser()->getId()))
					{
						$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
						// charter owner is same person as selected line manager
						if ($charterOwnerEqualsLM)
						{
							// bypass ajax 'approve' button locking mechanism
							//$entry['ApproveBypassLock'] = TRUE;
							$form['ApproveBypassLock']->setValue(TRUE);
							$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
						}
					}
					else
					{
						$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
						if (!$oldBudget)
							$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
					}
				}

				$this->session['TGRows'] = $TGrow;
				$this->session['SUBRows'] = $SUBrow;
				
				if ($oldBudget && !$charterOwnerEqualsLM)
				{
					$budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($oldBudget->getProjectCharterID(), $oldBudget->getVersion());
					foreach ($budgetItems as $budgetItem)
					{
						if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL)
						{
							$source = 'Travel';
							if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
								$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
							else
								$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
						}
						else if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL)
						{
							$source = 'Material';
							if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
								$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
							else
								$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
						}
						else if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
						{
							for ($row = 0; $row < $this->session['TGRows']; $row++)
							{
								$source = 'TGRow' . $row;
								if ($form[$source]['TechnicalGroup']->getValue() == $budgetItem->getTechnicalGroup())
								{
									if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
										$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
									else
										$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
									break;
								}
							}
						}
						else if ($budgetItem->getResourceType() == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING)
						{
							for ($row = 0; $row < $this->session['SUBRows']; $row++)
							{
								$source = 'SUBRow' . $row;
								if ($form[$source]['Subcontractor']->getValue() == $budgetItem->getSubcontractor())
								{
									if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
										$this->setBudgetItemFields($form, TRUE, $source, $budgetItem);
									else
										$this->setBudgetItemFields($form, FALSE, $source, $budgetItem);
									break;
								}
							}
						}
					}
					// set comment
					if (strtolower($oldBudget->getResponsible()) == strtolower($this->getUser()->getId()))
						$budgetComment = $oldBudget->getComment();
				}
			}

			$form['ProjectCharter_ID']->setValue($charter->getID());
		}

		// lock budget - must be done after TGRows and SUBRows session sections
		// are correctly set up
		// CL is not enabled to edit budget anymore
		$this->lockBudget($form);
		$this->template->budgetLocked = TRUE;
		
		// define list of budget items source types
		$arrBudgetSource = array('Travel', 'Material');
		// technical groups
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			array_push($arrBudgetSource, 'TGRow' . $row);
		// subcontractors
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			array_push($arrBudgetSource, 'SUBRow' . $row);
		// owner part must be readonly
		foreach ($arrBudgetSource as $source)
		{
			$form[$source]['ownerQ1']->setAttribute('readonly', 'readonly');
			$form[$source]['ownerQ2']->setAttribute('readonly', 'readonly');
			$form[$source]['ownerQ3']->setAttribute('readonly', 'readonly');
			$form[$source]['ownerQ4']->setAttribute('readonly', 'readonly');
		}

		// if exists uploaded file show it
		if ($form['TempFile']->getValue())
		{
			$this->template->tempFile = $this->getPathToTmpFile($form['TempFile']->getValue());
			$this->template->tempFileName = $this->getFileName($form['TempFile']->getValue());
		}
		if ($form['TempSummaryFile']->getValue())
		{
			$this->template->tempSummaryFile = $this->getPathToTmpFile($form['TempSummaryFile']->getValue());
			$this->template->tempSummaryFileName = $this->getFileName($form['TempSummaryFile']->getValue());
		}

		$this->template->charterInfo = $charterInfo;
		$this->template->arrFileInfo = $arrFileInfo;
		$this->template->arrSummaryFileInfo = $arrSummaryFileInfo;
		$this->template->TGRows = $this->session['TGRows'];
		$this->template->SUBRows = $this->session['SUBRows'];
		
		
		$form['PlannedYear']->setValue($charter->getYear());
		$form['Status']->setValue($charter->getStatus());

		$latestFile = $this->context->projectCharterFileHistoryRepository->findLatestVersion($charter->getID());
		$latestSummaryFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($charter->getID());
		$latestBudget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->getID());
		$userID = strtolower($this->getUser()->getId());
		
		// if exists changes made by CL disable choice approve.
		$DISABLE_APPROVE = FALSE;
		// latest file was saved by CL
		if ($latestFile && (strtolower($latestFile->getResponsible()) == $userID) && !$latestFile->getCommittedBool())
		{
			$DISABLE_APPROVE =  TRUE;
			$form['ApproveLocked']->setValue('locked');
		}
		// latest summary file was saved by CL
		if ($latestSummaryFile && (strtolower($latestSummaryFile->getResponsible()) == $userID) && !$latestSummaryFile->getCommittedBool())
		{
			$DISABLE_APPROVE =  TRUE;
			$form['ApproveLocked']->setValue('locked');
		}
		// latest budget was saved by CL
		if ($latestBudget && (strtolower($latestBudget->getResponsible()) == $userID))
		{
			$ownerBudget = $this->context->projectCharterBudgetHistoryRepository->find($latestBudget->getProjectCharterID(), $latestBudget->getVersion() - 1);
			if ($ownerBudget)
			{
				$latestBudgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($latestBudget->getProjectCharterID(), $latestBudget->getVersion());
				$ownerBudgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($ownerBudget->getProjectCharterID(), $ownerBudget->getVersion());
				
				$allSame = 0;
				foreach ($latestBudgetItems as $item1)
				{
					$same = FALSE;
					foreach ($ownerBudgetItems as $item2)
					{
						// same type
						if ($item1->getResourceType() === $item2->getResourceType())
						{
							if ($item1->getTechnicalGroup() && $item2->getTechnicalGroup())
							{
								if (($item1->getTechnicalGroup() === $item2->getTechnicalGroup()) && ($item1->getBudgetQ1() === $item2->getBudgetQ1()) && ($item1->getBudgetQ2() === $item2->getBudgetQ2()) && ($item1->getBudgetQ3() === $item2->getBudgetQ3()) && ($item1->getBudgetQ4() === $item2->getBudgetQ4()))
									$same = TRUE;
							}
							else if ($item1->getSubcontractor() && $item2->getSubcontractor())
							{
								if (($item1->getSubcontractor() === $item2->getSubcontractor()) && ($item1->getBudgetQ1() === $item2->getBudgetQ1()) && ($item1->getBudgetQ2() === $item2->getBudgetQ2()) && ($item1->getBudgetQ3() === $item2->getBudgetQ3()) && ($item1->getBudgetQ4() === $item2->getBudgetQ4()))
									$same = TRUE;
							}
							else
							{
								if (($item1->getBudgetQ1() === $item2->getBudgetQ1()) && ($item1->getBudgetQ2() === $item2->getBudgetQ2()) && ($item1->getBudgetQ3() === $item2->getBudgetQ3()) && ($item1->getBudgetQ4() === $item2->getBudgetQ4()))
									$same = TRUE;
							}
						}
					}
					
					if ($same)
						$allSame++;
				}
				
				$latestTG = array();
				$latestSUB = array();
				foreach ($latestBudgetItems as $item)
				{
					if ($item->getTechnicalGroup())
						$latestTG[$item->getTechnicalGroup()] = $item->getTechnicalGroup();
					if ($item->getSubcontractor())
						$latestSUB[$item->getSubcontractor()] = $item->getSubcontractor();
				}
				$ownerTG = array();
				$ownerSUB = array();
				foreach ($ownerBudgetItems as $item)
				{
					if ($item->getTechnicalGroup())
						$ownerTG[$item->getTechnicalGroup()] = $item->getTechnicalGroup();
					if ($item->getSubcontractor())
						$ownerSUB[$item->getSubcontractor()] = $item->getSubcontractor();
				}
				
				$diff = array_diff($latestTG, $ownerTG);
				$diff2 = array_diff($latestSUB, $ownerSUB);
				
				if (count($diff) || count($diff2))
				{
					$DISABLE_APPROVE = TRUE;
					$form['ApproveLocked']->setValue('locked');
				}
					
				if ($allSame !== count($latestBudgetItems))
					$DISABLE_APPROVE = TRUE;				
			}
		}
		
		if ($form['TempFile']->getValue())
		{
			$form['approve']->setDisabled(TRUE);
			$form['ApproveLocked']->setValue('locked');
		}
		
		if ($form['TempSummaryFile']->getValue())
		{
			$form['approve']->setDisabled(TRUE);
			$form['ApproveLocked']->setValue('locked');
		}
		
		if ($DISABLE_APPROVE || $form['ApproveLocked']->getValue())
		{
			$form['approve']->setDisabled(TRUE);
		}
		
		$arrTGSource = array();
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			array_push($arrTGSource, 'TGRow' . $row);		
		// add rule for unique value in form
		foreach ($arrTGSource as $current)
		{
			foreach ($arrTGSource as $other)
			{
				if ($current != $other)
				{
					$form[$current]['TechnicalGroup']->addRule(~Form::EQUAL, 'Each technical group may be selected only once.', $form[$other]['TechnicalGroup']);
				}
			}
		}
		
		$arrSUBSource = array();
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			array_push($arrSUBSource, 'SUBRow' . $row);		
		// add rule for unique value in form
		foreach ($arrSUBSource as $current)
		{
			foreach ($arrSUBSource as $other)
			{
				if ($current != $other)
				{
					$form[$current]['Subcontractor']->addRule(~Form::EQUAL, 'Each subcontractor may be selected only once.', $form[$other]['Subcontractor']);
				}
			}
		}

		$this->attachLaborRates($this->template, $charter);
		
		// Checklist
		$checklistItems = $this->context->projectCharterChecklistRepository->findByProjectCharterID($charter->ID);
		foreach ($checklistItems as $item)
		{
			$tempArray = array();
			for ($i = 1; $i <= 5; $i++)
				$tempArray[$i] = ($item->NeededArray[$i]) ? 'Yes' : 'No';
			$item->NeededArray = $tempArray;
			$checklistItems[$item->Title] = $item;
		}
		if (count($checklistItems))
			$this->template->checklist = $checklistItems;
	}
	
	/**
	 * Creates form for selecting of base name for new project charter.
	 * @param string $name Name of the form.
	 * @return AppForm Returns AppForm.
	 */
	public function createComponentChooseCharterBaseForm($name)
	{
		$form = new AppForm($this, $name);
		
		$temp_projects = $this->context->projectRepository->findAllProjects();
		$sbesConfig = Environment::getConfig('charter_sbes');
		$projects = array();
		foreach ($temp_projects as $project)
		{
			// only active projects
			if (($project->getPhase() != Project::PROJECT_PHASE_CLOSED) && isset($sbesConfig[$project->ResponsibleUnit]))
				$projects[$project->getID()] = $project->getName();
		}
		$form->addSelect('Project', 'Project', $this->addBookmarksToArray($projects))
				->setPrompt('(NEW PROJECT)')
				->setAttribute('style', 'width: 300px;');
		$form->addText('CharterName', HPTForm::createLabel('Charter Name', 'Should be the same as name of the project.'), 47, 100)
				->addConditionOn($form['Project'], ~Form::FILLED)
					->addRule(Form::FILLED, 'Charter name must be filled.');

		$form->addSubmit('back', 'Go back')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'backChooseCharterBaseFormSubmitted');
		
		$form->addSubmit('create', 'Create charter')
				->setDisabled(TRUE);
				//->onClick[] = array($this, 'createChooseCharterBaseFormSubmitted');

		return $form;
	}

	/**
	 * Adds row for working hours of technical group per year quarters.
	 * @param AppForm $form Destination form.
	 * @param int $row It should be positive integer. Identification of the row.
	 */
	public function addTGRowToForm(AppForm $form, $row)
	{
		// if container already exists do nothing
		if (isset($form['TGRow' . $row]))
			return;
		// add container into the form
		$TGRow = $form->addContainer('TGRow' . $row);
		$TGRow->addHidden('ProjectCharterBudget_ID');
			$sbes = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
			$technicalGroups = array();
			foreach ($sbes as $orgUnitID)
			{
				$sbe = $this->context->organizationUnitRepository->find($orgUnitID->ID);
				$tempTechGroups = $this->context->organizationUnitRepository->findAllChildrenByPath($sbe->Path);
				foreach($tempTechGroups as $tg)
					$technicalGroups[$sbe->Short][$tg->getID()] = $tg->getName();
			}
		$TGRow->addSelect('TechnicalGroup', 'Technical group', $technicalGroups)
				->setPrompt('')
				->addRule(FORM::FILLED, 'Technical group must be filled.');
		$TGRow->addText('ownerQ1', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q1 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q1 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('managerQ1', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q1 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q1 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('ownerQ2', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q2 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q2 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('managerQ2', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q2 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q2 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('ownerQ3', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q3 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q3 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('managerQ3', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q3 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q3 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('ownerQ4', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q4 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q4 belonging to one of technical group must contain positive integer.', '[0-9]*');
		$TGRow->addText('managerQ4', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q4 belonging to one of technical group must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q4 belonging to one of technical group must contain positive integer.', '[0-9]*');
		// Button delete
		$TGRow->addSubmit('deleteTGRow', 'Delete')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'deleteTGRowSubmitted');
				$TGRow['deleteTGRow']
						->setAttribute('onclick', 'deleteTGRow(' . $row . ')'); // JS sets row ID
	}

	/**
	 * Adds row for budget for subcontractor per year quarters.
	 * @param AppForm $form Destination form.
	 * @param int $row It should be positive integer. Identification of the row.
	 */
	public function addSUBRowToForm(AppForm $form, $row)
	{
		// if container already exists do nothing
		if (isset($form['SUBRow' . $row]))
			return;
		// add container into the form
		$SUBRow = $form->addContainer('SUBRow' . $row);
		$SUBRow->addHidden('ProjectCharterBudget_ID');
			//
			$temp_subcontractors = $this->context->subcontractorRepository->findAll();
			$subcontractors = array();
			foreach ($temp_subcontractors as $id => $subcontractor)
			{
				// from 2016 HTS subcontractors are planned as a labour!!!
				if(substr($subcontractor->Name, 0, 3) !== 'HTS')
					$subcontractors[$subcontractor->getID()] = $subcontractor->getName() . ' (' . $subcontractor->getShort() . ')';
				// D&S other can't be used as a subcontractor for new charters, yet it remains in the DB for older ones
				if($subcontractor->Name === 'D&S other')
					unset($subcontractors[$id]);
			}
		$SUBRow->addSelect('Subcontractor', 'Subcontractor', $subcontractors)
				->setPrompt('')
				->addRule(FORM::FILLED, 'Subcontractor must be filled.');
		$SUBRow->addText('ownerQ1', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q1 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q1 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('managerQ1', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q1 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q1 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('ownerQ2', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q2 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q2 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('managerQ2', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q2 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q2 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('ownerQ3', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q3 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q3 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('managerQ3', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q3 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q3 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('ownerQ4', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q4 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q4 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		$SUBRow->addText('managerQ4', NULL, 7)
				->setDefaultValue(0)
				->addRule(FORM::RANGE, 'Field Q4 belonging to one of subcontractor must contain positive integer.', array(0, NULL))
				->addRule(FORM::PATTERN, 'Field Q4 belonging to one of subcontractor must contain positive integer.', '[0-9]*');
		// Button delete
		$SUBRow->addSubmit('deleteSUBRow', 'Delete')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'deleteSUBRowSubmitted');
				$SUBRow['deleteSUBRow']
						->setAttribute('onclick', 'deleteSUBRow(' . $row . ')'); // JS sets row ID
	}

	/**
	 *
	 */
	public function createComponentFilterCharterForm($name)
	{
		$form = new AppForm($this, $name);

		$form->addText('Name', 'Name')
				->setAttribute('size', '30');
		$form->addText('Owner', 'Owner')
				->setAttribute('class', 'autocomplete')
				->setAttribute('size', '7');
		$form->addText('Owner_info')
				->setAttribute('readonly', 'readonly')
				->setAttribute('size', '20');
		$form->addText('SAP_ID', 'SAP ID')
				->setAttribute('size', '30');
		$form->addText('LineManager', 'Cluster leader for approval')
				->setAttribute('class', 'autocomplete')
				->setAttribute('size', '7');
		$form->addText('LineManager_info')
				->setAttribute('readonly', 'readonly')
				->setAttribute('size', '20');
		// operator + is used instead array_merge(), because this function interprets array keys as numbers,
		// so key '2013' disappears by default numbering.. it is wrong for this purpose.
		$years = array('' => '*') + $this->context->projectCharterRepository->getYears();
		$form->addSelect('Year', 'Planning for year', $years)
				->setAttribute('style', 'min-width: 80px;');

		$statuses = array('' => '*') + ProjectCharter::getStatusesFullNames();
		$form->addMultiSelect('Status', 'Status', $statuses)
				->setDefaultValue('');
				$form['Status']
					->setAttribute('size', '8')
					->setAttribute('style', 'width: 200px;');

		/**
		 * ProgramID and CallID are dynamically related with CustomerID.
		 * Each customer has some related programs and each program has some related calls.
		 * Here are fetched all of programs as well as all calls, but displayed items are
		 * filtered in render method according to current selection.
		 * It is simpler to obtain values from form if there are fetched all items.
		 */
		$all_customers = $this->context->customerRepository->findAll();
		$customers = array();
		foreach ($all_customers as $customer)
			$customers[$customer->ID] = $customer->Name;
		$form->addSelect('Customer_ID', 'Customer', $customers)
				->setPrompt('*');
		
		$all_customerPrograms = $this->context->customerProgramRepository->findAll();
		$customerPrograms = array();
		foreach ($all_customerPrograms as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
		$form->addSelect('ProgramID', 'Program', $customerPrograms)
				->setPrompt('*')
				->setAttribute('style', 'width: 200px;');
		
		$all_customerProgramCalls = $this->context->customerProgramCallRepository->findAll();
		$customerProgramCalls = array();
		foreach ($all_customerProgramCalls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
		$form->addSelect('CallID', 'Call', $customerProgramCalls)
				->setPrompt('*')
				->setAttribute('style', 'width: 200px;');

		$temp_responsibilities = $this->context->responsibilityRepository->findAll();
		$responsibilities = array('' => '*');
		foreach ($temp_responsibilities as $responsibility)
			$responsibilities[$responsibility->ID] = $responsibility->Name;
		$form->addSelect('Responsibility_ID', 'Responsibility', $responsibilities);

		$form->addCheckbox('OwnedByCurrentUser', 'My charters only', FALSE);

		$form->addSubmit('submit', 'Search');
		$form->addSubmit('reset', 'Reset Filter')
			->onClick[] = callback($this, 'filterCharterFormReset');
		$form->addSubmit('resetMyAction', 'Reset Filter')
			->onClick[] = callback($this, 'filterCharterFormMyActionReset');
		$form->getElementPrototype()->class('ajax');

		$form->onSuccess[] = array($this, 'filterCharterFormSubmitted');

	}

	/**
	 * Creates general charter form.
	 * @param string $name Name of the form.
	 * @return AppForm Returns AppForm.
	 */
	public function createComponentCharterForm($name)
	{
		$form = new AppForm($this, $name);
		// Hidden fields
		$form->addHidden('ProjectCharter_ID');
		$form->addHidden('Status');
		$form->addHidden('TempFile');
		$form->addHidden('TempFile2');
		$form->addHidden('FileVersion');
		$form->addHidden('TempSummaryFile');
		$form->addHidden('TempSummaryFile2');
		$form->addHidden('SummaryFileVersion');
		$form->addHidden('BudgetVersion');
		$form->addHidden('TGRowToRemove');
		$form->addHidden('SUBRowToRemove');
		// Name
		$form->addText('Name', 'Charter Name', 30, 100)
				->addRule(Form::FILLED, 'Charter name must be filled.');
		// Owner
		$people = $this->context->personRepository->findAll();
		$possibleOwners = array();
		foreach ($people as $person)
		{
			$possibleOwners[] = array($person->ID => $person->ReversedFullNameWithEID);
		}
		
		$form->addSelect('Owner', 'Charter owner', $possibleOwners)
				->setAttribute('style', 'width: 200px;');
		// SAP ID
		$form->addText('SAP_ID', 'SAP ID')
				->setAttribute('readonly', 'readonly')
				->setAttribute('size', '30');
		// Line manager
		$temp_lineManagers = $this->context->personRepository->getLeadersByOrgUnitTypeAndSBE(array(OrganizationUnit::ORG_UNIT_TYPE_CLUS, OrganizationUnit::ORG_UNIT_TYPE_DEP, OrganizationUnit::ORG_UNIT_TYPE_SBE), (array)Environment::getConfig('charter_sbes'));
		$lineManagers = array();
		foreach ($temp_lineManagers as $manager)
			$lineManagers[$manager->getID()] = $manager->getSurname() . ', ' . $manager->getFirstname() . ' (' . strtoupper ($manager->getID()) . ') ';
		$form->addSelect('LineManager', HPTForm::createLabel('Cluster leader for approval', 'Should be cluster leader (CL) in common case.'), $lineManagers)
				->setPrompt('');
				$form['LineManager']->setAttribute('style', 'width: 205px;');
		// STRAP Item
		$temp_strapItem = $this->context->itemRepository->findAll();
		$strapItem = array();
		foreach($temp_strapItem as $item)
			$strapItem[$item->ID] = $item->Name;
		$form->addSelect('StrapItem', 'Related STRAP Item', $strapItem)
				->setPrompt('');
		// Export Control
		$licences = array();
		$all_licences = $this->context->ecLicenceRepository->findAll();
		foreach ($all_licences as $licence)
			$licences[$licence->ECLicenceID] = $licence->Name;
		$form->addSelect('ECLicenceID', HPTForm::createLabel('Licence', 'Export control licence determines importance of given data. It usually depends on target customer.'), $licences)
				->setPrompt('')
				->setAttribute('style', 'width: 205px;');
				$form['ECLicenceID']->addRule(Form::FILLED, 'Licence must be filled.');
		// Cost share
		$form->addText('CostShare', HPTForm::createLabel('Estimated Average Cost Share', 'Percentage of costs covered by the customer.'))
				->addRule(Form::FILLED, 'Cost share must be filled.')
				->addRule(Form::PATTERN, 'Cost share must be number in rage <0,100>.', '[0-9]|[0-9][0-9]|100'); // percentage
				$form['CostShare']->setAttribute('size', '10');
		// Year
		$form->addText('Year', 'Planning for year')
				->setAttribute('readonly', 'readonly');
		/**
		 * ProgramID and CallID are dynamically related with CustomerID.
		 * Each customer has some related programs and each program has some related calls.
		 * Here are fetched all of programs as well as all calls, but displayed items are
		 * filtered in render method according to current selection.
		 * It is simpler to obtain values from form if there are fetched all items.
		 */
		$all_customers = $this->context->customerRepository->findAll();
		$customers = array();
		foreach ($all_customers as $customer)
			$customers[$customer->ID] = $customer->Name;
		$form->addSelect('Customer_ID', 'Customer', $customers)
				->setPrompt('')
				->addRule(Form::FILLED, 'Customer must be filled.');
		
		$all_customerPrograms = $this->context->customerProgramRepository->findAll();
		$customerPrograms = array();
		foreach ($all_customerPrograms as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
		$form->addSelect('ProgramID', 'Program', $customerPrograms)
				->setPrompt('')
				->setAttribute('style', 'width: 200px;');
		
		$all_customerProgramCalls = $this->context->customerProgramCallRepository->findAll();
		$customerProgramCalls = array();
		foreach ($all_customerProgramCalls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
		$form->addSelect('CallID', 'Call', $customerProgramCalls)
				->setPrompt('')
				->setAttribute('style', 'width: 200px;');
		
		// Responsibility
		$temp_responsibilities = $this->context->responsibilityRepository->findAll();
		$responsibilities = array();
		foreach ($temp_responsibilities as $responsibility)
			$responsibilities[$responsibility->ID] = $responsibility->Name;
		$form->addSelect('Responsibility_ID', 'Responsibility', $responsibilities)
				->setPrompt('')
				->addRule(Form::FILLED, 'Responsibility must be filled.');
		// File
		$form->addUpload('File', 'Add Charter Document')
				->setAttribute('size', '30');
		$form->addTextArea('OwnerFileComment', 'Comment to Charter Document', 75, 2);
		$form->addTextArea('OwnerTempFileComment', 'Comment to new Charter Document', 75, 2)
				->setAttribute('readonly', 'readonly');
		$form->addHidden('LineManagerFileComment');
		// Summary File
		$form->addUpload('SummaryFile', 'Add Charter Summary')
				->setAttribute('size', '30');
		$form->addTextArea('OwnerSummaryFileComment', 'Comment to Charter Summary', 75, 2);
		$form->addTextArea('OwnerTempSummaryFileComment', 'Comment to new Charter Summary', 75, 2)
				->setAttribute('readonly', 'readonly');
		$form->addHidden('LineManagerSummaryFileComment');
		// Budget
		$form->addTextArea('BudgetComment', 'Comment to Budget', 75, 2);
		// Technical groups
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			$this->addTGRowToForm($form, $row);
		// Subcontractors
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			$this->addSUBRowToForm($form, $row);
		// Travel
		$Travel = $form->addContainer('Travel');
			$Travel->addHidden('ProjectCharterBudget_ID');
			$Travel->addText('ownerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('ownerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('ownerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('ownerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to travel must contain positive integer.', '[0-9]*');
		// Material
		$Material = $form->addContainer('Material');
			$Material->addHidden('ProjectCharterBudget_ID');
			$Material->addText('ownerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('ownerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('ownerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('ownerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to material must contain positive integer.', '[0-9]*');
		// Checklist
		for ($i = 0; $i < count(self::$checklistArray); $i++)
		{
			$checklist = $form->addContainer('Checklist' . $i);
				$checklist->addSelect('Needed1', '', array(0 => 'No', 1 => 'Yes'))
						->setDefaultValue(0);
				$checklist->addSelect('Needed2', '', array(0 => 'No', 1 => 'Yes'))
						->setDefaultValue(0);
				$checklist->addSelect('Needed3', '', array(0 => 'No', 1 => 'Yes'))
						->setDefaultValue(0);
				$checklist->addSelect('Needed4', '', array(0 => 'No', 1 => 'Yes'))
						->setDefaultValue(0);
				$checklist->addSelect('Needed5', '', array(0 => 'No', 1 => 'Yes'))
						->setDefaultValue(0);
		}
		// Button upload - add document
		$form->addSubmit('upload', 'Add document')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'uploadCharterFileSubmitted');
		// Button upload - add document
		$form->addSubmit('uploadSummary', 'Add summary')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'uploadCharterSummaryFileSubmitted');
		// Button add technical group
		$form->addSubmit('addTGRow', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addTGRowSubmitted');
		// Button add subcontractor
		$form->addSubmit('addSUBRow', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addSUBRowSubmitted');
		// Button go back
		$form->addSubmit('back', 'Go back')
				->setValidationScope(FALSE);
		// Button save
		$form->addSubmit('save', 'Save');
		// Button submit for approval
		$form->addSubmit('submitForApproval', 'Submit for approval');
				$form['submitForApproval']->setAttribute('onclick', '
					if (document.getElementsByName("TempSummaryFile")[0].value || document.getElementsByName("SummaryFileVersion")[0].value)
						return confirm("Are you sure to submit the charter for approval?");
					else {
						alert("Charter summary must be added.");
						return false;
					}
				');

		return $form;
	}

	/**
	 * Creates charter form for creation.
	 * @param string $name Name of the form.
	 * @return AppForm Returns AppForm.
	 */
	public function createComponentCreateCharterForm($name)
	{
		$form = $this->createComponentCharterForm($name);
		
		$form['LineManager']->addRule(Form::FILLED, 'Cluster leader must be filled.');
		// Button go back
		$form['back']
			->onClick[] = callback($this, 'backDefault');
		// Button save
		$form['save']
				->onClick[] = array($this, 'createCharterFormSubmitted');
		// Button submit for approval
		$form['submitForApproval']
				->onClick[] = array($this, 'createCharterFormSubmitted');

		return $form;
	}

	/**
	 * Creates charter form for editation.
	 * @param string $name Name of the form.
	 * @return AppForm Returns AppForm.
	 */
	public function createComponentEditCharterForm($name)
	{
		$form = $this->createComponentCharterForm($name);
		// Button go back
		$form['back']
			->onClick[] = callback($this, 'backMyAction');
		// Button save
		$form['save']
				->onClick[] = array($this, 'editCharterFormSubmitted');
		// Button submit for approval
		$form['submitForApproval']
				->onClick[] = array($this, 'editCharterFormSubmitted');
		// Button cancel
		$form->addSubmit('cancel', 'Entirely cancel')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'cancelCharterFormSubmitted');
				$form['cancel']->setAttribute('onclick', '
						return confirm("Are you sure to cancel the charter?");
				');

		return $form;
	}

	/**
	 * Creates charter form for line manager editation.
	 * @param string $name Name of the form.
	 * @return AppForm Returns AppForm.
	 */
	public function createComponentLMApprovalForm($name)
	{
		$form = new AppForm($this, $name);
		// Hidden fields
		$form->addHidden('ProjectCharter_ID');
		$form->addHidden('Status');
		$form->addHidden('TempFile');
		$form->addHidden('PlannedYear');
		$form->addHidden('FileVersion');
		$form->addHidden('TempSummaryFile');
		$form->addHidden('SummaryFileVersion');
		$form->addHidden('BudgetVersion');
		$form->addHidden('TGRowToRemove');
		$form->addHidden('SUBRowToRemove');
		$form->addHidden('ApproveLocked');
		// used for bypassing 'Approve' button's lock, when charter's owner
		// is the same person as charter's LM
		$form->addHidden('ApproveBypassLock');
		// File
		$form->addUpload('File', 'Add Charter Document')
				->setAttribute('size', '30');
		$form->addHidden('OwnerFileComment');
		$form->addTextArea('LineManagerFileComment', NULL, 75, 2);
		$form->addTextArea('LineManagerTempFileComment', 'Comment to proposed Charter Document', 75, 2)
				->setAttribute('readonly', 'readonly');
		// File
		$form->addUpload('SummaryFile', 'Add Charter Summary')
				->setAttribute('size', '30');
		$form->addHidden('OwnerSummaryFileComment');
		$form->addTextArea('LineManagerSummaryFileComment', NULL, 75, 2);
		$form->addTextArea('LineManagerTempSummaryFileComment', 'Comment to proposed Charter Summary', 75, 2)
				->setAttribute('readonly', 'readonly');
		// Technical groups
		for ($row = 0; $row < $this->session['TGRows']; $row++)
			$this->addTGRowToForm($form, $row);
		// Subcontractors
		for ($row = 0; $row < $this->session['SUBRows']; $row++)
			$this->addSUBRowToForm($form, $row);
		// Travel
		$Travel = $form->addContainer('Travel');
			$Travel->addHidden('ProjectCharterBudget_ID');
			$Travel->addText('ownerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('ownerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('ownerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('ownerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to travel must contain positive integer.', '[0-9]*');
			$Travel->addText('managerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to travel must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to travel must contain positive integer.', '[0-9]*');
		// Material
		$Material = $form->addContainer('Material');
			$Material->addHidden('ProjectCharterBudget_ID');
			$Material->addText('ownerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ1', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q1 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q1 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('ownerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ2', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q2 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q2 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('ownerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ3', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q3 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q3 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('ownerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to material must contain positive integer.', '[0-9]*');
			$Material->addText('managerQ4', NULL, 7)
					->setDefaultValue(0)
					->addRule(FORM::RANGE, 'Field Q4 belonging to material must contain positive integer.', array(0, NULL))
					->addRule(FORM::PATTERN, 'Field Q4 belonging to material must contain positive integer.', '[0-9]*');
		// Button upload - add document
		$form->addSubmit('upload', 'Add document')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'uploadCharterFileSubmitted');
		// Button upload - add document
		$form->addSubmit('uploadSummary', 'Add summary')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'uploadCharterSummaryFileSubmitted');
		// Button add technical group
		$form->addSubmit('addTGRow', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addTGRowSubmitted');
		// Button add subcontractor
		$form->addSubmit('addSUBRow', 'Add')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'addSUBRowSubmitted');
		// Budget comment
		$form->addTextArea('BudgetComment', 'Comment to Budget', 75, 2);
		// Button go back
		$form->addSubmit('back', 'Go back')
				->setValidationScope(FALSE)
				->onClick[] = callback($this, 'backMyAction');
		// Button save
		$form->addSubmit('save', 'Save')
				->onClick[] = array($this, 'saveLMApprovalFormSubmitted');
		// Button approve
		$form->addSubmit('approve', 'Approve')
				->onClick[] = array($this, 'approveLMApprovalFormSubmitted');
				$form['approve']->setAttribute('onclick', '
					return confirm("Are you sure to approve the charter?");
				');
		// Button disapprove
		$form->addSubmit('disapprove', 'Send back to PIPM')
				->onClick[] = array($this, 'disapproveLMApprovalFormSubmitted');
				$form['disapprove']->setAttribute('onclick', '
					return confirm("Are you sure to send the charter back to PIPM?");
				');

		return $form;
	}
	
	/**
	 * Redirects user if he tries to do more things (create charter, edit existing)
	 * simultaneously. That way those tabs won't interfere and break session information,
	 * for instance budget. Better way of doing this would be using HTML5's sessionStorage via
	 * Javascript to store all data in a form on client-side.
	 * @return
	 */
	private function creatingOrEditingFlagRedirect()
	{
		// limit Charter - Create/Edit instance to only one
		if(!empty($this->session->creatingOrEditingFlag) && (time() - $this->session->creatingOrEditingFlag) < self::AJAX_BEACON_TIMEOUT)
		{
			$this->flashMessage('You can create or edit only one charter at a time. We apologize for any inconvenience caused.', 'warning');
			if($this->getAction() === 'edit')
				$this->redirect('ProjectCharter:detail', array('id' => (int)$this['editCharterForm']->parent->params['id']));
			else
				$this->redirect('ProjectCharter:default');
		}
	}

	/**
	 *
	 */
	public function saveLMApprovalFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			// check accessibility for user
			$this->secureAccess($entry['ProjectCharter_ID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL), FALSE);
			
			$isSaveButton = ($button->getHtmlName() == 'save') ? TRUE : FALSE;

			$charter = $this->context->projectCharterRepository->find($entry['ProjectCharter_ID']);
			if (!$charter)
			{
				$charter = new ProjectCharter;
				$year = (int) date('Y');
				// Year of planning is required for selecting labor rate.
				// Common year of planning is next year, so if charter cannot be found value of next year is selected.
				$charter->setYear($year + 1);
				unset($year);
			}

			// is uploaded any file
			if ($entry['TempFile'])
			{
				
				$createNewVersion = FALSE;                
				$latestFile = $this->context->projectCharterFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				// latest version has been committed, so new version will be created
				if (!$latestFile || ($latestFile && ($latestFile->getCommittedBool())))
					$createNewVersion = TRUE;

				// get file name from teporary file
				$fileName = $this->getFileName($entry['TempFile']);
				
				// set version
				$version = 1;
				if ($latestFile && $createNewVersion)
					$version = $latestFile->getVersion() + 1;
				else if ($entry['FileVersion'])
					$version = $entry['FileVersion'];
				
				// set creation datetime
				$created = new DateTime;
				if (!$createNewVersion && $latestFile)
					$created = $latestFile->getCreationDateTime();
				
				// set comments
				$arrComment = array(
					'OwnerComment' => ($createNewVersion) ? NULL : $entry['OwnerFileComment'],
					'LineManagerComment' => $entry['LineManagerTempFileComment']
				);
				
				// basic file information
				$file = new ProjectCharterFileHistory();
				$file->setProjectCharterID($entry['ProjectCharter_ID']);
				$file->setVersion($version);
				$file->setCreationDateTime($created);
				$file->setFileName($fileName);
				$file->setResponsible($this->getUser()->getId());
				$file->setOwnerComment($arrComment['OwnerComment']);
				$file->setLineManagerComment($arrComment['LineManagerComment']);
				$file->setLatestVersion(1);
				$file->setCommitted(($isSaveButton) ? FALSE : TRUE);
				
				// set path to storage
				$storage =
					__DIR__ . $this->charters_storage_dir .
					$this->charter_dir_prefix . $file->getProjectCharterID() . '/' .
					$this->version_dir_prefix . $file->getVersion() . '/'
				;
				// current file version stays and previous file with same version exists, so it should be removed.
				if (!$createNewVersion && $entry['FileVersion'])
				{
					foreach(glob($storage . '*.*') as $garbage)
					   unlink($garbage);
				}
				// initialization of flag of latest version
				$this->context->projectCharterFileHistoryRepository->initLatestVersion($entry['ProjectCharter_ID']);
				// current file version stays and previous file with same version exists.. means update
				if (!$createNewVersion && $entry['FileVersion'])
				{
					if($this->context->projectCharterFileHistoryRepository->update($file))
						$this->moveTmpFile($storage, $entry['TempFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
				else
				{
					if($this->context->projectCharterFileHistoryRepository->save($file))
						$this->moveTmpFile($storage, $entry['TempFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
			}
			else
			{
				$latestFile = $this->context->projectCharterFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				if ($latestFile)
				{
					$latestFile->setLineManagerComment($entry['LineManagerFileComment'] ? $entry['LineManagerFileComment'] : NULL);
					if (strtolower($latestFile->getResponsible()) == strtolower($this->getUser()->getId()))
						if (!$latestFile->getCommittedBool())
							$latestFile->setCommitted(($isSaveButton) ? FALSE : TRUE);
					
					if(!$this->context->projectCharterFileHistoryRepository->update($latestFile))
							$this->flashMessage("Saving file comment failed.", 'error');
				}
			}
			
			// is uploaded any summary file
			if ($entry['TempSummaryFile'])
			{
				$createNewVersion = FALSE;                
				$latestFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				// latest version has been committed, so new version will be created
				if (!$latestFile || ($latestFile && ($latestFile->getCommittedBool())))
					$createNewVersion = TRUE;

				// get file name from teporary file
				$fileName = $this->getFileName($entry['TempSummaryFile']);
				
				// set version
				$version = 1;
				if ($latestFile && $createNewVersion)
					$version = $latestFile->getVersion() + 1;
				else if ($entry['SummaryFileVersion'])
					$version = $entry['SummaryFileVersion'];
				
				// set creation datetime
				$created = new DateTime;
				if (!$createNewVersion && $latestFile)
					$created = $latestFile->getCreationDateTime();
				
				// set comments
				$arrComment = array(
					'OwnerComment' => ($createNewVersion) ? NULL : $entry['OwnerSummaryFileComment'],
					'LineManagerComment' => $entry['LineManagerTempSummaryFileComment']
				);
				
				// basic file information
				$file = new ProjectCharterSummaryFileHistory();
				$file->setProjectCharterID($entry['ProjectCharter_ID']);
				$file->setVersion($version);
				$file->setCreationDateTime($created);
				$file->setFileName($fileName);
				$file->setResponsible($this->getUser()->getId());
				$file->setOwnerComment($arrComment['OwnerComment']);
				$file->setLineManagerComment($arrComment['LineManagerComment']);
				$file->setLatestVersion(1);
				$file->setCommitted(($isSaveButton) ? FALSE : TRUE);
				
				// set path to storage
				$storage =
					__DIR__ . $this->charters_storage_dir .
					$this->charter_dir_prefix . $file->getProjectCharterID() . '/summary/' .
					$this->version_dir_prefix . $file->getVersion() . '/'
				;
				// current file version stays and previous file with same version exists, so it should be removed.
				if (!$createNewVersion && $entry['SummaryFileVersion'])
				{
					foreach(glob($storage . '*.*') as $garbage)
					   unlink($garbage);
				}
				// initialization of flag of latest version
				$this->context->projectCharterSummaryFileHistoryRepository->initLatestVersion($entry['ProjectCharter_ID']);
				// current file version stays and previous file with same version exists.. means update
				if (!$createNewVersion && $entry['SummaryFileVersion'])
				{
					if($this->context->projectCharterSummaryFileHistoryRepository->update($file))
						$this->moveTmpFile($storage, $entry['TempSummaryFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
				else
				{
					if($this->context->projectCharterSummaryFileHistoryRepository->save($file))
						$this->moveTmpFile($storage, $entry['TempSummaryFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
			}
			else
			{
				$latestFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				if ($latestFile)
				{
					$latestFile->setLineManagerComment($entry['LineManagerSummaryFileComment'] ? $entry['LineManagerSummaryFileComment'] : NULL);
					if (strtolower($latestFile->getResponsible()) == strtolower($this->getUser()->getId()))
						if (!$latestFile->getCommittedBool())
							$latestFile->setCommitted(($isSaveButton) ? FALSE : TRUE);
					
					if(!$this->context->projectCharterSummaryFileHistoryRepository->update($latestFile))
							$this->flashMessage("Saving file comment failed.", 'error');
				}
			}

			// define list of budget items resource types
			$arrBudgetItemSource = array('Travel', 'Material');
			for ($row = 0; $row < $this->session['TGRows']; $row++)
				array_push($arrBudgetItemSource, 'TGRow' . $row);
			for ($row = 0; $row < $this->session['SUBRows']; $row++)
				array_push($arrBudgetItemSource, 'SUBRow' . $row);
			$prevBudget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
			$createNewVersion = FALSE;
			if (strtolower($prevBudget->getResponsible()) != strtolower($this->getUser()->getId()))
			{
				$createNewVersion = TRUE;
			}
			// do it only during editation
			if (!$createNewVersion)
			{
				// get saved budget items
				$budgetItemsCurrentlySaved = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($entry['ProjectCharter_ID'], (($entry['BudgetVersion']) ? $entry['BudgetVersion'] : 1));
				$IDOfBudgetItemsCurrentlySaved = array();
				foreach ($budgetItemsCurrentlySaved as $budgetItemCurrentlySaved)
				{
					$IDOfBudgetItemsCurrentlySaved[] = $budgetItemCurrentlySaved->getID();
				}
				// get budget items from form
				$IDOfBudgetItemsFromForm = array();
				foreach ($arrBudgetItemSource as $source)
				{
					$IDOfBudgetItemsFromForm[] = $entry[$source]['ProjectCharterBudget_ID'];
				}
				// delete removed budget items
				$IDOfBudgetItemsToDelete = array_diff($IDOfBudgetItemsCurrentlySaved, $IDOfBudgetItemsFromForm);
				foreach ($IDOfBudgetItemsToDelete as $IDToDelete)
				{
					$this->context->projectCharterBudgetRepository->delete($IDToDelete);
				}
			}

			foreach ($arrBudgetItemSource as $source)
			{
				if (!$entry[$source]['managerQ1']) { $entry[$source]['managerQ1'] = 0; }
				if (!$entry[$source]['managerQ2']) { $entry[$source]['managerQ2'] = 0; }
				if (!$entry[$source]['managerQ3']) { $entry[$source]['managerQ3'] = 0; }
				if (!$entry[$source]['managerQ4']) { $entry[$source]['managerQ4'] = 0; }
			}


			// fill basic information for header (ProjectCharterBudgetHistory) of budget items
			$budget = new ProjectCharterBudgetHistory();
			$budget->setProjectCharterID($entry['ProjectCharter_ID']);
				// if does not exists any budget record, set version 1
				$budget->setVersion((($entry['BudgetVersion']) ? $entry['BudgetVersion'] : 1));
			$budget->setCreationDateTime(new DateTime);
			$budget->setResponsible($this->getUser()->getId());
			$budget->setComment($entry['BudgetComment']);
			$budget->setLatestVersion(1);
			// try to save budget header information
			if ($createNewVersion)
			{
				$budget->setVersion($prevBudget->getVersion()+1);
				$this->context->projectCharterBudgetHistoryRepository->initLatestVersion($entry['ProjectCharter_ID']);
				if (!$this->context->projectCharterBudgetHistoryRepository->save($budget))
				{
					$this->flashMessage("Saving budget failed.", 'error');
					return;
				}
			}
			else
			{
				$this->context->projectCharterBudgetHistoryRepository->initLatestVersion($entry['ProjectCharter_ID']);
				if (!$this->context->projectCharterBudgetHistoryRepository->update($budget))
				{
					$this->flashMessage("Saving budget failed.", 'error');
					return;
				}
			}

			// fill basic information for single budget item
			$budgetItem = new ProjectCharterBudget();
			$budgetItem->setProjectCharterID($budget->getProjectCharterID()); // common data for all items
			$budgetItem->setVersion($budget->getVersion());
			// fill another budget item basic information according to resource type of budget item
			foreach ($arrBudgetItemSource as $source)
			{
				$budgetItem->setTechnicalGroup(NULL);
				$budgetItem->setSubcontractor(NULL);
				// set resource type for budget item
				if ($source == 'Travel')
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL);
				}
				elseif ($source == 'Material')
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL);
				}
				elseif (preg_match('/TGRow[0-9]+/', $source))
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS);
					$budgetItem->setTechnicalGroup($entry[$source]['TechnicalGroup']);

					// Copy frozen average rate
					if ($entry[$source]['ProjectCharterBudget_ID'])
					{
						$oldBudgetItem = $this->context->projectCharterBudgetRepository->find($entry[$source]['ProjectCharterBudget_ID']);
						if ($oldBudgetItem)
							$budgetItem->setRate($oldBudgetItem->getRate());

						unset($oldBudgetItem);
					}

					// Freeze average rate if it is not frozen yet
					if (!$budgetItem->getRate())
					{
						$average_rates = $this->getOrgUnitAverageLaborRates($entry[$source]['TechnicalGroup']);
						$budgetItem->setRate($average_rates[$charter->getYear()]);
						unset($average_rates);
					}
				}
				elseif (preg_match('/SUBRow[0-9]+/', $source))
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING);
					$budgetItem->setSubcontractor($entry[$source]['Subcontractor']);
				}

				if ($entry[$source]['ProjectCharterBudget_ID'])
					$budgetItem->setID($entry[$source]['ProjectCharterBudget_ID']);
				$budgetItem->setBudgetQ1($entry[$source]['managerQ1']);
				$budgetItem->setBudgetQ2($entry[$source]['managerQ2']);
				$budgetItem->setBudgetQ3($entry[$source]['managerQ3']);
				$budgetItem->setBudgetQ4($entry[$source]['managerQ4']);

				// try to save single budget item
				if ($entry[$source]['ProjectCharterBudget_ID'] && !$createNewVersion)
				{
					if (!$this->context->projectCharterBudgetRepository->update($budgetItem))
						$this->flashMessage("Saving budget item failed.", 'error');
				}
				else
				{
					if (!$this->context->projectCharterBudgetRepository->save($budgetItem))
						$this->flashMessage("Saving budget item failed.", 'error');
				}
				// Init rate before processing next budget item
				$budgetItem->setRate(NULL);
			}
			
			// fetch charter
			if ($charter)
			{
				$this->flashMessage("Project charter '" . $charter->getName() . "' updated.", 'success');
			}
			// initialization of number of available rows for budgets of technical groups in the form
			$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
			$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
			if ($button->getHtmlName() == 'save' || $button->getHtmlName() == 'upload')
				$this->redirect('ProjectCharter:LMApproval', array('id' => $entry['ProjectCharter_ID']));

			return;
		}

		$this->flashMessage("Error occured during submitting the form. You has been redirected.", 'error');
		$this->redirect('ProjectCharter:myAction');
	}

	/**
	 *
	 */
	public function approveLMApprovalFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			// check accessibility for user
			$this->secureAccess($entry['ProjectCharter_ID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL), FALSE);
			
			// save form content
			$this->saveLMApprovalFormSubmitted($button);
			
			// update charter status
			$charter = $this->context->projectCharterRepository->find($entry['ProjectCharter_ID']);
			$charter->setStatus(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION);
			if (!$this->context->projectCharterRepository->update($charter))
			{
				$this->flashMessage("Saving failed.", 'error');
				return;
			}
			$this->redirect('ProjectCharter:detail', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form. You has been redirected.", 'error');
		$this->redirect('ProjectCharter:myAction');
	}

	/**
	 *
	 */
	public function disapproveLMApprovalFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			
			// check accessibility for user
			$this->secureAccess($entry['ProjectCharter_ID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL), FALSE);
			
			// save form content
			$this->saveLMApprovalFormSubmitted($button);
			
			// update charter status
			$charter = $this->context->projectCharterRepository->find($entry['ProjectCharter_ID']);
			$charter->setStatus(ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION);
			if (!$this->context->projectCharterRepository->update($charter))
			{
				$this->flashMessage("Saving failed.", 'error');
				return;
			}
			$this->redirect('ProjectCharter:detail', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form. You has been redirected.", 'error');
		$this->redirect('ProjectCharter:myAction');
	}

	
	/**
	 * Adds row for budget of technical group to the create charter form.
	 * @param SubmitButton $button
	 */
	public function addTGRowSubmitted(SubmitButton $button)
	{
		$form = $button->form;
		$this->addTGRowToForm($form, $this->session['TGRows']);
		$this->session['TGRows'] = $this->session['TGRows'] + 1;
		// save form and redirect
		$this->session['Form'] = $form->getValues(TRUE);
		$this->redirect('this');
	}

	/**
	 * Adds row for budget of technical group to the create charter form.
	 * @param SubmitButton $button
	 */
	public function addSUBRowSubmitted(SubmitButton $button)
	{
		$form = $button->form;
		$this->addSUBRowToForm($form, $this->session['SUBRows']);
		$this->session['SUBRows'] = $this->session['SUBRows'] + 1;
		// save form and redirect
		$this->session['Form'] = $form->getValues(TRUE);
		$this->redirect('this');
	}

	/**
	 * Removes row for budget of technical group from the create charter form.
	 * @param SubmitButton $button
	 */
	public function deleteTGRowSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues(TRUE);
		$form = $button->form;

		$row = $entry['TGRowToRemove'];
		$budgetId = $entry['TGRow' . $row]['ProjectCharterBudget_ID'];
		if ($budgetId)
		{
			$budgetItem = $this->context->projectCharterBudgetRepository->find($budgetId);
			if ($budgetItem)
			{
				$charter = $this->context->projectCharterRepository->find($budgetItem->ProjectCharterID);
				if ($charter)
				{
					$budgetCreatedByOwner = $this->context->projectCharterBudgetRepository
						->findLatestByCharterIdAndResponsible($charter->ID, $charter->Owner);

					$isCreatedByOwner = FALSE;
					foreach ($budgetCreatedByOwner as $budget) {
						if ($budget->TechnicalGroup == $entry['TGRow' . $row]['TechnicalGroup'])
							$isCreatedByOwner = TRUE;
					}

					// User is not owner and this budget item was created by owner
					if ((strtolower($this->getUser()->getId()) != strtolower($charter->Owner))
						&& $isCreatedByOwner)
					{
						$form['TGRow' . $row]['ownerQ1']->setValue(0);
						$form['TGRow' . $row]['managerQ1']->setValue(0);
						$form['TGRow' . $row]['ownerQ2']->setValue(0);
						$form['TGRow' . $row]['managerQ2']->setValue(0);
						$form['TGRow' . $row]['ownerQ3']->setValue(0);
						$form['TGRow' . $row]['managerQ3']->setValue(0);
						$form['TGRow' . $row]['ownerQ4']->setValue(0);
						$form['TGRow' . $row]['managerQ4']->setValue(0);
						$skipDeletion = TRUE;
					}
				}
			}
		}

		if (!isset($skipDeletion))
		{
			// delete given row
			if ($this->session['TGRows'] != 1)
			{
				// for each row behind deleted row, copy data to the row ahead of itself
				for ($row = $entry['TGRowToRemove']; $row < $this->session['TGRows']; $row++)
				{
					// is there any following row?
					$nextRow = $row + 1;
					if (!isset($entry['TGRow' . $nextRow]))
						break;

					$form['TGRow' . $row]['ProjectCharterBudget_ID']->setValue($entry['TGRow' . $nextRow]['ProjectCharterBudget_ID']);
					$form['TGRow' . $row]['TechnicalGroup']->setValue($entry['TGRow' . $nextRow]['TechnicalGroup']);
					$form['TGRow' . $row]['ownerQ1']->setValue($entry['TGRow' . $nextRow]['ownerQ1']);
					$form['TGRow' . $row]['managerQ1']->setValue($entry['TGRow' . $nextRow]['managerQ1']);
					$form['TGRow' . $row]['ownerQ2']->setValue($entry['TGRow' . $nextRow]['ownerQ2']);
					$form['TGRow' . $row]['managerQ2']->setValue($entry['TGRow' . $nextRow]['managerQ2']);
					$form['TGRow' . $row]['ownerQ3']->setValue($entry['TGRow' . $nextRow]['ownerQ3']);
					$form['TGRow' . $row]['managerQ3']->setValue($entry['TGRow' . $nextRow]['managerQ3']);
					$form['TGRow' . $row]['ownerQ4']->setValue($entry['TGRow' . $nextRow]['ownerQ4']);
					$form['TGRow' . $row]['managerQ4']->setValue($entry['TGRow' . $nextRow]['managerQ4']);
				}
			}
		}

		$lock = FALSE;
		$id = $form['TGRow' . $entry['TGRowToRemove']]['ProjectCharterBudget_ID']->getValue();
		$budget = $this->context->projectCharterBudgetRepository->find($id);
		if ($budget)
		{
			$budget_history = $this->context->projectCharterBudgetHistoryRepository->find($budget->ProjectCharterID, $budget->Version);
			if ($budget_history)
			{
				if ($budget_history->Responsible != $this->getUser()->getId())
					$lock = TRUE;
			}
		}

		// lock if deleted proposal was not owned by current user
		$entry = $form->getValues(TRUE);
		$entry['ApproveLocked'] = ($lock) ? 'locked' : NULL;
		// decrease number of available rows for budgets of technical groups in the form
		if (!isset($skipDeletion))
		{
			$this->session['TGRows'] = $this->session['TGRows'] - 1;
		}
		// save form and redirect
		$this->session['Form'] = $entry;
		$this->redirect('this');
	}

	/**
	 * Removes row for budget of technical group from the create charter form.
	 * @param SubmitButton $button
	 */
	public function deleteSUBRowSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues(TRUE);
		$form = $button->form;

		 $row = $entry['SUBRowToRemove'];
		$budgetId = $entry['SUBRow' . $row]['ProjectCharterBudget_ID'];
		if ($budgetId)
		{
			$budgetItem = $this->context->projectCharterBudgetRepository->find($budgetId);
			if ($budgetItem)
			{
				$charter = $this->context->projectCharterRepository->find($budgetItem->ProjectCharterID);
				if ($charter)
				{
					$budgetCreatedByOwner = $this->context->projectCharterBudgetRepository
						->findLatestByCharterIdAndResponsible($charter->ID, $charter->Owner);

					$isCreatedByOwner = FALSE;
					foreach ($budgetCreatedByOwner as $budget) {
						if ($budget->Subcontractor == $entry['SUBRow' . $row]['Subcontractor'])
							$isCreatedByOwner = TRUE;
					}

					if ((strtolower($this->getUser()->getId()) != strtolower($charter->Owner))
						&& $isCreatedByOwner)
					{
						$form['SUBRow' . $row]['ownerQ1']->setValue(0);
						$form['SUBRow' . $row]['managerQ1']->setValue(0);
						$form['SUBRow' . $row]['ownerQ2']->setValue(0);
						$form['SUBRow' . $row]['managerQ2']->setValue(0);
						$form['SUBRow' . $row]['ownerQ3']->setValue(0);
						$form['SUBRow' . $row]['managerQ3']->setValue(0);
						$form['SUBRow' . $row]['ownerQ4']->setValue(0);
						$form['SUBRow' . $row]['managerQ4']->setValue(0);
						$skipDeletion = TRUE;
					}
				}
			}
		}

		if (!isset($skipDeletion))
		{
			// delete given row
			if ($this->session['SUBRows'] != 1)
			{
				// for each row behind deleted row, copy data to the row ahead of itself
				for ($row = $entry['SUBRowToRemove']; $row < $this->session['SUBRows']; $row++)
				{
					// is there any following row?
					$nextRow = $row + 1;
					if (!isset($entry['SUBRow' . $nextRow]))
						break;

					$form['SUBRow' . $row]['ProjectCharterBudget_ID']->setValue($entry['SUBRow' . $nextRow]['ProjectCharterBudget_ID']);
					$form['SUBRow' . $row]['Subcontractor']->setValue($entry['SUBRow' . $nextRow]['Subcontractor']);
					$form['SUBRow' . $row]['ownerQ1']->setValue($entry['SUBRow' . $nextRow]['ownerQ1']);
					$form['SUBRow' . $row]['managerQ1']->setValue($entry['SUBRow' . $nextRow]['managerQ1']);
					$form['SUBRow' . $row]['ownerQ2']->setValue($entry['SUBRow' . $nextRow]['ownerQ2']);
					$form['SUBRow' . $row]['managerQ2']->setValue($entry['SUBRow' . $nextRow]['managerQ2']);
					$form['SUBRow' . $row]['ownerQ3']->setValue($entry['SUBRow' . $nextRow]['ownerQ3']);
					$form['SUBRow' . $row]['managerQ3']->setValue($entry['SUBRow' . $nextRow]['managerQ3']);
					$form['SUBRow' . $row]['ownerQ4']->setValue($entry['SUBRow' . $nextRow]['ownerQ4']);
					$form['SUBRow' . $row]['managerQ4']->setValue($entry['SUBRow' . $nextRow]['managerQ4']);
				}
			}
		}
		
		$lock = FALSE;
		$id = $form['SUBRow' . $entry['SUBRowToRemove']]['ProjectCharterBudget_ID']->getValue();
		$budget = $this->context->projectCharterBudgetRepository->find($id);
		if ($budget)
		{
			$budget_history = $this->context->projectCharterBudgetHistoryRepository->find($budget->ProjectCharterID, $budget->Version);
			if ($budget_history)
			{
				if ($budget_history->Responsible != $this->getUser()->getId())
					$lock = TRUE;
			}
		}
		
		// lock if deleted proposal was not owned by current user
		$entry = $form->getValues(TRUE);
		$entry['ApproveLocked'] = ($lock) ? 'locked' : NULL;
		// decrease number of available rows for budgets of technical groups in the form
		if (!isset($skipDeletion))
		{
			$this->session['SUBRows'] = $this->session['SUBRows'] - 1;
		}
		// save form and redirect
		$this->session['Form'] = $entry;
		$this->redirect('this');
	}

	/**
	 * Creates new project charter from data from the createCharterForm.
	 * @param SubmitButton $button
	 */
	public function createCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();
			$isSubmitForApprovalButton = ($button->getHtmlName() == 'submitForApproval');

			// basic project charter informations
			$charter = new ProjectCharter();
			$charter->setName($entry['Name']);
			$charter->setSAP_ID($entry['SAP_ID']);
			$charter->setLineManager($entry['LineManager']);
			$charter->setStrapItemID($entry['StrapItem']);
			$charter->setOwner($entry['Owner']);
			$charter->setCreationDateTime(new DateTime);
			$charter->setCustomerID($entry['Customer_ID']);
			$charter->setProgramID($entry['ProgramID']);
			$charter->setCallID($entry['CallID']);
			$charter->setResponsibilityID($entry['Responsibility_ID']);
			$charter->setYear($entry['Year']);
			$charter->setECLicenceID($entry['ECLicenceID']);
			$charter->setDaysForFinalApproval(0);
				// default value is zero, so don't set any value if not necessary
			if ($entry['CostShare'])
				$charter->setCostShare((float)($entry['CostShare'] / 100));
				// set status according to submit button
			$charter->setStatus($isSubmitForApprovalButton ? ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL : ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION);
			// save project charter
			if ($this->context->projectCharterRepository->save($charter))
			{
				if(!(!$entry['TempFile'] && $isSubmitForApprovalButton))
					$this->flashMessage("Project charter '" . $charter->getName() . "' saved.", 'success');
			}
			else
			{
				$this->flashMessage("Saving of project charter '" . $charter->getName() . "' failed.", 'error');
				return;
			}
			// check if charter summary file is selected
			if(!$entry['TempSummaryFile'] && $isSubmitForApprovalButton)
			{
				if(is_null($latestFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID'])))
				{
					$this->flashMessage("Select project charter summary.", 'error');
					$this->redirect('ProjectCharter:edit', array('id' => $charter->getID()));
				}
			}

			// processing of charter file
			if ($entry['TempFile'])
			{
				$fileName = $this->getFileName($entry['TempFile']);
				// basic file informations
				$file = new ProjectCharterFileHistory();
				$file->setProjectCharterID($charter->getID());
				$file->setVersion(1); // the first version
				$file->setCreationDateTime(new DateTime);
				$file->setFileName($fileName);
				$file->setResponsible($entry['Owner']);
				$file->setOwnerComment($entry['OwnerFileComment']);
				$file->setLatestVersion(TRUE);
				$file->setCommitted(($isSubmitForApprovalButton) ? TRUE : FALSE);
				// set path to storage
				$storage =
					__DIR__ . $this->charters_storage_dir .
					$this->charter_dir_prefix . $file->getProjectCharterID() . '/' .
					$this->version_dir_prefix . $file->getVersion() . '/'
				;
				// save file
				if($this->context->projectCharterFileHistoryRepository->save($file))
					$this->moveTmpFile($storage, $entry['TempFile'], $fileName);
				else
					$this->flashMessage("Saving file failed.", 'error');
			}
			
			// processing of charter summary file
			if ($entry['TempSummaryFile'])
			{
				$fileName = $this->getFileName($entry['TempSummaryFile']);
				// basic file informations
				$file = new ProjectCharterSummaryFileHistory();
				$file->setProjectCharterID($charter->getID());
				$file->setVersion(1); // the first version
				$file->setCreationDateTime(new DateTime);
				$file->setFileName($fileName);
				$file->setResponsible($entry['Owner']);
				$file->setOwnerComment($entry['OwnerSummaryFileComment']);
				$file->setLatestVersion(TRUE);
				$file->setCommitted(($isSubmitForApprovalButton) ? TRUE : FALSE);
				// set path to storage
				$storage =
					__DIR__ . $this->charters_storage_dir .
					$this->charter_dir_prefix . $file->getProjectCharterID() . '/summary/' .
					$this->version_dir_prefix . $file->getVersion() . '/'
				;
				// save file
				if($this->context->projectCharterSummaryFileHistoryRepository->save($file))
					$this->moveTmpFile($storage, $entry['TempSummaryFile'], $fileName);
				else
					$this->flashMessage("Saving file failed.", 'error');
			}

			// define list of budget items source types
			$arrBudgetSource = array('Travel', 'Material');
			// technical groups
			for ($row = 0; $row < $this->session['TGRows']; $row++)
				array_push($arrBudgetSource, 'TGRow' . $row);
			// subcontractors
			for ($row = 0; $row < $this->session['SUBRows']; $row++)
				array_push($arrBudgetSource, 'SUBRow' . $row);

			// basic budget informations
			$budget = new ProjectCharterBudgetHistory();
			$budget->setProjectCharterID($charter->getID());
			$budget->setVersion(1); // first version
			$budget->setCreationDateTime(new DateTime);
			$budget->setResponsible($entry['Owner']);
			$budget->setComment($entry['BudgetComment'] ? $entry['BudgetComment'] : NULL);
			$budget->setLatestVersion(1);
			// save budget informations
			if (!$this->context->projectCharterBudgetHistoryRepository->save($budget))
			{
				$this->flashMessage("Saving budget failed.", 'error');
				return;
			}

			// common basic information for single budget item
			$budgetItem = new ProjectCharterBudget();
			$budgetItem->setProjectCharterID($budget->getProjectCharterID());
			$budgetItem->setVersion(1); // first version
			$params = $this->request->getParameters();
			// for all sources
			foreach ($arrBudgetSource as $source)
			{
				$budgetItem->setTechnicalGroup(NULL);
				$budgetItem->setSubcontractor(NULL);
				// set resource type for budget item
				if ($source == 'Travel')
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL);
				}
				elseif ($source == 'Material')
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL);
				}
				elseif (preg_match('/TGRow[0-9]+/', $source))
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS);
					$budgetItem->setTechnicalGroup($entry[$source]['TechnicalGroup']);

					// Freeze average rate
					$average_rates = $this->getOrgUnitAverageLaborRates($entry[$source]['TechnicalGroup'], $params['id']);
					$budgetItem->setRate($average_rates[$entry['Year']]);
					unset($average_rates);
				}
				elseif (preg_match('/SUBRow[0-9]+/', $source))
				{
					$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING);
					$budgetItem->setSubcontractor($entry[$source]['Subcontractor']);
				}

				$budgetItem->setBudgetQ1($entry[$source]['ownerQ1']);
				$budgetItem->setBudgetQ2($entry[$source]['ownerQ2']);
				$budgetItem->setBudgetQ3($entry[$source]['ownerQ3']);
				$budgetItem->setBudgetQ4($entry[$source]['ownerQ4']);
				// save single budget item
				if (!$this->context->projectCharterBudgetRepository->save($budgetItem))
				{
					$this->flashMessage("Saving budget item failed.", 'error');
				}
				// Init rate before processing next budget item
				$budgetItem->setRate(NULL);
			}
			// save Checklist
			for ($i = 0; $i < count(self::$checklistArray); $i++)
			{
				$checklistItem = new ProjectCharterChecklist();
				$checklistItem->ProjectCharterID = $charter->ID;
				$checklistItem->Title = self::$checklistArray[$i];
				
				$neededArray = array();
				for ($j = 1; $j <= 5; $j++)
				{
					$neededArray[$j] = $entry['Checklist' . $i]['Needed' . $j];
				}
				$checklistItem->NeededArray = $neededArray;
				$this->context->projectCharterChecklistRepository->save($checklistItem);
			}

			// initialization of number of available rows of technical groups and subcontractors
			$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
			$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;

			// update planning state
			// id supplied by URL is Project ID
			$planningState = $this->context->projectPlanningCharterStateRepository->find($params['id'], $this->year);

			if ($planningState !== NULL)
			{
				$planningState->ProjectCharterID = $charter->ID;
				$planningState->State = ProjectPlanningCharterState::PROJECT_PLANNING_CHARTER_STATE_CHARTEREXISTS;

				$this->context->projectPlanningCharterStateRepository->update($planningState);
			}
			
			
			// if creating charter for someone else, then you're not allowed to edit it after creation
			if ($isSubmitForApprovalButton || $entry['Owner'] !== strtoupper($this->getUser()->getID()))
				$this->redirect('ProjectCharter:detail', array('id' => $charter->getID()));
			else
				$this->redirect('ProjectCharter:edit', array('id' => $charter->getID()));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		return;
	}

	/**
	 * Creates new project charter from data from the createCharterForm.
	 * @param SubmitButton $button
	 */
	public function editCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues();

			// check accessibility for user
			$this->secureAccess($entry['ProjectCharter_ID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION), TRUE);
			
			$isSubmitForApprovalButton = ($button->getHtmlName() == 'submitForApproval');

			// basic project charter informations
			$charter = new ProjectCharter();
			$charter->setID($entry['ProjectCharter_ID']);
			$charter->setName($entry['Name']);
			$charter->setSAP_ID($entry['SAP_ID']);
			//$charter->setLineManager($entry['LineManager']);
			$charter->setStrapItemID($entry['StrapItem']);
			$charter->setOwner($this->getUser()->getId());
			$charter->setCustomerID($entry['Customer_ID']);
			$charter->setProgramID($entry['ProgramID']);
			$charter->setCallID($entry['CallID']);
			$charter->setResponsibilityID($entry['Responsibility_ID']);
			$charter->setYear($entry['Year']);
			$charter->setECLicenceID($entry['ECLicenceID']);
			//Project charter second approval duration hack
			$tempCharter = $this->context->projectCharterRepository->find($entry['ProjectCharter_ID']);
			$charter->setLineManager($tempCharter->getLineManager());
			if($tempCharter)
				$charter->setDaysForFinalApproval($tempCharter->DaysForFinalApproval);
			else
				$charter->setDaysForFinalApproval(0);
				// default value is zero, so don't set any value if not necessary
			if ($entry['CostShare'])
				$charter->setCostShare((float)($entry['CostShare'] / 100));
				// set status according to submit button
			$charter->setStatus($isSubmitForApprovalButton ? ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL : ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION);
			// check if charter summary file is selected
			if(!$entry['TempSummaryFile'] && $isSubmitForApprovalButton)
			{
				if(is_null($latestFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID'])))
				{
					$this->flashMessage("Select project charter summary.", 'error');
					$this->redirect('ProjectCharter:edit', array('id' => $charter->getID()));
				}
			}
			// save project charter
			if ($this->context->projectCharterRepository->update($charter))
			{
				$this->flashMessage("Project charter '" . $charter->getName() . "' updated.", 'success');
			}
			else
			{
				$this->flashMessage("Updating of project charter '" . $charter->getName() . "' failed.", 'error');
				return;
			}

			// processing of charter file
			if ($entry['TempFile'])
			{
				$createNewVersion = FALSE;

				$latestFile = $this->context->projectCharterFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				// latest version has been committed, so new version will be created
				if (!$latestFile || ($latestFile && ($latestFile->getCommittedBool())))
					$createNewVersion = TRUE;

				// get file name from teporary file
				$fileName = $this->getFileName($entry['TempFile']);
				
				// set version
				$version = 1;
				if ($latestFile && $createNewVersion)
					$version = $latestFile->getVersion() + 1;
				else if ($entry['FileVersion'])
					$version = $entry['FileVersion'];
				
				// set creation datetime
				$created = new DateTime;
				if (!$createNewVersion && $latestFile)
					$created = $latestFile->getCreationDateTime();
				
				// set comments
				$arrComment = array(
					'OwnerComment' => $entry['OwnerTempFileComment'] ? $entry['OwnerTempFileComment'] : NULL,
					'LineManagerComment' => ($createNewVersion) ? NULL : $entry['LineManagerFileComment']
				);
				
				// basic file informations
				$file = new ProjectCharterFileHistory();
				$file->setProjectCharterID($charter->getID());
				$file->setVersion($version);
				$file->setCreationDateTime($created);
				$file->setFileName($fileName);
				$file->setResponsible($this->getUser()->getId());
				$file->setOwnerComment($arrComment['OwnerComment']);
				$file->setLineManagerComment($arrComment['LineManagerComment']);
				$file->setLatestVersion(1);
				$file->setCommitted(($isSubmitForApprovalButton) ? TRUE : FALSE);
				
				// set path to storage
				$storage =
					__DIR__ . $this->charters_storage_dir .
					$this->charter_dir_prefix . $file->getProjectCharterID() . '/' .
					$this->version_dir_prefix . $file->getVersion() . '/'
				;
				// current file version stays and previous file with same version exists, so it should be removed.
				if (!$createNewVersion && $entry['FileVersion'])
				{
					foreach(glob($storage . '*.*') as $garbage)
					   unlink($garbage);
				}
				// initialization of flag of latest version
				$this->context->projectCharterFileHistoryRepository->initLatestVersion($charter->getID());
				// current file version stays and previous file with same version exists.. means update
				if (!$createNewVersion && $entry['FileVersion'])
				{
					if($this->context->projectCharterFileHistoryRepository->update($file))
						$this->moveTmpFile($storage, $entry['TempFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
				else
				{
					if($this->context->projectCharterFileHistoryRepository->save($file))
						$this->moveTmpFile($storage, $entry['TempFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
			}
			else
			{
				$latestFile = $this->context->projectCharterFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				if ($latestFile)
				{
					$latestFile->setOwnerComment($entry['OwnerFileComment'] ? $entry['OwnerFileComment'] : NULL);
					if (strtolower($latestFile->getResponsible()) == strtolower($this->getUser()->getId()))
						if (!$latestFile->getCommittedBool())
							$latestFile->setCommitted(($isSubmitForApprovalButton) ? TRUE : FALSE);
					
					if(!$this->context->projectCharterFileHistoryRepository->update($latestFile))
							$this->flashMessage("Saving file comment failed.", 'error');
				}
			}
			
			// processing of charter file
			if ($entry['TempSummaryFile'])
			{
				$createNewVersion = FALSE;

				$latestFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				// latest version has been committed, so new version will be created
				if (!$latestFile || ($latestFile && ($latestFile->getCommittedBool())))
					$createNewVersion = TRUE;

				// get file name from teporary file
				$fileName = $this->getFileName($entry['TempSummaryFile']);
				
				// set version
				$version = 1;
				if ($latestFile && $createNewVersion)
					$version = $latestFile->getVersion() + 1;
				else if ($entry['SummaryFileVersion'])
					$version = $entry['SummaryFileVersion'];
				
				// set creation datetime
				$created = new DateTime;
				if (!$createNewVersion && $latestFile)
					$created = $latestFile->getCreationDateTime();

				// set comments
				$arrComment = array(
					'OwnerComment' => $entry['OwnerTempSummaryFileComment'] ? $entry['OwnerTempSummaryFileComment'] : NULL,
					'LineManagerComment' => ($createNewVersion) ? NULL : $entry['LineManagerSummaryFileComment']
				);
				
				// basic file informations
				$file = new ProjectCharterSummaryFileHistory();
				$file->setProjectCharterID($charter->getID());
				$file->setVersion($version);
				$file->setCreationDateTime($created);
				$file->setFileName($fileName);
				$file->setResponsible($this->getUser()->getId());
				$file->setOwnerComment($arrComment['OwnerComment']);
				$file->setLineManagerComment($arrComment['LineManagerComment']);
				$file->setLatestVersion(1);
				$file->setCommitted(($isSubmitForApprovalButton) ? TRUE : FALSE);
				
				// set path to storage
				$storage =
					__DIR__ . $this->charters_storage_dir .
					$this->charter_dir_prefix . $file->getProjectCharterID() . '/summary/' .
					$this->version_dir_prefix . $file->getVersion() . '/'
				;
				// current file version stays and previous file with same version exists, so it should be removed.
				if (!$createNewVersion && $entry['SummaryFileVersion'])
				{
					foreach(glob($storage . '*.*') as $garbage)
					   unlink($garbage);
				}
				// initialization of flag of latest version
				$this->context->projectCharterSummaryFileHistoryRepository->initLatestVersion($charter->getID());
				// current file version stays and previous file with same version exists.. means update
				if (!$createNewVersion && $entry['SummaryFileVersion'])
				{
					if($this->context->projectCharterSummaryFileHistoryRepository->update($file))
						$this->moveTmpFile($storage, $entry['TempSummaryFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
				else
				{
					if($this->context->projectCharterSummaryFileHistoryRepository->save($file))
						$this->moveTmpFile($storage, $entry['TempSummaryFile'], $fileName);
					else
						$this->flashMessage("Saving file failed.", 'error');
				}
			}
			else
			{
				$latestFile = $this->context->projectCharterSummaryFileHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
				if ($latestFile)
				{
					$latestFile->setOwnerComment($entry['OwnerSummaryFileComment'] ? $entry['OwnerSummaryFileComment'] : NULL);
					if (strtolower($latestFile->getResponsible()) == strtolower($this->getUser()->getId()))
						if (!$latestFile->getCommittedBool())
							$latestFile->setCommitted(($isSubmitForApprovalButton) ? TRUE : FALSE);
					
					if(!$this->context->projectCharterSummaryFileHistoryRepository->update($latestFile))
							$this->flashMessage("Saving file comment failed.", 'error');
				}
			}

			$createNewVersion = FALSE;
			$latestBudget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);
			if ($latestBudget)
				// current user is not responsible for latest version of budget so new version will be created
				if (strtolower($latestBudget->getResponsible()) != strtolower($this->getUser()->getId()))
					$createNewVersion = TRUE;

			// define list of budget items source types
			$arrBudgetSource = array('Travel', 'Material');
			// technical groups
			for ($row = 0; $row < $this->session['TGRows']; $row++)
				array_push($arrBudgetSource, 'TGRow' . $row);
			// subcontractors
			for ($row = 0; $row < $this->session['SUBRows']; $row++)
				array_push($arrBudgetSource, 'SUBRow' . $row);

			// delete removed budget items (technical groups and subcontractors)
			if (!$createNewVersion)
			{
				// get saved budget items
				$currentlySaved = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($charter->getID(), (($entry['BudgetVersion']) ? $entry['BudgetVersion'] : 1));
				$currentlySavedID = array();
				foreach ($currentlySaved as $savedBudgetItem)
					$currentlySavedID[] = $savedBudgetItem->getID();

				// get budget items from form
				$fromFormID = array();
				foreach ($arrBudgetSource as $source)
					$fromFormID[] = $entry[$source]['ProjectCharterBudget_ID'];

				// delete removed budget items
				$toDeleteID = array_diff($currentlySavedID, $fromFormID);
				foreach ($toDeleteID as $id)
					$this->context->projectCharterBudgetRepository->delete($id);
			}

				// basic budget informations
				$budget = new ProjectCharterBudgetHistory();
				$budget->setProjectCharterID($charter->getID());
					// set new version number or leave current version number or it is the first version
					if ($createNewVersion)
						$budget->setVersion($latestBudget->getVersion()+1);
					else
						$budget->setVersion((($entry['BudgetVersion']) ? $entry['BudgetVersion'] : 1));
				$budget->setCreationDateTime(new DateTime);
				$budget->setComment($entry['BudgetComment'] ? $entry['BudgetComment'] : NULL);
				$budget->setResponsible($this->getUser()->getId());
				$budget->setLatestVersion(1);

				$this->context->projectCharterBudgetHistoryRepository->initLatestVersion($charter->getID());
				// current budget version stays and previous budget with same version exists.. means update
				if (!$createNewVersion && $entry['BudgetVersion'])
				{
					if (!$this->context->projectCharterBudgetHistoryRepository->update($budget))
					{
						$this->flashMessage("Saving budget failed.", 'error');
						return;
					}
				}
				else
				{
					if (!$this->context->projectCharterBudgetHistoryRepository->save($budget))
					{
						$this->flashMessage("Saving budget failed.", 'error');
						return;
					}
				}

				// common basic informations for single budget item
				$budgetItem = new ProjectCharterBudget();
				$budgetItem->setProjectCharterID($budget->getProjectCharterID());
				$budgetItem->setVersion($budget->getVersion());
				// for all sources
				foreach ($arrBudgetSource as $source)
				{
					$budgetItem->setTechnicalGroup(NULL);
					$budgetItem->setSubcontractor(NULL);
					// set resource type for budget item
					if ($source == 'Travel')
					{
						$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL);
					}
					elseif ($source == 'Material')
					{
						$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL);
					}
					elseif (preg_match('/TGRow[0-9]+/', $source))
					{
						$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS);
						$budgetItem->setTechnicalGroup($entry[$source]['TechnicalGroup']);

						// Copy frozen average rate
						if ($entry[$source]['ProjectCharterBudget_ID'])
						{
							$oldBudgetItem = $this->context->projectCharterBudgetRepository->find($entry[$source]['ProjectCharterBudget_ID']);
							if ($oldBudgetItem)
								$budgetItem->setRate($oldBudgetItem->getRate());

							unset($oldBudgetItem);
						}
						
						// Freeze average rate if it is not frozen yet
						if (!$budgetItem->getRate())
						{
							$average_rates = $this->getOrgUnitAverageLaborRates($entry[$source]['TechnicalGroup']);
							$budgetItem->setRate($average_rates[$entry['Year']]);
							unset($average_rates);
						}
					}
					elseif (preg_match('/SUBRow[0-9]+/', $source))
					{
						$budgetItem->setResourceType(ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING);
						$budgetItem->setSubcontractor($entry[$source]['Subcontractor']);
					}

					if ($entry[$source]['ProjectCharterBudget_ID'])
						$budgetItem->setID($entry[$source]['ProjectCharterBudget_ID']);
					$budgetItem->setBudgetQ1($entry[$source]['ownerQ1']);
					$budgetItem->setBudgetQ2($entry[$source]['ownerQ2']);
					$budgetItem->setBudgetQ3($entry[$source]['ownerQ3']);
					$budgetItem->setBudgetQ4($entry[$source]['ownerQ4']);

					// save single budget item
					if (!$createNewVersion && $entry[$source]['ProjectCharterBudget_ID'])
					{
						if (!$this->context->projectCharterBudgetRepository->update($budgetItem))
							$this->flashMessage("Saving budget item failed.", 'error');
					}
					else
					{
						if (!$this->context->projectCharterBudgetRepository->save($budgetItem))
							$this->flashMessage("Saving budget item failed.", 'error');
					}
					// Init rate before processing next budget item
					$budgetItem->setRate(NULL);
				}
				
			// Checklist
			$checklistItems = $this->context->projectCharterChecklistRepository->findByProjectCharterID($charter->ID);
			if (count($checklistItems))
			{
				for ($i = 0; $i < count(self::$checklistArray); $i++)
				{
					if ($entry['Checklist' . $i])
					{
						$checklistItem = $checklistItems[self::$checklistArray[$i]];
						$checklistItem->ProjectCharterID = $charter->ID;
						$checklistItem->Title = self::$checklistArray[$i];

						$neededArray = array();
						for ($j = 1; $j <= 5; $j++)
						{
							$neededArray[$j] = $entry['Checklist' . $i]['Needed' . $j];
						}
						$checklistItem->NeededArray = $neededArray;
						$this->context->projectCharterChecklistRepository->update($checklistItem);
					}
				}
			}

			// initialization of number of available rows of technical groups and subcontractors
			$this->session['TGRows'] = self::DEFAULT_TGROWS_NUMBER;
			$this->session['SUBRows'] = self::DEFAULT_SUBROWS_NUMBER;
			//
			if ($isSubmitForApprovalButton)
				$this->redirect('ProjectCharter:detail', array('id' => $charter->getID()));
			else
				$this->redirect('ProjectCharter:edit', array('id' => $charter->getID()));
			return;
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		return;
	}

	/**
	 * Sets project charter state to: entirely cancelled
	 * @param SubmitButton $button
	 */
	public function cancelCharterFormSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues();

		// check accessibility for user
		$this->secureAccess($entry['ProjectCharter_ID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION), TRUE);

		// update charter status
		$charter = $this->context->projectCharterRepository->find($entry['ProjectCharter_ID']);
		$charter->setStatus(ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED);
		if (!$this->context->projectCharterRepository->update($charter))
		{
			$this->flashMessage("Saving failed.", 'error');
			return;
		}

		$this->flashMessage("Project charter '" . $charter->getName() . "' cancelled.", 'success');
		$this->redirect('ProjectCharter:detail', array('id' => $entry['ProjectCharter_ID']));
	}

	/**
	 * Processes entries from chooseCharterBaseForm.
	 * @param AppForm $button
	 */
	public function createChooseCharterBaseFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);

			if ($entry['Project'])
				$this->redirect('ProjectCharter:createStepII', array('id' => $entry['Project']));
			else
				$this->redirect('ProjectCharter:createStepII', array('name' => $entry['CharterName']));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		return;
	}
	
	/**
	 * Redirects back to dashbord
	 * @param AppForm $form
	 */
	public function backChooseCharterBaseFormSubmitted(SubmitButton $button)
	{
		$this->redirect('Dashboard:default');
	}

	/**
	 * Handler triggered when Reset button is pressed. It will drop
	 * remembered filter.
	 */
	public function filterCharterFormReset()
	{
		$this->context->formStateRepository->drop('filterCharterForm');
		$this->redirect('this');
	}

	/**
	 * Handler triggered when Reset button is pressed. It will drop
	 * remembered filter.
	 */
	public function filterCharterFormMyActionReset()
	{
		$this->context->formStateRepository->drop('filterCharterFormMyAction');
		$this->redirect('this');
	}

	/**
	 * Charter form submitted.
	 * @param AppForm $form
	 */
	public function filterCharterFormSubmitted(AppForm $form)
	{
		$entry = $form->getValues(TRUE);

		if ($entry['OwnedByCurrentUser'])
			$entry['Owner'] = $this->getUser()->getId();

		$this->payload->filter = $entry;
		if ($this->isAjax())
			$this->invalidateControl('charters');
	}

	/**
	 * Creates form for restoring charter from end states.
	 * @param string $name Name of the form.
	 * @return AppForm Returns AppForm.
	 */
	public function createComponentDetailCharterForm($name)
	{
		$form = new AppForm($this, $name);
		$form->addHidden('ProjectCharter_ID');
		$form->addSubmit('edit', 'Edit charter')
				->onClick[] = array($this, 'editDetailCharterFormSubmitted');
		$form->addSubmit('restore', 'Redefine charter')
				->onClick[] = array($this, 'restoreDetailCharterFormSubmitted');
				$form['restore']->setAttribute('onclick',
						'return confirm("Do you really want to restore the charter for redefinition?");'
					);
		$form->addSubmit('finalApproversDefinition', 'Final approvers definition')
				->onClick[] = array($this, 'finalApproversDefinitionDetailCharterFormSubmitted');
		$form->addSubmit('clApprover', 'CL approval')
				->onClick[] = array($this, 'clApproverDetailCharterFormSubmitted');
		$form->addSubmit('missingApproval', 'Missing approval')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'missingApprovalDetailCharterFormSubmitted');
		$form->addSubmit('finalApproval', 'Final approval')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'finalApprovalDetailCharterFormSubmitted');
		$form->addSubmit('back', 'Go back')
				->setValidationScope(FALSE)
				->onClick[] = array($this, 'backDefault');

		return $form;
	}

	/**
	 * Redirects to charter edit page.
	 * @param SubmitButton $button
	 */
	public function editDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharter_ID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			$this->redirect('ProjectCharter:edit', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}
	
	/**
	 * Redirects to missing approval page.
	 * @param SubmitButton $button
	 */
	public function missingApprovalDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharter_ID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			$this->redirect('ProjectCharterApproval:missingApproval', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}
	
	/**
	 * Redirects to approver page.
	 * @param SubmitButton $button
	 */
	public function finalApprovalDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharter_ID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			$this->redirect('ProjectCharterApproval:approver', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}
	
	/**
	 * Redirects to final approvers definition page.
	 * @param SubmitButton $button
	 */
	public function finalApproversDefinitionDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharter_ID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			$this->redirect('ProjectCharterApproval:definition', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}
	
	/**
	 * Redirects to LM approval page.
	 * @param SubmitButton $button
	 */
	public function clApproverDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharter_ID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			$this->redirect('ProjectCharter:LMapproval', array('id' => $entry['ProjectCharter_ID']));
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}

	/**
	 * Restores charter from end states.
	 * @param SubmitButton $button
	 */
	public function restoreDetailCharterFormSubmitted(SubmitButton $button)
	{
		if ($button->form->isValid())
		{
			$entry = $button->form->getValues(TRUE);
			
			// important TODO: remove this hard-coded hack!
			if($this->context->projectCharterRepository->find($entry['ProjectCharter_ID'])->year <= 2015)
			{
				$this->flashMessage('Modifying of project charters from year 2015 and older has been disabled. '
					. 'Please, create new charter instead.', 'error');
				return;
			}
			
			// check accessibility for user
			$this->secureAccess($entry['ProjectCharter_ID'], array(ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED, ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED, ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION, ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED), TRUE);
			
			$charter = $this->context->projectCharterRepository->find($entry['ProjectCharter_ID']);
			if ($charter)
			{
				$succes = true;
				if($charter->status == ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED)
				{
					$creation_date = date("Y-m-d H:i:s");
					$oldApproverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($entry['ProjectCharter_ID']);

					// Creates new ProjectCharterApproverHistory from old one
					$newApproverHistory = new ProjectCharterApproverHistory();
					$newApproverHistory->ProjectCharterID = $oldApproverHistory->ProjectCharterID;
					$newApproverHistory->Version = $oldApproverHistory->Version + 1;
					$newApproverHistory->CreationDateTime = $creation_date;
					$newApproverHistory->Responsible = $oldApproverHistory->Responsible;
					$newApproverHistory->LatestVersion = 1;
					$newApproverHistory->BudgetVersion = $oldApproverHistory->BudgetVersion;
					$newApproverHistory->FileVersion = $oldApproverHistory->FileVersion;
					$newApproverHistory->EmailContent = null;
					$newApproverHistory->FinalApprovalDate = null;
					// Update old ProjectCharterApproverHistory;
					$oldApproverHistory->LatestVersion = 0;

					/* New list of final approvers */
					$oldApprovers = $this->context->projectCharterApproverRepository->findAllApprover($entry['ProjectCharter_ID'], $newApproverHistory->Version - 1);
					$newApprovers = array();
					foreach($oldApprovers as $oldApprover)
					{
						$newApprovers[$oldApprover->projectCharterApproverID] = new ProjectCharterApprover();
						$newApprovers[$oldApprover->projectCharterApproverID]->ProjectCharterID = $oldApprover->ProjectCharterID;
						$newApprovers[$oldApprover->projectCharterApproverID]->Version = ($oldApprover->Version + 1);
						$newApprovers[$oldApprover->projectCharterApproverID]->ApproverEID = $oldApprover->ApproverEID;
						$newApprovers[$oldApprover->projectCharterApproverID]->Surname = $oldApprover->Surname;
						$newApprovers[$oldApprover->projectCharterApproverID]->Name = $oldApprover->Name;
						$newApprovers[$oldApprover->projectCharterApproverID]->Email = $oldApprover->Email;
						$newApprovers[$oldApprover->projectCharterApproverID]->Role = $oldApprover->Role;
						$newApprovers[$oldApprover->projectCharterApproverID]->Approval = ProjectCharterApprover::DECISION_PENDING;
						$newApprovers[$oldApprover->projectCharterApproverID]->DecisionDateTime = null;
						$newApprovers[$oldApprover->projectCharterApproverID]->Comment = null;
						$newApprovers[$oldApprover->projectCharterApproverID]->AutoApproved = null;
					}
					if($this->context->projectCharterApproverHistoryRepository->update($oldApproverHistory) && $this->context->projectCharterApproverHistoryRepository->save($newApproverHistory))
					{
						foreach($newApprovers as $approver)
						{
							if(!$this->context->projectCharterApproverRepository->save($approver))
							{
								$succes = false;
							}
						}
					}
					else
					{
						$succes = false;
					}
				}
				$charter->setStatus(ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION);
				if ($this->context->projectCharterRepository->update($charter) && $succes)
					$this->flashMessage("Project charter '" . $charter->getName() . "' restored.", 'success');
				else
					$this->flashMessage("Restoring of project charter '" . $charter->getName() . "' failed.", 'error');
			}
			$this->redirect('this');
		}

		$this->flashMessage("Error occured during submitting the form.", 'error');
		$this->redirect('this');
	}

	/**
	 * Redirects back to charters overview.
	 */
	public function backDefault()
	{
		$this->redirect('ProjectCharter:default');
	}

	/**
	 * Redirects back to charters waiting for my action.
	 */
	public function backMyAction()
	{
		$this->redirect('ProjectCharter:myAction');
	}
	
	/**
	 *
	 * @param SubmitButton $button
	 */
	public function uploadCharterFileSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues(TRUE);
		if ($entry['File']->getContentType())
		{
			if ($entry['File']->isOk())
			{
				//get file name
				$fileName = $entry['File']->getSanitizedName();
				// set storage path
				$storage = __DIR__ . $this->charters_temp_storage_dir;
				// remove previous file if exists
				$garbage = $storage . $entry['TempFile'];
				if ($entry['TempFile'] && is_file($garbage))
				   unlink($garbage);
				// find free temp name
				$prefix = $this->charter_temp_prefix . mt_rand(0, 9999);
				$tempFileName = $storage . $prefix . '_' . $fileName;
				while (is_file($tempFileName))
				{
					$prefix = $this->charter_temp_prefix . mt_rand(0, 9999);
					$tempFileName = $storage . $prefix . '_' . $fileName;
				}
				// move file to temp storage
				try
				{
					$entry['File']->move($tempFileName);
					$entry['TempFile'] = $prefix . '_' . $fileName;
					unset($entry['File']);
					$this->flashMessage("File '" . $fileName ."' uploaded.", 'success');
				}
				catch (Exception $e)
				{
					$this->flashMessage("Saving file failed.", 'error');
				}
			}
			else
			{
				$this->flashMessage("Saving file failed.", 'error');
			}
		}

		$this->session['Form'] = $entry;
		$this->redirect('this');
	}
	
	/**
	 *
	 * @param SubmitButton $button
	 */
	public function uploadCharterSummaryFileSubmitted(SubmitButton $button)
	{
		$entry = $button->form->getValues(TRUE);
		if ($entry['SummaryFile']->getContentType())
		{
			if ($entry['SummaryFile']->isOk())
			{
				//get file name
				$fileName = $entry['SummaryFile']->getSanitizedName();
				// set storage path
				$storage = __DIR__ . $this->charters_temp_storage_dir;
				// remove previous file if exists
				$garbage = $storage . $entry['TempSummaryFile'];
				if ($entry['TempSummaryFile'] && is_file($garbage))
				   unlink($garbage);
				// find free temp name
				$prefix = $this->charter_temp_prefix . mt_rand(0, 9999);
				$tempFileName = $storage . $prefix . '_' . $fileName;
				while (is_file($tempFileName))
				{
					$prefix = $this->charter_temp_prefix . mt_rand(0, 9999);
					$tempFileName = $storage . $prefix . '_' . $fileName;
				}
				// move file to temp storage
				try
				{
					$entry['SummaryFile']->move($tempFileName);
					$entry['TempSummaryFile'] = $prefix . '_' . $fileName;
					unset($entry['SummaryFile']);
					$this->flashMessage("File '" . $fileName ."' uploaded.", 'success');
				}
				catch (Exception $e)
				{
					$this->flashMessage("Saving file failed.", 'error');
				}
			}
			else
			{
				$this->flashMessage("Saving file failed.", 'error');
			}
		}
		$this->session['Form'] = $entry;
		$this->redirect('this');
	}

	/**
	 * Adds labor rates for technical groups to template
	 *
	 * @param ITemplate $template
	 * @param ProjectCharter [$charter=null]
	 */
	private function attachLaborRates($template, $charter = null)
	{
		$rates = new StdClass();
		$rates->average = $this->getAverageLabourRates();
		$rates->frozen = $this->getFrozenLaborRates($charter);

		$template->labor_rates = $rates;
	}

	/**
	 * Gets frozen labor rate for all technical groups assigned to given charter
	 *
	 * @param ProjectCharter $charter
	 * @return string JSON encoded array: $arr[group_id][year] = frozen_rate
	 */
	private function getFrozenLaborRates($charter)
	{
		$frozen_rates = array();

		if (!$charter)
			return json_encode($frozen_rates);

		$latest_budget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->ID);

		if ($latest_budget)
		{
			$budget_items = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($charter->ID, $latest_budget->Version);

			foreach ($budget_items as $item)
			{
				if ($item->ResourceType === ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
					$frozen_rates[$item->TechnicalGroup][$charter->Year] = $item->Rate;
			}
		}

		return json_encode($frozen_rates);
	}
	
	/**
	 * Gets average labor rate for all technical groups for all possible years of charter definition
	 *
	 * @return string JSON encoded array: $arr[group_id][year] = average_rate
	 */
	private function getAverageLabourRates()
	{
		$params = $this->request->getParameters();
		
		if ($params['action'] !== 'createStepII')
		{
			// we are supplied by charter's ID instead of Project ID
			$planningState = $this->context->projectPlanningCharterStateRepository->findByProjectCharterIDAndYear($params['id'], $this->year);
			$projectID = $planningState->ProjectID;
		}
		else
		{
			$projectID = $params['id'];
		}
			
		$all_groups = $this->context->organizationUnitRepository->findAll();
		$average_rates = array();
		
		foreach ($all_groups as $group)
			$average_rates[$group->ID] = $this->getOrgUnitAverageLaborRates($group->ID, $projectID);

		return json_encode($average_rates);
	}

	/**
	 * Gets average labor rate for all years of possible charter definition
	 * Average rate is calculated as arithmetic average of rates of all people in given organization unit.
	 *
	 * @param int $unit_id Organization unit id
	 * @param bool $getCorrectedRate Takes in account information from planning module to get proper labour rate
	 * for charter's TGs. Can be used only, if we can rely on the planning module -
	 * e.g. 'Create charter'.
	 * 
	 * Corrected Rate:
	 * When coming from the planning module a person belonging to a certain technical group
	 * can have different rate than the rate calculated as an average for this TG, because
	 * his rate is calculated from his work center and this is sometimes different.
	 * 
	 * @return array Average rates per years: $arr[year] = average_rate
	 * If rate cannot be calculated for specific year, zero value will be returned.
	 */
	private function getOrgUnitAverageLaborRates($unit_id, $projectID = NULL)
	{
		$all_rates = array();
		$average_rates = array($this->year => 0);
		$result = $this->planningHelper->getOrgUnitLabourRate($unit_id, $projectID);
		
		$average_rates[$this->year] = $result['LabourRate'];
		
		return $average_rates;
	}
	
	/**
	 * Filters and sets programs as well as calls according to selected customer.
	 * @param AppForm $form Form with fields dynamically related with field Customer.
	 * @param bool $emptyPrompt Changes promt of selectboxes. TRUE => '', FALSE => '*'. Is set to FALSE by default.
	 */
	protected function setCustomerDependencies($form, $emptyPrompt = FALSE)
	{
		// capture selected customer and related program. To be able to filter list of programs and list of calls
		$customerID = $form['Customer_ID']->getValue();
		$programID = $form['ProgramID']->getValue();
		
		// filter related programs according to selected customer
		$programs = $this->context->customerProgramRepository->findByCustomer($customerID);
		$customerPrograms = ($emptyPrompt) ? array('' => '') : array('' => '*');
		foreach ($programs as $program)
			$customerPrograms[$program->ProgramID] = $program->Program;
		$form['ProgramID']->setItems($customerPrograms);
		
		// filter related calls according to selected program
		$calls = $this->context->customerProgramCallRepository->findByProgram($programID);
		$customerProgramCalls = ($emptyPrompt) ? array('' => '') : array('' => '*');
		foreach ($calls as $call)
			$customerProgramCalls[$call->CallID] = $call->Call;
		$form['CallID']->setItems($customerProgramCalls);
	}

	private function preloadPlanningData($form)
	{
		$costShare = 0;
		$totalUSD = 0;
		
		$params = $this->request->getParameters();
		// id supplied by URL is Project ID
		$planningState = $this->context->projectPlanningCharterStateRepository->find($params['id'], $this->year);
		
		if ($planningState === NULL || $planningState->WorkpackageID === NULL)
		{
			$this->flashMessage('Couldn\'t find planning state or its Work Package. Both are required to create Project Charter.', 'error');
			$this->redirect('Dashboard:default');
		}
		
		$planningWPs = $this->context->workPackageRepository->findByParentID($planningState->WorkpackageID);
		$TGs = array();
		$subCounter = $taskWPCounter = 0;
		foreach ($planningWPs as $WP)
		{
			$tasks = $this->context->taskRepository->findAllWorkPackageTasks($WP->ID);
			switch ($WP->name)
			{
				case 'Subcontractors':
					// each task means one subcontractor
					// subcontractors never exist after init, so we can add them
					foreach ($tasks as $task)
					{
						$budgetArray = array();
						$assignments = $this->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
						foreach ($assignments as $assignment)
						{
							$budgetArray[$assignment->month] = $assignment->cost;
							$costShare += $assignment->cost * $task->costShare;
							$totalUSD += $assignment->cost;
						}

						$subID = explode(' ', $task->SAPID);
						$subID = $subID[1];

						$subcontractor = $this->context->subcontractorRepository->find($subID);
						if ($subcontractor === NULL)
						{
							$this->flashMessage('Couldn\'t preload subcontractor from Task with Task SAP ID: '
								. $task->SAPID, 'error');
							continue;
						}
						
						if (!isset($form['SUBRow' . $subCounter]))
						{
							$this->addSUBRowToForm($form, $this->session['SUBRows']);
							$this->session['SUBRows'] = $this->session['SUBRows'] + 1;
						}
						
						$this->planningHelper->convertToFormQuartals($budgetArray, $form['SUBRow' . $subCounter]);
						$form['SUBRow' . $subCounter]['Subcontractor']->setValue($subcontractor->ID);
						$subCounter++;
					}
					break;

				case 'MAT & TRV':
					foreach ($tasks as $task)
					{
						$budgetArray = array();
						$assignments = $this->context->assignmentsCostRepository->findTaskYearAssignments($task->ID, $this->year);
						foreach ($assignments as $assignment)
						{
							$budgetArray[$assignment->month] = $assignment->cost;
							$costShare += $assignment->cost * $task->costShare;
							$totalUSD += $assignment->cost;
						}

						if ($task->Name == 'MAT ' . $this->year)
						{
							$this->planningHelper->convertToFormQuartals($budgetArray, $form['Material']);
						}
						else if ($task->Name == 'TRV ' . $this->year)
						{
							$this->planningHelper->convertToFormQuartals($budgetArray, $form['Travel']);
						}
						else
						{
							$this->flashMessage('Couldn\'t preload material or travel from Task with Task SAP ID: '
								. $task->SAPID, 'error');
						}
					}
					break;

				default:
					// Task/WP tables with custom names
					foreach ($tasks as $task)
					{
						$assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($task->ID, $this->year);

						foreach ($assignments as $EID => $assignmentArray)
						{
							$person = $this->context->personRepository->find($EID);
//							$orgUnit = $this->context->organizationUnitRepository->find($person->OrganizationUnitID);
							$orgUnitID = $person->OrganizationUnitID;
							
//							if (!$orgUnit || $orgUnit->Type !== OrganizationUnit::ORG_UNIT_TYPE_TG)
//							{
//								$this->flashMessage('Couldn\'t find Technical Group associated with EID: ' . $person->ID . '.', 'warning');
//								continue;
//							}
							
							$labourRate = $this->getOrgUnitAverageLaborRates($orgUnitID, $params['id']);
							$labourRate = $labourRate[$this->year];
							
							$hourArray = array();
							foreach ($assignmentArray as $assignment)
							{
								$hourArray[$assignment->month] = $assignment->hours;
								$USD = $assignment->hours * $labourRate;
								$costShare += $USD * $task->costShare;
								$totalUSD += $USD;
							}
							
							if (isset($TGs[$orgUnitID]))
							{
								// we will be adding values to this TG
								$this->planningHelper->convertToFormQuartals($hourArray, $form['TGRow' . $TGs[$orgUnitID]]);
							}
							else
							{
								// we'll be creating a new TG-row
								$TGs[$orgUnitID] = $taskWPCounter;
								
								if (!isset($form['TGRow' . $taskWPCounter]))
								{
									$this->addTGRowToForm($form, $this->session['TGRows']);
									$this->session['TGRows'] = $this->session['TGRows'] + 1;
								}

								$this->planningHelper->convertToFormQuartals($hourArray, $form['TGRow' . $taskWPCounter]);
								$form['TGRow' . $taskWPCounter]['TechnicalGroup']->setValue($orgUnitID);
								$taskWPCounter++;
							}
						}
					}
			}
		}
		
		if ($totalUSD != 0)
			return $costShare / $totalUSD;
		else
			return 0;
	}
	
	/**
	 * Function recalculates frozen labour rates of project charter budget items for given year.
	 * It can takes long time - extension of maximum time limit is needed
	 * 
	 * @param int $year
	 * @return boolean True if success
	 */
	private function recalculateAllBudgetItems($year)
	{
		if(is_null($year))
		{
			$year = (int)Date('Y') + 1;
		}
		$success = true;
		$allCharters = $this->context->projectCharterRepository->findAll($year);
		$budgetItems = Array();
		foreach($allCharters as $charter)
		{
			$latest_budget = $this->context->projectCharterBudgetHistoryRepository->findLatestVersion($charter->ID);
			$budgetItems = $this->context->projectCharterBudgetRepository->findByProjecCharterIDAndVersion($charter->ID, $latest_budget->Version);
			foreach ($budgetItems as $item)
			{
				if ($item->ResourceType === ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
				{
					$rates = $this->getOrgUnitAverageLaborRates($item->TechnicalGroup);
					$item->Rate = $rates[$year];
					if(!$this->context->projectCharterBudgetRepository->update($item))
					{
						$success = false;
						break(2);
					}

				}
			}
		}
		return $success;
	}
}
