<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for administration of tasks.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TaskPresenter  extends SecuredPresenter
{
    /**
     * eCharge service for importing actuals
     * @var EChargeImportService 
     */
    private $eChargeImportService;
    
	/**
	 * Method runs iASAP actuals import
	 * 
	 * @param Task $task
	 */
	private function runiASAPImport($task)
	{
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'iasap_actuals_import-'.$logs_suffix.'.html';
		$batch_id = 'iasap_actuals-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
		$filelog = $this->context->createFileLogger($filelog_path, $batch_id, "IASAP ACTUALS") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "IASAP ACTUALS"));

		$import_result = FALSE;
		$import_ex_error = '';

		$tasks = array();
		$tasks[$task->SAPID] = $task;

		if($task->isLabourTask())
		{
			$iASAPActualsImportService = new IASAPActualsImportServiceSAPID($this->context, $tasks, $multilog);
			//$iASAPActualsImportService->setRange(Date('Y') . '-01-01', date_format(new DateTime(), 'Y-m-d'));

			try
			{
				switch ($iASAPActualsImportService->runLaborImport())
				{
					case iASAPActualsImportServiceSAPID::RESULT_SUCCESS:
						$this->flashMessage('iASAP actuals import was successful.', 'success');
						break;
					case iASAPActualsImportServiceSAPID::RESULT_WARNING:
						$this->flashMessage('iASAP actuals import was successful with warnings.', 'warning');
						break;
					case iASAPActualsImportServiceSAPID::RESULT_ERROR:
						$this->flashMessage('iASAP actuals import failed!', 'error');
						break;
					case iASAPActualsImportServiceSAPID::RESULT_NO_DATA:
						$this->flashMessage('For Task SAP ID: "' . $task->SAPID. '" was not found any actuals!', 'warning');
						break;
					default: break;
				}
			}
			catch (IASAPActualsImportSAPIIDServiceException $e)
			{
				$import_result = FALSE;
				$import_ex_error = $e->getMessage();
				$multilog->log('iASAP actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
				Debugger::log('iASAP actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
				$this->flashMessage('Actuals imported unsuccessfully.', 'error');
			}
			$filelog->close();
		}
		else
		{
			$iASAPActualsImportService = new IASAPActualsImportServiceSAPID($this->context, $tasks, $multilog);
			//$iASAPActualsImportService->setRange(Date('Y') . '-01-01', date_format(new DateTime(), 'Y-m-d'));

			try
			{
				switch ($iASAPActualsImportService->runCostImport())
				{
					case iASAPActualsImportServiceSAPID::RESULT_SUCCESS:
						$this->flashMessage('iASAP actuals import was successful.', 'success');
						break;
					case iASAPActualsImportServiceSAPID::RESULT_WARNING:
						$this->flashMessage('iASAP actuals import was successful with warnings.', 'warning');
						break;
					case iASAPActualsImportServiceSAPID::RESULT_ERROR:
						$this->flashMessage('iASAP actuals import failed!', 'error');
						break;
					case iASAPActualsImportServiceSAPID::RESULT_NO_DATA:
						$this->flashMessage('For Task SAP ID: "' . $task->SAPID. '" was not found any actuals!', 'warning');
						break;
					default: break;
				}
			}
			catch (IASAPActualsImportSAPIDServiceException $e)
			{
				$import_result = FALSE;
				$import_ex_error = $e->getMessage();
				$multilog->log('iASAP actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
				Debugger::log('iASAP actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
				$this->flashMessage('Actuals imported unsuccessfully.', 'error');
			}
			$filelog->close();
		}
	}
	
	/**
	 * Method runs E-Cahrge actuals import
	 * 
	 * @param Task $task
	 */
	private function runEChargeImport($task)
	{
		if($task->isLabourTask())
		{
			/* init logging */
			$logs_suffix = time().'-'. rand(1, 9);
			$current_log_file_name = 'echarge_import-'.$logs_suffix.'.html';
			$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
			$batch_id = 'echarge-'.$logs_suffix;
			$filelog = $this->context->createFileLogger($filelog_path, $batch_id, "ECHARGE ACTUALS") ;
			$multilog = new MultiLogger();
			$multilog->add($filelog);
			$multilog->add($this->context->createLogger($batch_id, "ECHARGE ACTUALS"));

			$import_result = FALSE;
			$import_ex_error = '';

			$this->eChargeImportService = new EChargeImportService($this->context, $multilog);

			try
			{
				$import_result = $this->eChargeImportService->runImportforTask($task);
			}
			catch (EChargeImportServiceException $e)
			{
				Debugger::log('eCharge import failed: ' . $e->getMessage(), Debugger::ERROR);
				$multilog->log('eCharge import failed: ' . $e->getMessage(), ILogger::ERROR);
				$import_result = FALSE;
				$import_ex_error = $e->getMessage();
			}
			$filelog->close();
			
			if($import_result)
				$this->flashMessage('E-Charge actuals import was successful.', 'success');
			else
				$this->flashMessage('E-Charge actuals import failed! ' . $import_ex_error, 'error');
		}
		else
			$this->flashMessage('E-charge actuals import cannot be done for cost task!', 'error');
	}
	
	/**
	 * Method runs TLC actuals import
	 * 
	 * @param Task $task
	 */
	private function runTLCImport($task)
	{
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'tlc_actuals_task_import-'.$logs_suffix.'.html';
		$batch_id = 'tlc_actuals_task_import-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "TLC ACTUALS TASK IMPORT") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "TLC ACTUALS TASK IMPORT"));

		//  time admin table import
        $iASAPActualsImportServiceTLC = new IASAPActualsImportServiceTLC($this->context, $multilog);
        
        try
        {
			switch ($iASAPActualsImportServiceTLC->runTaskImport($task))
			{
				case IASAPActualsImportServiceTLC::RESULT_SUCCESS:
					$this->flashMessage('TLC task actuals import was successful.', 'success');
					break;
				case IASAPActualsImportServiceTLC::RESULT_NO_DATA:
					$this->flashMessage('TLC task actuals import was successful but no actuals was found.', 'warning');
					break;
				case IASAPActualsImportServiceTLC::RESULT_WARNING:
					$this->flashMessage('TLC task actuals import was successful with warnings.', 'warning');
					break;
				case IASAPActualsImportServiceTLC::RESULT_ERROR:
					$this->flashMessage('TLC task actuals import failed!', 'error');
					break;
				default: break;
			}
        }
        catch (IASAPActualsImportServiceTLCException $e)
        {
			$this->flashMessage("TLC task actuals import failed: " . $e->getMessage(), 'error');
			$multilog->log('TLC task actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('TLC task actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
		}
        $filelog->close();
	}

	/**
	 * Shows form for task creation
	 */
	public function renderCreate($projectID = null)
	{
		if(is_null($projectID) && isset($this->payload->projectID))
			$projectID = $this->payload->projectID;
		$project = $this->context->projectRepository->find($projectID);
		if(!$project)
		{
			$this->flashMessage('Project with ID: ' . $projectID . ' was not found', 'notice');
			$this->redirect('Project:default');
		}
		$form = $this['createTaskForm'];
//		$findHtsTrvTaskSapIDForm = $this['findHtsTrvTaskSapIDForm'];
		
		$defaultValues = array();
		
		//sets hts trv task sap id if defined.
//		if(isset($this->payload->sapIDFound))
//		{
//			$defaultValues['SAP_ID'] = $this->payload->sapID;
//			$defaultValues['Def'] = $this->payload->def;
//		}
//		else
//		{
		$form['SAP_ID']->setValue($this->context->taskRepository->getNextTaskID());
		$form['Def']->setValue('0010');
//		}
		
//		if(isset($this->payload->sapIDFound))
//		{
//			$this->flashMessage('HTS TRV Task: ' . $this->payload->taskName . '  has been found! SAP ID was filled up.', 'success');
//		}
//		if(isset($this->payload->sapIDNotFound))
//		{
//			$this->flashMessage('HTS TRV Task SAP ID has not been found!', 'error');
//		}

		$defaultValues['ProjectID'] = $projectID;
		$defaultValues['TaskPlan'] = Date('Y');
		$form->setDefaults($defaultValues);
		$this->template->form = $form;
//		$this->template->findHtsTrvTaskSapIDForm = $findHtsTrvTaskSapIDForm;
//        $this->template->linkToFindSignal = str_replace('?', '?do=findHtsTrvTaskSapIDForm&', $this->link('Task:create'));
	}
	
    /**
     * Load values to second select at detail form
	 * @param int Item id
	 * @param int Activity id
     * @param string Activity type
     */
    public function handleFindHtsTrvTaskSapIDForm($projectID, $prefix, $sapID, $suffix, $def)
    {
		// connect iASAP and find SAP ID
		//$wpSAPID = 'AE-00002996-003-0002';
		$wpSAPID = $prefix . '-' . $sapID . '-' . $suffix . '-' . $def;
		$htsTrvTaskSAPIDFinder = new HtsTrvTaskSAPID($wpSAPID);
		$htsTrvTaskSAPID = $htsTrvTaskSAPIDFinder->findHtsTrvTaskSAPID();

		$this->payload->projectID = $projectID;
		if($htsTrvTaskSAPID)
		{
			$this->payload->sapID = $htsTrvTaskSAPID['network'];
			$this->payload->def = $htsTrvTaskSAPID['def'];
			$this->payload->taskName = $htsTrvTaskSAPID['taskName'];
			$this->payload->sapIDFound = true;
		}
		else
		{
			$this->payload->sapIDNotFound = true;
		}
		$this->invalidateControl('flashMessages');
		$this->invalidateControl('createTaskFormSnippet');
    }
	
    /**
     * Shows detail of Task.
     * 
     * @param int $id TaskPlan ID.
	 * @param int $taskID Task ID.
     */
    public function renderShowDetail($taskID)
    {
        $task = $this->context->taskRepository->find($taskID);
		if($task == null)
		{
			$this->flashMessage('Task with ID: ' . $taskID . ' not found.', 'error');
			$this->redirect('Project:default');
		}
		
		$form = $this['editTaskForm'];
		// set only active work centers
        $tmp = $this->context->costCenterRepository->findAllCostCenters();
        $costCenters = array();
        foreach ($tmp as $costCenter)
        {
			if($costCenter->Status == 1 || $task->CostCenterID == $costCenter->ID)
			{
				$costCenters[$costCenter->ID] = $costCenter->Name . " (" . $costCenter->ID . ")";
			}
        }
		$form['CostCenter']->setItems($costCenters);
		
		$formDefaults = array(
			'TaskID' => $task->ID,
			'SAP_ID' => $task->Network,
			'Def' => $task->Definition,
			'Name' => $task->Name,
			'Date' => $task->Date,
			'CostCenter' => $task->CostCenterID,
			'Remark' => $task->Remark,
			'CostShare' => $task->SAPCostSharePercentage			
		);
		if($task->LabourTaskFlag)
			$formDefaults['LaborTask'] = 'Yes';
		else
			$formDefaults['LaborTask'] = 'No';
		
//		if($task->ActualsImport)
//		{
//			$formDefaults['Import'] = '1';
//			$formDefaults['ImportSource'] = $task->ActualsImportDataSource;
//		}
//		else
//			$formDefaults['Import'] = '0';
		
		if($task->ActualsImportDataSource == Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL)
		{
			$formDefaults['DataSource'] = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL;
		}
		else
		{
			$formDefaults['DataSource'] = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC;
		}
		
		$taskYears = array();
		$taskPlans = $this->context->taskPlanRepository->findTaskPlans($task->ID);
		foreach($taskPlans as $taskPlan)
		{
			if($taskPlan->Year >= Date('Y'))
				$taskYears[$taskPlan->Year] = $taskPlan->Year;
		}
		$formDefaults['TaskPlan'] = $taskYears;
		
		$form->setDefaults($formDefaults);
		$this->template->existingPlans = $taskYears;
		$this->template->editTaskForm = $form;
		$this->template->task = $task;
    }
	
    /**
     * Creates general Task form.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentTaskForm($name)
    {
        $form = new AppForm($this, $name);

        $form->addText('SAP_ID', HPTForm::createLabel('SAP ID', 'Automatically generated'), 10)
				->setAttribute('maxlength', '10');
				//->addRule(Form::LENGTH, 'First part of SAP ID must be 10 characters!', 10);
		$form->addText('Def', 'Acitvity definition', 4)
				->setAttribute('maxlength', '4');
				//->addRule(Form::LENGTH, 'Second part of SAP ID must be 4 characters!', 4);
        
		$form->addHidden('ProjectID');
		
		$form->addText('Name', 'Task Name', 30, 500);
		$form['Name']->setRequired('Task name is mandatory!');
		
		$form->addText('Date', 'Date')
			->addCondition(Form::FILLED, TRUE)
				->addRule(Form::PATTERN, '%value is not valid %label!', '2[0-9]{3}-(0[0-9]{1}|1[012]{1})-([012]{1}[0-9]|3[01]{1})');
        $form['Date']->getControlPrototype()->addClass('datepicker');
		
        $tmp = $this->context->costCenterRepository->findAllCostCenters();
        $costCenters = array();
        foreach ($tmp as $costCenter)
		{
			if($costCenter->Status == 1)
			{
				$costCenters[$costCenter->ID] = $costCenter->Name . " (" . $costCenter->ID . ")";
			}
		}
        $form->addSelect2('CostCenter', 'Work Center', $costCenters)
				->setPrompt('*')
				->setAttribute('style', 'width: 380px;');
		
		$form->addTextArea('Remark', 'Remark', 35, 5);
        
		$form->addText('CostShare', HPTForm::createLabel('Cost Share', 'Percentage of costs sharing among customer and supplier.'), 3, 3)
				    ->addRule(Form::RANGE, '%label should be number in range <0,100>', array(0, 100));
		
		$form->addRadioList('LaborTask', 'Labor Task', array('Yes' => 'Yes', 'No' => 'No'));
		$form['LaborTask']->getSeparatorPrototype()->setName(NULL);
		$form->setDefaults(array('LaborTask' => 'Yes'));
		$form['LaborTask']->setRequired('Labor task is mandatory!');
		
		$form['CostCenter']->addConditionOn($form['LaborTask'], Form::EQUAL, 'Yes')
					        ->addRule(Form::FILLED, 'Work Center is mandatory!');
		
		$year = Date('Y');
		$years = array();
		for($i = ($year); $i <= ($year + 10); $i++)
		{
			$years[$i] = $i;
		}
		$form->addMultiSelect('TaskPlan', 'Task Plan', $years)
				->setDefaultValue('');
		$form['TaskPlan']
			->setAttribute('size', '4')
			->setAttribute('style', 'width: 150px;');
		
//		$form->addRadioList('Import', HPTForm::createLabel('Import Actuals', 'Import actuals by inserted SAP ID'), array(1 => 'Yes', 0 => 'No'));
//		$form['Import']->getSeparatorPrototype()->setName(NULL);
//		$form->setDefaults(array('Import' => 0));
//		$form['Import']->setRequired('Import actuals is mandatory!');
//		$form['Import']->addCondition($form::EQUAL, TRUE)
//			->toggle('DataSource');
//		$form['Import']->addCondition($form::EQUAL, FALSE)
//			->toggle('DataSourceHidden');
//		
//		$form->addRadioList('ImportSource', HPTForm::createLabel('Import Source', 'Choose source of imported data'), array(
//			Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE => Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE,
//			Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP => Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP,
//			Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL => Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL
//			));
//		$form['ImportSource']->getSeparatorPrototype()->setName(NULL);
//		$form['ImportSource']->addConditionOn($form['Import'], Form::EQUAL, TRUE)
//				->addRule(Form::FILLED, 'Import data source is mandatory!');

		$form->addRadioList('DataSource',HPTForm::createLabel('Data Source', 'Choose source of actuals. TLC - Imports actuals only if there are some actuals for created task (Charge number and Task SAP ID matches)'), array(
			Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC => Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC,
			Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL => Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL
			));
		$form['DataSource']->getSeparatorPrototype()->setName(NULL);
		$form['DataSource']->setRequired('Data source is mandatory!');
		$form->setDefaults(array('DataSource' => Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_TLC));
		
        return $form;
    }
	
    /**
     * Creates form for creating Task based on createComponentTaskForm.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentCreateTaskForm($name)
    {
        $form = $this->createComponentTaskForm($name);
		
//		$form->addButton('findHtsTrvTaskSapID', 'Find HTS TRV SAP ID')
//			->setAttribute('onclick', '$.fancybox({content: $("#findHtsTrvTaskSapIDBox").html()});');
		
        $form->addSubmit('save', 'Save')
                ->onClick[] = callback($this, 'detailTaskFormSubmittedSave');
        $form->addSubmit('back', 'Back to project')
                ->setValidationScope(FALSE)
                ->onClick[] = callback($this, 'backClicked');
   
        return $form;
    }
	
    /**
     * Creates form for editing Task based on createComponentTaskForm.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentEditTaskForm($name)
    {
        $form = $this->createComponentTaskForm($name);
		
		$form->addHidden('TaskID');
		
		$form['LaborTask']->setDisabled(TRUE);
		
        $form->addSubmit('save', 'Save')
                ->onClick[] = callback($this, 'detailTaskFormSubmittedEdit');
		
        $form->addSubmit('back', 'Back to Project')
                ->setValidationScope(FALSE)
                ->onClick[] = callback($this, 'backProjectClicked');
   
		$form->addSubmit('delete', 'Delete Task')
                ->onClick[] = callback($this, 'deleteTaskClicked');
		$form['delete']->setAttribute('onclick', '
			return confirm("Are you sure to delete the task?");
		');
		
        return $form;
    }
	
    /**
     * Task Form Submitted - Save.
     * 
     * @param SubmitButton $button 
     */
    public function detailTaskFormSubmittedSave(SubmitButton $button)
    {
        $entry = $button->form->getValues();
		if ($button->form->isValid())
		{
			$task = new Task();
			$task->Name = $entry['Name'];
			$task->ProjectID = $entry['ProjectID'];
			// get task's project
			$project = $this->context->projectRepository->find($entry['ProjectID']);
			// SAP ID has to be unique
			$taskAlreadyExists = $this->context->taskRepository->findBySAPID($entry['SAP_ID'] . ' ' . $entry['Def']);
			if($taskAlreadyExists)
			{
				$project = $this->context->projectRepository->findProjectByTaskId($taskAlreadyExists->ID);
				$this->flashMessage("Task with SAP ID: " . $entry['SAP_ID'] . ' ' . $entry['Def'] . ' already exists in project: ' . $project->Name . ' (' . $project->SAP_ID . ').', 'error');
				$this->redirect('Task:create', $entry['ProjectID']);
			}
			$task->SAPID = $entry['SAP_ID'] . ' ' . $entry['Def'];
			$task->Network = $entry['SAP_ID'];
			$task->Definition = $entry['Def'];
			$task->Remark = $entry['Remark'];
			$task->SAPCostSharePercentage = (strlen($entry['CostShare'])) ? $entry['CostShare'] : null;
			$task->CostShare = $project->RealCostShare;
			$task->CostAsLabour = 0;
			if(Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL == $entry['DataSource'])
			{
				$task->ActualsImport = 1;
				$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL;
			}
			else
			{
				$task->ActualsImport = 0;
				$task->ActualsImportDataSource = null;
			}
//			$task->ActualsImport = $entry['Import'];
//			//Set null import data source if import flag is not selected
//			if($entry['Import'] == 0)
//				$task->ActualsImportDataSource = null;
//			else
//				$task->ActualsImportDataSource = $entry['ImportSource'];

			$task->Date = date('Y-m-d');
			$task->Fake = 1;
			if($entry['LaborTask'] == 'Yes')
				$task->CostCenterID = $entry['CostCenter'];

			// If project is placeholder project then create root WP
			if($project->Fake)
			{
				$wpRoot = $this->context->workPackageRepository->findBySAPID($project->SAP_ID);
				if(!$wpRoot) // if root doesnt exist, wp needs to be created, create WP for labour and cost tasks also
				{
					$wpRoot = new WorkPackage();
					$wpRoot->ProjectID = $project->ID;
					$wpRoot->SAPID = $project->SAP_ID;
					$wpRoot->ParentID = 0;
					$wpRoot->Name = $project->Name;
					$wpRoot->Date = $task->Date;
					if($this->context->workPackageRepository->save($wpRoot))
					{
						$this->flashMessage("Root work package for project " . $project->Name . " created.", 'success');
					}
					else
					{
						$this->flashMessage("Root work package creation failed.", 'error');
						$this->redirect('Task:create', $entry['ProjectID']);
					}
					// Placeholder WP
					$placeholderWP = new WorkPackage();
					$placeholderWP->ProjectID = 0;
					if($project->SAP_ID)
						$placeholderWP->SAPID = $project->SAP_ID . '-Placeholder';
					else
						$placeholderWP->SAPID = $project->Name . '-Placeholder';

					$placeholderWP->ParentID = $wpRoot->ID;
					$placeholderWP->Name = $wpRoot->Name;
					$placeholderWP->Date = $task->Date;
					if($this->context->workPackageRepository->save($placeholderWP))
					{
						$this->flashMessage("Placeholder work package created.", 'success');
					}
					else
					{
						$this->flashMessage("Placeholder work package creation failed", 'error');
						$this->redirect('Task:create', $entry['ProjectID']);
					}
				}
				else // Get task WP if root WP already exits
				{
					if($project->SAP_ID)
						$placeholderWP = $this->context->workPackageRepository->findBySAPID($project->SAP_ID . '-Placeholder');
					else
						$placeholderWP = $this->context->workPackageRepository->findBySAPID($project->Name . '-Placeholder');
				}
			}
			else // Saves task for real project
			{
				$wpRoot = $this->context->workPackageRepository->findBySAPID($project->SAP_ID);
				if(!$wpRoot)
				{
					$this->flashMessage("Task creation failed. Root work package doesn't exists.", 'error');
					$this->redirect('project:showDetial', $entry['ProjectID']);
				}
				else
				{
					$placeholderWP = $this->context->workPackageRepository->findBySAPID($project->SAP_ID . '-Placeholder');
					if(!$placeholderWP) // if WP doesnt exists, creates one
					{
						// labour WP
						$placeholderWP = new WorkPackage();
						$placeholderWP->ProjectID = 0;
						$placeholderWP->SAPID = $project->SAP_ID . '-Placeholder';
						$placeholderWP->ParentID = $wpRoot->ID;
						$placeholderWP->Name = $wpRoot->Name . '-Placeholder';
						$placeholderWP->Date = $task->Date;
						if($this->context->workPackageRepository->save($placeholderWP))
						{
							$this->flashMessage("Placehoder work package created.", 'success');
						}
						else
						{
							$this->flashMessage("Placeholder work package creation failed", 'error');
							$this->redirect('Task:create', $entry['ProjectID']);
						}
					}
				}
			}
			$task->WorkPackageID = $placeholderWP->ID;

			//now is time to save task
			if($this->context->taskRepository->save($task))
			{
				$this->flashMessage("Task " . $task->Name . " created.", 'success');
			}
			else
			{
				$this->flashMessage("Task creation failed", 'error');
				$this->redirect('Task:create', $entry['ProjectID']);
			}

			// there is also need to create task plan for just created task
			foreach ($entry['TaskPlan'] as $year)
			{
				$taskPlan = new TaskYearPlan();
				$taskPlan->Year = $year;
				$taskPlan->TaskID = $task->ID;
				for ($i = 1; $i <= 12; $i++) 
					$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));
				if($this->context->taskPlanRepository->save($taskPlan))
				{
					$this->flashMessage("Task Plan for task " . $task->Name . " and year " . $year . " created.", 'success');
				}
				else
				{
					$this->flashMessage("Task Plan creation failed", 'error');
					$this->redirect('Task:create', $entry['ProjectID']);
				}
			}
			if($entry['LaborTask'] == 'Yes')
				$task->LabourTaskFlag = true;
			else
				$task->LabourTaskFlag = false;
			$this->context->taskRepository->update($task);
			
			// Import actuals
//			if($task->ActualsImport)
//			{
//				switch($task->ActualsImportDataSource)
//				{
//					case Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP:
//						$this->runiASAPImport($task);
//						break;
//					case Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE:
//						$this->runEChargeImport($task);
//						break;
//					default:
//						break;
//				}
//			}
			if(($task->ActualsImport == 0) && ($entry['LaborTask'] == 'Yes'))
			{
				$this->runTLCImport($task);
			}
		}
		else
		{
			$this->flashMessage("Invalid form values.");
		}
        $this->redirect('Project:detail', $entry['ProjectID']);
    }
	
    /**
     * Go back button redirects back to :default.
     * 
     * @param SubmitButton $button Go back button.
     */
    public function backClicked(SubmitButton $button)
    {
		$projectID = $this->getHttpRequest()->getQuery('projectID');
        $this->redirect('Project:detail', $projectID);
    }
	
    /**
     * Navugaiton Form Submitted - Delete.
     * @param SubmitButton $button 
     */
    public function deleteTaskClicked(SubmitButton $button)
    {
        $entry = $button->form->getValues();
		$project = $this->context->projectRepository->findProjectByTaskId($entry['TaskID']);
		if ($this->context->taskRepository->delete($entry['TaskID']))
		{
			$this->flashMessage("Task " . $entry['TaskID'] . " deleted.", 'success');
			$this->redirect('Project:detail', $project->ID);
		}
		else
		{
			$this->flashMessage("Error: Person " . $entry['TaskID'] . " not deleted.", 'error');
		}
       $this->redirect('Project:detail', $project->ID);
    }
	
    /**
     * Go back button redirects back to :default.
     * 
     * @param SubmitButton $button Go back button.
     */
    public function backProjectClicked(SubmitButton $button)
    {
		$project = $this->context->projectRepository->findProjectByTaskId($this->getHttpRequest()->getQuery('taskID'));
		$this->redirect('Project:detail', $project->ID);
    }
	
    /**
     * Task Form Submitted - Edit.
     * 
     * @param SubmitButton $button 
     */
    public function detailTaskFormSubmittedEdit(SubmitButton $button)
    {
        $entry = $button->form->getValues();
		$project = $this->context->projectRepository->findProjectByTaskId($entry['TaskID']);
		if ($button->form->isValid())
		{
			$task = $this->context->taskRepository->find($entry['TaskID']);
			$taskAlreadyExists = $this->context->taskRepository->findBySAPID($entry['SAP_ID'] . ' ' . $entry['Def']);
			if($this->context->taskRepository->findBySAPID($entry['SAP_ID'] . ' ' . $entry['Def']) && $task->SAPID != $entry['SAP_ID'] . ' ' . $entry['Def'])
			{
				$project = $this->context->projectRepository->findProjectByTaskId($taskAlreadyExists->ID);
				$this->flashMessage("Task with SAP ID: " . $entry['SAP_ID'] . ' ' . $entry['Def'] . ' already exists in project: ' . $project->Name . ' (' . $project->SAP_ID . ').', 'error');
				$this->redirect('Task:showDetail', $task->ID);
			}
			$task->SAPID = $entry['SAP_ID'] . ' ' . $entry['Def'];
			$task->Network = $entry['SAP_ID'];
			$task->Definition = $entry['Def'];
			$task->Name = $entry['Name'];
			$task->SAPCostSharePercentage = (strlen($entry['CostShare'])) ? $entry['CostShare'] : null;
			$task->CostShare = $project->RealCostShare;
			$task->Remark = $entry['Remark'];
			$task->CostAsLabour = 0;

//			//remove actuals if import flag is cancelled or if import data source is changed
//			if(($task->ActualsImport == 1 && $entry['Import'] == 0) || ($task->ActualsImportDataSource != $entry['ImportSource']))
//			{
//				if($task->isLabourTask())
//					$this->context->actualsRepository->deleteActualsByTaskSAPID($task->SAPID);
//				else
//					$this->context->actualsCostRepository->deleteDataForTask($task->ID);
//			}
//			$task->ActualsImport = $entry['Import'];
//			//remove import data source if import flag is not selected
//			if($entry['Import'] == 0)
//				$task->ActualsImportDataSource = null;
//			else
//				$task->ActualsImportDataSource = $entry['ImportSource'];
			
			// deletes actuals if manual data source has been removed.
			if(($task->ActualsImportDataSource == Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL) && ($entry['DataSource'] != Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL))
			{
				if($task->isLabourTask())
					$this->context->actualsRepository->deleteActualsByTaskSAPIDAndSource($task->SAPID, Actuals::SOURCE_MANUAL);
				else
					$this->context->actualsCostRepository->deleteDataForTaskAndSource($task->ID, ActualsCost::SOURCE_MANUAL);
				//Get SAP(e-charge, isasap, timeadmin, yspdetal) actuals from temp table
				$this->context->actualsSAPTempRepository->moveIntoActuals($task->SAPID);
				$this->context->actualsSAPTempRepository->deleteActualsByTaskSAPID($task->SAPID);
			}
			
			if(Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL == $entry['DataSource'])
			{
				$task->ActualsImport = 1;
				$task->ActualsImportDataSource = Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_MANUAL;
				//move Sap actuals to ActualsSapTemp table
				$this->context->actualsSAPTempRepository->moveIntoActualsSapTemp($task->SAPID);
				$this->context->actualsRepository->deleteActualsByTaskSAPID($task->SAPID);
			}
			else
			{
				$task->ActualsImport = 0;
				$task->ActualsImportDataSource = null;
			}
			
			$wcCantBeChanged = false;
			if($task->isLabourTask())
			{
				if($task->CostCenterID != $entry['CostCenter'])
				{
				if($this->context->personTaskAssignmentRepository->hasTaskAssignmentAtAll($entry['TaskID']))
					{
						$wcCantBeChanged = true;
					}
					else
						$task->CostCenterID = $entry['CostCenter'];
				}
			}
			if($this->context->taskRepository->update($task))
			{
				if($wcCantBeChanged)
				{
					$this->flashMessage("Task was updated, however update was not fully completed!", 'success');
					$this->flashMessage("Task has some assigment. Assignment must be removed before change of work center. Work center has not been changed!", 'warning');
				}
				else
					$this->flashMessage("Task was updated.", 'success');
			}
			else
				$this->flashMessage("Task was not updated. Action failed.", 'error');
			
			// there is also need to updates task plans
			$taskPlans = array();
			$taskPlans = $this->context->taskPlanRepository->findTaskPlans($task->ID);
			//Delete task plans
			foreach ($taskPlans as $taskPlan)
			{
				 if(!in_array($taskPlan->Year, $entry['TaskPlan']) && Date('Y') <= $taskPlan->Year)
				 {
					 if($this->context->taskPlanRepository->delete($taskPlan->TaskPlanID))
					 {
						 // deletes actuals assignment
						 if($task->isLabourTask())
						 {
							 $this->context->personTaskAssignmentRepository->deleteByTaskAndYearAssignment($task->ID, $taskPlan->Year);
							 $this->context->actualsRepository->deleteActualsByTaskSAPIDAndYear($task->ID, $taskPlan->Year);
						 }
						 else
						 {
							$this->context->assignmentsCostRepository->deleteByYear($task->ID, $taskPlan->Year);
							$this->context->actualsCostRepository->deleteIASAPActualsByTaskID($task->ID, $taskPlan->Year);
						 }
						 $this->flashMessage("Task Plan for task " . $task->Name . " and year " . $taskPlan->Year . " deleted.", 'success');
					 }
					 else
					 {
						$this->flashMessage("Failed to delete Task Plan for task " . $task->Name . " and year " . $taskPlan->Year . ".", 'error');
						$this->redirect('Project:detail', $project->ID);
					 }
				 }
			}
			//Create new task plans
			foreach ($entry['TaskPlan'] as $year)
			{
				 if(!array_key_exists($year, $taskPlans))
				 {
					$taskPlan = new TaskYearPlan();
					$taskPlan->Year = $year;
					$taskPlan->TaskID = $task->ID;
					for ($i = 1; $i <= 12; $i++) 
						$taskPlan->setMonthPlan($i, new MonthPlan(0, 0));
					if($this->context->taskPlanRepository->save($taskPlan))
					{
						$this->flashMessage("Task Paln for task " . $task->Name . " and year " . $year . " created.", 'success');
					}
					else
					{
						$this->flashMessage("Task Plan creation failed", 'error');
						$this->redirect('Project:detail', $project->ID);
					}
				 }
			}

			// Import actuals from IASAP database
//			if($task->ActualsImport)
//			{
//				switch($task->ActualsImportDataSource)
//				{
//					case Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_IASAP:
//						$this->runiASAPImport($task);
//						break;
//					case Task::TASK_ACTUALS_IMPORT_DATA_SOURCE_ECHARGE:
//						$this->runEChargeImport($task);
//						break;
//					default:
//						break;
//				}
//			}
			if(($task->ActualsImport == 0) && ($task->isLabourTask()))
			{
				$this->runTLCImport($task);
			}
			
			$this->redirect('Task:showDetail', $task->ID);
		}
		else
		{
			$this->flashMessage("Invalid form values.");
		}
        $this->redirect('Project:detail', $project->ID);
    }
	
	/**
	 * Finds HTS TRV Task SAP ID form.
	 * @param string $name Name of component.
     * @return AppForm Returns AppForm.
	 */
	public function createComponentFindHtsTrvTaskSapIDForm($name)
	{
		$form = new AppForm($this, $name);
		
		$form->addText('Prefix', 'Prefix', 2)
				//->setRequired('SAP ID is mandatory.')
				->setAttribute('maxlength', '2');

        $form->addText('SAP_ID', 'SAP ID', 10)
				//->setRequired('SAP ID is mandatory.')
				->setAttribute('maxlength', '10');
		
		$form->addText('Suffix', 'Suffix', 3)
				//->setRequired('SAP ID is mandatory.')
				->setAttribute('maxlength', '3');

		$form->addText('Def', 'Activity definition', 4)
				->setAttribute('maxlength', '4');
		
		$form->addButton('cancel', 'Cancel')
		        ->setAttribute('onclick', '$.fancybox.close()')
				->setAttribute('class', 'rightFloat');
		
		$form->addButton('ok', 'Get iASAP ID')
				->setAttribute('onclick', 'findHtsTrvTaskSapID()');
		
		return $form;
	}
}
