<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing actualsTemp from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsTempDibiMapper extends DibiMapper implements IActualsTempMapper
{

    /**
     * Creates ActualsTemp instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return	ActualsTemp 
     * @todo rewrite using annotations
     */
    private function load(DibiRow $row)
    {
        $actuals = new ActualsTemp();
        $actuals->Eid             = $row->EID;
        $actuals->Charge_number   = $row->ChargeNumber;
        $actuals->Year            = $row->Year;
        $actuals->Fiscal_month    = $row->FiscalMonth;
        $actuals->Hours           = $row->Hours;
        $actuals->Date            = $row->Date;
        $actuals->Project_sap_id  = $row->Project;
        $actuals->Source          = $row->Source;
        return $actuals;
    }	

    /**
     * Returns all actualsTemp.
     * 
	 * @param int $source Should be in range of ActualsTemp::SOURCE_*
     * @return array Associative array of actualsTemp.
     */
    public function findAll($source)
    {
        $actuals_db_result = $this->conn->query('SELECT * FROM [ActualsTemp] WHERE [Source] = %i', $source)->fetchAll();
        $actuals = array();
        foreach ($actuals_db_result as $row)
        {
                $actual = $this->load($row);
                $actuals[] = $actual;
        }
        return $actuals;
    }
	
    /**
     * Returns number all actuals.
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
	 * @param int $source Should be in range of ActualsTemp::SOURCE_*
     * @return int Number of stored actuals.
     */
    public function countActualsByRangeAndType(DateTime $startDate, DateTime $endDate, $source)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		
        $actuals_db_result = $this->conn->query('SELECT * FROM [ActualsTemp] WHERE
												(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i 
												AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i AND
												[Source] = %i', 
												$start_month_idx,
												$end_month_idx,
												$source)->fetchAll();
		$count = 0;
        foreach ($actuals_db_result as $row)
        {
            $count++;
        }
        return $count;
    }
	
	/**
     * Moves actualsTemp record.
     * 
	 * @param int $source Should be in range of ActualsTemp::SOURCE_*
     * @return boolean Returns true if move successfull, otherwiste returns false.
     */
    public function moveAll($source)
    {
		try
		{
			$result = $this->conn->query('INSERT INTO [Actuals]
												(EID, ChargeNumber, Year, FiscalMonth, Hours, Date, Project, Source) 
												SELECT * FROM [ActualsTemp] WHERE [Source] = %i', $source);
			Debugger::log("Result moving temp actuals: " . $result, Debugger::ERROR);
			if ($result >= 0)
			{
				Debugger::log("ActualsTemp was moved.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Moving of ActualsTemp failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Moving of ActualsTemp failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
	
    /**
     * Deletes all actualsTemp.
     * 
	 * @param int $source Should be in range of ActualsTemp::SOURCE_*
	 * @return boolean
     */
    public function deleteAll($source)
    {
		try
		{
		    $result = $this->conn->delete('ActualsTemp')->where('[Source] = %i', $source)->execute();
			if ($result >= 0)
			{
				Debugger::log("ActualsTemp was deleted.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Deleting of ActualsTemp failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of ActualsTemp failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }

    /**
     * Saves new ActualsTemp record.
     * 
     * @param ActualsTemp $actual
     * @return boolean Returns true if save successfull, otherwiste returns false.
     */
    public function save(ActualsTemp $actual)
    {
        try
        {
            $array = array();
            $array['EID'] = $actual->Eid;
            $array['ChargeNumber'] = $actual->Charge_number;
            $array['Year'] = $actual->Year;
            $array['FiscalMonth'] = $actual->Fiscal_month;
            $array['Hours'] = $actual->Hours;
            $array['Project'] = $actual->Project_sap_id;
            $array['Source'] = ($actual->Source) ? $actual->Source : ActualsTemp::SOURCE_ECHARGE;
            $db_result = $this->conn->insert('ActualsTemp', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving actualsTemp record failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
    }

    /**
     * Adds new ActualsTemp record.
     * If ActualsTemp record does not exist, new one is inserted.
     * If ActualsTemp record exists, it is updated (hours are added).
     * 
     * @param ActualsTemp $actual
     * @return boolean Returns true if adding successfull, otherwiste returns false.
     */
    public function add(ActualsTemp $actual)
    {
        try
        {
            $db_result = $this->conn->query(
                'SELECT * FROM [ActualsTemp] WHERE [EID] = %s AND [ChargeNumber] = %s AND [Year] = %i AND [FiscalMonth] = %i',
                $actual->Eid,
                $actual->Charge_number,
                $actual->Year,
                $actual->Fiscal_month
            )->fetch();
            if ($db_result)
            {
                $array = $db_result->toArray();
                unset($array['Hours']);
                $this->conn->query(
                    'UPDATE [ActualsTemp] SET',
                    array('Hours' => $db_result->Hours + $actual->Hours),
                    'WHERE [EID] = %s AND [ChargeNumber] = %s AND [Year] = %i AND [FiscalMonth] = %i',
                    $actual->Eid,
                    $actual->Charge_number,
                    $actual->Year,
                    $actual->Fiscal_month
                );
            }
            else
            {
                $array = array();
                $array['EID'] = $actual->Eid;
                $array['ChargeNumber'] = $actual->Charge_number;
                $array['Year'] = $actual->Year;
                $array['FiscalMonth'] = $actual->Fiscal_month;
                $array['Hours'] = $actual->Hours;
                $array['Project'] = $actual->Project_sap_id;
				$array['Source'] = ($actual->Source) ? $actual->Source : ActualsTemp::SOURCE_ECHARGE;
                $db_result = $this->conn->insert('ActualsTemp', $array)->execute();
            }
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Adding actualsTemp record failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
}
