<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing costs task assignments
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AssignmentsCostDibiMapper extends DibiMapper implements IAssignmentsCostMapper
{
	/**
	 * Creates AssignmentsCost instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	AssignmentsCost 
	 */
    private function loadAssignmentsCost(DibiRow $row)
    {
		$assignment = new AssignmentsCost();
		$assignment->ID = $row->AssignmentsCost_ID;
		$assignment->TaskID = $row->Task_ID;
		$assignment->Year = $row->Year;
		// month are stored with indexes 0..11
		$assignment->Month = $row->Month + 1;
		$assignment->Cost = $row->Cost;
		return $assignment;
    }

	/**
	 * Saves costs task assignment.
	 * 
	 * @param AssignmentsCost $assignment
	 * @return	boolean Returns TRUE if success, otherwise false.
	 */
	public function save(AssignmentsCost $assignment)
	{
		$values = array();
		$values['Cost'] = $assignment->Cost;
		try
		{
			if (!is_null($assignment->ID))
			{
				$this->conn->update('AssignmentsCost', $values)->where('[AssignmentsCost_ID] = %i', $assignment->ID)->execute();
			}
			else
			{
//				if ($assignment->Cost <> 0) // we do not save new empty assignments | It turns out we do :)
//				{
					$values['Task_ID'] = $assignment->TaskID;
					$values['Year'] = $assignment->Year;
					$values['Month'] = $assignment->Month - 1; // Month index in DB is in range <0,11> 
					$this->conn->insert('AssignmentsCost', $values)->execute();
					$assignment->setID($this->conn->getInsertId());
//				}
			}
			return true;
		}
		catch  (DibiException $e)
        {
            Debugger::log("Saving costs task assignment with ID= " . $assignment->getID() . " , TaskID = " . $assignment->getTaskID() . " , Year = " . $assignment->getYear() . " , Month = " . $assignment->getMonth(). " ) failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds month assignment by id of assignment.
	 * 
	 * @param int $id task assignment ID.
	 * @return	AssignmentsCost 
	 */
	public function find($id)
	{
		$db_row = $this->conn->select('*')->from('AssignmentsCost')->where('[AssignmentsCost_ID] = %u', $id)->fetch();
		if ($db_row->count() > 0)
			return $this->loadAssignmentsCost($db_row);
		else
			return NULL;
	}

	/**
	 * Deletes assignment for task in given year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @return bool TRUE on success, FALSE otherwise.
	 */
	public function deleteByYear($task_id, $year)
	{
		$deleted = $this->conn->delete('AssignmentsCost')->where('[Task_ID] = %u AND [Year] = %u', $task_id, $year)->execute();
		return ($deleted > 0);
	}
	
	/**
	 * Returns array of arrays of twelve task assigments 
	 * that is assigned to given task ID in given year.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @return	array or 12 AssignmentsCost 
	 *					where keys are month indexes from 0 to 11.
	 */
	public function findTaskYearAssignments($task_id, $year)
	{
		$db_result = $this->conn->query('SELECT * FROM [AssignmentsCost] 
			WHERE [Task_ID] = %u AND [Year] = %u', $task_id, $year)->fetchAll();
		$assignments = array();
		foreach ($db_result as $row)
			$assignments[$row->Month] = $this->find($row->AssignmentsCost_ID);
		return $assignments;
	}
	
    /**
     * Returns all assignments costs for single task
     * 
     * @return array Array of arrays (years) of arrays (fiscal months)
     * of Assignments Cost records for single task.
     */
    public function findAllForTask($taskID)
    {
        $assignmentsCost_db_result = $this->conn->query('SELECT * FROM [AssignmentsCost] WHERE [Task_ID] = %i', $taskID)->fetchAll();
        $assignmentsCosts = array();
        foreach ($assignmentsCost_db_result as $row)
        {
			$assignmentsCost = $this->loadAssignmentsCost($row);
			$assignmentsCosts[$assignmentsCost->Year][$assignmentsCost->Month] = $assignmentsCost;
        }
        return $assignmentsCosts;
    }
}
