<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * CurrencyExchangeRate is the ration between source currency
 * and target currency.
 * 
 * Currency code must be defined according to ISO 4217.
 * 
 * Exchange rate of two same currencies is 1.
 * 
 * E.g.:
 * If value of CZK (source) to USD (target) exchange rate is 10,
 * it means that you need 10 CZK to get 1 USD respectively
 * you get 10 CZK for 1 USD.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class CurrencyExchangeRate extends Object
{
	const RATE_UNDEFINED = 0.0;
	
	/** var string 3-letter of source currency code e.g. USD, CZK, GBP, ... */
	private $sourceCurrency;
	/** var string 3-letter of target currency code e.g. USD, CZK, GBP, ... */
	private $targetCurrency;
	/** var float positive value of exchange rate (negative means unknown) */
	private $value = 1;
    /** @var int Year of exchange rate */
    private $year;
    /** @var int Month of exchange rate */
    private $month;
	

	public function __construct()
	{
	}
	
	/**
	 * Sets source currency code.
	 * 
	 * @param string $curr_code 3-letter ISO 4217 currency code.
	 */
	public function setSourceCurrencyCode($curr_code)
	{
		if ($this->validCurrencyCode($curr_code))
			$this->sourceCurrency = $curr_code;
		else
			throw new InvalidArgumentException('Currency code must be 3-letter code (capitals). Invalid value: ' . $curr_code);
	}

	/**
	 * Gets source currency code.
	 * 
	 * @return string
	 */
	public function getSourceCurrencyCode()
	{
		return $this->sourceCurrency;
	}

	/**
	 * Sets target currency code.
	 * 
	 * @param string $curr_code 3-letter ISO 4217 currency code.
	 */
	public function setTargetCurrencyCode($curr_code)
	{
		if ($this->validCurrencyCode($curr_code))
			$this->targetCurrency = $curr_code;
		else
			throw new InvalidArgumentException('Currency code must be 3-letter code (capitals). Invalid value: ' . $curr_code);
	}
	
	/**
	 * Gets target currency code.
	 * 
	 * @return string
	 */
	public function getTargetCurrencyCode()
	{
		return $this->targetCurrency;
	}

	/**	 
	 * Sets exchange rate.
	 * 
	 * @param float $value
	 */
	public function setValue($value)
	{
		if (is_numeric($value) && ($value >= 0))
			$this->value = $value;
		else
			throw new InvalidArgumentException('Exchange rate value must be non-negative float. Invalid value: ' . $value);
	}

	/**	 
	 * Gets exchange rate.
	 * 
	 * @return float
	 */
	public function getValue()
	{
		return $this->value;
	}
	
	/**	 
	 * Returns TRUE if exchange rate value between currencies is undefined.
	 * 
	 * @return boolean
	 */
	public function isUndefined()
	{
		return ($this->value === self::RATE_UNDEFINED);
	}

	/**	 
	 * Returns TRUE currency code is valid.
	 * 
	 * @param string $code 
	 * @return boolean
	 */
	public static function validCurrencyCode($code)
	{
		return (preg_match('/^[A-Z]{3}$/', $code) ? true : false);
	}
	
    /**
     * Sets year of exchange rate.
     * 
     * @param int $year
     */
    public function setYear($year)
    {
		if (NumericUtils::isPositiveInteger($year))
			$this->year = 0 + $year;
		else
			throw new InvalidArgumentException("Year must be positive integer. Value: " . $year);
    }

    /**
     * Gets year of exchange rate.
     * 
     * @return int
     */
	public function getYear()
	{
		return $this->year;
	}
    
	/**
	 * Gets month of exchange rate.
	 * 
	 * @return int
	 */
	public function getMonth()
	{
        return $this->month;
	}

	/**
	 * Sets month of exchange rate.
	 * 
	 * @param int $month
	 */
	public function setMonth($month)
	{
		if (is_int($month) && ($month >= 1) && ($month <= 12))
			$this->month = 0 + $month;
		else
			throw new InvalidArgumentException('Month must be spedified as a number in range <1,12>. Rate must be numeric value.');
	}
}


