<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository for working with exchange rates.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ExchangeRatesRepository extends Object
{
	/** @var IExchangeRateMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IExchangeRateMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ExchangeRates-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Exchange Rates"));
    }	

	/**
	 * Find exchange rate between currencies in defined month in a year.
	 * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * @param int $month
	 * @param int $year
	 * @return CurrencyExchangeRate or NULL in unknown.
	 */
	public function find($source_curr, $target_curr, $month, $year)
	{
		return $this->mapper->find($source_curr, $target_curr, $month, $year);
	}

	/**
	 * Finds exchange rates between currencies in defined year.
	 * Objects with undefined rates may be returned if rate is unknown
	 * of undefined for a month.
	 * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * @param int $year
	 * @return @return MonthArray of CurrencyExchangeRate
	 */
	public function findYearRates($source_curr, $target_curr, $year)
	{
		return $this->mapper->findYearRates($source_curr, $target_curr, $year);
	}
    
    /**
     * Gets all years of ExchangeRate.
     * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * 
     * @return array Array of available years
     */
    public function findExchangeRateYears($source_curr, $target_curr)
    {
		return $this->mapper->findExchangeRateYears($source_curr, $target_curr);
	}
    
    /**
     * Saves exchange rate.
     * 
     * @param CostCenter $currencyExchangeRate exchange rate.
     * @return boolean Returns true if exchange rate is successfully saved, otherwise false.
     */
    public function save(CurrencyExchangeRate $currencyExchangeRate)
    {
        if ($this->mapper->save($currencyExchangeRate))
		{
			$this->multilog->log('Exchange Rate with Source curr.: ' . $currencyExchangeRate->SourceCurrencyCode . ' and Target curr.: ' . $currencyExchangeRate->TargetCurrencyCode . ' and Year: ' . $currencyExchangeRate->Year . ' and Month: ' . $currencyExchangeRate->Month . ' saved.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }

    /**
     * Updates exchange rate.
     * 
     * @param CostCenter $currencyExchangeRate exchange rate.
     * @return boolean Returns true if exchange rate is successfully updated, otherwise false.
     */
    public function update(CurrencyExchangeRate $currencyExchangeRate)
    {
		$oldObject = $this->find($currencyExchangeRate->SourceCurrencyCode, $currencyExchangeRate->TargetCurrencyCode, $currencyExchangeRate->Month, $currencyExchangeRate->Year);
        if ($this->mapper->update($currencyExchangeRate))
		{
			$this->multilog->log('Exchange Rate with Source curr.: ' . $currencyExchangeRate->SourceCurrencyCode . ' and Target curr.: ' . $currencyExchangeRate->TargetCurrencyCode . ' and Year: ' . $currencyExchangeRate->Year . ' and Month: ' . $currencyExchangeRate->Month . ' updated. ' . $this->multilog->differences($oldObject, $currencyExchangeRate), ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
    }
}