<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models/Fiscal
 */

/**
 * Represents financial/fiscal month in financial/fiscal year.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class FiscalMonth extends Object 
{

    /** Lowest index of fiscal month in a year. */
    const FIRST_INDEX = 0;

    /** Highest index of fiscal month in a year. */
    const LAST_INDEX = 11;

    /** @var int Index of the month in a fiscal year must be number from 0 to 11. */
    private $index = self::FIRST_INDEX;

    /** @var int Index of the year. Usually the same as regular calendar year. */
    private $year;
    
    /** @var string Starting date of fiscal month in FiscalCalendar::DATE_FORMAT. */
    private $from;

    /** @var string Ending date of a fiscal month in FiscalCalendar::DATE_FORMAT. */
    private $to;

    /** @var int Number in hours of Annual Operational Plan (AOP) in current fiscal month. */
    private $hours = 0;

    /**
     * New instance of fiscal month.
     *
     * @param  int  $index	Index of the month in fiscal year (non-negative number). Identifier within a year.
     * @param  int  $from	Unix timestamp, representing start of the fiscal month.
     * @param  int  $to		Unix timestamp, representing end of the fiscal month.
     * @param  int  $hours	Non-negative integer value, representing AOP hours in the month.
     * @throws InvalidArgumentException
     */
    public function __construct($index, $year, $from, $to, $hours = 0) 
    {
		$this->setIndex($index);
		$this->setYear($year);
		$this->setFrom($from);
		$this->setTo($to);
		if (strcmp($from, $to) > 0)
			throw new InvalidArgumentException("Fiscal month cannot end before it's starts. Assertion ".$from." < ".$to." is not true.");
		$this->setAOPHours($hours);
    }

    /**
     * Sets index of the month. Must be number in range <0,11>.
     * 
     * @param int $idx Index
     * @throws InvalidArgumentException on invalid input. 
     */
    public function setIndex($idx)
    {
		if (is_numeric($idx) && ((0 + $idx) >= self::FIRST_INDEX) && ((0 + $idx) <= self::LAST_INDEX) )
			$this->index = 0 + $idx; // convert to int
		else
			throw new InvalidArgumentException("Index must be number in range <".self::FIRST_INDEX.",".self::LAST_INDEX.">. Wrong value: " . $idx);
	}

    /**
     * Returns numeric index of the month in a year. Integer in range <0, 11>.
     * 
     * @return int
     */
    public function getIndex()
    {
        return $this->index;
    }

    /**
     * Returns month name according to it's index.
	 * Fiscal month name usually corresponds to the dominant regular month
	 * in fiscal month time range (From, To).
	 * 
     * @return string Month name (regular calendar month name).
     */
    public function getMonthName() 
    {
		return FiscalCalendar::fiscalMonthIndexToName($this->index, true);
    }
	
    /**
     * Sets index of the fiscal year. Must be integer greter than zero.
     * 
     * @param int $idx Year number.
     * @throws InvalidArgumentException on invalid input. 
     */
    public function setYear($year)
    {
		if (is_numeric($year) && ((0 + $year) > 0))
			$this->year = 0 + $year; // convert to int
		else
			throw new InvalidArgumentException("Year index must be number positive integer. Wrong value: " . $year);
    }

    /**
     * Returns numeric index of the fiscal year.
     * 
     * @return int
     */
    public function getYear()
    {
        return $this->year;
    }

    
    /**
     * Sets start of fiscal month.
	 * If $date argument includes time, only date is stored.
     * 
     * @param string $date Date.
     * @throws InvalidArgumentException on invalid input. 
     */
    public function setFrom($date)
    {
		if (FiscalCalendar::isValidDate($date))
			$this->from = $date;
		else
			if (FiscalCalendar::isValidDateTime($date))
			{
				$date_array = explode(' ', $date);
				$this->from = $date_array[0];
				
			}
			else
				throw new InvalidArgumentException("Fiscal from date must be valid date in correct format. Wrong value: " . $date);
    }

    /**
     * Returns start date of fiscal month in FiscalCalendar::DATE_FORMAT.
     * 
     * @return string
     */
    public function getFrom()
    {
        return $this->from;
    }
	
    /**
     * Returns start date of fiscal month.
     * 
     * @return DateTime
     */
    public function getFromDateTime()
    {
		return DateTime::createFromFormat(FiscalCalendar::DATE_FORMAT, $this->from);
    }
	
    /**
     * Sets end of fiscal month.
	 * If $date argument includes time, only date is stored.
     * 
     * @param string $date Date.
     * @throws InvalidArgumentException on invalid input. 
     */
    public function setTo($date)
    {
		if (FiscalCalendar::isValidDate($date))
			$this->to = $date;
		else
			if (FiscalCalendar::isValidDateTime($date))
			{
				$date_array = explode(' ', $date);
				$this->to = $date_array[0];
				
			}
			else
				throw new InvalidArgumentException("Fiscal end date must be valid date in correct format. Wrong value: " . $date);
    }

    /**
     * Returns end date of fiscal month in FiscalCalendar::DATE_FORMAT.
     * 
     * @return string
     */
    public function getTo()
    {
        return $this->to;
    }
	
	/**
     * Returns end date of fiscal month.
     * 
     * @return DateTime
     */
    public function getToDateTime()
    {
		return DateTime::createFromFormat(FiscalCalendar::DATE_FORMAT, $this->to);
    }

    /**
     * Sets assigned hours of Annual Operation Plan for the month.
     * 
	 * @param int $idx Index
     * @throws InvalidArgumentException on invalid input. 
     */
    public function setAOPHours($hours)
    {
		if (is_numeric($hours) && ((0 + $hours) >= 0))
			$this->hours = 0 + $hours; // conversion to int
		else
			throw new InvalidArgumentException("Hours must be non-negative integer number. Wrong value: " . $hours);
    }

    /**
     * Returns assigned hours of Annual Operation Plan for the month.
     * 
     * @return int
     */
    public function getAOPHours()
    {
        return $this->hours;
    }

    /**
     * Whether fiscal month includes date/time provided as a UNIX timestamp.
     * 
	 * @param int $ts UNIX timestamp.
     * @return boolean	Returns TRUE if selected date/time is within month's From, To interval.
     */
    public function includesTimestamp($ts)
    {
		$date = date(FiscalCalendar::DATE_FORMAT, $ts);
		if ($date === FALSE)
			throw new InvalidArgumentException("Provided argument is not a UNIX timestamp.");
        return $this->includesDate($date);
    }

    /**
     * Whether fiscal month includes date.
     * 
	 * @param string $date Date in fiscal date format.
     * @return boolean	Returns TRUE if selected date is within month's From, To interval.
     */
    public function includesDate($date)
    {
		if (FiscalCalendar::isValidDate($date))
			return ((strcmp($this->from, $date) <= 0) && (strcmp($date, $this->to) <= 0)); // lze testovat lexikalne dik formatu data
		else
			throw new InvalidArgumentException("Provided argument is not valid fiscal date.");
    }
	
}