<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for importing data - actual hours - from iASAP.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class IASAPActualsImportServiceSAPID extends Object
{
	/** @const php.ini maximum execution time */
	const MAXIMUM_EXECUTION_TIME = 720; // 12 minutes
	/** @const Lock key */
	const LOCK_KEY = 'IASAP_ACTUALS_IMPORT_SAP_ID';
	/** @const Possible results of import */
	const RESULT_SUCCESS = 0,
		  RESULT_WARNING = 1,
		  RESULT_ERROR = 2,
		  RESULT_NO_DATA = 3;
	
	/** @var DIContainer DI context with application repositories access */
    private $context;
	/** @var ILogger Logging facility */
    private $logger;
	/** @var ActualsReaderSAPID iASAP access class */
	private $reader;
	/** @var DateTime Creation date of oldest requested data. Default value is stored in db table GlobalSetting. */
	private $start_date;
	/** @var DateTime Creation date of latest requested data. Default value is time of import invocation. */
	private $end_date;
	
	/**
	 * @param DIContainer $context
	 * @param array $tasks Array of PO numbers used as mapping keys
	 * @param ILogger $logger
	 */
	public function __construct(DIContainer $context, $tasks, ILogger $logger = NULL)
    {
		$this->context  = $context;
		$this->logger   = ($logger === NULL) ? (new NullLogger('')) : $logger;
		
		$this->reader = new ActualsReaderSAPID($tasks, $logger);
		
		$this->start_date = new DateTime(GlobalSettingReaderHelper::get('iASAP.actuals.update.startDate'));
		$this->end_date = new DateTime();
	}
	
	/**
	 * Sets window between oldest and latest requested data item
	 * @param string $start_date Date in format YYYY-MM-DD
	 * @param string $end_date Optional date in format YYYY-MM-DD
	 * @throws InvalidArgumentException
	 */
	public function setRange($start_date, $end_date = NULL)
	{
		if ($this->validateDate($start_date) && $this->validateDate($end_date))
		{
		    $this->start_date = new DateTime($start_date);
		    $this->end_date = new DateTime($end_date);
		}
        else
		    throw new InvalidArgumentException('Start date and End date must be in format YYYY-MM-DD. Wrong params: ' . $start_date . ', ' . $end_date);
	}
	
	/**
	 * Runs actuals import from iASAP.
	 * Import is divided into time blocks. Each block represents fiscal month in given time range.
	 * @return int Result of operation in range of IASAPActualsImportService::RESULT_*
	 * @throws IASAPActualsImportSAPIDServiceException
	 */
	public function runLaborImport()
	{
		// adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new IASAPActualsImportSAPIDServiceException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
		$lock = $this->context->createLock(self::LOCK_KEY);
		if (!$lock->lock())
            throw new IASAPActualsImportSAPIDServiceException("iASAP labor actuals import is already running.");
		
        // open logfile
        $this->logger->log('iASAP SAP ID labor actuals import start: ' . date('Y-m-d H:i:s'), ILogger::INFO);
		
		if (!count($this->reader->tasks))
		{
			$msg = "IASAPActualsImportService: Cannot get actuals. List of labor tasks is empty.";
			$this->logger->log($msg, ILogger::WARNING);
			Debugger::log($msg, Debugger::WARNING);
			$lock->unlock();
			return self::RESULT_WARNING;
		}
		
		try
		{
			$succ = 0; // number of successfully imported records
            $fail = 0; // number of records that couldn't be stored in db
			
			$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->start_date->format('U'));
			$end_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->end_date->format('U'));
			$task_info = array();
			foreach($this->reader->tasks as $task)
			{
				$task_info[$task->SAPID]['Year'] = Date('Y');
				$task_info[$task->SAPID]['TaskSAP_ID'] = $task->SAPID;
				$task_info[$task->SAPID]['ProjectSAP_ID'] = $this->context->projectRepository->findProjectByTaskId($task->ID)->SAP_ID;
			}
			if (!$task_info || !$fiscal_month || !$end_fiscal_month)
			{
				$msg = "IASAPActualsImportService: Labor import failed. Cannot get needed input data to import.";
				$this->logger->log($msg, ILogger::ERROR);
				Debugger::log($msg, Debugger::ERROR);
				$lock->unlock();
				return self::RESULT_ERROR;
			}

			$charge_numbers = array();
			foreach ($task_info as $info)
				$charge_numbers[] = $info['TaskSAP_ID'];
			
			// delete existing data that will be imported again
			if (!$this->context->actualsRepository->deleteIASAPImport($fiscal_month->FromDateTime, $end_fiscal_month->ToDateTime, $charge_numbers))
			{
				$msg = 'IASAPActualsImportService: Cleanup of previously imported labor data failed. Data may be inconsistent after import.';
				$this->logger->log($msg, ILogger::ERROR);
				Debugger::log($msg, Debugger::ERROR);
				$lock->unlock();
				return self::RESULT_ERROR;
			}

			while ($fiscal_month->ToDateTime <= $end_fiscal_month->ToDateTime)
			{
				foreach ($this->reader->readLabor($fiscal_month->FromDateTime, $fiscal_month->ToDateTime) as $taskSAP_ID => $assigned)
				{
					if (isset($task_info[$taskSAP_ID]))
					{
						foreach ($assigned as $eid => $months)
						{
							foreach ($months as $month => $hours)
							{
								
								$actuals = new Actuals();
								$actuals->Eid = $eid;
								$actuals->Charge_number = $task_info[$taskSAP_ID]['TaskSAP_ID'];
								$actuals->Year = $fiscal_month->year;
								$actuals->Fiscal_month = $month;
								$actuals->Hours = $hours;
								$actuals->Project_sap_id = $task_info[$taskSAP_ID]['ProjectSAP_ID'];
								$actuals->Source = Actuals::SOURCE_IASAP;
								if ($this->context->actualsRepository->add($actuals))
									$succ++;
								else
								    $fail++;
							}
						}
					}
					else
					{
						// cannot save without information about task
						foreach ($assigned as $eid)
							$fail += count($eid);
					}
				}
				
				$fiscal_month = FiscalCalendar::getNextFiscalMonth($fiscal_month);
				if ($fiscal_month === NULL)
				{
					$msg = "IASAPActualsImportService: Labor import failed. Cannot get next fiscal month.";
					$this->logger->log($msg, ILogger::ERROR);
					Debugger::log($msg, Debugger::ERROR);
					$lock->unlock();
					return self::RESULT_ERROR;
				}
			}
			
			$lock->unlock();

			if ($fail == 0)
			{
				$msg = "IASAPActualsImportService: All $succ labor actuals from " . $this->start_date->format('Y-m-d') . " till " . $this->end_date->format('Y-m-d') . " imported successfully";
				$this->logger->log($msg, ILogger::NOTICE);
				Debugger::log($msg, Debugger::INFO);
			}
			else
			{
				$msg = "IASAPActualsImportService: $succ labor actuals imported, $fail labor actuals thrown away (" . $this->start_date->format('Y-m-d') . " - " . $this->end_date->format('Y-m-d') . ")";
				$this->logger->log($msg, ILogger::WARNING);
				Debugger::log($msg, Debugger::WARNING);
				return self::RESULT_WARNING;
			}
		}
        catch (Exception $e)
		{
			$this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			return self::RESULT_ERROR;
		}
		
		if($succ == 0)
		{
			return self::RESULT_NO_DATA;
		}
		return self::RESULT_SUCCESS;
	}
	
/**
	 * Runs actuals import from iASAP.
	 * Import is divided into time blocks. Each block represents fiscal month in given time range.
	 * @return int Result of operation in range of IASAPActualsImportService::RESULT_*
	 * @throws IASAPActualsImportSAPIDServiceException
	 */
	public function runCostImport()
	{
		// adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new IASAPActualsImportSAPIDServiceException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
		$lock = $this->context->createLock(self::LOCK_KEY);
		if (!$lock->lock())
            throw new IASAPActualsImportSAPIDServiceException("iASAP cost actuals import is already running.");
		
        // open logfile
        $this->logger->log('iASAP SAP ID cost actuals import start: ' . date('Y-m-d H:i:s'), ILogger::INFO);
		
		if (!count($this->reader->tasks))
		{
			$msg = "IASAPActualsImportService: Cannot get actuals. List of cost tasks is empty.";
			$this->logger->log($msg, ILogger::WARNING);
			Debugger::log($msg, Debugger::WARNING);
			$lock->unlock();
			return self::RESULT_WARNING;
		}
		
		try
		{
			$succ = 0; // number of successfully imported records
            $fail = 0; // number of records that couldn't be stored in db
			
			$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->start_date->format('U'));
			$end_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int)$this->end_date->format('U'));
			$task_info = array();
			$allTaskIDs = array();
			foreach ($this->reader->tasks as $task)
			{
				$task_info[$task->SAPID]['TaskSAP_ID'] = $task->SAPID;
				$task_info[$task->SAPID]['Name'] = $task->Name;
				$task_info[$task->SAPID]['ProjectSAP_ID'] = $this->context->projectRepository->findProjectByTaskId($task->ID)->SAP_ID;
				$task_info[$task->SAPID]['Task_ID'] = $task->ID;
				$allTaskIDs[] = $task->ID;
			}
			
			if (!$task_info || !$fiscal_month || !$end_fiscal_month)
			{
				$msg = "IASAPActualsImportService: Cost import failed. Cannot get needed input data to import.";
				$this->logger->log($msg, ILogger::ERROR);
				Debugger::log($msg, Debugger::ERROR);
				$lock->unlock();
				return self::RESULT_ERROR;
			}
			
			// delete existing data that will be imported again
			if (!$this->context->actualsCostRepository->deleteIASAPImport($fiscal_month->FromDateTime, $end_fiscal_month->ToDateTime, $allTaskIDs))
			{
				$msg = 'IASAPActualsImportService: Cleanup of previously imported cost data failed. Data may be inconsistent after import.';
				$this->logger->log($msg, ILogger::ERROR);
				Debugger::log($msg, Debugger::ERROR);
				$lock->unlock();
				return self::RESULT_ERROR;
			}
			
			while ($fiscal_month->ToDateTime <= $end_fiscal_month->ToDateTime)
			{
				foreach ($this->reader->readCost($fiscal_month->FromDateTime, $fiscal_month->ToDateTime) as $taskSAP_ID => $dbData)
				{
					if (isset($task_info[$taskSAP_ID]))
					{
						$actualsCost = new ActualsCost();
						$actualsCost->Project_sap_id = $task_info[$taskSAP_ID]['ProjectSAP_ID'];
						$actualsCost->Task_id = $task_info[$taskSAP_ID]['Task_ID'];
						$actualsCost->Activity_name = $task_info[$taskSAP_ID]['Name'];
						$actualsCost->Creation_date = $dbData['ExpenseDate']->format('Y-m-d'); // Q: Switch dates?!?
						$actualsCost->Posting_date = $dbData['InputDate']->format('Y-m-d');
						$actualsCost->Year = $dbData['ExpenseDate']->format('Y');
						$actualsCost->Fiscal_month = $dbData['FiscalMonth'];
						$actualsCost->Description = '';
						$actualsCost->Category = '';
						$actualsCost->Quantity = 0;
						$actualsCost->Value_usd = round($dbData['Amount'], 2); // HPT use only USD values //if it changes, here is need to be done rate recalculation
						$actualsCost->Value_czk = 0;
						$actualsCost->Value = 0;
						$actualsCost->Currency = $dbData['Currency'];
						$actualsCost->Source = ActualsCost::SOURCE_IASAP;
						if ($this->context->actualsCostRepository->save($actualsCost))
							$succ++;
						else
							$fail++;
					}
					else
					{
						$fail++;
					}
				}
				
				$fiscal_month = FiscalCalendar::getNextFiscalMonth($fiscal_month);
				if ($fiscal_month === NULL)
				{
					$msg = "IASAPActualsImportService: Cost import failed. Cannot get next fiscal month.";
					$this->logger->log($msg, ILogger::ERROR);
					Debugger::log($msg, Debugger::ERROR);
					$lock->unlock();
					return self::RESULT_ERROR;
				}
			}
			$lock->unlock();
			if ($fail == 0)
			{
				$msg = "IASAPActualsImportService: All $succ cost actuals hour from " . $this->start_date->format('Y-m-d') . " till " . $this->end_date->format('Y-m-d') . " imported successfully";
				$this->logger->log($msg, ILogger::NOTICE);
				Debugger::log($msg, Debugger::INFO);
			}
			else
			{
				$msg = "IASAPActualsImportService: $succ cost actuals imported, $fail cost actuals thrown away (" . $this->start_date->format('Y-m-d') . " - " . $this->end_date->format('Y-m-d') . ")";
				$this->logger->log($msg, ILogger::WARNING);
				Debugger::log($msg, Debugger::WARNING);
				return self::RESULT_WARNING;
			}
		}
        catch (Exception $e)
		{
			$this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			return self::RESULT_ERROR;
		}
		
		if ($succ == 0)
		{
			return self::RESULT_NO_DATA;
		}
		return self::RESULT_SUCCESS;
	}
	
	/**
	 * Validates date format
	 * @param string $date To be validated
	 * @param string $format Validation pattern
	 * @return bool TRUE if date match format. FALSE otherwise.
	 */
	protected function validateDate($date, $format = 'Y-m-d')
	{
		if ($date === NULL)
			return TRUE;
		
	    $d = DateTime::createFromFormat($format, $date);
	    return $d && ($d->format($format) == $date);
	}
}

class IASAPActualsImportSAPIDServiceException extends Exception {}