<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for importing data - person - from iASAP.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PersonImportService extends Object
{
	/** @const php.ini maximum execution time */
	const MAXIMUM_EXECUTION_TIME = 720; // 12 minutes
	/** @const Lock key */
	const LOCK_KEY = 'IASAP_PERSON_IMPORT';
	/** @const Possible results of import */
	const RESULT_SUCCESS = 0,
		  RESULT_WARNING = 1,
		  RESULT_ERROR = 2,
		  RESULT_NO_DATA = 3;
	
	/** @var DIContainer DI context with application repositories access */
    private $context;
	/** @var ILogger Logging facility */
    private $logger;
	/** @var PersonReader iASAP access class */
	private $reader;
	/** Array Array of all cost centers*/
	private $cachedWWorkCenters;
	
	/**
	 * @param DIContainer $context
	 * @param ILogger $logger
	 */
	public function __construct(DIContainer $context, ILogger $logger = NULL)
    {
		$this->context  = $context;
		$this->logger   = ($logger === NULL) ? (new NullLogger('')) : $logger;
		
		$this->reader = new PersonReader($logger);
		
		$this->cachedWWorkCenters = $this->context->costCenterRepository->findAllCostCenters();
	}
	
	/**
	 * Runs import person from iASAP.
	 * @return int Result of operation in range of PersonImportService::RESULT_*
	 * @throws PersonImportException
	 */
	public function runPersonImport()
	{
		// adjust maximum execution time (php.ini)
        $maximum_execution_time = self::MAXIMUM_EXECUTION_TIME;
		ini_set('max_execution_time', $maximum_execution_time);
        if (ini_get('max_execution_time') != $maximum_execution_time)
            throw new IASAPActualsImportServiceTLCException("Setting maximum execution time to '$maximum_execution_time' failed, check server (php.ini) settings");
		
		$lock = $this->context->createLock(self::LOCK_KEY);
		if (!$lock->lock())
            throw new IASAPActualsImportServiceTLCException("Person import is already running.");
        //open logfile
        $this->logger->log('Person import start: ' . date('Y-m-d H:i:s'), ILogger::INFO);

		try
		{
			$succ = 0; // number of successfully imported records
			
			// get all labour tasks activity
			$taskSAPIDs = $this->context->taskRepository->findDistinctLaborTaskActivity();
			$activities = array();
			foreach ($taskSAPIDs as $sapID => $task)
			{
				$activities[$task->Network] = $task->Network;
			}
			$personsEID = array();
			
			//get all existing persons EID
			$existingPerson = $this->context->personRepository->findAll();
			foreach ($existingPerson as $eid => $person)
			{
				$personsEID[$eid] = $eid;
			}
			//get all exisitng person in PersounFound table EID
			$tempPerson = $this->context->personFoundRepository->findAll();
			foreach ($tempPerson as $eid => $person)
			{
				$personsEID[$eid] = $eid;
			}

			foreach ($this->reader->readPerson() as $taskActivity => $record)
			{
				if (array_key_exists($taskActivity, $activities))// in array
				{
					foreach ($record as $eid => $personRecord)
					{
						// Person what is not in Person OR PersonFound tables
						if(!array_key_exists($eid, $personsEID))
						{
							// Split name into name and surname
							$names = explode(" ", $personRecord['Name']);
							if(isset($names[0]))
								$name = $names[0];
							else
								$name = '-';
							if(isset($names[1]))
								$surname = $names[1];
							else
								$surname = '-';
							
							// Get Org unit according to supervisor and his infos
							$supervisor = $this->context->personRepository->find($personRecord['SupervisorEID']);
							if($supervisor)
							{
								$supervisorEID = $supervisor->ID;
								$supervisorName = $supervisor->Firstname;
								$supervisorSurname = $supervisor->Surname;
								$orgUnits = $this->context->organizationUnitRepository->findByLeader($supervisor->ID);
								if($orgUnits)
								{
									$orgUnit = reset($orgUnits);
									$organizationUnitID = $orgUnit->ID;
								}
								else
								{
									$organizationUnitID = null;
								}
							}
							else
							{
								$supervisorEID = $personRecord['SupervisorEID'];
								$supervisorName = null;
								$supervisorSurname = null;
								$organizationUnitID = null;
							}
							
							// Fill person found objec
							$personFound = new PersonFound();
							$personFound->EID = $personRecord['EID'];
							$personFound->Name = $name;
							$personFound->Surname = $surname;
							$personFound->WorkCenterID = $personRecord['WorkCenterID'];
							$personFound->SupervisorEID = $supervisorEID;
							$personFound->SupervisorName = $supervisorName;
							$personFound->SupervisorSurname = $supervisorSurname;
							$personFound->OrganizationUnitID = $organizationUnitID;
							$personFound->Blacklisted = 0;
							if ($this->context->personFoundRepository->save($personFound))
								$succ++;
						}
					}
				}

			}
			$lock->unlock();

			$msg = "Person Import: $succ person imported.";
			$this->logger->log($msg, ILogger::NOTICE);
			Debugger::log($msg, Debugger::INFO);
		}
        catch (Exception $e)
		{
			$this->logger->log($e->getMessage(), ILogger::ERROR);
            Debugger::log($e->getMessage(), Debugger::ERROR);
			$lock->unlock();
			return self::RESULT_ERROR;
		}

		if($succ == 0)
		{
			return self::RESULT_NO_DATA;
		}
		return self::RESULT_SUCCESS;
	}
}

class PersonImportServiceException extends Exception {}
