<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Node of organization chart.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class OrganizationChartNode extends Object
{
	/** @var DIContainer DI context with application repositories access */
	private $context;
	/** @var int ID of organization unit */
	private $id;
	/** @var Person Leader of organization unit */
	private $leader;
	/** @var string Information about leadership */
	private $leadershipInfo;
	/** @var string Short of organization unit type */
	private $type;
	/** @var string Name of organization unit */
	private $name;
	/** @var string Short of organization unit name */
	private $short;
	/** @var bool True if organization unit is virtual */
	private $virtual;
	/** @var bool True if organization unit is active */
	private $status;
	/** @var array Array of Person. Persons assigned as staff of current organization unit */
	private $staff;
	/** @var array Array of OrganizationChartNode. Succestors of current organization unit */
	private $children;
	/** @var int Number of end nodes, which are under current node of organization unit  */
	private $leavesCount;

	/**
	 * Initializes new node of organization chart
	 * @param OrganizationUnit $org_unit
	 * @param OrganizationUnit $load_staff
	 * @param array $children Array of OrganizationChartNode. Succestors of current organization unit
	 * @param int $leavesCount Number of end nodes, which are under current node of organization unit
	 */
	public function __construct(OrganizationUnit $org_unit, $load_staff = FALSE, $children = NULL, $leavesCount = 0)
	{
		$this->context = Environment::getContext();
		$this->id = $org_unit->ID;
		$this->type = $org_unit->Type;
		$this->name = $org_unit->Name;
		$this->short = $org_unit->Short;
		$this->virtual = ($org_unit->Virtual) ? TRUE : FALSE;
		$this->status = ($org_unit->Status) ? TRUE : FALSE;
		$this->children = $children;
		$this->leavesCount = $leavesCount;
		
		$this->staff = array();
		
		if ($load_staff)
		{
			$leader = $this->context->personRepository->find($org_unit->Leader);
		    $this->leader = ($leader) ? $leader : (new Person());
		    $this->leadershipInfo = $org_unit->LeadershipInfo;
			
			$persons = $this->context->personRepository->findByOrganizationUnitID($org_unit->ID);
			foreach ($persons as $person)
				if ($person->ID != $org_unit->Leader) // leader of unit should not be displayed as staff
					$this->staff[] = $person;
		}				
	}
	
	/**
	 * Gets ID of organization unit
	 * @return int
	 */
	public function getID()
	{
		return $this->id;
	}
	
	/**
	 * Gets leader of organization unit
	 * @return Person
	 */
	public function getLeader()
	{
		return $this->leader;
	}
	
	/**
	 * Gets information about leadership
	 * @return string
	 */
	public function getLeadershipInfo()
	{
		return $this->leadershipInfo;
	}
	
	/**
	 * Gets Short of organization unit type
	 * @return string
	 */
	public function getType()
	{
		return $this->type;
	}
	
	/**
	 * Gets Name of organization unit
	 * @return string
	 */
	public function getName()
	{
		return $this->name;
	}
	
	/**
	 * Gets Short name of organization unit
	 * @return string
	 */
	public function getShort()
	{
		return $this->short;
	}
	
	/**
	 * Gets true if organization unit is virtual
	 * @return bool
	 */
	public function getVirtual()
	{
		return $this->virtual;
	}
	
	/**
	 * Gets true if organization unit is active
	 * @return bool
	 */
	public function getStatus()
	{
		return $this->status;
	}
	
	/**
	 * Gets persons assigned as staff of current organization unit
	 * @return array Array of Persons
	 */
	public function getStaff()
	{
		return $this->staff;
	}
	
	/**
	 * Gets succestors of current organization unit
	 * @return array Array of OrganizationChartNode
	 */
	public function getChildren()
	{
		return $this->children;
	}
	
	/**
	 * Gets number of end nodes, which are under current node of organization unit
	 * @return int
	 */
	public function getLeavesCount()
	{
		return $this->leavesCount;
	}
	
}
