<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Organization unit repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class OrganizationUnitRepository extends Object
{
    /** @var IOrganizationUnitMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IOrganizationUnitMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'OrganizationUnit-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Organization Unit"));
    }
	
	/**
	 * Function returns array of only active organizations unit.
	 * 
	 * @param array $orgUnitArray from find methods.
	 * @return array Array of OrgUnits where OrgUnit ID is key or null if nothing left
	 */
	private function popInactiveOrgUnits($orgUnitArray)
	{
		foreach ($orgUnitArray as $id => $orgUnit)
		{
			if(!$orgUnit->Status)
				unset($orgUnitArray[$id]);
		}
		if($orgUnitArray)
			return $orgUnitArray;
		else
			return array();
	}
    
	/**
	 * Saves organization unit.
	 * 
	 * @param OrganizationUnit $organizationUnit
	 */
	public function save(OrganizationUnit $organizationUnit)
	{
		if ($this->mapper->save($organizationUnit))
		{
			$this->multilog->log('Organization unit with ID: ' . $organizationUnit->ID . ' and Name: ' . $organizationUnit->Name . ' saved.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}

	/**
	 * Updates organization unit.
	 * 
	 * @param OrganizationUnit $organizationUnit
	 */
	public function update(OrganizationUnit $organizationUnit)
	{
		$oldObject = $this->find($organizationUnit->ID);
		if ($this->mapper->update($organizationUnit))
		{
			$this->multilog->log('Organization unit with ID: ' . $organizationUnit->ID . ' and Name: ' . $organizationUnit->Name . ' updated. ' . $this->multilog->differences($oldObject, $organizationUnit), ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Updates parent id of set of organization units.
	 * Used for move of children to another parent after delete of original parent.
	 * 
	 * @param array $children Of organization unit ids.
	 * @param int $parent_id New parent organization unit id.
	 */
	public function updateParentInBatch(array $children, $parent_id)
	{
		$oldObject = $this->find(reset($children));
		
		$updated = '';
		foreach($children as $child)
			$updated .= $child . '; ';
		
		if ($this->mapper->updateParentInBatch($children, $parent_id))
		{
			if ($oldObject)
			    $this->multilog->log('Organization units with ID: ' . $updated . 'updated. ' . 'parentID: "' . $oldObject->Parent_ID . '" -> "' . $parent_id . '"', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Deletes existing organization unit.
	 * 
	 * @param int $OrganizationUnit_id
	 * @return boolean
	 */
	public function delete($organizationUnit_id)
	{
		if ($this->mapper->delete($organizationUnit_id))
		{
			$this->multilog->log('Organization unit with ID: ' . $organizationUnit_id . ' deleted.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
    
	/**
	 * Finds organization unit by id.
	 * 
	 * @param int $id organization unit ID.
	 * @return	OrganizationUnit or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
    
    /**
	 * Finds all organization units.
	 * 
	 * @param int $id organization unit ID.
	 * @param boolean $status Decides if funtion returns inactive units or not.
	 * @return	array of OrganizationUnit where id is key or NULL if not found.
	 */
	public function findAll($status = false)
	{
		if($status)
			return $this->mapper->findAll();
		else
			return $this->popInactiveOrgUnits($this->mapper->findAll());
	}
    
	/**
	 * Finds all children unit of organization unit given by id.
	 * 
	 * @param int $id organization unit ID.
	 * @param boolean $status Decides if funtion returns inactive units or not.
	 * @return	array of OrganizationUnit where id is key or NULL if not found.
	 */
	public function findAllChildren($id, $status = false)
	{
		if($status)
			return $this->mapper->findAllChildren($id);
		else
			return $this->popInactiveOrgUnits($this->mapper->findAllChildren($id));
		
	}
	
    /**
	 * Finds all children unit of organization unit by given path.
	 * 
	 * @param int $path organization unit path.
	 * @param boolean $status Decides if funtion returns inactive units or not.
	 * @return	array of OrganizationUnit where id is key or NULL if not found.
	 */
	public function findAllChildrenByPath($path, $status = false)
	{
		if($status)
			return $this->mapper->findAllChildrenByPath($path);
		else
			return $this->popInactiveOrgUnits($this->mapper->findAllChildrenByPath($path));
	}
	
	/**
	 * Finds all children unit of organization unit given by id and type of children.
	 * 
	 * @param int $id organization unit ID.
	 * @param string $type Type of organization unit in range of OrganizationUnit::ORG_UNIT_TYPE_*.
	 * @param boolean $status Decides if funtion returns inactive units or not.
	 * @return	array of OrganizationUnit where id is key or empty array.
	 */
	public function findAllChildrenByType($id, $type, $status = false)
	{
		if($status)
			return $this->mapper->findAllChildrenByType($id, $type);
		else
			return $this->popInactiveOrgUnits($this->mapper->findAllChildrenByType($id, $type));
	}
    
    /**
	 * Finds all subordinate of organization unit given by id.
	 * 
	 * @param int $id organization unit ID.
	 * @return	array of string(EID) or NULL if not found.
	 */
	public function findAllSubordinate($id)
	{
		return $this->mapper->findAllSubordinate($id);
	}
    
	/**
     * Gets all Organization units by type.
	 * 
     * @param string Type of org unit
	 * @param boolean $status Decides if funtion returns inactive units or not.
	 * @return array Array of OrganizationUnit where Organization unit ID is key.
	 */
	public function findAllOrgUnitByType($type, $status = false)
	{
		if($status)
			return $this->mapper->findAllOrgUnitByType($type);
		else
			return $this->popInactiveOrgUnits($this->mapper->findAllOrgUnitByType($type));
		
	}
    
	/**
	 * Finds all technical groups working on given project charter.
	 * 
	 * @param int $charter_ID ProjectCharter ID.
	 * @return array Array of OrganizationUnit where key is OrganizationUnit_ID or NULL if not found.
	 */
	public function findAllTechnicalGroupOnCharter($charter_ID)
    {
        return $this->popInactiveOrgUnits($this->mapper->findAllTechnicalGroupOnCharter($charter_ID));
    }
	
	/**
	 * Finds all Organization units by type and leader.
	 * 
     * @param string $type Type of organization unit
     * @param string $eid EID of leader of organization unit
	 * @return array Array of OrganizationUnit where Organization unit ID is key.
	 */
	public function findByTypeAndLeader($type, $eid)
	{
		return $this->popInactiveOrgUnits($this->mapper->findByTypeAndLeader($type, $eid));
	}
	
	/**
	 * Finds first ancestor of given organization unit
	 * 
	 * @param int $id Organization unit ID
	 * @return OrganizationUnit Or NULL in case that no ancestor was found.
	 */
	public function findFirstAncestor($id)
	{
		return $this->mapper->findFirstAncestor($id);
	}
	
	/**
	 * Finds all ancestors of given organization unit.
	 * 
	 * @param int $id Organization unit id
	 * @return array Array of OrganizationUnit. For each level of organizational
	 * structure carries OrganizationUnit or NULL depending on existence of
	 * the level for given organization unit id.
	 * $array[OrganizationUnit::ORG_UNIT_TYPE_*] => OrganizationUnit | NULL
	 */
	public function findAllAncestor($id)
	{
		$result = array(
			OrganizationUnit::ORG_UNIT_TYPE_SBE => NULL,
			OrganizationUnit::ORG_UNIT_TYPE_DEP => NULL,
			OrganizationUnit::ORG_UNIT_TYPE_CLUS => NULL,
			OrganizationUnit::ORG_UNIT_TYPE_TG => NULL
		);
		
		$unit = $this->mapper->find($id);
		if ($unit)
		{
			$result[$unit->Type] = $unit;
			$run = $unit->Type != OrganizationUnit::ORG_UNIT_TYPE_SBE;
			while ($run)
			{
				$unit = $this->mapper->find($unit->Parent_ID);
				if ($unit)
				{
					$result[$unit->Type] = $unit;
					$run = $unit->Type != OrganizationUnit::ORG_UNIT_TYPE_SBE;
				}
				else
					$run = FALSE;
			}
		}
		
		return $result;
	}
	
	/**
	 * Finds organization unit by path.
	 * 
	 * @param int $id organization unit Path.
	 * @return	OrganizationUnit or NULL if not found.
	 */
	public function findSBEByPath($id)
	{
		return $this->mapper->findSBEByPath($id);
	}
	
	/**
	 * Finds organization unit by short.
	 * 
	 * @param string $short organization unit Short.
	 * @return	OrganizationUnit or NULL if not found.
	 */
	public function findSBEByShort($short)
	{
		return $this->mapper->findSBEByShort($short);
	}

	/**
     * Finds all organization units where given person is leader
     *
     * @param string $person_id
     * @return array Array of OrganizationUnits indexed by their id or empty array.
     */
	public function findByLeader($person_id)
	{
		return $this->mapper->findByLeader($person_id);
	}

	/**
     * Tells if given person is leader of at least one organization unit
     *
     * @param string $person_id
     * @return boolean True if given person is leader, false otherwise
     */
	public function isLeader($person_id)
	{
		return $this->mapper->isLeader($person_id);
	}
}