<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing persons/employees from database using dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class PersonDibiMapper extends DibiMapper implements IPersonMapper
{

	/**
	 * Creates Person instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	Person 
	 * @todo rewrite using annotations
	 */
    private function load(DibiRow $row)
    {
		$person = new Person();
		$person->ID = $row->EID;
		$person->Surname = $row->Surname;
		$person->Firstname = $row->Name;
		$person->CostCenterID = $row->CostCenter;
		$person->Email = $row->Email;
        $person->OrganizationUnitID = $row->OrganizationUnit_ID;
        $person->Comment = $row->Comment;
		$person->Status = $row->Status;
		// Lock depends on Organization Unit
		$orgUnitLock_db_result = $this->conn->query('SELECT [Locked] FROM [OrganizationUnit] WHERE [OrganizationUnit_ID] = %i', $person->OrganizationUnitID)->fetch();
		if($orgUnitLock_db_result !== FALSE)
			$person->Locked = $orgUnitLock_db_result['Locked'];
		else
			$person->Locked = false;
		$person->NationalityID = $row->Nationality_ID;
		// find person supervisor
		$person_db_result = $this->conn->query('SELECT [Leader], [Parent_ID] FROM [OrganizationUnit] WHERE [OrganizationUnit_ID] = %i', $person->OrganizationUnitID)->fetch();
		if ($person_db_result !== FALSE)
		{
			if($person->ID == $person_db_result['Leader'])
			{
				$person_db_result = $this->conn->query('SELECT [Leader] FROM [OrganizationUnit] WHERE [OrganizationUnit_ID] = %i', $person_db_result['Parent_ID'])->fetch();
				if ($person_db_result !== FALSE)
					$person->SupervisorID = $person_db_result['Leader'];
				else
					$person->SupervisorID = null;
			}
			else
			{
				$person->SupervisorID = $person_db_result['Leader'];
			}
		}
		else
			$person->SupervisorID = null;
		return $person;
    }	

	/**
	 * Saves person.
	 * 
	 * @param Person $person
	 */
	public function save(Person $person)
	{
        try
        {
            $array = array();
            $array['EID'] = $person->getID();
            $array['Name'] = $person->getFirstname();
            $array['Surname'] = $person->getSurname();
            $array['OrganizationUnit_ID'] = $person->getOrganizationUnitID();
            $array['CostCenter'] = $person->getCostCenterID();
            $array['Email%sN'] = $person->getEmail();
            $array['Comment'] = $person->getComment();
			$array['Status'] = 1;
			$array['Nationality_ID'] = $person->getNationalityID();
            $person_db_result = $this->conn->insert('Person', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving person with id = ".$person->ID." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Updates person.
	 * 
	 * @param Person $person
	 */
	public function update(Person $person)
	{
        try
        {
            $array = array();
            $array['Name'] = $person->getFirstname();
            $array['Surname'] = $person->getSurname();
            $array['OrganizationUnit_ID'] = $person->getOrganizationUnitID();
            $array['CostCenter'] = $person->getCostCenterID();
            $array['Email%sN'] = $person->getEmail();
            $array['Comment'] = $person->getComment();
			$array['Status'] = $person->getStatus();
			$array['Nationality_ID'] = $person->getNationalityID();
            $person_db_result = $this->conn->update('Person', $array)->where('[EID] = %s', $person->ID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Person update with id = ".$person->ID." failed. ".$e->getMessage());
            return false;
        }
	}
	
	/**
	 * Updates person EID.
	 * 
	 * @param string $originEID person ID.
	 * @param string $newEID person ID.
	 */
	public function updateEID($originEID, $newEID)
	{
        try
        {
            $array = array();
            $array['EID'] = $newEID;
            $person_db_result = $this->conn->update('Person', $array)->where('[EID] = %s', $originEID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Person update with id = ".$originEID." failed. ".$e->getMessage());
            return false;
        }
	}
	
	/**
	 * Finds person by id.
	 * 
	 * @param string $id person ID.
	 * @return	Person or NULL if not found.
	 */
	public function find($id)
	{
		$person_db_result = $this->conn->query('SELECT * FROM [Person] WHERE [EID] = %s', $id)->fetch();
		if ($person_db_result !== FALSE)
			return $this->load($person_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds persons by workcenter.
	 * 
	 * @param string $id workcenter ID.
	 * @return	Person or NULL if not found.
	 */
	public function findAllByWorkCenter($id)
	{
        $persons_array = array();
		$persons_db_result = $this->conn->query('SELECT * FROM [Person] WHERE [CostCenter] = %s ORDER BY [Surname]', $id)->fetchAll();
		foreach ($persons_db_result as $person)
        {
			$persons_array[$person->EID] = $this->load($person);
        }
        return $persons_array;
	}
	
	/**
	 * Finds all Person.
	 * 
	 * @return array Array of Persons where EID is key.
	 */
	public function findAll()
	{
        $persons_array = array();
		$persons_db_result = $this->conn->query('SELECT * FROM [Person] ORDER BY [Surname]')->fetchAll();
		foreach ($persons_db_result as $person)
        {
			$persons_array[$person->EID] = $this->load($person);
        }
        return $persons_array;
	}
	
	/**
	 * Finds all Person by given sbe path.
	 * 
	 * @param int $path organization unit path.
	 * @return array Array of Persons where EID is key.
	 */
	public function findAllByOrgUnitPath($path)
	{
        $persons_array = array();
		$persons_db_result = $this->conn->query('SELECT [Person].* FROM [OrganizationUnit] JOIN
													[Person] ON [Person].[OrganizationUnit_ID] = [OrganizationUnit].[OrganizationUnit_ID] 
														WHERE [OrganizationUnit].[Path] LIKE %like~ ORDER BY [Person].[Surname]', $path)->fetchAll();
		foreach ($persons_db_result as $person)
        {
			$persons_array[$person->EID] = $this->load($person);
        }
        return $persons_array;
	}
	
    /**
     * Finds all people included into specific organization unit.
     * 
     * @param int $id ID of Organization unit.
     * @return array Array of Persons where EID is key.
     */
    public function findByOrganizationUnitID($id)
    {
        $persons_array = array();
        $db_result = $this->conn->query('SELECT * FROM [Person] WHERE [OrganizationUnit_ID] = %i ORDER BY [Surname]', $id)->fetchAll();
        foreach ($db_result as $person)
            $persons_array[$person->EID] = $this->load($person);
        
        return $persons_array;
    }

	/**
	 * Finds all active people included into specific organization unit.
	 * 
	 * @param int $id ID of Organization unit.
	 * @return array Array of Persons where EID is key.
	 */
	public function findActiveByOrganizationUnitID($id)
	{
        $persons = array();
		
        $db_result = $this->conn
            ->query('
                SELECT *
                FROM [Person]
                WHERE [OrganizationUnit_ID] = %i
                      AND [Status] >= 1
                ORDER BY [Surname]
        ', $id)
            ->fetchAll();

		foreach ($db_result as $person)
			$persons[$person->EID] = $this->load($person);
		
        return $persons;
	}
	
    /**
     * Gets all persons assigned to project from given year to future.
     * @param int $projectID project ID.
	 * @param int $year year.
     * @return array of Persons. Return array if find succesfull, otherwise returns null. 
     */
    public function findAllPersonsAssignedToProjectInYearToFuture($projectID, $year)
    {
        $db_result = $this->conn->query("SELECT DISTINCT [Person].* FROM [Person]
                                        JOIN [Person_Task] ON [Person_Task].[EID] = [Person].[EID]
                                            JOIN [Task] ON [Task].[Task_ID] = [Person_Task].[Task_ID]
                                                WHERE [Person_Task].[Year] >= %i AND [Task].[Project_ID] = %i", $year, $projectID)->fetchAll();
        $persons = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
                $persons[$row->EID] = $this->load($row);
            return $persons;        
        }
        else
            return NULL;
    }
	
	/**
     * Gets all persons assigned to task.
     * @param int $taskID task ID.
     * @return array of Persons. Return array if find succesfull, otherwise returns empty array. 
     */
    public function findAllPersonsAssignedToTask($taskID)
    {
        $db_result = $this->conn->query("SELECT DISTINCT [Person].* FROM [Person]
                                        JOIN [Person_Task] ON [Person_Task].[EID] = [Person].[EID]
                                                WHERE [Person_Task].[Task_ID] = %i", $taskID)->fetchAll();
        $persons = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
                $persons[$row->EID] = $this->load($row);
                    
        }
        return $persons;
    }
	
	/**
     * Gets all persons eid charged on task.
     * @param string $chargeNumber Charge number.
     * @return array of EID. Return array if find succesfull, otherwise returns empty array. 
     */
    public function findAllPersonsChargedOnTask($chargeNumber)
    {
        $db_result = $this->conn->query("SELECT [Actuals].[EID] FROM [Actuals]
                                        WHERE [Actuals].[ChargeNumber] = %s", $chargeNumber)->fetchAll();
        $eids = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
                $eids[$row->EID] = $row->EID;
        }
        return $eids;
    }
	
	/**
     * Gets all persons assigned to task in given year.
     * @param int $taskID task ID.
	 * @param int $year year.
     * @return array of Persons. Return array if find succesfull, otherwise returns null. 
     */
    public function findAllPersonsAssignedToTaskInYear($taskID, $year)
    {
        $db_result = $this->conn->query("SELECT DISTINCT [Person].* FROM [Person]
                                        JOIN [Person_Task] ON [Person_Task].[EID] = [Person].[EID]
                                            JOIN [TaskPlan] ON [TaskPlan].[Task_ID] = [Person_Task].[Task_ID]
                                                WHERE [Person_Task].[Year] = %i AND [TaskPlan].[Task_ID] = %i", $year, $taskID)->fetchAll();
        $persons = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
                $persons[$row->EID] = $this->load($row);
            return $persons;        
        }
        else
            return NULL;
    }
	
	/**
     * Gets all persons eid charged on task in given year.
     * @param string $chargeNumber Charge number.
	 * @param int $year year.
     * @return array of EID. Return array if find succesfull, otherwise returns empty array. 
     */
    public function findAllPersonsChargedOnTaskInYear($chargeNumber, $year)
    {
        $db_result = $this->conn->query("SELECT [Actuals].[EID] FROM [Actuals]
                                        WHERE [Actuals].[ChargeNumber] = %s
											AND [Actuals].[Year] = %i", $chargeNumber, $year)->fetchAll();
        $eids = array();
        if ($db_result !== FALSE)
        {
            foreach($db_result as $row)
                $eids[$row->EID] = $row->EID;
        }
        return $eids;
    }
	
	/**
	 * Gets all people, who have the 'TG' prefix in their EID.
	 * In fact, they are placeholders to shadow Technical Groups.
	 * 
	 * @return array 
	 */
	public function findAllTGs()
	{
        $persons_array = array();
		$persons_db_result = $this->conn->query('SELECT * FROM [Person] WHERE [EID] LIKE \'TG%\' ORDER BY [Surname]')->fetchAll();
		foreach ($persons_db_result as $person)
        {
			$persons_array[$person['OrganizationUnit_ID']] = $this->load($person);
        }
        return $persons_array;
	}
    
    /**
     * Finds active persons by their role on project.
     * @param string $role Should be in range of Person::ROLE_*
     * @param array $org_units Array of IDs of organization unit where people are members
     * @param array $project_phases Array of IDs of responsible organization unit for project
     * @return array Of Person or empty array.
     */
    public function findByProjectRole($role, $org_units = array(), $project_phases = array())
    {
        $db_result = $this->conn->select('[Person].*')->from('[Person], [Project]');
        $db_result->where('[Person].[Status] = 1');
        
        switch ($role)
        {
            case Person::ROLE_PIPM:
                $db_result->where('[Person].[EID] = [Project].[PIPM]');
                break;
            case Person::ROLE_PMOE:
                $db_result->where('[Person].[EID] = [Project].[PMOE]');
                break;
            case Person::ROLE_PPaC:
                $db_result->where('[Person].[EID] = [Project].[PPaC]');
                break;
            default:
                break;
        }
        
        $persons = array();
        
        if (count($org_units))
            $db_result->where('[Project].[ResponsibleUnit] IN %in', $org_units);
        else
            return $persons;
        
        if (count($project_phases))
            $db_result->where('[Project].[Phase] IN %in', $project_phases);
        else
            return $persons;
        
        $db_result->fetchAll();
		foreach ($db_result as $person)
			$persons[$person->EID] = $this->load($person);
        return $persons;
    }
    
    /**
     * Finds active persons who own resources
     * @param array $org_units Array of IDs of organization unit where people are members
     * @return array Of Person or empty array.
     */
    public function findResourceOwners($org_units = array())
    {
        $db_result = $this->conn->select('[Person].*')->from('[Person], [OrganizationUnit]');
        $db_result->where('[Person].[Status] = 1');
        $db_result->where('[Person].[EID] = [OrganizationUnit].[Leader]');
        $db_result->where('(SELECT COUNT(*) FROM [Person] WHERE [OrganizationUnit_ID] = [OrganizationUnit].[OrganizationUnit_ID] AND [EID] != [OrganizationUnit].[Leader]) > 0');

        $persons = array();
        
        if (count($org_units))
        {
            $cond = array();
            foreach ($org_units as $unit_id)
            {
                $orgUnit = $this->conn->select('[Path]')->from('[OrganizationUnit]')->where('[OrganizationUnit_ID] = %i', $unit_id)->fetch();
				if($orgUnit)
					$cond[] = array('[OrganizationUnit].[Path] LIKE %like~', $orgUnit['Path']);
            }
            $db_result->where('(%or)', $cond);
        }
        else
            return $persons;
        
        $db_result->fetchAll();
		foreach ($db_result as $person)
			$persons[$person->EID] = $this->load($person);
        return $persons;
    }
	
	/**
	 * Deletes person by id.
	 * 
	 * @param string $id person ID.
	 * @return boolean Return true if delete successfull, otherwiste returns false.
	 */
	public function delete($id)
	{
        try
        {
            $person_db_result = $this->conn->delete('Person')->where('[EID] = %s', $id)->execute();
            Debugger::log(print_r(dibi::$sql, true), Debugger::DEBUG);
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Person delete with id = $id failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
    
    /**
     * Gets Persons by PersonFilter. Returns every person that matches all
	 * set conditions.
     *
     * @param PersonFilter $personFilter
     * @return array of Person
     */
    public function getPersonsByFilter(PersonFilter $personFilter)
    {
        $textValues = array("EID" => "ID", "Surname" => "Surname", "Name" => "Firstname");
        $selectValues = array("CostCenter" => "CostCenterID");
        
		$db_result = $this->conn
                ->select('*')
                ->from('[Person]');

        foreach ($textValues as $dbKey => $key)
			if ($personFilter->$key)
				$db_result->where("[$dbKey] LIKE %~like~", $personFilter->$key);
            
        foreach ($selectValues as $dbKey => $key)
            if ($personFilter->$key)
                switch ($dbKey)
                {
                    case 'CostCenter':
                        $db_result->where("[$dbKey] = %s", $personFilter->$key);
                        break;
                    default:
                        $db_result->where("[$dbKey] = %i", $personFilter->$key);
                }
				
        if ($personFilter->Status)
        {
            $statuses = $personFilter->getStatus();
			if(count($statuses) != 2)
				foreach ($statuses as $value => $status)
				{
					if($status == 'Active')
						$db_result->where("[Status] = %i", 1);
					else
						$db_result->where("[Status] = %i", 0);
				}
        }
		else
		{
			//No status selected, no person should be displayed.
			$db_result->where("[Status] = %i", 2);
		}
        $db_result->fetchAll();
        
        $arrayOfPersons = array();
        foreach($db_result as $row)
            $arrayOfPersons[$row->EID] = $this->load($row);
        
        return $arrayOfPersons;
    }
	
    /**
     * Gets Persons by PersonFilter. Returns every person that matches
	 * at least one set condition in the filter.
     *
     * @param PersonFilter $personFilter
     * @return array of Person
     */
    public function getPersonsByOrFilter(PersonFilter $personFilter)
    {
        $textValues = array("EID" => "ID", "Surname" => "Surname", "Name" => "Firstname");
        $selectValues = array("OrganizationUnit_ID" => "OrganizationUnitID", "CostCenter" => "CostCenterID");
        
		$where_cond = 'SELECT * FROM [Person] WHERE ([EID] IS NULL)';
		$where_args = array();
		foreach ($textValues as $dbKey => $key)
			if ($personFilter->$key !== NULL)
			{
	            $where_cond .= " OR ([$dbKey] LIKE %~like~)";
				$where_args[] = $personFilter->$key;
			}
            
        foreach ($selectValues as $dbKey => $key)
            if ($personFilter->$key !== NULL)
                switch ($dbKey)
                {
                    case 'CostCenter':
					{
						$where_cond .= " OR ([$dbKey] = %s)";
						$where_args[] = $personFilter->$key;
						break;
					}
                    default:
					{
						$where_cond .= " OR ([$dbKey] = %i)";
						$where_args[] = $personFilter->$key;
						break;
					}
                }
		if (count($where_args) == 0)
			return array();
        $db_result = $this->conn->query(array_merge(array($where_cond), $where_args))->fetchAll();
        
        $arrayOfPersons = array();
        foreach($db_result as $row)
            $arrayOfPersons[$row->EID] = $this->load($row);
        
        return $arrayOfPersons;
    }

    /**
     * Returns leaders of organization units according to type of organization units.
     * @param array $orgUnitType Should be array of values in range of ORG_UNIT_TYPE_* defined in OrganizationUnit.
     * @return array of Person or empty array in case of empty result.
     */
    public function getLeadersByOrgUnitType(array $orgUnitType)
    {
        $db_result = $this->conn->select('[Person].*')->from('[Person], [OrganizationUnit]');
        $db_result->where("([Person].[EID] = [OrganizationUnit].[Leader])");
        $orgUnitTypeCondition = array();
        foreach ($orgUnitType as $type)
            array_push($orgUnitTypeCondition, array("[OrganizationUnit].[Type] = %s", $type));
        // add whole condition for statuses
        $db_result->where('(%or)', $orgUnitTypeCondition);
        $db_result->orderBy('[Person].[Surname]');
        $db_result->groupBy('[Person].[EID]');
        
        $db_result->fetchAll();
        
        $arrayPersons = array();
            foreach($db_result as $row)
                $arrayPersons[$row->EID] = $this->load($row);

        return $arrayPersons;            
    }
	
    /**
     * Returns leaders of organization units according to type of organization units and SBE.
     * @param array $orgUnitType Should be array of values in range of ORG_UNIT_TYPE_* defined in OrganizationUnit.
	 * @param int $sbeIDs Organizaiton unit ID
     * @return array of Person or empty array in case of empty result.
     */
    public function getLeadersByOrgUnitTypeAndSBE(array $orgUnitType, array $sbeIDs)
    {
        $db_result = $this->conn->select('[Person].*')->from('[Person], [OrganizationUnit]');
        $db_result->where("([Person].[EID] = [OrganizationUnit].[Leader])");
        $orgUnitTypeCondition = array();
        foreach ($orgUnitType as $type)
            array_push($orgUnitTypeCondition, array("[OrganizationUnit].[Type] = %s", $type));
		
		$orgUnitSBECondition = array();
        foreach ($sbeIDs as $sbeID)
		{
			$org_unit_path = $this->conn->query('SELECT [OrganizationUnit].[Path] FROM [OrganizationUnit] WHERE [OrganizationUnit].[OrganizationUnit_ID] = %i', $sbeID)->fetch();
			if($org_unit_path)
				array_push($orgUnitSBECondition, array('[Path] LIKE %like~', $org_unit_path['Path']));
		}
        // add whole condition for statuses
        $db_result->where('(%or)', $orgUnitTypeCondition);
		$db_result->where('(%or)', $orgUnitSBECondition);
        $db_result->orderBy('[Person].[Surname]');
        $db_result->groupBy('[Person].[EID]');
        
        $db_result->fetchAll();
        
        $arrayPersons = array();
            foreach($db_result as $row)
                $arrayPersons[$row->EID] = $this->load($row);

        return $arrayPersons;            
    }
}