<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing task assignments
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class PersonTaskAssignmentDibiMapper extends DibiMapper implements IPersonTaskAssignmentMapper
{
	/**
	 * Creates PersonTaskMonthAssignment instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	PersonTaskMonthAssignment 
	 */
    private function loadPersonTaskMonthAssignment(DibiRow $row)
    {
		$assignment = new PersonTaskMonthAssignment();
		$assignment->ID = $row->Person_Task;
		$assignment->PersonID = $row->EID;
		$assignment->TaskID = $row->Task_ID;
		$assignment->Year = $row->Year;
		// month are stored with indexes 0..11
		$assignment->Month = $row->Month + 1;
		$assignment->Hours = $row->Hours;
		$assignment->Version = $row->Version;
		$assignment->Modified = $row->Modified; // should be read-only
		return $assignment;
    }	

	/**
	 * Saves persons task assignment.
	 * 
	 * @param PersonTaskMonthAssignment $assignment
	 * @return	boolean Returns TRUE if success, otherwise false.
	 */
	public function save(PersonTaskMonthAssignment $assignment)
	{
		$values = array();
		$values['Hours'] = $assignment->getHours();
		// $values['Version'] = $assignment->getVersion(); // Not used yet
		$values['Modified'] = date('Y-m-d H:i:s');
		try
		{
			if ($assignment->getID() > 0)
				$this->conn->update('Person_Task', $values)->where('[Person_Task] = %u', $assignment->getID())->execute();
			else
			{
				if ($assignment->getHours() >= 0) // we do not save new empty assignments
				{
					$values['EID'] = $assignment->getPersonID();
					$values['Task_ID'] = $assignment->getTaskID();
					$values['Year'] = $assignment->getYear();
					$values['Month'] = $assignment->getMonth() - 1; // Month index in DB is in range <0,11> 
					$this->conn->insert('Person_Task', $values)->execute();
					$assignment->setID($this->conn->getInsertId());
				}
			}
			return true;
		}
		catch  (DibiException $e)
        {
            Debugger::log("Saving person assignment with ID=".$assignment->getID()." (Person=".$assignment->getPersonID().",TaskID=".$assignment->getTaskID().",Year=".$assignment->getYear().",Month=".$assignment->getMonth().") failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds month assignment by id of assignment.
	 * 
	 * @param int $id task assignment ID.
	 * @return	PersonTaskMonthAssignment 
	 */
	public function find($id)
	{
		$db_row = $this->conn->select('*')->from('Person_Task')->where('[Person_Task] = %u', $id)->fetch();
		if ($db_row !== FALSE)
			return $this->loadPersonTaskMonthAssignment($db_row);
		else
			return NULL;
	}

	/**
	 * Deletes all person assignment from task in given year.
	 * 
	 * @param int $person_id Person ID.
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 */
	public function deleteByPersonAndYear($person_id, $task_id, $year)
	{
		$deleted = $this->conn->delete('Person_Task')->where('[EID] = %s AND [Task_ID] = %u AND [Year] = %u', $person_id, $task_id, $year)->execute();
		return ($deleted > 0);
	}
	
	/**
	 * Deletes all assignment from task in given year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 */
	public function deleteByTaskAndYearAssignment($task_id, $year)
	{
		$deleted = $this->conn->delete('Person_Task')->where('[Task_ID] = %u AND [Year] = %u', $task_id, $year)->execute();
		return ($deleted > 0);
	}
	
	/**
	 * Deleteserson assignment from task in given year and month.
	 * 
	 * @param int $person_id Person ID.
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @param int $month Month.
	 */
	public function deleteByPersonAndYearAndMonth($person_id, $task_id, $year, $month)
	{
		$deleted = $this->conn->delete('Person_Task')->where('[EID] = %s AND [Task_ID] = %u AND [Year] = %u AND [Month] = %u', $person_id, $task_id, $year, ($month - 1))->execute();
		return ($deleted > 0);
	}
	
	/**
	 * Returns array task assigments 
	 * for every task person is assigned to in given month only if it
	 * is more than 0 hours.
	 * Task assignments are ordered by (grouped by) project they belong to.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month.
	 * Month indexes are 1 (January) - 12 (December). 
	 * 
	 * @param int $person_id Person ID.
	 * @param int $month month index (1-12).
	 * @param int $year year.
	 * @return	array or PersonTaskMonthAssignment 
	 *					where outer array keys are task IDs.
	 */
    public function findPersonMonthAssignments($person_id, $month, $year)
	{
		if (NumericUtils::isInteger($month) && ($month >= 1) && ($month <= 12))
		{
			$db_result = $this->conn->query('SELECT [Person_Task].*,[Task].[Project_ID] FROM [Person_Task],[Task] 
				WHERE [Person_Task].[EID] = %s AND [Person_Task].[Month] = %u AND [Person_Task].[Year] = %u
				AND [Person_Task].[Hours] > 0 AND ([Person_Task].[Task_ID] = [Task].[Task_ID])
				ORDER BY [Project_ID] ASC', $person_id, $month - 1, $year)->fetchAll();
            $assignments = array();
			foreach ($db_result as $row)
			{
				$assignment = $this->loadPersonTaskMonthAssignment($row);
				$assignments[$assignment->TaskID] = $assignment;
			}
			return $assignments;
		}
		else 
		{
			throw new InvalidArgumentException("Month parameter must be integer in range <1,12>. Given value:" . $month);
		}
	}

	/**
	 * Returns twelve task assigments for a person for given task ID in given year.
	 * Each for every fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array of 12 PersonTaskMonthAssigment where key is a month index.
	 */
	public function findPersonTaskYearAssignment($task_id, $year, $person_id)
	{
		$db_result = $this->conn->query('SELECT * FROM [Person_Task] 
			WHERE [Task_ID] = %u AND [Year] = %u AND [EID] = %s 
			ORDER BY [Month] ASC', $task_id, $year, $person_id)->fetchAll();
		$assignments = array();
		for ($i = 1; $i <= 12; $i++)
		{
			$assignments[$i] = new PersonTaskMonthAssignment();
			$assignments[$i]->setMonth($i);
			$assignments[$i]->setPersonID($person_id);
			$assignments[$i]->setTaskID($task_id);
			$assignments[$i]->setYear($year);
		}
		foreach ($db_result as $row)
		{
			$assignment = $this->loadPersonTaskMonthAssignment($row);
			$assignments[$assignment->Month] = $assignment;
		}
		return $assignments;
	}
	
	/**
	 * Returns twelve task assigments for a person for given task ID in given year.
	 * Each for every fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array of 12 PersonTaskMonthAssigment where key is a month index.
	 */
	public function findPersonTaskYearAndFutureAssignment($task_id, $year, $person_id)
	{
		$db_result = $this->conn->query('SELECT * FROM [Person_Task] 
			WHERE [Task_ID] = %u AND [Year] >= %u AND [EID] = %s 
			ORDER BY [Month] ASC', $task_id, $year, $person_id)->fetchAll();
		$assignments = array();
		for ($i = 1; $i <= 12; $i++)
		{
			$assignments[$i] = new PersonTaskMonthAssignment();
			$assignments[$i]->setMonth($i);
			$assignments[$i]->setPersonID($person_id);
			$assignments[$i]->setTaskID($task_id);
			$assignments[$i]->setYear($year);
		}
		foreach ($db_result as $row)
		{
			$assignment = $this->loadPersonTaskMonthAssignment($row);
			$assignments[$assignment->Month] = $assignment;
		}
		return $assignments;
	}

	
	/**
	 * Returns array of arrays of twelve task assigments 
	 * for every person that is assigned to given task ID in given year.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @param string $person_id Person/Employee ID.
	 * @return	array or arrays of 12 PersonTaskMonthAssignment 
	 *					where outer array keys are person IDs
	 *					and inner arrays keys are month indexes.
	 */
	public function findTaskYearAssignments($task_id, $year)
	{
		$db_result = $this->conn->query('SELECT DISTINCT [EID] FROM [Person_Task] 
			WHERE [Task_ID] = %u AND [Year] = %u', $task_id, $year)->fetchAll();
		$assignments = array();
		foreach ($db_result as $row)
			$assignments[$row->EID] = $this->findPersonTaskYearAssignment($task_id, $year, $row->EID);
		return $assignments;
	}

	/**
	 * Returns array of twelve values with total hours assigned every month
	 * to given person.
	 * 
	 * Month indexes are 1 (January) - 12 (December). 
	 * 
	 * @param int $person_id Person ID.
	 * @param int $year year.
	 * @return	array or integers where keys are month indexes.
	 */
    public function findPersonTotalYearAssignments($person_id, $year)
	{
		$assignments = array();
		for ($i = 1; $i <= 12; $i++)
			$assignments[$i] = 0;
		$db_result = $this->conn->query('SELECT SUM([Hours]) AS [Hours], [Month] FROM [Person_Task] 
			WHERE [EID]=%s AND [Year]=%u GROUP BY [Month] ORDER BY [Month] ASC', $person_id, $year)->fetchAll();
		foreach ($db_result as $row)
			$assignments[$row->Month + 1] = $row->Hours;
		return $assignments;
	}
	
	/**
	 * Returns wheter person is assigned to to any task year.
	 * 
	 * @param int $person_id Person ID.
	 * @return	boolean
	 */
    public function isPersonAssignedToAnyTasks($person_id)
    {
		$db_result = $this->conn->select('*')->from('[Person_Task]')->where('[EID] = %s', $person_id)->fetchAll();
        return (bool) count($db_result);
    }
	
	/**
	 * Returns whether task has any assignment in all years
	 * 
	 * @param int $task_id Tasl ID.
	 * @return	boolean
	 */
	public function hasTaskAssignmentAtAll($task_id)
	{
		$db_result = $this->conn->query('SELECT [Task_ID] FROM [Person_Task] 
			WHERE [Task_ID] = %u', $task_id)->fetchAll();
		return (bool) count($db_result);
	}
	
	/**
	 * Returns whether task assignment was transfered.
	 * 
	 * @param int $fromTaskID Task ID.
	 * @param int $toTaskID Task ID.
	 * @param int $year Year.
	 * @return	boolean
	 */
	public function transferTaskAssignment($fromTaskID, $toTaskID, $year)
	{
		try
		{
			$db_result = $this->conn->query('UPDATE [Person_Task] SET [Task_ID] = %i WHERE [Task_ID] = %i AND [Year] = %i', $toTaskID, $fromTaskID, $year);
			return true;
		}
        catch (DibiException $e)
        {
            Debugger::log("Task assignment was not transfered. From task with ID: " . $fromTaskID . ' to task with ID: ' . $toTaskID . ' ' . $e->getMessage());
            return false;
        }
	}
	
	/**
	 * Returns whether person task assignment was transfered.
	 * 
	 * @param int $fromTaskID Task ID.
	 * @param int $toTaskID Task ID.
	 * @param int $eid Person EID.
	 * @param int $year Year.
	 * @return	boolean
	 */
	public function transferPersonTaskAssignment($fromTaskID, $toTaskID, $eid, $year)
	{
		try
		{
			$db_result = $this->conn->query('UPDATE [Person_Task] SET [Task_ID] = %i WHERE [Task_ID] = %i AND [EID] = %i AND [Year] = %i', $toTaskID, $fromTaskID, $eid, $year);
			return true;
		}
        catch (DibiException $e)
        {
            Debugger::log("Person with EID: " . $eid . " task assignment was not transfered. From task with ID: " . $fromTaskID . ' to task with ID: ' . $toTaskID . ' ' . $e->getMessage());
            return false;
        }
	}
}
