<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project definition.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class Project extends Object
{
	/* possible project phases */
	const PROJECT_PHASE_PLANNED = 'Planned';
	const PROJECT_PHASE_ACTIVE = 'Active';
	const PROJECT_PHASE_CLOSED = 'Closed';
	const PROJECT_PHASE_CANCELLED = 'Cancelled';

	/* possible project funding types */
	const PROJECT_TYPE_BnP = 'B&P';
	const PROJECT_TYPE_INTERNAL = 'Internal';
	const PROJECT_TYPE_EXTERNAL = 'External';
	const PROJECT_TYPE_CAPITAL = 'Capital';
    const PROJECT_TYPE_COST_SHARE = 'Cost-share';
	const PROJECT_TYPE_CSC = 'CSC';
	
    /** @var int ID */
	private $id;
    /** @var string SAP ID */
    private $sapID;
    /** @var string EU SAP ID */
    private $euSapID;
	/** @var string EID of PIPM */
	private $pipm;
	/** @var string EID of PMOE */
    private $pmoe;
	/** @var string EID of PPaC */
    private $ppac;
	/** @var string EID of TM */
    private $tm;
	/** @var string name of project */
    private $name;
	/** @var string type of project */
    private $type = self::PROJECT_TYPE_INTERNAL;
	/** @var int ID of customer (organization) that is funding the project */
    private $customer;
	/** @var int Id of department responsible for the project */
    private $responsibility;
	/** @var string description the project */
    private $description;
	/** @var string date of creation of the project */
    private $createDate;
	/** @var string current project phase */
    private $phase = self::PROJECT_PHASE_PLANNED;
	/** @var string current project state */
    private $state = 0;
	/** @var string project remark */
    private $remark;
	/** @var string project ECO remark */
    private $ecoRemark;
    private $lastStateChangeDateTime;
	/** @var string project KeyCode ??? (TODO add better explanation) */
    private $keyCode;
    /** @var string project EUKeyCode ??? (TODO add better explanation) */
    private $euKeyCode;
	/** @var string level of project support by PMO department */
    private $pmoSupport;
	/** @var string Project start date */
    private $startDate;
	/** @var string Project end date */
    private $endDate;
	/** @var int ID of customer program */
	private $programID;
	/** @var int ID of customer program call */
	private $callID;
	/** @var int ID of responsible organization unit */
	private $responsibleUnit;
	/** @var int ECLicence id */
	private $ECLicenceID;
	/** @var boolean Fake if project is fake or not.*/ 
 	private $fake = 0;
 	/** @var float Percentage of costs sharing among customer and supplier */
 	private $realCostShare;
	/** @var int Project Plant*/
	private $plant;
    
   /**
     * Set ID of project.
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('Project ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of project.
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}

   /**
     * Set ID of project.
     * @param int $id Must be positive integer.
     */
	public function setProjectID($id)
	{
		$this->setID($id);
	}

	/**
	 * Gets ID of project.
	 * @return int
	 */
	public function getProjectID()
    {
        return $this->getID();
    }

	
    /**
     * Sets SAP ID.
     * 
     * @param string $id
     */
	public function setSAP_ID($id)
	{
		$this->sapID = $id;
	}

    /**
     * Gets SAP ID.
     * @return string
     */
	public function getSAPID()
	{
		return $this->getSAP_ID();
	}	
    
    /**
     * Gets SAP ID.
     * @return string
     */
	public function getSAP_ID()
	{
		return $this->sapID;
	}		
	
    /**
     * Sets EU SAP ID.
     * 
     * @param string $id
     */
	public function setEU_SAP_ID($id)
	{
		$this->euSapID = $id;
	}

    /**
     * Gets EU SAP ID.
     * @return string
     */
	public function getEU_SAP_ID()
	{
		return $this->euSapID;
	}

    
    /**
     * Sets name.
     * @param string $name
     */
	public function setName($name)
	{
		$this->name = $name;
	}

    /**
     * Gets name.
     * @return string
     */
	public function getName()
	{
		return $this->name;
	}	

    /**
     * Sets description.
     * @param string $description
     */
	public function setDescription($description)
	{
		$this->description = $description;
	}

    /**
     * Gets description.
     * @return string
     */
	public function getDescription()
	{
		return $this->description;
	}

    /**
     * Sets customer ID.
     * @param mixed $customer	Positive integer (Customer ID) or NULL.
     */
	public function setCustomer($customer)
	{
		if (($customer == '') || ($customer === NULL))
			$this->customer = null;
		elseif (NumericUtils::isPositiveInteger($customer))
			$this->customer = $customer;
		else
			throw new InvalidArgumentException('Customer must be null or positive integer.');
	}

    /**
     * Gets customer ID if set.
     * @return mixed	Positive integer (Customer ID) or NULL.
     */
	public function getCustomer()
	{
		return $this->customer;
	}	
	
    /**
     * Sets PIPM (Project Investigator and Project Manager) EID.
     * @param string $eid
     */
	public function setPIPM($eid)
	{
		$this->pipm = Person::processEID($eid);
	}

    /**
     * Gets PIPM (Project Investigator and Project Manager) EID.
     * @return string
     */
	public function getPIPM()
	{
		return $this->pipm;
	}	

    /**
     * Sets PMOE EID.
     * @param string $eid
     */
	public function setPMOE($eid)
	{
		$this->pmoe = Person::processEID($eid);
	}

    /**
     * Gets PMOE EID.
     * @return string
     */
	public function getPMOE()
	{
		return $this->pmoe;
	}	

    /**
     * Sets PPaC EID.
     * @param string $eid
     */
	public function setPPaC($eid)
	{
		$this->ppac = Person::processEID($eid);
	}

    /**
     * Gets PPaC EID.
     * @return string
     */
	public function getPPaC()
	{
		return $this->ppac;
	}	

    /**
     * Sets TM (Technical Manager) EID. Now Cluster Leader.
     * @param string $eid
     */
	public function setTM($eid)
	{
		$this->tm = Person::processEID($eid);
	}

    /**
     * Gets TM (Technical Manager) EID. Now Cluster Leader.
     * @return string
     */
	public function getTM()
	{
		return $this->tm;
	}	

    /**
     * Sets PMO Support level (Full or Basic or undefined (empty string)).
     * @param string $level
     */
	public function setPMOSupport($level)
	{
		$this->pmoSupport = $level;
	}

    /**
     * Gets PMO Support level (Full or Basic or undefined (empty string)).
     * @return string
     */
	public function getPMOSupport()
	{
		return $this->pmoSupport;
	}
	
    /**
     * Sets KeyCode code.
     * @param string $code
     */
	public function setKeyCode($code)
	{
		$this->keyCode = $code;
	}

    /**
     * Gets KeyCode code.
     * @return string
     */
	public function getKeyCode()
	{
		return $this->keyCode;
	}	

    /**
     * Sets EuKeyCode code.
     * @param string $code
     */
	public function setEuKeyCode($code)
	{
		$this->euKeyCode = $code;
	}

    /**
     * Gets EuKeyCode code.
     * @return string
     */
	public function getEuKeyCode()
	{
		return $this->euKeyCode;
	}
	
	/**
     * Set date of project creation.
     * @param string $date Date in YYYY-MM-DD formatD.
	 * @todo Check format of $date.
     */
	public function setDate($date)
	{
		$this->createDate = $date;
	}
	
    /**
     * Gets date of project creation.
     * 
     * @return string	Date in YYYY-MM-DD format
     */
	public function getDate()
	{
		return $this->createDate;
	}	

	/**
     * Sets last state date time of project.
     * @param string $date Date in YYYY-MM-DD HH:MM:SS format (db timestamp).
	 * @todo Check format of $date.
     */
	public function setLastStateChange($date)
	{
		$this->lastStateChangeDateTime = $date;
	}
	
    /**
     * Gets last state date time of project.
     * 
     * @return string	Date in YYYY-MM-DD HH:MM:SS format (db timestamp)
     */
	public function getLastStateChange()
	{
		return $this->lastStateChangeDateTime;
	}	
	
	/**
     * Sets phase of project.
     * 
     * @param string $remark
     */
	public function setPhase($phase)
	{
		if (in_array($phase, array_keys(Project::getPhases())))
			$this->phase = $phase;
		else
			throw new InvalidArgumentException('Unknown project phase.');
	}

    /**
     * Gets phase of project.
     * 
     * @return string
     */
	public function getPhase()
	{
		return $this->phase;
	}		

    /**
     * Returns array of available phases.
     * 
     * @return array
     */
	public static function getPhases()
	{
		return array(	self::PROJECT_PHASE_PLANNED => self::PROJECT_PHASE_PLANNED, 
						self::PROJECT_PHASE_ACTIVE => self::PROJECT_PHASE_ACTIVE, 
						self::PROJECT_PHASE_CLOSED => self::PROJECT_PHASE_CLOSED,
						self::PROJECT_PHASE_CANCELLED => self::PROJECT_PHASE_CANCELLED);
	}		
	
	/**
     * Sets state of project.
     * 
     * @param integer $state
     */
	public function setState($state)
	{
		if (NumericUtils::isNonNegativeInteger($state))
			$this->state = $state;
		else
			throw new InvalidArgumentException('Unknown project state.');
	}

    /**
     * Gets state of project.
     * 
     * @return integer
     */
	public function getState()
	{
		return $this->state;
	}		

	/**
     * Sets type of project.
     * 
     * @param string $type
     */
	public function setType($type)
	{
		if (in_array($type, array_keys(Project::getTypes())))
			$this->type = $type;
		else
			throw new InvalidArgumentException('Unknown project type.');
	}

    /**
     * Gets type of project.
     * 
     * @return string
     */
	public function getType()
	{
		return $this->type;
	}		

    /**
     * Returns array of available phases.
     * 
     * @return array
     */
	public static function getTypes()
	{
		return array(	self::PROJECT_TYPE_BnP => self::PROJECT_TYPE_BnP, 
						self::PROJECT_TYPE_INTERNAL => self::PROJECT_TYPE_INTERNAL, 
						self::PROJECT_TYPE_EXTERNAL => self::PROJECT_TYPE_EXTERNAL, 
						self::PROJECT_TYPE_CAPITAL => self::PROJECT_TYPE_CAPITAL,
                        self::PROJECT_TYPE_COST_SHARE => self::PROJECT_TYPE_COST_SHARE,
						self::PROJECT_TYPE_CSC => self::PROJECT_TYPE_CSC);
	}		
	
	/**
     * Sets remarks
     * 
     * @param string $remark
     */
	public function setRemark($remark)
	{
		$this->remark = $remark;
	}

    /**
     * Gets remarks
     * 
     * @return string
     */
	public function getRemark()
	{
		return $this->remark;
	}	
	
	/**
     * Sets eco remarks
     * 
     * @param string $remark
     */
	public function setECORemark($remark)
	{
		$this->ecoRemark = $remark;
	}

    /**
     * Gets eco remarks
     * 
     * @return string
     */
	public function getECORemark()
	{
		return $this->ecoRemark;
	}	
	
    /**
     * Sets responsibility.
     * @param mixed $responsibility Ingeger or null.
     */
	public function setResponsibility($responsibility)
	{
		if (($responsibility == '') || ($responsibility === NULL))
			$this->responsibility = null;
		elseif (NumericUtils::isPositiveInteger($responsibility))
			$this->responsibility = $responsibility;
		else
			throw new InvalidArgumentException('Responsibility must be null or positive integer.');
	}

    /**
     * Gets responsibility.
     * @return mixed integer or null
     */
	public function getResponsibility()
	{
		return $this->responsibility;
	}	

	/**
     * Set date of project start.
     * 
     * @param string $date Date in YYYY-MM-DD formatD.
	 * @todo Check format of $date.
     */
	public function setStart($date)
	{
		$this->startDate = $date;
	}
	
    /**
     * Gets date of project start.
     * 
     * @return string	Date in YYYY-MM-DD format
     */
	public function getStart()
	{
		return $this->startDate;
	}


    /**
     * Set date of project end.
     * 
     * @param string $date Date in YYYY-MM-DD format.
	 * @todo Check format of $date.
     */
	public function setEnd($date)
	{
		$this->endDate = $date;
	}
	
    /**
     * Gets date of project end.
     * 
     * @return string	Date in YYYY-MM-DD format
     */
	public function getEnd()
	{
		return $this->endDate;
	}	
	
    /**
     * Sets ID of customer program.
     * 
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
    public function setProgramID($id)
    {
		if (($id > 0) || ($id === NULL))
			$this->programID = $id;
		else
			throw new InvalidArgumentException('Program ID must be integer greater than 0. Wrong param: '.$id);
    }
	
    /**
     * Gets ID of customer program.
     * 
     * @return int
     */
    public function getProgramID()
    {
        return $this->programID;
    }	
	
    /**
     * Sets ID of customer program call.
     * 
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
    public function setCallID($id)
    {
		if (($id > 0) || ($id === NULL))
			$this->callID = $id;
		else
			throw new InvalidArgumentException('Call ID must be integer greater than 0. Wrong param: '.$id);
    }
	
    /**
     * Gets ID of customer program call.
     * 
     * @return int
     */
    public function getCallID()
    {
        return $this->callID;
    }	
	
    /**
     * Sets ID of responsible organization unit.
     * 
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
    public function setResponsibleUnit($id)
    {
		if (($id > 0) || ($id === NULL))
			$this->responsibleUnit = $id;
		else
			throw new InvalidArgumentException('ResponsibleUnit must be integer greater than 0. Wrong param: '.$id);
    }
	
    /**
     * Gets ID of responsible organization unit.
     * 
     * @return int
     */
    public function getResponsibleUnit()
    {
        return $this->responsibleUnit;
    }
	
	/**
     * Sets ECLicence id.
     * 
     * @param int $id
     */
	public function setECLicenceID($id)
	{
		if (($id > 0) || ($id === NULL))
			$this->ECLicenceID = $id;
		else
			throw new InvalidArgumentException('ECLicence ID must be integer greater than 0. Wrong param: ' . $id);
	}

    /**
     * Gets ECLicence id.
     * 
     * @return string
     */
	public function getECLicenceID()
	{
		return $this->ECLicenceID;
	}
	
	/**
	 * Sets fake flag. 
	 * 
	 * @param boolean $fake 
	 */ 
	public function setFake($fake) 
	{ 
		$this->fake = $fake; 
	} 

	/** 
	 * Gets fake flag. 
	 * 
	 * @return boolean 
	 */ 
	public function getFake() 
	{ 
		return ($this->fake) ? TRUE : FALSE; 
	}

	/**
	 * Sets real cost share.
	 * Precision will be rounded to 2 decimal positions.
	 * Value 1.6 means 160%
	 *
	 * @param float $value Should be number in range <0,10)
	 * @throws InvalidArgumentException
	 */
	public function setRealCostShare($value)
	{
        if ((is_numeric($value) && ($value >= 0) && ($value < 10)) || ($value === null))
        	$this->realCostShare = $value;
        else
        	throw new InvalidArgumentException('Cost Share must be float in range <0,10).'); 	
	}

	/**
	 * Sets real cost share.
	 * Treats with given value as it would be percentage representation of cost share.
	 * Value 160 will be transformed to 1.6
	 *
	 * @param int $value Should be number in range <0,1000)
	 * @throws InvalidArgumentException
	 */
	public function setRealCostSharePercentage($value)
	{
        if (is_numeric($value) && ($value >= 0) && ($value < 1000))
        	$this->realCostShare = $value / 100;
        elseif ($value === null)
        	$this->realCostShare = null;
        else
        	throw new InvalidArgumentException('Cost share must be integer in range <0,1000).'); 	
	}

	/**
	 * Gets real cost share
	 *
	 * @return float
	 */
	public function getRealCostShare()
	{
		return $this->realCostShare;
	}
	
	/**
	 * Gets real cost share as percentage value
	 *
	 * @return float
	 */
	public function getRealCostSharePercentage()
	{
		return ($this->getRealCostShare() !== null) ? ($this->getRealCostShare() * 100) : null;
	}
	
    /**
     * Sets project plant.
     * 
     * @param $plant int
     */
	public function setPlant($plant)
	{
		if ($plant >= 0)
			$this->plant = $plant;
		else
			throw new InvalidArgumentException('Project Plant must be integer greater than 0. Wrong param: '.$plant);
	}

    /**
     * Gets project plant.
     * 
     * @return int
     */
	public function getPlant()
	{
		return $this->plant;
	}	
}
