<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter definition.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharter extends Object
{
    /* possible project charter types */
    const PROJECT_CHARTER_STATUS_DEFINITION = 'Definition';
    const PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL = 'WaitingForLMApproval';
    const PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION = 'ApproversDefinition';
    const PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL = 'WaitingForFinalApproval';
    const PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED = 'DisapproveReceived';
    const PROJECT_CHARTER_STATUS_CANCELLED = 'Cancelled';
    const PROJECT_CHARTER_STATUS_APPROVED = 'Approved';
    
    /* full names of statuses */
    const FULL_NAME_STATUS_DEFINITION = 'Definition';
    const FULL_NAME_STATUS_WAITING_FOR_LM_APPROVAL = 'Cluster Leader Approval';
    const FULL_NAME_STATUS_APPROVERS_DEFINITION = 'Final Approvers Definition';
    const FULL_NAME_STATUS_WAITING_FOR_FINAL_APPROVAL = 'Final Approvals';
    const FULL_NAME_STATUS_DISAPPROVE_RECEIVED = 'Missing approvals';
    const FULL_NAME_STATUS_CANCELLED = 'Cancelled';
    const FULL_NAME_STATUS_APPROVED = 'Approved';
	
	// flight ops - checklist
	const CHECKLIST_NAME = 'Flight Ops - Checklist';

    /** @var int ID */
    private $id;
    /** @var string SAP ID of Project*/
    private $sapID;
    /** @var string Name of Project */
    private $name;
    /** @var string EID of Owner */
    private $owner;
    /** @var string EID of Line manager */
    private $lineManager;
    /** @var string Status */
    private $status = self::PROJECT_CHARTER_STATUS_DEFINITION;
    /** @var string Project charter create date */
    private $CreationDateTime;
    /** @var string Percentage of costs sharing among customer and supplier. */
    private $costShare = 0;
    /** @var int */
    private $customerID;
    /** @var int */
    private $programID;
    /** @var int */
    private $callID;
    /** @var int */
    private $responsibilityID;
    /** @var string */
    private $year;
    /** @var int */
    private $daysForFinalApproval;
	/** @var int ECLicence id */
	private $ECLicenceID;
	/** @var int Id of CL's organization unit */ 
    private $OrganizationUnitID;
		/** @var int Id of related STRAP item */ 
    private $strapItemID;
    
   /**
     * Sets ID of project charter.
     * @param int $id Must be positive integer.
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('Project charter ID must be integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of project charter.
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}
    
    /**
     * Sets SAP ID.
     * 
     * @param string $id
     */
	public function setSAP_ID($id)
	{
		$this->sapID = $id;
	}	
    
    /**
     * Gets SAP ID.
     * @return string
     */
	public function getSAP_ID()
	{
		return $this->sapID;
	}

    /**
     * Sets name.
     * @param string $name
     */
	public function setName($name)
	{
		$this->name = $name;
	}

    /**
     * Gets name.
     * @return string
     */
	public function getName()
	{
		return $this->name;
	}	
    
    /**
     * Sets owner EID.
     * @param string $eid
     */
	public function setOwner($eid)
	{
		$this->owner = Person::processEID($eid);
	}

    /**
     * Gets leader EID.
     * @return string
     */
	public function getOwner()
	{
		return $this->owner;
	}
    
    /**
     * Sets owner EID.
     * @param string $eid
     */
	public function setLineManager($eid)
	{
		$this->lineManager = Person::processEID($eid);
	}

    /**
     * Gets leader EID.
     * @return string
     */
	public function getLineManager()
	{
		return $this->lineManager;
	}
    
	/**
     * Sets status of project charter.
     * 
     * @param string $status
     */
	public function setStatus($status)
	{
		if (in_array($status, array_keys(ProjectCharter::getStatuses())))
			$this->status = $status;
		else
			throw new InvalidArgumentException('Unknown project charter status.');
	}

    /**
     * Gets type of project charter.
     * 
     * @return string
     */
	public function getStatus()
	{
		return $this->status;
	}
    
	/**
     * Set date of project charter creation.
     * 
     * @param string $date Date in YYYY-MM-DD formatD.
     */
	public function setCreationDateTime($date)
	{
		$this->CreationDateTime = $date;
	}
	
    /**
     * Gets date of project charter creation.
     * 
     * @return string	Date in YYYY-MM-DD format
     */
	public function getCreationDateTime()
	{
		return $this->CreationDateTime;
	}
	
	/**
     * Sets costs sharing percentage between customer and supplier.
	 * Must be float in range <0,1>. Percetage should be set as float.
	 * E.g.: 60% -> 0.6
	 * 
	 * Precision will be rounded to 2 decimal positions.
     * 
     * @param float $percent_as_float
     */
	public function setCostShare($percent_as_float)
	{
		if (is_numeric($percent_as_float)
				&& ($percent_as_float >= 0) && ($percent_as_float <= 1.0))
			$this->costShare = $percent_as_float;
		else
			throw new InvalidArgumentException('Cost sharing must be float in range <0,1>.');
	}
    
    /**
     * Gets cost share.
     * 
     * @return float
     */
	public function getCostShare()
	{
		return $this->costShare;
	}
    
    /**
     * Gets cost share expressed in percentage.
     * 
     * @return float
     */
	public function getCostSharePercentage()
	{
	    return $this->costShare * 100;
	}
        
    /**
     * Sets customerID.
     * @param int $customerID Must be positive integer.
     */
	public function setCustomerID($customerID)
	{
		$this->customerID = $customerID;
	}

    /**
     * Gets customerID.
     * @return int
     */
	public function getCustomerID()
	{
		return $this->customerID;
	}
	
	/**
     * Sets ID of program.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setProgramID($id)
	{
		if (($id > 0) || ($id === NULL))
			$this->programID = $id;
		else
			throw new InvalidArgumentException('Program ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of program.
     * @return int
     */
	public function getProgramID()
	{
		return $this->programID;
	}
	
	/**
     * Sets ID of call.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setCallID($id)
	{
		if (($id > 0) || ($id === NULL))
			$this->callID = $id;
		else
			throw new InvalidArgumentException('Call ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of call.
     * @return int
     */
	public function getCallID()
	{
		return $this->callID;
	}
	
    /**
     * Sets DaysForFinalApproval.
     * @param int $daysForFinalApproval Must be positive integer.
     */
	public function setDaysForFinalApproval($daysForFinalApproval)
	{
		$this->daysForFinalApproval = $daysForFinalApproval;
	}

    /**
     * Gets DaysForFinalApproval.
     * @return int
     */
	public function getDaysForFinalApproval()
	{
		return $this->daysForFinalApproval;
	}
        
    /**
     * Sets responsibilityID.
     * @param int $responsibilityID Must be positive integer.
     */
	public function setResponsibilityID($responsibilityID)
	{
		$this->responsibilityID = $responsibilityID;
	}

    /**
     * Gets responsibilityID.
     * @return int
     */
	public function getResponsibilityID()
	{
		return $this->responsibilityID;
	}
        
    /**
     * Sets year.
     * @param string $year Year in YYYY format.
     */
	public function setYear($year)
	{
		$this->year = $year;
	}

    /**
     * Gets year.
     * @return string
     */
	public function getYear()
	{
		return $this->year;
	}
    
    /**
     * Returns array of available statuses
     * @return array arr[string] == string .. arr[PROJECT_CHARTER_STATUS_*] == PROJECT_CHARTER_STATUS_*
     */
	public static function getStatuses()
	{
            $arrStatuses = array(
                self::PROJECT_CHARTER_STATUS_DEFINITION => self::PROJECT_CHARTER_STATUS_DEFINITION,
                self::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL => self::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL,
                self::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION => self::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION,
                self::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL => self::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL,
                self::PROJECT_CHARTER_STATUS_CANCELLED => self::PROJECT_CHARTER_STATUS_CANCELLED,
                self::PROJECT_CHARTER_STATUS_APPROVED => self::PROJECT_CHARTER_STATUS_APPROVED,
                self::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED => self::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED
            );
            // sort array
            asort($arrStatuses);
            return $arrStatuses;
	}
        
    /**
     * Returns array of available statuses.
     * @return array arr[string] == string .. arr[PROJECT_CHARTER_STATUS_*] == full status name
     */
        public static function getStatusesFullNames()
        {
            $arrStatuses = array(
                self::PROJECT_CHARTER_STATUS_DEFINITION => self::FULL_NAME_STATUS_DEFINITION,
                self::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL => self::FULL_NAME_STATUS_WAITING_FOR_LM_APPROVAL,
                self::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION => self::FULL_NAME_STATUS_APPROVERS_DEFINITION,
                self::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL => self::FULL_NAME_STATUS_WAITING_FOR_FINAL_APPROVAL,
                self::PROJECT_CHARTER_STATUS_CANCELLED => self::FULL_NAME_STATUS_CANCELLED,
                self::PROJECT_CHARTER_STATUS_APPROVED => self::FULL_NAME_STATUS_APPROVED,
                self::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED => self::FULL_NAME_STATUS_DISAPPROVE_RECEIVED
            );
            // sort array
            asort($arrStatuses);
            return $arrStatuses;
        }
        
    /**
     * Returns full name of status of project charter according to shortcut.
     * @param string Type of organization unit in range of self::PROJECT_CHARTER_STATUS_*
     * @return string Full name of status or empty string if type doesn't exist.
     */
    public static function getFullStatusName($status)
    {
        switch($status)
        {
            case self::PROJECT_CHARTER_STATUS_DEFINITION:
                return self::FULL_NAME_STATUS_DEFINITION;
            case self::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL:
                return self::FULL_NAME_STATUS_WAITING_FOR_LM_APPROVAL;
            case self::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION:
                return self::FULL_NAME_STATUS_APPROVERS_DEFINITION;
            case self::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL:
                return self::FULL_NAME_STATUS_WAITING_FOR_FINAL_APPROVAL;
            case self::PROJECT_CHARTER_STATUS_CANCELLED:
                return self::FULL_NAME_STATUS_CANCELLED;
            case self::PROJECT_CHARTER_STATUS_APPROVED:
                return self::FULL_NAME_STATUS_APPROVED;
            case self::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED:
                return self::FULL_NAME_STATUS_DISAPPROVE_RECEIVED;
            default:
                return '';
        }
    }
	
	/**
     * Sets ECLicence id.
     * @param int $id
     */
	public function setECLicenceID($id)
	{
		if ($id > 0)
			$this->ECLicenceID = $id;
		else
			throw new InvalidArgumentException('ECLicence ID must be integer greater than 0. Wrong param: ' . $id);
	}

    /**
     * Gets ECLicence id.
     * @return string
     */
	public function getECLicenceID()
	{
		return $this->ECLicenceID;
	}
	
	/**
     * Sets OrganizationUnit ID. 
     *  
     * @param int $id  
     */ 
    public function setOrganizationUnitID($id) 
    { 
            $this->OrganizationUnitID = $id; 
    } 

    /** 
     * Gets OrganizationUnit ID. 
     *  
     * @return int 
     */ 
    public function getOrganizationUnitID() 
    { 
            return $this->OrganizationUnitID; 
    }

	/**
     * Sets StrapItem ID. 
     *  
     * @param int $id  
     */ 
    public function setStrapItemID($id) 
    { 
            $this->strapItemID = $id; 
    } 

    /** 
     * Gets StrapItem ID. 
     *  
     * @return int 
     */ 
    public function getStrapItemID() 
    { 
            return $this->strapItemID; 
    }
}