<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter approver definition.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterApprover extends Object
{
    /** @var int ID of record */
    private $projectCharterApproverID;
    /** @var int ID of ProjectCharter */
    private $projectCharterID;
    /** @var int Version of round of approval */
    private $version;
    /** @var string Role of approver in company structure */
    private $role;
    /** @var string Approver surname */
    private $approverEID;
    /** @var string Approver surname */
    private $surname;
    /** @var string Approver name */
    private $name;
    /** @var string Email of person*/
    private $email;
    /** @var int Decision result */
    private $approval = self::DECISION_PENDING;
    /** @var string Decision date */
    private $decisionDateTime;
    /** @var string Comment */
    private $comment;
    /** @var boolean Auto approved */
    private $autoApproved = FALSE;
	
	/** @const string List of roles of approvers */
	const APPROVER_ROLE_ATE_DIRECTOR = 'ATE Director', // Advanced Technology Europe Director
	      APPROVER_ROLE_DEP_LEADER = 'Department Leader',
		  APPROVER_ROLE_MandPM = 'M&PM', // Marketing & Product Management
	      APPROVER_ROLE_PEC = 'PEC', // Programs & External Customers, kept for legacy charters
          APPROVER_ROLE_PROGRAM_MANAGER = 'Program Manager', // from now on, PEC is Program Manager
	      APPROVER_ROLE_PMO = 'PMO', // Project Management Office
	      APPROVER_ROLE_RTC_DIRECTOR = 'RTC Director'; // Research & Technology Center Director

	/** @const int Approver decisions */
	const 
		DECISION_PENDING = 0,
		DECISION_DISAPPROVED = 1,
		DECISION_APPROVED = 2;
	
    /**
     * Sets ID of project charter approver record.
     * @param int $id Must be positive integer.
     */
    public function setProjectCharterApproverID($id)
    {
        if ($id > 0)
            $this->projectCharterApproverID = $id;
		else
            throw new InvalidArgumentException('Project charter approver ID must be integer greater than 0. Wrong param: '.$id);
    }
    
    /**
     * Gets ID of project charter approver record.
     * @return int
     */
    public function getProjectCharterApproverID()
    {
        return $this->projectCharterApproverID;
    }
    
   /**
    * Sets ID of project charter.
    * @param int $id Must be positive integer.
    */
    public function setProjectCharterID($id)
    {
        if ($id > 0)
            $this->projectCharterID = $id;
        else
            throw new InvalidArgumentException('Project charter ID must be integer greater than 0. Wrong param: '.$id);
    }
	
    /**
     * Gets ID of project charter.
     * @return int
     */
    public function getProjectCharterID()
    {
        return $this->projectCharterID;
    }
    
   /**
    * Sets version of project charter approval round.
    * @param int $version Must be positive integer.
    */
    public function setVersion($version)
    {
        if ($version > 0)
            $this->version = $version;
        else
            throw new InvalidArgumentException('Version must be integer greater than 0. Wrong param: '.$version);
    }
	
    /**
     * Gets version of project charter approval round.
     * @return int
     */
    public function getVersion()
    {
        return $this->version;
    }
	
	/**
    * Sets role of approver
    * @param int $role
    */
    public function setRole($role)
    {
            $this->role = $role;
    }
	
    /**
     * Gets role of approver
     * @return int
     */
    public function getRole()
    {
        return $this->role;
    }
    
    /**
     * Sets approver EID.
     * @param string $eid
     */
    public function setApproverEID($eid)
    {
        $this->approverEID = Person::processEID($eid);
    }

    /**
     * Gets approver EID.
     * @return string
     */
    public function getApproverEID()
    {
        return $this->approverEID;
    }
    
    /**
     * Sets approver surname.
     * @param string $surname
     */
    public function setSurname($surname)
    {
        $this->surname = $surname;
    }

    /**
     * Gets approver surname.
     * @return string
     */
    public function getSurname()
    {
        return $this->surname;
    }
    
    /**
     * Sets approver name.
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * Gets approver name.
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }
    
    /**
     * Sets email.
     * @param string $email
     */
    public function setEmail($email)
    {
        $this->email = $email;
    }

    /**
     * Gets email.
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }
    
    /**
     * Sets decision result. 
	 * Accepts only values DECISION_PENDING, DECISION_DISAPPROVED, DECISION_APPROVED
     * @param int $decision
     */
    public function setApproval($decision)
    {
		if (($decision == self::DECISION_PENDING)
			|| ($decision == self::DECISION_DISAPPROVED)
			|| ($decision == self::DECISION_APPROVED))
			$this->approval = $decision;
		else
			throw new InvalidArgumentException("Approval decision is invalid: " . $decision);
    }

    /**
     * Gets decision result.
     * @return int
     */
    public function getApproval()
    {
        return $this->approval;
    }
    
    /**
     * Sets date of decision.
     * @param string $decisionDate Date in YYYY-MM-DD format.
     * @todo Check format of $decisionDate.
     */
    public function setDecisionDateTime($decisionDate)
    {
        $this->decisionDateTime = $decisionDate;
    }
	
    /**
     * Gets date of decision. 
     * @return string	Date in YYYY-MM-DD format
     */
    public function getDecisionDateTime()
    {
        return $this->decisionDateTime;
    }
    
    /**
     * Set comment of current version.
     * @param string $comment
     */
    public function setComment($comment)
    {
        $this->comment = $comment;
    }
        
    /**
     * Gets comment of current version.
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }
    
    /**
	 * Set flag whether decision (approval) was setup by system and not by person.
     * @param boolean $autoApproved
     */
    public function setAutoApproved($autoApproved)
    {
        $this->autoApproved = ($autoApproved) ? TRUE : FALSE;
    }

    /**
	 * Gets flag whether decision (approval) was set by system and not by person.
     * @return boolean
     */
    public function getAutoApproved()
    {
        return $this->autoApproved;
    }
}