<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project charter approver history definition.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterApproverHistory extends Object
{
    /** @var int ProjectCharter ID */
    private $projectCharterID;
    /** @var int Version of round of approval */
    private $version;
    /** @var string */
    private $creationDateTime;
    /** @var string EID of responsible person */
    private $responsible;
    /** @var boolean */
    private $latestVersion;
    /** @var int Version of budget for approval */
    private $budgetVersion;
    /** @var int Version of file for approval*/
    private $fileVersion;
    /** @var string Content of email sent to approvers */
    private $emailContent;
    /** @var string Timestamp of date until everybody should decide for approve or disapprove */
    private $finalApprovalDate;
    
    /**
     * Sets ID of project charter.
     * @param int $id Should be positive integer.
     * @throws InvalidArgumentException
     */
    public function setProjectCharterID($id)
    {
        if ($id > 0)
            $this->projectCharterID = $id;
	else
            throw new InvalidArgumentException('Project charter ID must be integer greater than 0. Wrong param: '.$id);
    }
    
    /**
     * Gets ID of project charter.
     * @return int
     */
    public function getProjectCharterID()
    {
        return $this->projectCharterID;
    }
    
    /**
     * Sets version of round of approval.
     * @param int $version Should be positive integer.
     * @throws InvalidArgumentException
     */
    public function setVersion($version)
    {
        if ($version > 0)
            $this->version = $version;
	else
            throw new InvalidArgumentException('Version must be integer greater than 0. Wrong param: '.$version);
    }
    
    /**
     * Gets version of round of approval.
     * @return int
     */
    public function getVersion()
    {
        return $this->version;
    }
    
    /**
     * Sets date of creation.
     * @param string $date Date in YYYY-MM-DD format.
     * @todo Check format of $date.
     */
    public function setCreationDateTime($date)
    {
        $this->creationDateTime = $date;
    }
	
    /**
     * Gets date of decision. 
     * @return string	Date in YYYY-MM-DD format
     */
    public function getCreationDateTime()
    {
        return $this->creationDateTime;
    }
    
    /**
     * Sets EID of responsible person.
     * @param string $eid
     */
    public function setResponsible($eid)
    {
        $this->responsible = Person::processEID($eid);
    }

    /**
     * Gets EID of responsible person.
     * @return string
     */
    public function getResponsible()
    {
        return $this->responsible;
    }
    
    /**
     * Sets flag of latest version.
     * @param int $latest
     */
    public function setLatestVersion($latest)
    {
        $this->latestVersion = ($latest) ? TRUE : FALSE;
    }
    
    /**
     * Gets flag of latest version.
     * @return boolean
     */
    public function getLatestVersion()
    {
        return $this->latestVersion;
    }
    
    /**
     * Sets version of budget.
     * @param int $version Should be positive integer.
     * @throws InvalidArgumentException
     */
    public function setBudgetVersion($version)
    {
        if ($version == NULL || $version > 0)
            $this->budgetVersion = $version;
		else
            throw new InvalidArgumentException('Budget version must be integer greater than 0. Wrong param: '.$version);
    }
    
    /**
     * Gets version of budget.
     * @return int
     */
    public function getBudgetVersion()
    {
        return $this->budgetVersion;
    }
    
    /**
     * Sets version of file.
     * @param int $version Should be positive integer.
     * @throws InvalidArgumentException
     */
    public function setFileVersion($version)
    {
        if ($version == NULL || $version > 0)
            $this->fileVersion = $version;
		else
            throw new InvalidArgumentException('File version must be integer greater than 0. Wrong param: '.$version);
    }
    
    /**
     * Gets version of file.
     * @return int
     */
    public function getFileVersion()
    {
        return $this->fileVersion;
    }
    
    /**
     * Sets content of email.
     * @param string $text.
     */
    public function setEmailContent($text)
    {
        $this->emailContent = $text;
    }
	
    /**
     * Gets content of email. 
     * @return string
     */
    public function getEmailContent()
    {
        return $this->emailContent;
    }
    
    /**
     * Sets date and time deadline for approval.
     * @param int $date Date and time as Unix timestamp.
     */
    public function setFinalApprovalDate($timestamp)
    {
        $this->finalApprovalDate = $timestamp;
    }
	
    /**
     * Gets deadline for final approval as timestamp. 
     * @return int Date as plain Unix timestamp.
     */
    public function getFinalApprovalDate()
    {
        return $this->finalApprovalDate;
    }
	
    /**
     * Gets date previous of date of last day for approval in local time. 
     * @return string Date in YYYY-MM-DD format
     */
    public function getFinalApprovalDateFormatted()
    {
        return date('Y-m-d', strtotime('-1 day', $this->finalApprovalDate));
    }
	
	/**
	 * Gets date of last day for approval worldwide.
	 * @return array If $finalApprovalDate is not set or error occured returns empty array else
	 *     arr[]['City'] - Name of city in time zone
	 *     arr[]['Date'] - Final approval date converted to time zone defined by city name
	 *     arr[]['UTC_Offset'] - e.g. "UTC +5"
	 */
	public function getFinalApprovalDateWorldwide()
	{	
		$result = array();
		
		$zones = array(
			'Pacific/Honolulu', // next are Zulu -x
			'America/Anchorage',
			'America/Phoenix',
			'America/Chicago',
			'America/New_York',
			'America/Halifax',
			'Europe/London',    // Zulu
			'Europe/Paris',     // next are Zulu +x
			'Europe/Helsinki',
			'Europe/Moscow',
			'Asia/Karachi',
			'Asia/Yekaterinburg',
			'Asia/Bangkok',
			'Asia/Shanghai',
			'Asia/Tokyo',
			'Australia/Sydney',
			'Asia/Vladivostok',
			'Pacific/Auckland',
		);
		
		if ($this->finalApprovalDate)
		{
			$datetime = new DateTime(date('Y-m-d H:i:s', $this->finalApprovalDate));
			$i = 0;
			foreach ($zones as $city) {
				$datetime->setTimezone(new DateTimeZone($city));
				$offsetValue = $datetime->getOffset() / 3600;
				$offset = ($offsetValue > 0) ? '+' . $offsetValue : $offsetValue;
				
				$result[$i]['City'] = preg_replace('/_/', ' ', $city);
				$result[$i]['Date'] = $datetime->format('Y-m-d H:i:s');
				$result[$i]['UTC_Offset'] = ($offset != '0') ? 'UTC ' . $offset : 'UTC';
				$i++;
			}
		}
		
		return $result;
	}
}
