<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Description of projectCharterDashboardService
 * 
 * Contract of the class:
 * Use method prepareProjectCharterDashboardService() first for component preparation
 * and then use getProjectCharterDashboardService() to obtain the data.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @todo Do not inject context into class but rather inject single repositories
 */
class ProjectCharterDashboardService
{
	/** prepared list of charters */
	private $service = NULL;

	/** @var DIContainer Access to model repositories */
	private $context;

    public function __construct(DIContainer $context)
    {
		$this->context = $context;
	}
	
    /**
     * Associative array of Project charter arrays where key is project charet ID. 
     * Each second level array have following keyed values:
     * 
	 * 'ProjectCharter_ID'
	 * 'Name' - project charter name
	 * 'CreationDate'
	 * 'Owner' - short name
     * 'Owner_EID'
     * 'Owner_Full' - full name
	 * 'SAP_ID'
	 * 'LineManager' - short name
     * 'LineManager_EID'
     * 'LineManager_Full' - full name
	 * 'Customer_ID'
     * 'Responsibility_ID'
     * 'Year'
     * 'Status'
     * 'Status_Full'
     * 'Approval' - only if charter status is WaitingForFinalApprover else is not defined.
     * 
	 * @return array
     */
    public function filteredChartersToArray($charters, $filtered, $approver = null)
    {
        foreach ($filtered as $charter)
        {
            $id = $charter->getID();
            $charters[$id]['ProjectCharter_ID'] = $id;
            $charters[$id]['Name'] = $charter->getName();
            $charters[$id]['CreationDate'] = date_format($charter->getCreationDateTime(), 'Y-m-d');
            $owner = $this->context->personRepository->find($charter->getOwner());
            $charters[$id]['Owner'] = $owner->getSurname() . ', ' . substr($owner->getFirstname(), 0, 1) . '.';
            $charters[$id]['Owner_EID'] = strtolower($owner->getID());
            $charters[$id]['Owner_Full'] = $owner->getSurname() . ', ' . $owner->getFirstname() . ' (' . strtoupper($owner->getID()) . ')';
            $charters[$id]['SAP_ID'] = $charter->getSAP_ID();
            $lineManager = $this->context->personRepository->find($charter->getLineManager());
            $charters[$id]['LineManager'] = $lineManager->getSurname() . ', ' . substr($lineManager->getFirstname(), 0, 1) . '.';
            $charters[$id]['LineManager_EID'] = strtolower($lineManager->getID());
            $charters[$id]['LineManager_Full'] = $lineManager->getSurname() . ', ' . $lineManager->getFirstname() . ' (' . strtoupper($lineManager->getID()) . ')';
            $charters[$id]['Customer_ID'] = $this->context->customerRepository->find($charter->getCustomerID())->getName();
            $charters[$id]['Responsibility_ID'] = $this->context->responsibilityRepository->find($charter->getResponsibilityID())->getName();
            $charters[$id]['Year'] = $charter->getYear();
            $charters[$id]['Status'] = $charter->getStatus();
            $charters[$id]['Status_Full'] = ProjectCharter::getFullStatusName($charter->getStatus());
            if($charter->status == ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL && $approver != null)
                $charters[$id]['Approval'] = $approver->Approval;
			else
				$charters[$id]['Approval'] = 'none';
        }        
        return $charters;
    }
    
   /**
     * Prepares list of project chartes where is logged user involved.
	 * 
     * @param $employee_id string Employee ID.
	 * @return void	
     */
    public function prepareProjectCharterDashboardService($employee_id) 
    {
        $charters = array();
		
		// dashboard should only show Project charters from this and upcoming year
		$yearRange = array((int) date('Y'), (int) date('Y') + 1);
		// all statuses except Cancelled
		$statuses = array(
			ProjectCharter::PROJECT_CHARTER_STATUS_DEFINITION,
			ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_LM_APPROVAL,
			ProjectCharter::PROJECT_CHARTER_STATUS_APPROVERS_DEFINITION,
			ProjectCharter::PROJECT_CHARTER_STATUS_WAITING_FOR_FINAL_APPROVAL,
			ProjectCharter::PROJECT_CHARTER_STATUS_APPROVED,
			ProjectCharter::PROJECT_CHARTER_STATUS_DISAPPROVE_RECEIVED,
		);
		
        // I am owner
        $charterFilter = new ProjectCharterFilter();
        $charterFilter->setOwner($employee_id);
		$charterFilter->setYear($yearRange);
		$charterFilter->setStatus($statuses);
        
        // get project charters according to values of filter
        $chartersFiltered = $this->context->projectCharterRepository->getChartersByFilter($charterFilter);
        $charters = $this->filteredChartersToArray($charters, $chartersFiltered);
        
        // I am line manager
        $charterFilter = new ProjectCharterFilter();
        $charterFilter->setLineManager($employee_id);
		$charterFilter->setYear($yearRange);
		$charterFilter->setStatus($statuses);
        
        // get project charters according to values of filter
        $chartersFiltered = $this->context->projectCharterRepository->getChartersByFilter($charterFilter);
        $charters = $this->filteredChartersToArray($charters, $chartersFiltered);
        
        // I am final approver
        $approverCharters = $this->context->projectCharterApproverRepository->findAllProjectCharterByApprover($employee_id);
        foreach($approverCharters as $approverCharter)
        {
            $charter = array();
            $charter[] = $this->context->projectCharterRepository->find($approverCharter->ProjectCharterID);
			
			// we have to get rid of $approverCharters with year out of the range, too
			if(!in_array($charter[0]->getYear(), $yearRange) || $charter[0]->status == ProjectCharter::PROJECT_CHARTER_STATUS_CANCELLED)
			{
				continue;
			}
			
            $charters = $this->filteredChartersToArray($charters, $charter, $approverCharter);
        }
		
        $this->service = $charters;
	}

    /**
     * Returns service prepared by prepareProjectCharterDashboardService method.
	 * 
	 * @return array or null
     */
    public function getProjectCharterDashboardService() 
    {
        return $this->service;
	}
}

