<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing projectCharterFileHistory from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterFileHistoryDibiMapper extends DibiMapper implements IProjectCharterFileHistoryMapper
{
	/**
	 * Creates ProjectCharterFileHistory instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return ProjectCharterFileHistory 
	 */
	private function load(DibiRow $row)
	{
		$projectCharterFileHistory = new ProjectCharterFileHistory();
		$projectCharterFileHistory->projectCharterID = $row->ProjectCharter_ID;
		$projectCharterFileHistory->version = $row->Version;
		$projectCharterFileHistory->creationDateTime = $row->CreationDateTime;
		$projectCharterFileHistory->fileName = $row->FileName;
		$projectCharterFileHistory->responsible = $row->Responsible;		
		$projectCharterFileHistory->ownerComment = $row->OwnerComment;		
		$projectCharterFileHistory->lineManagerComment = $row->LineManagerComment;		
		$projectCharterFileHistory->latestVersion = $row->LatestVersion;		
		$projectCharterFileHistory->setCommitted($row->Committed);
		return $projectCharterFileHistory;
	}
        
	/**
	 * Saves new project charter file history record.
	 * 
	 * @param ProjectCharterFileHistory $projectCharterFileHistory
	 * @return boolean
	 */
	public function save(ProjectCharterFileHistory $projectCharterFileHistory)
	{
        try
        {
            $array = array();
            $array['ProjectCharter_ID'] = $projectCharterFileHistory->getProjectCharterID();
            $array['Version'] = $projectCharterFileHistory->getVersion();
            $array['CreationDateTime'] = new DateTime;
            $array['FileName'] = $projectCharterFileHistory->getFileName();
            $array['Responsible'] = $projectCharterFileHistory->getResponsible();
            $array['OwnerComment'] = $projectCharterFileHistory->getOwnerComment();
            $array['LineManagerComment'] = $projectCharterFileHistory->getLineManagerComment();
            $array['LatestVersion'] = $projectCharterFileHistory->getLatestVersion();
            $array['Committed'] = $projectCharterFileHistory->getCommitted();
            
            $this->conn->insert('ProjectCharterFileHistory', $array)->execute();
			return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert of projectCharterFileHistory with ProjectCharter_ID = ".$projectCharterFileHistory->getProjectCharterID()." and Version = ".$projectCharterFileHistory->getVersion()." failed: ".$e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
        
	/**
	 * Updates existing project charter file history record.
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterFileHistory $projectCharterFileHistory
	 * @return boolean
	 */
	public function update(ProjectCharterFileHistory $projectCharterFileHistory)
	{
        try
        {
            $array = array();
            $array['FileName'] = $projectCharterFileHistory->getFileName();
            $array['OwnerComment'] = $projectCharterFileHistory->getOwnerComment();
            $array['LineManagerComment'] = $projectCharterFileHistory->getLineManagerComment();
            $array['LatestVersion'] = $projectCharterFileHistory->getLatestVersion();
            $array['Committed'] = $projectCharterFileHistory->getCommitted();
            $this->conn->update('ProjectCharterFileHistory', $array)->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterFileHistory->getProjectCharterID(), $projectCharterFileHistory->getVersion())->execute();
			return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of projectCharterFileHistory with ProjectCharter_ID = ".$projectCharterFileHistory->getProjectCharterID()." and Version = ".$projectCharterFileHistory->getVersion()." failed: ".$e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
        
	/**
	 * Deletes existing project charter file history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		try
		{
		    $result = $this->conn
		        ->delete('ProjectCharterFileHistory')
				->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)
				->execute();
			
			if ($result)
				Debugger::log("ProjectCharterFileHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of projectCharterFileHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of projectCharterFileHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." failed: ".$e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
        
	/**
	 * Gets project charter file history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return ProjectCharterFileHistory
	 */
	public function find($projectCharterID, $version)
	{
		$file_history_db_result = $this->conn
		    ->select('*')
			->from('ProjectCharterFileHistory')
			->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)
			->fetch();
		
		if ($file_history_db_result !== FALSE)
			return $this->load($file_history_db_result);
		else
			return NULL;
	}
        
	/**
	 * Gets all project charter history records by project charter id.
	 * 
	 * @param int $projectCharterID
	 * @return array Array of ProjectCharterFileHistory or empty array if error occured.
	 */
	public function findByProjectCharterID($projectCharterID)
	{
		$file_history_db_result = $this->conn
		    ->select('*')
			->from('ProjectCharterFileHistory')
			->where('[ProjectCharter_ID] = %i', $projectCharterID)
			->fetchAll();
		
		$array_file_history = array();
		if ($file_history_db_result !== FALSE)
		{
			foreach ($file_history_db_result as $file_history)
				$array_file_history[] = $this->load($file_history);
			return $array_file_history;
		}
		else
		    return $array_file_history;
	}
        
	/**
	 * Gets project charter file history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterFileHistory
	 */
	public function findLatestVersion($projectCharterID)
	{
		$file_history_db_result = $this->conn
            ->select('*')
            ->from('ProjectCharterFileHistory')
            ->where('[ProjectCharter_ID] = %i AND [LatestVersion] = 1', $projectCharterID)
            ->fetch();
		
		if ($file_history_db_result !== FALSE)
			return $this->load($file_history_db_result);
		else
			return NULL;
	}
        
	/**
	 * Initializes flag LatestVersion
	 * 
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID)
	{
		try
		{
			$array = array();
			$array['LatestVersion'] = 0;
			$this->conn
			    ->update('ProjectCharterFileHistory', $array)
				->where('[ProjectCharter_ID] = %i AND [Version] IS NOT NULL', $projectCharterID)
				->execute();
			
			return TRUE;
		}
		catch (DibiException $e)
		{
			Debugger::log("Initializing of LatestVersion in ProjectCharterFileHistory failed: ".$e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
}
