<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Project planning charter item repository access class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProjectPlanningCharterItemRepository extends Object
{
	/** @var IProjectPlanningCharterItemMapper */
	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

	public function __construct(IProjectPlanningCharterItemMapper $mapper)
	{
		$this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectPlanningCharterItem-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Planning Charter Item"));
	}
	
	/**
	 * Saves project planning charter item.
	 * 
     * @param ProjectPlanningCharterItem $projectPlanningCharterItem Project planning charter
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterItem $projectPlanningCharterItem)
	{
		if($this->mapper->save($projectPlanningCharterItem))
		{
			$this->multilog->log('Project Planning Charter Item with ID: ' . $projectPlanningCharterItem->ID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Updates project planning charter item.
	 * 
	 * @param ProjectPlanningCharterItem $projectPlanningCharterItem Project planning charter item
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterItem $projectPlanningCharterItem)
	{
		$oldObject = $this->find($projectPlanningCharterItem->ID);
		if($this->mapper->update($projectPlanningCharterItem))
		{
			$this->multilog->log('Project Planning Charter Item with ID: ' . $projectPlanningCharterItem->ID . ' updated. ' . $this->multilog->differences($oldObject, $projectPlanningCharterItem), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charter item.
	 * 
	 * @param int $projectPlanningCharterItem_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharterItem_id)
	{
		if($this->mapper->delete($projectPlanningCharterItem_id))
		{
			$this->multilog->log('Project planning charter item with ID: ' . $projectPlanningCharterItem_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Returns project planning charter item by its id.
	 * 
	 * @param int $id ProjectPlanningCharterItem ID.
	 * @return ProjectPlanningCharterItem
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Returns the count of rows with specified type
	 * 
	 * @param int $projectPlanningCharterID Project ID.
	 * @param array $types Type of the records to count, e.g. 'Person', 'OrganizationUnit'.
	 * @return int The count of rows with specified type.
	 */
	public function countRecordsByProjectPlanningCharterIDAndType($projectPlanningCharterID, $types)
	{
		return $this->mapper->countRecordsByProjectPlanningCharterIDAndType($projectPlanningCharterID, $types);
	}
	
	/**
	 * Returns rows related to Planning Project Charter ID with specified Type
	 * 
	 * @param int $projectPlanningCharterID Planning Project Charter ID.
	 * @param string $type Type of the records to find, e.g. 'Person', 'OrganizationUnit', 'Subcontractor', 'Other'.
	 * @return array $projectPlanningCharterItem_array
	 */
	public function findByProjectPlanningCharterIDAndType($projectPlanningCharterID, $type)
	{
		return $this->mapper->findByProjectPlanningCharterIDAndType($projectPlanningCharterID, $type);
	}
	
	/**
	 * Returns project planning charter item by project planning charter id.
	 * 
	 * @param int $projectPlanningCharterID ProjectPlanningCharter ID.
	 * @return array of ProjectPlanningCharterItem or empty array if not found
	 */
	public function findByProjectPlanningCharterID($projectPlanningCharterID)
	{
		return $this->mapper->findByProjectPlanningCharterID($projectPlanningCharterID);
	}

	/**
	 * Returns all project planning charter items.
	 * 
	 * @return array of ProjectPlanningCharterItem or empty array if not found
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * 
	 * @param type $projectPlanningCharterID
	 * @return type
	 */
	public function findAllPeopleByProjectPlanningCharterID($projectPlanningCharterID)
	{
		return $this->mapper->findAllPeopleByProjectPlanningCharterID($projectPlanningCharterID);
	}
	
	/**
	 * 
	 * @param type $projectPlanningCharterID
	 * @param type $orgUnitID
	 * @return type
	 */
	public function findAllOrgUnitsByProjectPlanningCharterIDByOrganizationUnit_ID($projectPlanningCharterID, $orgUnitID)
	{
		return $this->mapper->findAllOrgUnitsByProjectPlanningCharterIDByOrganizationUnit_ID($projectPlanningCharterID, $orgUnitID);
	}
}
