<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing projectCharterSummaryFileHistory from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterSummaryFileHistoryDibiMapper extends DibiMapper implements IProjectCharterSummaryFileHistoryMapper
{
	/**
	 * Creates ProjectCharterSummaryFileHistory instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return ProjectCharterSummaryFileHistory 
	 */
	private function load(DibiRow $row)
	{
		$projectCharterSummaryFileHistory = new ProjectCharterSummaryFileHistory();
		$projectCharterSummaryFileHistory->projectCharterID = $row->ProjectCharter_ID;
		$projectCharterSummaryFileHistory->version = $row->Version;
		$projectCharterSummaryFileHistory->creationDateTime = $row->CreationDateTime;
		$projectCharterSummaryFileHistory->fileName = $row->FileName;
		$projectCharterSummaryFileHistory->responsible = $row->Responsible;			
		$projectCharterSummaryFileHistory->ownerComment = $row->OwnerComment;		
		$projectCharterSummaryFileHistory->lineManagerComment = $row->LineManagerComment;
		$projectCharterSummaryFileHistory->latestVersion = $row->LatestVersion;		
		$projectCharterSummaryFileHistory->setCommitted($row->Committed);
		return $projectCharterSummaryFileHistory;
	}
        
	/**
	 * Saves new project charter file history record.
	 * 
	 * @param ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory
	 * @return boolean
	 */
	public function save(ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory)
	{
        try
        {
            $array = array();
            $array['ProjectCharter_ID'] = $projectCharterSummaryFileHistory->getProjectCharterID();
            $array['Version'] = $projectCharterSummaryFileHistory->getVersion();
            $array['CreationDateTime'] = new DateTime;
            $array['FileName'] = $projectCharterSummaryFileHistory->getFileName();
            $array['Responsible'] = $projectCharterSummaryFileHistory->getResponsible();
            $array['OwnerComment'] = $projectCharterSummaryFileHistory->getOwnerComment();
            $array['LineManagerComment'] = $projectCharterSummaryFileHistory->getLineManagerComment();
            $array['LatestVersion'] = $projectCharterSummaryFileHistory->getLatestVersion();
            $array['Committed'] = $projectCharterSummaryFileHistory->getCommitted();
            
            $this->conn->insert('ProjectCharterSummaryFileHistory', $array)->execute();
			return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert of projectCharterSummaryFileHistory with ProjectCharter_ID = ".$projectCharterSummaryFileHistory->getProjectCharterID()." and Version = ".$projectCharterSummaryFileHistory->getVersion()." failed: ".$e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
        
	/**
	 * Updates existing project charter file history record.
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory
	 * @return boolean
	 */
	public function update(ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory)
	{
        try
        {
            $array = array();
            $array['FileName'] = $projectCharterSummaryFileHistory->getFileName();
            $array['OwnerComment'] = $projectCharterSummaryFileHistory->getOwnerComment();
            $array['LineManagerComment'] = $projectCharterSummaryFileHistory->getLineManagerComment();
            $array['LatestVersion'] = $projectCharterSummaryFileHistory->getLatestVersion();
            $array['Committed'] = $projectCharterSummaryFileHistory->getCommitted();
            $this->conn->update('ProjectCharterSummaryFileHistory', $array)->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterSummaryFileHistory->getProjectCharterID(), $projectCharterSummaryFileHistory->getVersion())->execute();
			return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of projectCharterSummaryFileHistory with ProjectCharter_ID = ".$projectCharterSummaryFileHistory->getProjectCharterID()." and Version = ".$projectCharterSummaryFileHistory->getVersion()." failed: ".$e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
        
	/**
	 * Deletes existing project charter file history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		try
		{
		    $result = $this->conn
		        ->delete('ProjectCharterSummaryFileHistory')
				->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)
				->execute();
			
			if ($result)
				Debugger::log("ProjectCharterSummaryFileHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of projectCharterSummaryFileHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of projectCharterSummaryFileHistory with ProjectCharter_ID = ".$projectCharterID." and Version = ".$version." failed: ".$e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
        
	/**
	 * Gets project charter file history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return ProjectCharterSummaryFileHistory
	 */
	public function find($projectCharterID, $version)
	{
		$file_history_db_result = $this->conn
		    ->select('*')
			->from('ProjectCharterSummaryFileHistory')
			->where('[ProjectCharter_ID] = %i AND [Version] = %i', $projectCharterID, $version)
			->fetch();
		
		if ($file_history_db_result !== FALSE)
			return $this->load($file_history_db_result);
		else
			return NULL;
	}
        
	/**
	 * Gets all project charter history records by project charter id.
	 * 
	 * @param int $projectCharterID
	 * @return array Array of ProjectCharterSummaryFileHistory or empty array if error occured.
	 */
	public function findByProjectCharterID($projectCharterID)
	{
		$file_history_db_result = $this->conn
		    ->select('*')
			->from('ProjectCharterSummaryFileHistory')
			->where('[ProjectCharter_ID] = %i', $projectCharterID)
			->fetchAll();
		
		$array_file_history = array();
		if ($file_history_db_result !== FALSE)
		{
			foreach ($file_history_db_result as $file_history)
				$array_file_history[] = $this->load($file_history);
			return $array_file_history;
		}
		else
		    return $array_file_history;
	}
        
	/**
	 * Gets project charter file history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterSummaryFileHistory
	 */
	public function findLatestVersion($projectCharterID)
	{
		$file_history_db_result = $this->conn
            ->select('*')
            ->from('ProjectCharterSummaryFileHistory')
            ->where('[ProjectCharter_ID] = %i AND [LatestVersion] = 1', $projectCharterID)
            ->fetch();
		
		if ($file_history_db_result !== FALSE)
			return $this->load($file_history_db_result);
		else
			return NULL;
	}
        
	/**
	 * Initializes flag LatestVersion
	 * 
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID)
	{
		try
		{
			$array = array();
			$array['LatestVersion'] = 0;
			$this->conn
			    ->update('ProjectCharterSummaryFileHistory', $array)
				->where('[ProjectCharter_ID] = %i AND [Version] IS NOT NULL', $projectCharterID)
				->execute();
			
			return TRUE;
		}
		catch (DibiException $e)
		{
			Debugger::log("Initializing of LatestVersion in ProjectCharterSummaryFileHistory failed: ".$e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
}
