<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing newly found projects stubs in import from SAP
 * via dibi library in db.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectFoundDibiMapper extends DibiMapper implements IProjectFoundMapper
{

	/**
	 * Creates ProjectFoundInfo instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @param ProjectFoundInfo If instance given. Properties are populated.
	 * @return	ProjecFoundInfo 
	 */
    private function loadInfo(DibiRow $row, ProjectFoundInfo $project = NULL)
    {
		if ($project == NULL)
			$project = new ProjectFoundInfo();
		$project->SAP_ID = $row->SAP_ID;
		$project->Name = $row->Name;
		return $project;
    }	

	/**
	 * Creates ProjectFoundPlan instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectFoundPlan 
	 */
    private function loadPlan(DibiRow $row)
    {
		$project_plan = new ProjectFoundPlan($row->Year);
		$project_plan->SerializedPlan = $row->Plan;
		return $project_plan;
    }	

	/**
	 * Saves found project stub with plans.
	 * 
	 * @param ProjectFound $project
	 * @return boolean
	 */
	public function save(ProjectFound $project)
	{
        try
        {
            $array = array();
            $array['SAP_ID'] = $project->getSAPID();
            $array['Name'] = $project->getName();
			if ($this->findInfoBySAPID($project->getSAPID()) != NULL)
	            $project_db_result = $this->conn->update('ProjectFound', $array)->where('[SAP_ID] = %s', $project->getSAPID())->execute();
			else
	            $project_db_result = $this->conn->insert('ProjectFound', $array)->execute();
			$plans = $project->getPlans();
			foreach ($plans as $plan)
			{
				$plan_array = array();
				$plan_array['LastUpdated'] = $plan->getLastFoundDate();
				$plan_array['Plan'] = $plan->getSerializedPlan();
				
				$project_plan_result = $this->conn->query('SELECT * FROM [ProjectFoundPlan] WHERE [SAP_ID] = %s AND [Year]= %u', $project->getSAPID(), $plan->getYear())->fetch();
				if ($project_plan_result !== FALSE)
		            $update_db_result = $this->conn->update('ProjectFoundPlan', $plan_array)->where('[SAP_ID] = %s AND [Year]= %u', $project->getSAPID(), $plan->getYear())->execute();
				else
				{
					$plan_array['SAP_ID'] = $project->getSAPID();
					$plan_array['Year'] = $plan->getYear();
		            $update_db_result = $this->conn->insert('ProjectFoundPlan', $plan_array)->execute();
				}
			}
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving found project stub with SAP id = ".$project->getSAPID()." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
    
	/**
	 * Updates info for project stub.
	 * 
	 * @param ProjectFoundInfo $project
	 * @return boolean
	 */
	public function updateInfo(ProjectFoundInfo $project)
	{
        try
        {
            $array = array();
            $array['Name'] = $project->getName();
            $project_found_db_result = $this->conn->update('ProjectFound', $array)->where('[SAP_ID] = %s', $project->getSAPID())->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of found project stub with SAP id = ".$project->getSAPID()." failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Finds project stub by SAP id including plans if any.
	 * 
	 * @param string $sap_id SAP ID.
	 * @return	ProjectFound or NULL if not found.
	 */
	public function findBySAPID($sap_id)
	{
		$project_db_result = $this->conn->query('SELECT * FROM [ProjectFound] WHERE [SAP_ID] = %s AND [Blacklisted]=0', $sap_id)->fetch();
		if ($project_db_result !== FALSE)
		{
			$project = new ProjectFound();
			if ($this->loadInfo($project_db_result, $project))
			{
				$project_plans_db_result = $this->conn->query('SELECT * FROM [ProjectFoundPlan] WHERE [SAP_ID] = %s', $sap_id)->fetchAll();
				if ($project_plans_db_result !== FALSE)
				{
					foreach ($project_plans_db_result as $plan_db_result)
						$project->setPlan($this->loadPlan($plan_db_result));
					return $project;
				}
			}
			else
				return NULL;
		}
		else
			return NULL;
	}
	
	/**
	 * Finds info for project stub by SAP id.
	 * 
	 * @param string $sap_id SAP ID.
	 * @return	ProjectFoundInfo or NULL if not found.
	 */
	public function findInfoBySAPID($sap_id)
	{
		$project_db_result = $this->conn->query('SELECT * FROM [ProjectFound] WHERE [SAP_ID] = %s AND [Blacklisted]=0', $sap_id)->fetch();
		if ($project_db_result !== FALSE)
			return $this->loadInfo($project_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all infos for found project stubs.
	 * 
	 * @return array Array of ProjectFoundInfo where SAP ID is key.
	 */
	public function findAllInfos()
	{
        $projects_array = array();
		$projects_db_result = $this->conn->query('SELECT * FROM [ProjectFound] WHERE [Blacklisted]=0 ORDER BY [Name] ASC')->fetchAll();
		foreach ($projects_db_result as $project)
        {
			$projects_array[$project->SAP_ID] = $this->loadInfo($project);
        }
        return $projects_array;
	}
	
	/**
	 * Finds all infos for found project stubs.
	 * 
	 * @return array Array of ProjectFoundInfo where SAP ID is key.
	 */
	public function findAllBlacklistedInfos()
	{
        $projects_array = array();
		$projects_db_result = $this->conn->query('SELECT * FROM [ProjectFound] WHERE [Blacklisted]=1 ORDER BY [Name] ASC')->fetchAll();
		foreach ($projects_db_result as $project)
        {
			$projects_array[$project->SAP_ID] = $this->loadInfo($project);
        }
        return $projects_array;
	}
	
	/**
	 * Deletes found project stub by SAP ID.
	 * 
	 * @param string $id found project stubs SAP ID.
	 * @return boolean Return true if delete was successfull, otherwise returns false.
	 */
	public function delete($sap_id)
	{
        try
        {
            $project_db_result = $this->conn->delete('ProjectFound')->where('[SAP_ID] = %s', $sap_id)->execute();
            //Debugger::log(print_r(dibi::$sql, true), Debugger::DEBUG);
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Found project stub delete with SAP ID = $sap_id failed. ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Puts project stub identified by SAP ID on blacklisted.
	 * 
	 * @param string $id found project stubs SAP ID.
	 * @return	boolean
	 */
	public function putOnBlacklist($sap_id)
	{
        try
        {
            $array = array();
            $array['Blacklisted'] = 1;
            $db_result = $this->conn->update('ProjectFound', $array)->where('[SAP_ID] = %s', $sap_id)->execute();
            Debugger::log("Project stub with with SAP id = ".$sap_id." was blaclisted.", Debugger::INFO);
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Failed to blacklist project stub with with SAP id = ".$sap_id.": ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Returns whether project identified by SAP ID is blacklisted.
	 * 
	 * @return	boolean
	 */
	public function isBlacklisted($sap_id)
	{
		$project_db_result = $this->conn->query('SELECT [SAP_ID],[Blacklisted] FROM [ProjectFound] WHERE [SAP_ID] = %s AND [Blacklisted]=1', $sap_id)->fetch();
		if ($project_db_result !== FALSE)
			return true;
		else
			return false;
	}
	
	/**
	 * Deletes found project stubs and plans and leaves infos only for blacklisted.
	 * 
	 * @return boolean Return true if delete was successfull, otherwise returns false.
	 */
	public function deleteAllButBlacklisted()
	{
		try
		{
			$this->conn->query('TRUNCATE [ProjectFoundPlan]');
			$this->conn->query('DELETE FROM [ProjectFound] WHERE [Blacklisted]=0');
			return true;
		}
        catch (DibiException $e)
        {
            Debugger::log("Failed cleanup ProjectFoundPlan and ProjectFound tables: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	
}
