<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for distribution list report.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class DistributionListReportService extends Object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;
    /** @var string Report name */
    private $name;
    /** @var array List of person eids */
    private $eid_list;
    /** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
    
    /**
	 * Creates service for export of distribution list
	 * @param DIContainer $context Application context.
	 * @param string $name Report name.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, $name, $eid_list, $uiFreezeToken = NULL)
    {
        $this->context = $context;
		$this->name = strtolower($name);
		$this->eid_list = array_unique($eid_list);
		$this->uiFreezeToken = $uiFreezeToken;
    }
    
    /**
     * Generates distribution list report as a XLSX (MS-EXCEL file)
	 * File can be downloaded through browser
	 * @throws DistributionListReportServiceException
     */
    public function runDistributionListReportGeneration()
    {
		$objPHPExcel = new PHPExcel();
		
        $exec_time = 600;
	    if (!ini_set("max_execution_time", $exec_time))
            throw new DistributionListReportServiceException('Unable to setup maximum execution time to '.$exec_time.'.');
        
        // Initialization of person list
        $person_list = array();
        
        foreach ($this->eid_list as $eid)
        {
            $person = $this->context->personRepository->find($eid);
            
            if ($person !== NULL)
                $person_list[] = $person;
        }
        
        usort($person_list, array($this, 'compareSurname'));
        
		$sheet = $objPHPExcel->getActiveSheet();
        $row_iterator = $sheet->getRowIterator();
        $cell_iterator = $row_iterator->current()->getCellIterator();
		
		// Set column width
		$sheet->getColumnDimension('A')->setWidth(15);
        $sheet->getColumnDimension('B')->setWidth(28);
		$sheet->getColumnDimension('C')->setWidth(28);
		$sheet->getColumnDimension('D')->setWidth(55);
        
        // Set style
        $style = new stdClass();
        $style->header_color = 'FFFFFF';
        $style->header_background = '2E619A';
        $style->first_header_cell = 'A';
        $style->last_header_cell = 'D';
        
        $row = $cell_iterator->current()->getRow();
		$sheet->getStyle($style->first_header_cell . $row . ':' . $style->last_header_cell . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($style->header_background);
        $sheet->getStyle($style->first_header_cell . $row . ':' . $style->last_header_cell . $row)->getFont()->getColor()->setARGB($style->header_color);
        $sheet->getStyle($style->first_header_cell . $row . ':' . $style->last_header_cell . $row)->getFont()->setBold(TRUE);
		
		// Column headers
		$cell_iterator->current()->setValue('EID');
        $cell_iterator->next();
        $cell_iterator->current()->setValue('Surname');
        $cell_iterator->next();
        $cell_iterator->current()->setValue('Name');
        $cell_iterator->next();
        $cell_iterator->current()->setValue('E-mail Address');
        
        $row_iterator->next();
        $cell_iterator = $row_iterator->current()->getCellIterator();
        
		foreach ($person_list as $person)
        {
            $cell_iterator->current()->setValue($person->ID);
            $cell_iterator->next();
            $cell_iterator->current()->setValue($person->Surname);
            $cell_iterator->next();
            $cell_iterator->current()->setValue($person->Firstname);
            $cell_iterator->next();
            $cell_iterator->current()->setValue(strtolower($person->Email));
            
            $row_iterator->next();
            $cell_iterator = $row_iterator->current()->getCellIterator();
        }
        
		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);
        
        ob_clean();
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename=dl-'.$this->name.'.xlsx');
        header('Cache-Control: max-age=0');
        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
        $objWriter->save("php://output");
		
		exit();
    }
    
    /**
     * Helper for sorting array of Person
     * @param Person $person_1
     * @param Person $person_2
     * @return int
     */
    private function compareSurname($person_1, $person_2)
    {
        if (!($person_1 instanceof Person) || !($person_2 instanceof Person))
            return 0;
        
        return strcmp($person_1->Surname, $person_2->Surname);
    }
}

class DistributionListReportServiceException extends Exception {}