<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for project charter row data report.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ProjectCharterRowDataReportService extends object
{
    /** @var DIContainer DI context with application repositories access */
    private $context;

	/** @var ILogger Logging facility */
    private $logger;
	
	/** @var int Year to generation */
    private $year;
	
	/** @var String Value for freeze user interface during report generation. */
    private $uiFreezeToken;
	
    /** @var string Path to storage project charter row data report files. */
    private $projectCharterRowDataReport_storage_dir;
	
	/**
	 * Creates service forproject charter row data report.
	 *  
	 * @param $context Application context.
	 * @param ILogger Object for logging the import progress.
	 */
    public function  __construct(DIContainer $context, ILogger $logger, $year, $uiFreezeToken = null)
    {
        $this->context = $context;
		$this->logger = $logger;
		$this->year = $year;
		$this->uiFreezeToken = $uiFreezeToken;
		$this->projectCharterRowDataReport_storage_dir = Environment::getConfig('projectCharterRowDataReport_storage_dir');
    }
	
    /**
	 * 
     * Generates project charter row data report as a XLSX (MS-EXCEL file).
	 * File can be downloaded through browser and also file is saved to the public/ProjectCharterRowDataReport folder.
	 * 
     */
    public function runProjectCharterRowDataReportGeneration()
    {
		$objPHPExcel = new PHPExcel();
		
        $memory_limit = "512M";
        if (!ini_set("memory_limit", $memory_limit)) 
		{
			$this->flashMessage('Setup of memory limit failed. Contact administrator.', 'error');
			trigger_error('Unable to setup memory limit to ' . $memory_limit . '.', E_USER_ERROR);
		}

        $exec_time = 600;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}
		
		$lock = $this->context->createLock('ProjectCharter_RowDataReport', $exec_time);
		if (!$lock->lock())
		{
			$this->logger->log('Project Charter Row Data report generation failed - active lock detected: ' . date('m-d-Y H:i:s'), ILogger::INFO);
			throw new PCHRDReportServiceException("Generation of Project Charter Row Data report is already running.");
		}
		$this->logger->log('Project Charter Row Data report generation start: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		
		$sheet = $objPHPExcel->getActiveSheet();        
        $rowIterator = $sheet->getRowIterator();
        $cellIterator = $rowIterator->current()->getCellIterator();        
        // Header     
		$cellIterator->current()->setValue("SAP_ID");
        $cellIterator->next();
        $cellIterator->current()->setValue("Name");
        $cellIterator->next();
        $cellIterator->current()->setValue("CostShare");
        $cellIterator->next();
        $cellIterator->current()->setValue("LineManagerEID");
        $cellIterator->next();
        $cellIterator->current()->setValue("LineManagerName");
        $cellIterator->next();
        $cellIterator->current()->setValue("LineManagerSurname");
        $cellIterator->next();
        $cellIterator->current()->setValue("OwnerEID");
        $cellIterator->next();
        $cellIterator->current()->setValue("OwnerName");
        $cellIterator->next();
        $cellIterator->current()->setValue("OwnerSurname");
        $cellIterator->next();
        $cellIterator->current()->setValue("Status");
        $cellIterator->next();
        $cellIterator->current()->setValue("CreationDateTime");
        $cellIterator->next();
        $cellIterator->current()->setValue("Customer");
        $cellIterator->next();
        $cellIterator->current()->setValue("Program");
        $cellIterator->next();
        $cellIterator->current()->setValue("Call");        
        $cellIterator->next();
        $cellIterator->current()->setValue("Responsibility");
        $cellIterator->next();
        $cellIterator->current()->setValue("ECLicence");
        $cellIterator->next();
        $cellIterator->current()->setValue("Budget_Q1");
        $cellIterator->next();
        $cellIterator->current()->setValue("Budget_Q2");
        $cellIterator->next();
        $cellIterator->current()->setValue("Budget_Q3");
        $cellIterator->next();
        $cellIterator->current()->setValue("Budget_Q4");
        $cellIterator->next();
        $cellIterator->current()->setValue("Budget_SUM");
        $cellIterator->next();
        $cellIterator->current()->setValue("MAT_Q1");
        $cellIterator->next();
        $cellIterator->current()->setValue("MAT_Q2");
        $cellIterator->next();
        $cellIterator->current()->setValue("MAT_Q3");
        $cellIterator->next();
        $cellIterator->current()->setValue("MAT_Q4");
        $cellIterator->next();
        $cellIterator->current()->setValue("MAT_SUM");
        $cellIterator->next();
        $cellIterator->current()->setValue("TRV_Q1");
        $cellIterator->next();
        $cellIterator->current()->setValue("TRV_Q2");
        $cellIterator->next();
        $cellIterator->current()->setValue("TRV_Q3");
        $cellIterator->next();
        $cellIterator->current()->setValue("TRV_Q4");
        $cellIterator->next();
        $cellIterator->current()->setValue("TRV_SUM");
        $cellIterator->next();
		$cellIterator->current()->setValue("TOTAL_SUM");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverRole");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverEID");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverName");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverSurname");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverEmail");
        $cellIterator->next();
		$cellIterator->current()->setValue("ApproverDecision");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverDecisionDateTime");
		$cellIterator->next();
		$cellIterator->current()->setValue("ApproverComment");
		$cellIterator->next();
		$cellIterator->current()->setValue("AutoApproved");
		$cellIterator->next();
		for($i = 0; $i < 5; $i++)
		{
			$cellIterator->current()->setValue(ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_HONEYWELL_AC . " " . ($this->year + $i));
			$cellIterator->next();
		}
		for($i = 0; $i < 5; $i++)
		{
			$cellIterator->current()->setValue(ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_OTHER_AIRCRAFT . " " . ($this->year + $i));
			$cellIterator->next();
		}
		$cellIterator->current()->setValue("Generation time");
		$row = $cellIterator->current()->getRow();
		$col = $cellIterator->current()->getColumn();
		$sheet->setCellValue($col . ($row + 1), date('m-d-Y H:i:s'));
        $rowIterator->next();
        $cellIterator = $rowIterator->current()->getCellIterator();
		
		$projectCharters = $this->context->projectCharterRepository->findAll($this->year);
		foreach($projectCharters as $charter)
		{
			// Data generation
			$lineManager = $this->context->personRepository->find($charter->LineManager);
			$owner = $this->context->personRepository->find($charter->Owner);
			$customer = $this->context->customerRepository->find($charter->customerID);
			$program = $this->context->customerRepository->find($charter->programID);
			$call = $this->context->customerRepository->find($charter->callID);
			$responsibility = $this->context->responsibilityRepository->find($charter->responsibilityID);
			$ecLicence = $this->context->ecLicenceRepository->find($charter->ECLicenceID);
			$projectCharterBudget = $this->context->projectCharterBudgetRepository->findProjectCharterBudgetByProjectCharterID($charter->ID);
			$budgetArray = $trvArray = $matArray = array('budgetQ1' => 0, 'budgetQ2' => 0, 'budgetQ3' => 0, 'budgetQ4' => 0, 'budgetTotal' => 0);
			foreach($projectCharterBudget as $budget)
			{
				if($budget->resourceType == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_HOURS)
				{
					$budgetArray['budgetQ1'] += ($budget->BudgetQ1 * $budget->Rate);
					$budgetArray['budgetQ2'] += ($budget->BudgetQ2 * $budget->Rate);
					$budgetArray['budgetQ3'] += ($budget->BudgetQ3 * $budget->Rate);
					$budgetArray['budgetQ4'] += ($budget->BudgetQ4 * $budget->Rate);
					$budgetArray['budgetTotal'] += (($budget->BudgetQ1 + $budget->BudgetQ2 + $budget->BudgetQ3 + $budget->BudgetQ4) * $budget->Rate);
				}
				if($budget->resourceType == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_SUBCONTRACTING)
				{
					$budgetArray['budgetQ1'] += $budget->BudgetQ1;
					$budgetArray['budgetQ2'] += $budget->BudgetQ2;
					$budgetArray['budgetQ3'] += $budget->BudgetQ3;
					$budgetArray['budgetQ4'] += $budget->BudgetQ4;
					$budgetArray['budgetTotal'] += ($budget->BudgetQ1 + $budget->BudgetQ2 + $budget->BudgetQ3 + $budget->BudgetQ4);
				}
				if($budget->resourceType == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_MATERIAL)
				{
					$matArray['budgetQ1'] = $budget->BudgetQ1;
					$matArray['budgetQ2'] = $budget->BudgetQ2;
					$matArray['budgetQ3'] = $budget->BudgetQ3;
					$matArray['budgetQ4'] = $budget->BudgetQ4;
					$matArray['budgetTotal'] = $budget->BudgetQ1 + $budget->BudgetQ2 + $budget->BudgetQ3 + $budget->BudgetQ4;
				}
				if($budget->resourceType == ProjectCharterBudget::PROJECT_CHARTER_BUDGET_TRAVEL)
				{
					$trvArray['budgetQ1'] = $budget->BudgetQ1;
					$trvArray['budgetQ2'] = $budget->BudgetQ2;
					$trvArray['budgetQ3'] = $budget->BudgetQ3;
					$trvArray['budgetQ4'] = $budget->BudgetQ4;
					$trvArray['budgetTotal'] = $budget->BudgetQ1 + $budget->BudgetQ2 + $budget->BudgetQ3 + $budget->BudgetQ4;
				}
			}
			$totalBudget = $budgetArray['budgetTotal'] + $matArray['budgetTotal'] + $trvArray['budgetTotal'];
			$latestVersionFinalApprovers = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($charter->ID);
			// generate report
			if($latestVersionFinalApprovers)
			{
				$finalApprovers = $this->context->projectCharterApproverRepository->findAllApprover($charter->ID, $latestVersionFinalApprovers->Version);
				$iterationNumber = count($finalApprovers);
			}
			else
				$iterationNumber = 1;
			for($i = 1; $i <= $iterationNumber; $i++)
			{
				$cellIterator->current()->setValue($charter->SAP_ID);
				$cellIterator->next();
				$cellIterator->current()->setValue($charter->Name);
				$cellIterator->next();
				$cellIterator->current()->setValue($charter->CostShare);
				$cellIterator->next();
				$cellIterator->current()->setValue($charter->LineManager);
				$cellIterator->next();
				if($lineManager)
				{
					$cellIterator->current()->setValue($lineManager->Firstname);
					$cellIterator->next();
					$cellIterator->current()->setValue($lineManager->Surname);
					$cellIterator->next();
				}
				else
				{
					$cellIterator->next();
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($charter->Owner);
				$cellIterator->next();
				if($owner)
				{
					$cellIterator->current()->setValue($owner->Firstname);
					$cellIterator->next();
					$cellIterator->current()->setValue($owner->Surname);
					$cellIterator->next();
				}
				else
				{
					$cellIterator->next();
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($charter->Status);
				$cellIterator->next();
				$cellIterator->current()->setValue(date_format($charter->getCreationDateTime(), 'Y-m-d'));
				$cellIterator->next();
				if($customer)
					$cellIterator->current()->setValue($customer->Name);
				$cellIterator->next();
				if($program)
					$cellIterator->current()->setValue($program->Name);
				$cellIterator->next();
				if($call)
					$cellIterator->current()->setValue($call->Name);
				$cellIterator->next();
				if($responsibility)
					$cellIterator->current()->setValue($responsibility->Name);
				$cellIterator->next();
				if($ecLicence)
					$cellIterator->current()->setValue($ecLicence->Name);
				$cellIterator->next();
				$cellIterator->current()->setValue($budgetArray['budgetQ1']);
				$cellIterator->next();
				$cellIterator->current()->setValue($budgetArray['budgetQ2']);
				$cellIterator->next();
				$cellIterator->current()->setValue($budgetArray['budgetQ3']);
				$cellIterator->next();
				$cellIterator->current()->setValue($budgetArray['budgetQ4']);
				$cellIterator->next();
				$cellIterator->current()->setValue($budgetArray['budgetTotal']);
				$cellIterator->next();
				$cellIterator->current()->setValue($matArray['budgetQ1']);
				$cellIterator->next();
				$cellIterator->current()->setValue($matArray['budgetQ2']);
				$cellIterator->next();
				$cellIterator->current()->setValue($matArray['budgetQ3']);
				$cellIterator->next();
				$cellIterator->current()->setValue($matArray['budgetQ4']);
				$cellIterator->next();
				$cellIterator->current()->setValue($matArray['budgetTotal']);
				$cellIterator->next();
				$cellIterator->current()->setValue($trvArray['budgetQ1']);
				$cellIterator->next();
				$cellIterator->current()->setValue($trvArray['budgetQ2']);
				$cellIterator->next();
				$cellIterator->current()->setValue($trvArray['budgetQ3']);
				$cellIterator->next();
				$cellIterator->current()->setValue($trvArray['budgetQ4']);
				$cellIterator->next();
				$cellIterator->current()->setValue($trvArray['budgetTotal']);
				$cellIterator->next();
				$cellIterator->current()->setValue($totalBudget);
				$cellIterator->next();
				if($latestVersionFinalApprovers)
				{
					$approver = current($finalApprovers);
					next($finalApprovers);
					$cellIterator->current()->setValue($approver->Role);
					$cellIterator->next();
					$cellIterator->current()->setValue($approver->ApproverEID);
					$cellIterator->next();
					$cellIterator->current()->setValue($approver->Surname);
					$cellIterator->next();
					$cellIterator->current()->setValue($approver->Name);
					$cellIterator->next();
					$cellIterator->current()->setValue($approver->Email);
					$cellIterator->next();
					if($approver->Approval == ProjectCharterApprover::DECISION_APPROVED)
						$result = 'Approved';
					if($approver->Approval == ProjectCharterApprover::DECISION_DISAPPROVED)
						$result = 'Disapproved';
					if($approver->Approval == ProjectCharterApprover::DECISION_PENDING)
						$result = 'Pending';
					$cellIterator->current()->setValue($result);
					$cellIterator->next();
					if($approver->DecisionDateTime)
						$cellIterator->current()->setValue(date_format($approver->DecisionDateTime, 'Y-m-d'));
					$cellIterator->next();
					$cellIterator->current()->setValue($approver->Comment);
					$cellIterator->next();
					$cellIterator->current()->setValue($approver->AutoApproved);
					$cellIterator->next();
				}
				else
				{
					for($j = 1; $j < 10; $j++)
						$cellIterator->next();
				}
				
				$checklistItems = $this->context->projectCharterChecklistRepository->findByProjectCharterID($charter->ID);
				if(isset($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_HONEYWELL_AC]) 
						&& isset($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_HONEYWELL_AC]->NeededArray))
				{
					for($j = 0; $j < 5; $j++)
					{
						if(isset($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_HONEYWELL_AC]->NeededArray[$j+1]))
						{
							$cellIterator->current()->setValue(($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_HONEYWELL_AC]->NeededArray[$j+1]) ? 'Yes' : 'No');
							$cellIterator->next();
						}
						else
						{
							$cellIterator->next();
						}
					}
				}
				else
				{
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
				}
				if(isset($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_OTHER_AIRCRAFT])
						&& isset($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_OTHER_AIRCRAFT]->NeededArray))
				{
					for($j = 0; $j < 5; $j++)
					{
						if(isset($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_OTHER_AIRCRAFT]->NeededArray[$j+1]))
						{
							$cellIterator->current()->setValue(($checklistItems[ProjectCharterChecklist::PROJECT_CHARTER_CHECKLIST_TITLE_OTHER_AIRCRAFT]->NeededArray[$j+1]) ? 'Yes' : 'No');
							$cellIterator->next();
						}
						else
						{
							$cellIterator->next();
						}
					}
				}
				else
				{
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
					$cellIterator->current()->setValue('UNKNOWN');
					$cellIterator->next();
				}

				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
		}
		
		if(!is_null($this->uiFreezeToken))
			setcookie('fileDownloadToken', $this->uiFreezeToken);

        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		$objWriter->setPreCalculateFormulas(true);
        if (!file_exists($this->projectCharterRowDataReport_storage_dir)) 
          mkdir($this->projectCharterRowDataReport_storage_dir, 0777, TRUE);

		$objWriter->save($this->projectCharterRowDataReport_storage_dir . 'ProjectCharterRowDataReport_' . $this->year . '.xlsx');
		
		$this->logger->log('Project Charter Row Data report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		$lock->unlock();
    }
}

class PCHRDReportServiceException extends Exception {}

