<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing STRAP Advisor from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AdvisorDibiMapper extends DibiMapper implements IAdvisorMapper
{
	/**
	 * Creates Advisor instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	Advisor 
	 */
    private function load(DibiRow $row)
    {
		$advisor = new Advisor();
		$advisor->ID = $row->Advisor_ID;
		$advisor->Parent_ID = $row->Parent_ID;
		$advisor->Name = $row->Name;
        $advisor->Type = $row->Type;
		return $advisor;
    }
	
	/**
	 * Saves new STRAP Advisor.
	 * 
	 * @param Advisor $advisor
	 * @return boolean
	 */
	public function save(Advisor $advisor)
	{
        try
        {
            $array = array();
			$array['Advisor_ID'] = $advisor->ID;
			$array['Parent_ID'] = $advisor->Parent_ID;
			$array['Name'] = $advisor->Name;
			$array['Type'] = $advisor->Type;
            $this->conn->insert('Advisor', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new STRAP Item Advisor by user = " . $advisor->ID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing STRAP Advisor.
	 * 
	 * @param Advisor $advisor
	 * @return boolean
	 */
	public function update(Advisor $advisor)
	{
        try
        {
            $array = array();
			$array['Advisor_ID'] = $advisor->ID;
			$array['Parent_ID'] = $advisor->Parent_ID;
			$array['Name'] = $advisor->Name;
			$array['Type'] = $advisor->Type;
            $this->conn->update('Advisor', $array)->where('[Advisor_ID] = %i', $advisor->ID)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of STRAP Advisor with ID = " . $advisor->ID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing STRAP Advisor.
	 * 
	 * @param int $id
	 * @return boolean
	 */
	public function delete($id)
	{
		try
		{
		    $result = $this->conn->delete('Advisor')->where('[Advisor_ID] = %i', $id)->execute();
			if ($result)
				Debugger::log("Strap Advisor with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of STRAP Advisor with ID = " . $id . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of STRAP Advisor with ID = " . $id . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets STRAP Advisor by id.
	 * 
	 * @param int $id
	 * @return Advisor or null if not found
	 */
	public function find($id)
	{
		$advisor_db_result = $this->conn->select('*')->from('Advisor')->where('[Advisor_ID] = %i', $id)->fetch();
		if ($advisor_db_result !== FALSE)
			return $this->load($advisor_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets all STRAP Advisors by type.
	 * @param String $type
	 * 
	 * @return array Array of Advisor or null if not found.
	 */
	public function findAllByType($type)
	{
        $advisor_array = array();
		$advisor_db_result = $this->conn->select('*')->from('Advisor')->where('[Type] = %s', $type)->orderBy('Name')->fetchAll();        
		if ($advisor_db_result !== FALSE)			
		{
			foreach ($advisor_db_result as $row)
				$advisor_array[$row->Advisor_ID] = $this->load($row);
			return $advisor_array;
		}
		else
			return null;
	}
	
    /**
	 * Finds all children nodes of advisor by id.
	 * 
	 * @param int $id advisor ID.
	 * @return	array of Advisor where id is key or NULL if not found.
	 */
	public function findAllChildren($id)
	{
        $orgUnit_array = array();
		$orgUnit_db_result = $this->conn->select('*')->from('Advisor')->where('[Parent_ID] = %i', $id)->fetchAll();        
        if ($orgUnit_db_result !== FALSE)
        {
            foreach ($orgUnit_db_result as $orgUnit)
                $orgUnit_array[$orgUnit->Advisor_ID] = $this->load($orgUnit);
            return $orgUnit_array;
        }
        else
            return null;
	}
}
