<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing CustomerProgramCall from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class CustomerProgramCallDibiMapper extends DibiMapper implements ICustomerProgramCallMapper
{
	/**
	 * Creates CustomerProgramCall instance from DibiRow.
	 * @param DibiRow $row
	 * @return CustomerProgramCall
	 */
    private function load(DibiRow $row)
    {
		$programCall = new CustomerProgramCall();
		$programCall->CallID = $row->Call_ID;
		$programCall->ProgramID = $row->Program_ID;
		$programCall->Call = $row->Call;
		$programCall->Remarks = $row->Remarks;
		$programCall->OpeningDate = $this->dateOnly($row->OpeningDate);
		$programCall->ClosingDate = $this->dateOnly($row->ClosingDate);
		$programCall->CostShare = $row->CostShare;
		$programCall->Responsible = $row->Responsible;
		return $programCall;
    }
	
	/**
	 * Saves program call.
	 * @param CustomerProgramCall $programCall
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(CustomerProgramCall $programCall)
	{
        try
        {
            $array = array();
            $array['Program_ID'] = $programCall->ProgramID;
            $array['Call'] = $programCall->Call;
            $array['Remarks'] = $programCall->Remarks;
            $array['OpeningDate'] = $programCall->OpeningDate;
            $array['ClosingDate'] = $programCall->ClosingDate;
            $array['CostShare'] = $programCall->CostShare;
            $array['Responsible'] = $programCall->Responsible;
            $db_result = $this->conn->insert('CustomerProgramCall', $array)->execute();
            $programCall->CallID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving CustomerProgramCall with Call = " . $programCall->Call . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates program call.
	 * @param CustomerProgramCall $programCall
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(CustomerProgramCall $programCall)
	{
        try
        {
            $array = array();
            $array['Program_ID'] = $programCall->ProgramID;
            $array['Call'] = $programCall->Call;
            $array['Remarks'] = $programCall->Remarks;
            $array['OpeningDate'] = $programCall->OpeningDate;
            $array['ClosingDate'] = $programCall->ClosingDate;
            $array['CostShare'] = $programCall->CostShare;
            $array['Responsible'] = $programCall->Responsible;
            $db_result = $this->conn->update('CustomerProgramCall', $array)->where('[Call_ID] = %i', $programCall->CallID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("CustomerProgramCall update with ID = " . $programCall->CallID . " and Call = " . $programCall->Call . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes program call.
	 * @param int $id programCall ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('CustomerProgramCall')->where('[Call_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("CustomerProgramCall with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of CustomerProgramCall with ID = " . $id . " failed.", Debugger::ERROR);
			return $db_result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of CustomerProgramCall with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds program call by ID.
	 * @param int $id programCall ID.
	 * @return	CustomerProgramCall or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('CustomerProgramCall')->where('[Call_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/** 
	 * Finds all program calls.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('CustomerProgramCall')->orderBy('[Call]', 'ASC')->fetchAll();
		$call_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $call)
                $call_array[$call->Call_ID] = $this->load($call);
        }		
		return $call_array;
	}
	
	/** 
	 * Finds program calls by customer program ID.
	 * @param int $id customer program ID.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findByProgram($id)
	{
		$db_result = $this->conn->select('*')->from('CustomerProgramCall')->where('[Program_ID] = %i', $id)->fetchAll();
		$call_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $call)
                $call_array[$call->Call_ID] = $this->load($call);
        }		
		return $call_array;
	}
	
	/** 
	 * Finds program calls by customer ID.
	 * @param int $id customer ID.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findByCustomer($id)
	{
		$db_result = $this->conn->select('[CustomerProgramCall].*')->from('[CustomerProgramCall], [CustomerProgram]')->where('[CustomerProgramCall.Program_ID] = [CustomerProgram.Program_ID] AND [CustomerProgram.Customer_ID] = %i', $id)->fetchAll();
		$call_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $call)
                $call_array[$call->Call_ID] = $this->load($call);
        }		
		return $call_array;
	}
}
