<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing CustomerProgram from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class CustomerProgramDibiMapper extends DibiMapper implements ICustomerProgramMapper
{
	/**
	 * Creates CustomerProgram instance from DibiRow.
	 * @param DibiRow $row
	 * @return CustomerProgram
	 */
    private function load(DibiRow $row)
    {
		$program = new CustomerProgram();
		$program->ProgramID = $row->Program_ID;
		$program->CustomerID = $row->Customer_ID;
		$program->Program = $row->Program;
		$program->Remarks = $row->Remarks;
		return $program;
    }
	
	/**
	 * Saves program.
	 * @param CustomerProgram $program
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(CustomerProgram $program)
	{
        try
        {
            $array = array();
            $array['Customer_ID'] = $program->CustomerID;
            $array['Program'] = $program->Program;
            $array['Remarks'] = $program->Remarks;
            $db_result = $this->conn->insert('CustomerProgram', $array)->execute();
                        $program->ProgramID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving CustomerProgram with Program = " . $program->Program . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates program.
	 * @param CustomerProgram $program
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(CustomerProgram $program)
	{
        try
        {
            $array = array();
            $array['Customer_ID'] = $program->CustomerID;
            $array['Program'] = $program->Program;
            $array['Remarks'] = $program->Remarks;
            $db_result = $this->conn->update('CustomerProgram', $array)->where('[Program_ID] = %i', $program->ProgramID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("CustomerProgram update with ID = " . $program->ProgramID . " and Program = " . $program->Program . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes program.
	 * @param int $id program ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('CustomerProgram')->where('[Program_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("CustomerProgram with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of CustomerProgram with ID = " . $id . " failed.", Debugger::ERROR);
			return $db_result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of CustomerProgram with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds program by ID.
	 * @param int $id program ID.
	 * @return	CustomerProgram or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('CustomerProgram')->where('[Program_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/** 
	 * Finds all programs.
	 * @return array Array of CustomerProgram or empty array.	
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('CustomerProgram')->orderBy('Program', 'ASC')->fetchAll();
		$program_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $program)
                $program_array[$program->Program_ID] = $this->load($program);
        }		
		return $program_array;
	}
	
	/** 
	 * Finds programs by customer ID.
	 * @param int $id customer ID.
	 * @return array Array of CustomerProgram or empty array.	
	 */
	public function findByCustomer($id)
	{
		$db_result = $this->conn->select('*')->from('CustomerProgram')->where('[Customer_ID] = %i', $id)->fetchAll();
		$program_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $program)
                $program_array[$program->Program_ID] = $this->load($program);
        }		
		return $program_array;
	}
}
