<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing STRAP Item from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemDibiMapper extends DibiMapper implements IItemMapper
{
	/**
	 * Creates Item instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	Item 
	 */
    private function load(DibiRow $row)
    {
		$item = new Item();
		$item->ID = $row->StrapItem_ID;
		$item->CustomID = $row->Custom_ID;
		$item->Name = $row->Name;
		$item->ResponsibleOrgUnitID = $row->ResponsibleOrgUnit_ID;
		$item->TargetTRL = $row->TargetTRL;
        $item->TargetTRLYear = $row->TargetTRLYear;
		$item->Description = $row->Description;
		$item->Active = $row->Active;
		$item->ECLicenceID = $row->ECLicence_ID;
		return $item;
    }
	
	/**
	 * Saves new STRAP item.
	 * 
	 * @param Item $item
	 * @return boolean
	 */
	public function save(Item $item)
	{
        try
        {
            $array = array();
			$array['Custom_ID'] = $item->CustomID;
			$array['Name'] = $item->Name;
			$array['ResponsibleOrgUnit_ID'] = $item->ResponsibleOrgUnitID;
			$array['TargetTRL'] = $item->TargetTRL;
			$array['TargetTRLYear'] = $item->TargetTRLYear;
			$array['Description'] = $item->Description;
			$array['Active'] = $item->Active;
			$array['ECLicence_ID'] = $item->ECLicenceID;
            $this->conn->insert('StrapItem', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new STRAP Item with custom id = " . $item->CustomID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing STRAP item.
	 * 
	 * @param Item $item
	 * @return boolean
	 */
	public function update(Item $item)
	{
        try
        {
            $array = array();
			$array['Custom_ID'] = $item->CustomID;
			$array['Name'] = $item->Name;
			$array['ResponsibleOrgUnit_ID'] = $item->ResponsibleOrgUnitID;
			$array['TargetTRL'] = $item->TargetTRL;
			$array['TargetTRLYear'] = $item->TargetTRLYear;
			$array['Description'] = $item->Description;
			$array['Active'] = $item->Active;
			$array['ECLicence_ID'] = $item->ECLicenceID;
			$this->conn->update('StrapItem', $array)->where('[StrapItem_ID] = %i', $item->ID)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of STRAP Item with ID = " . $item->ID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing STRAP Item.
	 * 
	 * @param int $item
	 * @return boolean
	 */
	public function delete($item_id)
	{
		try
		{
		    $result = $this->conn->delete('StrapItem')->where('[StrapItem_ID] = %i', $item_id)->execute();
			if ($result)
				Debugger::log("Strap Item with ID = " . $item_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of STRAP Item with ID = " . $item_id . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of STRAP ITEM with ID = " . $item_id . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets STRAP Item by id.
	 * 
	 * @param int $item
	 * @return Item or null if not found
	 */
	public function find($item_id)
	{
		$item_db_result = $this->conn->select('*')->from('StrapItem')->where('[StrapItem_ID] = %u', $item_id)->fetch();
		if ($item_db_result !== FALSE)
			return $this->load($item_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets STRAP Item by custom id.
	 * 
	 * @param string $custom_id
	 * @return Item or null if not found
	 */
	public function findByCustomID($custom_id)
	{
		$item_db_result = $this->conn->select('*')->from('StrapItem')->where('[Custom_ID] = %s', $custom_id)->fetch();
		if ($item_db_result !== FALSE)
			return $this->load($item_db_result);
		else
			return NULL;
	}

	/**
	 * Gets all Strap Items.
	 * 
	 * @return array Array of Item where strapItem_ID is key or null if not found.
	 */
	public function findAll()
	{
        $items_array = array();
		$items_db_result = $this->conn->select('*')->from('StrapItem')->orderBy('Name', 'ASC')->fetchAll();        
		if ($items_db_result !== FALSE)			
		{
			foreach ($items_db_result as $item)
				$items_array[$item->StrapItem_ID] = $this->load($item);
			return $items_array;
		}
		else
			return null;
	}
	
	/**
	 * Gets all Strap Items by responsible organization unit.
	 * 
	 * @param int $tgID
	 * @return array Array of Item where strapItem_ID is key or null if not found.
	 */
	public function findAllByTechnicalGroupID($tgID)
	{
        $items_array = array();
		$items_db_result = $this->conn->query('SELECT * FROM [StrapItem] WHERE [ResponsibleOrgUnit_ID] = %i', $tgID)->fetchAll();
		if ($items_db_result !== FALSE)			
		{
			foreach ($items_db_result as $item)
				$items_array[$item->StrapItem_ID] = $this->load($item);
			return $items_array;
		}
		else
			return null;
	}
	
    /**
     * Gets Item by ItemFilter. Returns every items that matches all
     * set conditions.
     *
     * @param ItemFilter $itemFilter
     * @return array of Item
    */
    public function getItemsByFilter(ItemFilter $itemFilter)
    {
        $db_result = $this->conn->select('*')->from('[StrapItem]');

        if ($itemFilter->CustomID)
            $db_result->where("[Custom_ID] LIKE %~like~", $itemFilter->CustomID);
        if ($itemFilter->Name)
            $db_result->where("[Name] LIKE %~like~", $itemFilter->Name);
        if ($itemFilter->ResponsibleOrgUnitID)
		{
			$tgs = $itemFilter->ResponsibleOrgUnitID;
			if (!in_array('', $tgs)) // star selected == all TGs, so do not filter according to tg
			{
				$arrTGCondition = array();
				foreach ($tgs as $tg)
					array_push($arrTGCondition, array("[ResponsibleOrgUnit_ID] = %i", $tg));
				// add whole condition for statuses
				$db_result->where('%or', $arrTGCondition);
			}
		}
        if ($itemFilter->TargetTRL)
            $db_result->where("[TargetTRL] LIKE %i", $itemFilter->TargetTRL);
        if ($itemFilter->targetTRLYear)
            $db_result->where("[targetTRLYear] = %i", $itemFilter->TargetTRLYear);
		if ($itemFilter->Active)
			$db_result->where("[Active] = %i", $itemFilter->Active);
		
		$db_result->orderBy('[ResponsibleOrgUnit_ID]');

        $db_result->fetchAll();

        $arrayOfItems = array();
        foreach($db_result as $row)
            $arrayOfItems[] = $this->load($row);

        return $arrayOfItems;
    }
	
	/**
	 * Gets unique number for new item where CustomID has passed prefix.
	 * Number expresses order of items with the same prefix.
	 * 
	 * @param string $prefix May be defined as '[department short]-[technical group short]-'
	 * @return string '1' if prefix does not exist, first free number for passed prefix otherwise.
	 */
	public function getNextCustomIDNumber($prefix)
	{
		$db_result = $this->conn->query('
			SELECT *
			FROM [StrapItem]
			WHERE [Custom_ID] LIKE %like~',
		   $prefix
        )->fetchAll();
		
		if ($db_result === FALSE)
			return '1';
		else
	    {
			$number = count($db_result);
			return $number + 1;
		}
	}
}
