<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing PursuitRisk from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PursuitRiskDibiMapper extends DibiMapper implements IPursuitRiskMapper
{
	/**
	 * Creates PursuitRisk instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	PursuitRisk 
	 */
    private function load(DibiRow $row)
    {
		$pursuitRisk = new PursuitRisk();
		$pursuitRisk->ID = $row->Risk_ID;
		$pursuitRisk->PursuitID = $row->Pursuit_ID;
		$pursuitRisk->Risk = $row->Risk;
		$pursuitRisk->Owner = $row->Owner;
		$pursuitRisk->Deadline = $this->dateOnly($row->Deadline);
		$pursuitRisk->Status = $row->Status;
		$pursuitRisk->Remarks = $row->Remarks;
		return $pursuitRisk;
    }
	
	/**
	 * Saves new PursuitRisk.
	 * 
	 * @param PursuitRisk $pursuitRisk
	 * @return boolean
	 */
	public function save(PursuitRisk $pursuitRisk)
	{
        try
        {
            $array = array();
			$array['Pursuit_ID'] = $pursuitRisk->PursuitID;
			$array['Risk'] = $pursuitRisk->Risk;
			$array['Owner'] = $pursuitRisk->Owner;
			$array['Deadline'] = $pursuitRisk->Deadline;
			$array['Status'] = $pursuitRisk->Status;
			$array['Remarks'] = $pursuitRisk->Remarks;
            $this->conn->insert('PursuitRisk', $array)->execute();
			    $pursuitRisk->ID = $this->conn->getInsertId();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new PursuitRisk with risk: " . $pursuitRisk->Risk . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing PursuitRisk.
	 * 
	 * @param PursuitRisk $pursuitRisk
	 * @return boolean
	 */
	public function update(PursuitRisk $pursuitRisk)
	{
        try
        {
            $array = array();
			$array['Risk_ID'] = $pursuitRisk->ID;
			$array['Pursuit_ID'] = $pursuitRisk->PursuitID;
			$array['Risk'] = $pursuitRisk->Risk;
			$array['Owner'] = $pursuitRisk->Owner;
			$array['Deadline'] = $pursuitRisk->Deadline;
			$array['Status'] = $pursuitRisk->Status;
			$array['Remarks'] = $pursuitRisk->Remarks;
            $this->conn->update('PursuitRisk', $array)->where('[Risk_ID] = %i', $pursuitRisk->ID)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of PursuitRisk with ID = " . $pursuitRisk->ID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing PursuitRisk.
	 * 
	 * @param int $id PursuitRisk ID 
	 * @return boolean
	 */
	public function delete($id)
	{
		try
		{
		    $result = $this->conn->delete('PursuitRisk')->where('[Risk_ID] = %i', $id)->execute();
			if ($result)
				Debugger::log("PursuitRisk with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of PursuitRisk with ID = " . $id . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of PursuitRisk with ID = " . $id . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets PursuitRisk.
	 * 
	 * @param int $id PursuitRisk id
	 * @return PursuitRisk or null if not found
	 */
	public function find($id)
	{
		$pursuitRisk_db_result = $this->conn->select('*')->from('PursuitRisk')->where('[Risk_ID] = %i', $id)->fetch();
		if ($pursuitRisk_db_result !== FALSE)
			return $this->load($pursuitRisk_db_result);
		else
			return NULL;
	}
	
	/**
	 * Gets PursuitRisk by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitRisk or empty array if not found
	 */
	public function findByPursuitID($pursuitID)
	{
		$pursuitRisk_db_result = $this->conn->select('*')->from('PursuitRisk')->where('[Pursuit_ID] = %i', $pursuitID)->fetchAll();
		
		$pursuitRisk_array = array();
		if ($pursuitRisk_db_result !== FALSE)
		{
			foreach ($pursuitRisk_db_result as $row)
				$pursuitRisk_array[] = $this->load($row);
		}
		return $pursuitRisk_array;
	}
	
	/**
	 * Gets nearest PursuitRisk by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return PursuitPlan or null if not found
	 */
	public function findNearestByPursuitID($pursuitID)
	{
		$now = new DateTime();
		$pursuitRisk_db_result = $this->conn->query('SELECT * FROM [PursuitRisk]
														WHERE [Deadline] > %d
															AND [Pursuit_ID] = %i', $now, $pursuitID)->fetch();
		if ($pursuitRisk_db_result !== FALSE)
			return $this->load($pursuitRisk_db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds out if exists missed milestone.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return True or false
	 */
	public function existsMissedRisk($pursuitID)
	{
		$now = new DateTime();
		$pursuitPlan_db_result = $this->conn->query('SELECT * FROM [PursuitRisk]
														WHERE [Deadline] < %d
															AND [Pursuit_ID] = %i
																AND [Status] <> %s', $now, $pursuitID, PursuitRisk::PURSUIT_RISK_STATUS_CLOSED)->fetch();
		if ($pursuitPlan_db_result !== FALSE)
			return true;
		else
			return false;
	}
	
	/**
	 * Gets missed PursuitRisk by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of PursuitRisk or empty array if not found
	 */
	public function findMissedByPursuitID($pursuitID)
	{
		$now = new DateTime();
		$pursuitRisk_db_result = $this->conn->query('SELECT * FROM [PursuitRisk]
														WHERE [Deadline] < %d
															AND [Pursuit_ID] = %i
																AND [Status] <> %s', $now, $pursuitID, PursuitRisk::PURSUIT_RISK_STATUS_CLOSED)->fetchAll();
		
		$pursuitRisk_array = array();
		if ($pursuitRisk_db_result !== FALSE)
		{
			foreach ($pursuitRisk_db_result as $row)
				$pursuitRisk_array[] = $this->load($row);
		}
		return $pursuitRisk_array;
	}
}
