<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing SettingDisplayedContent from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class SettingDisplayedContentDibiMapper extends DibiMapper implements ISettingDisplayedContentMapper
{	
	/**
	 * Creates SettingDisplayedContent instance from DibiRow.
	 * @param DibiRow $row
	 * @return SettingDisplayedContent
	 */
    private function load(DibiRow $row)
    {
		$setting = new SettingDisplayedContent();
		$setting->EID = $row->EID;
		$setting->OrganizationUnitID = $row->OrganizationUnit_ID;
		return $setting;
    }
	
	/**
	 * Saves setting.
	 * @param SettingDisplayedContent $setting
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(SettingDisplayedContent $setting)
	{
        try
        {
            $array = array();
            $array['EID'] = $setting->EID;
            $array['OrganizationUnit_ID'] = $setting->OrganizationUnitID;
            $db_result = $this->conn->insert('SettingDisplayedContent', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving SettingDisplayedContent with EID = " . $setting->EID . " and OrganizationUnitID = " . $setting->OrganizationUnitID . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates setting.
	 * @param SettingDisplayedContent $setting
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(SettingDisplayedContent $setting)
	{
        try
        {
            $array = array();
            $array['EID'] = $setting->EID;
            $array['OrganizationUnit_ID'] = $setting->OrganizationUnitID;
            $db_result = $this->conn->update('SettingDisplayedContent', $array)->where('[EID] = %s AND [OrganizationUnit_ID] = %i', $setting->EID, $setting->OrganizationUnitID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("SettingDisplayedContent update with EID = " . $setting->EID . " and OrganizationUnitID = " . $setting->OrganizationUnitID . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes setting.
	 * @param string $eid User EID.
	 * @param int $orgUnitID Organization unit ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($eid, $orgUnitID)
	{
		try
		{
		    $db_result = $this->conn->delete('SettingDisplayedContent')->where('[EID] = %s AND [OrganizationUnit_ID] = %i', $eid, $orgUnitID)->execute();
			if ($db_result)
				Debugger::log("SettingDisplayedContent with EID = " . $eid . " and OrganizationUnitID = " . $orgUnitID . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of SettingDisplayedContent with EID = " . $eid . " and OrganizationUnitID = " . $orgUnitID . " failed.", Debugger::ERROR);
			return $db_result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of SettingDisplayedContent with EID = " . $eid . " and OrganizationUnitID = " . $orgUnitID . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds setting.
	 * @param string $eid User EID.
	 * @param int $orgUnitID Organization unit ID.
	 * @return	SettingDisplayedContent or NULL if not found.
	 */
	public function find($eid, $orgUnitID)
	{
		$db_result = $this->conn->select('*')->from('SettingDisplayedContent')->where('[EID] = %s AND [OrganizationUnit_ID] = %i', $eid, $orgUnitID)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds setting by EID.
	 * @param string $eid User EID.
	 * @return array Array of SettingDisplayedContent or empty array. Indexed by organization unit id.
	 */
	public function findByEID($eid)
	{
		$db_result = $this->conn->select('*')->from('SettingDisplayedContent')->where('[EID] = %s', $eid)->fetchAll();     
		$setting_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $setting_array[$row->OrganizationUnit_ID] = $this->load($row);
        }		
		return $setting_array;
	}
	
}
