<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing TaskPONumber from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TaskPONumberDibiMapper extends DibiMapper implements ITaskPONumberMapper
{	
	/**
	 * Creates TaskPONumber instance from DibiRow.
	 * @param DibiRow $row
	 * @return TaskPONumber
	 */
    private function load(DibiRow $row)
    {
		$poAttr = new TaskPONumber();
		$poAttr->ID = $row->TaskPONumber_ID;
		$poAttr->TaskID = $row->Task_ID;
		$poAttr->Year = $row->Year;
		$poAttr->PONumber = $row->PONumber;
		return $poAttr;
    }
	
	/**
	 * Saves TaskPONumber.
	 * @param TaskPONumber $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(TaskPONumber $poAttr)
	{
        try
        {
            $array = array();
            $array['Task_ID'] = $poAttr->TaskID;
            $array['Year'] = $poAttr->Year;
            $array['PONumber'] = $poAttr->PONumber;
            $db_result = $this->conn->insert('TaskPONumber', $array)->execute();
            $poAttr->ID = $this->conn->insertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving TaskPONumber with TaskID = " . $poAttr->TaskID . "and Year = " . $poAttr->Year . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates TaskPONumber.
	 * @param TaskPONumber $poAttr
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(TaskPONumber $poAttr)
	{
        try
        {
            $array = array();
            $array['Year'] = $poAttr->Year;
            $array['PONumber'] = $poAttr->PONumber;
            $db_result = $this->conn->update('TaskPONumber', $array)->where('[TaskPONumber_ID] = %i', $poAttr->ID)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("TaskPONumber update with ID = " . $poAttr->ID . " failed. " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing TaskPONumber.
	 * @param int $id TaskPONumber ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		try
		{
		    $db_result = $this->conn->delete('TaskPONumber')->where('[TaskPONumber_ID] = %i', $id)->execute();
			if ($db_result)
				Debugger::log("TaskPONumber with ID = " . $id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of TaskPONumber with ID = " . $id . " failed.", Debugger::ERROR);
			return $db_result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of TaskPONumber with ID = " . $id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Finds TaskPONumber by ID.
	 * @param int $id TaskPONumber ID.
	 * @return TaskPONumber or NULL if not found.
	 */
	public function find($id)
	{
		$db_result = $this->conn->select('*')->from('TaskPONumber')->where('[TaskPONumber_ID] = %i', $id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all TaskPONumbers.
	 * @return array Array of TaskPONumber or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('TaskPONumber')->orderBy('Task_ID', 'Year')->fetchAll();     
		$poAttr_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $poAttr_array[$row->TaskPONumber_ID] = $this->load($row);
        }		
		return $poAttr_array;
	}
	
	/**
	 * Finds all TaskPONumbers for task.
	 * @param int $id Task ID
	 * @return array Array of TaskPONumber or empty array.
	 */
	public function findByTaskID($id)
	{
		$db_result = $this->conn->select('*')->from('TaskPONumber')->where('[Task_ID] = %i', $id)->orderBy('[Year], [TaskPONumber_ID]')->fetchAll();     
		$poAttr_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $poAttr_array[$row->TaskPONumber_ID] = $this->load($row);
        }		
		return $poAttr_array;
	}
	
	/**
	 * Find TaskPONumber for task and year.
	 * @param int $id Task ID
	 * @param int $year Year
	 * @return TaskPONumber or NULL if not found.
	 */
	public function findByTaskIDandYear($id, $year)
	{
		$db_result = $this->conn->select('*')->from('TaskPONumber')->where('[Task_ID] = %i AND [Year] = %i', $id, $year)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Find TaskPONumber by PO number.
	 * @param String $po_number
	 * @return TaskPONumber or NULL if not found.
	 */
	public function findByPONumber($po_number)
	{
		$db_result = $this->conn->select('*')->from('TaskPONumber')->where('[PONumber] = %s', $po_number)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Find TaskPONumber by year.
	 * @param int $year
	 * @return array Array of TaskPONumber or empty array.
	 */
	public function findByYear($year)
	{
		$db_result = $this->conn->select('*')->from('TaskPONumber')->where('[Year] = %i', $year)->fetchAll();     
		$array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $array[$row->TaskPONumber_ID] = $this->load($row);
        }		
		return $array;
	}
	
	/**
	 * Finds relations between TaskPONumber, Task and Project by given PO numbers.
	 * @param array $po_numbers List of PO numbers
	 * @param int $year Year
	 * @return array
	 *     $array[PONumber] = array(Year => val, TaskSAP_ID => val, ProjectSAP_ID => val)
	 *     or NULL if not found.
	 */
	public function findIASAPActualsImportRelationsByPONumbers(array $po_numbers, $year)
	{
		if (!count($po_numbers))
			return NULL;
		
		$db_result = $this->conn
				->select('[TaskPONumber].[PONumber], [TaskPONumber].[Year], [Task].[SAP_ID] AS [TaskSAP_ID], [Project].[SAP_ID] AS [ProjectSAP_ID]')
				->from('[TaskPONumber], [Task], [Project]')
				->where('[TaskPONumber].[Task_ID] = [Task].[Task_ID] AND [Task].[Project_ID] = [Project].[Project_ID]')
				->where('[TaskPONumber].[PONumber] IN %in AND [TaskPONumber].[Year] = %i', $po_numbers, $year)
				->fetchAll();
		
		if ($db_result)
		{
			$relations = array();
			foreach ($db_result as $row)
			    $relations[$row->PONumber] = array(
				    'Year' => $row->Year,
				    'TaskSAP_ID' => $row->TaskSAP_ID,
				    'ProjectSAP_ID' => $row->ProjectSAP_ID
			    );
			
			return $relations;
		}
		
		return NULL;
	}
}
