<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Base ancestor for all application presenters.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
abstract class BasePresenter extends Presenter
{
	/** @const int Start year of HPT era */
	const YEAR_ZERO = 2001;

	/**
	 * Switches email development mode
	 * @param int $mode Current mode
	 */
	public function handleSwitchEmailMode($mode)
	{
		$this->context->globalSetting->set('mail.developmentMode', (($mode) ? FALSE : TRUE));
		$this->invalidateControl('serverInfo');
	}
	
	/**
	 * Switches login development mode
	 * @param int $mode Current mode
	 */
	public function handleSwitchLoginMode($mode)
	{
		$this->context->globalSetting->set('login.developmentMode', (($mode) ? FALSE : TRUE));
		$this->invalidateControl('serverInfo');
	}

	/**
     * Tracks used buttons and links
     *
     * @param string $name
     */
	public function handleStatisticsClick($name)
	{
		$detail = $this->getPresenter()->getName() . ':' . $this->getAction() . ':' . $name;
        $this->context->statistics->log($detail, IStatistics::CLICK);
		$this->terminate();
	}


    /**
     * Tracks visited pages
     */
	public function handleStatisticsVisit()
	{
        $detail = $this->getPresenter()->getName() . ':' . $this->getAction();
        $this->context->statistics->log($detail, IStatistics::VISIT);
		$this->terminate();
	}

	/**
	 * Prepares data for organization unit selector
	 * @param int $id ID of selected organization unit
	 */
	public function actionOrganizationUnitSelector($id)
	{		
		$org_unit_selector = new OrganizationUnitSelectorContainer;
		$this->payload->organizationUnitSelectorResult = $org_unit_selector->ajaxHandle($id);
		$this->terminate();
	}
	
	/**
	 * Prepares common helpers and default data
	 * 
	 * @param  string|NULL
	 * @return ITemplate
	 */
	protected function createTemplate($class = NULL)
    {
		$template = parent::createTemplate($class);
		// add default helpers
		$template->registerHelper('fromGlobalSetting', 'GlobalSettingReaderHelper::get');
		$template->registerHelper('round', 'RoundHelper::round');
		$template->registerHelper('coveragePercentageToCSS', 'CoverageStyleHelper::coveragePercentageToCSS');
		$template->registerHelper('coveragePercentageTypeToCSS', 'CoverageStyleHelper::coveragePercentageTypeToCSS');
		$template->registerHelper('coveragePercentageBoundary', 'CoverageStyleHelper::coveragePercentageBoundary');
		
		// set application version
		$template->applicationVersion = APPLICATION_VERSION;
		if (defined('APPLICATION_VERSION_REV') && APPLICATION_VERSION_REV != '')
			$template->applicationVersion .= ' (revision ' . APPLICATION_VERSION_REV . ')';
		// set cookie path
		// separator for directory in uri is always '/' see RFC 2396 (do not change)
		$template->cookiePath = '/' . TOOL_DIR . '/';
		$template->cacheControl = 'no-cache';
		//set server type
		$template->serverType = $this->context->parameters['serverType'];
		$template->mailDevelopmentMode = $this->context->globalSetting->get('mail.developmentMode');
		$template->loginDevelopmentMode = $this->context->globalSetting->get('login.developmentMode');
		$template->mailTesterList = $this->context->globalSetting->get('mail.testerList');
		
		//Echarge import - notification if it is running
		$template->echargeLock = $this->context->createLock('ECHARGE_IMPORT');
		
		return $template;
    }
	
	/**
	 * Sets year and store it to given session section
	 *
	 * @param int &$year 
	 * @param SessionSection [&$session]
	 */
	protected function setYear(&$year, &$session = NULL)
	{
		if (!isset($year) || !is_numeric($year))
			$year = (isset($session->year)) ? $session->year : (int) date('Y');

		if ($year < BasePresenter::YEAR_ZERO)
			$year = (int) date('Y');
        
        if ($session)
            $session->year = $year;
	}

	/**
	 * Forgets year stored in given session section
	 *
	 * @param SessionSection &$session
	 */
	protected function dropYear(&$session)
	{
        if (isset($session->year))
            unset($session->year);
	}
}
