<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Base all application presenters available after user has logged
 * which means all presenters that are not public.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
abstract class SecuredPresenter extends BasePresenter
{
	/** @const List of ATE Tool resources */
	const RESOURCE_PROJECT = 'Project',
		  RESOURCE_PERSON = 'Person',
	      RESOURCE_ORGANIZATION = 'Organization',
		  RESOURCE_WORK_CENTER = 'WorkCenter',
		  RESOURCE_LABOUR_RATE = 'LabourRate',
		  RESOURCE_EXCHANGE_RATE = 'ExchangeRate',
		  RESOURCE_IMPORT = 'Import',
		  RESOURCE_RESOURCE_LOCK = 'ResourceLock',
		  RESOURCE_PLACEHOLDER_TASK = 'PlaceHolderTask',
		  RESOURCE_PROJECT_REAL_COST_SHARE = 'ProjectRealCostShare',
		  RESOURCE_TASK_ASSIGNMENT_LOCK = 'TaskAssignmentLock',
		  RESOURCE_PERSON_ASSIGNMENT_TRANSFER = 'PersonAssignmentTransfer',
		  RESOURCE_TASK_ASSIGNMENT_TRANSFER = 'TaskAssignmentTransfer';
	/** @const List of STRAP Tool resources */
	const RESOURCE_STRAP_ITEM = 'STRAPItem',
			RESOURCE_STRAP_ACTIVITY = 'STRAPActivity';
	/** @const List of actions */
	const ACTION_VIEW = 'view',      // item detail, form, link etc.
		  ACTION_CREATE = 'create',  // save new record to db
		  ACTION_EDIT = 'edit',      // modify existing db record 
		  ACTION_DELETE = 'delete',  // delete existing db record
		  ACTION_RUN = 'run';        // run import, download report, lock resources etc.
			
	
	/** @var Permission */
	private $acl;
	
	
	protected function startup()
    {
        parent::startup();

        $user = Environment::getUser();
        if (!$user->isLoggedIn()) 
        {
			Debugger::log("not logged in", Debugger::DEBUG);
            if ($user->getLogOutReason() === IUserStorage::INACTIVITY)
            {
                $this->flashMessage('You have been logged out due to inactivity. Please login again.');
            }

            //$backlink = $this->getApplication()->storeRequest();
            //$this->redirect('Login:default', array('backlink' => $backlink));
            $this->template->user = Environment::getUser();
            $this->redirect('Login:default', array('backlink' => $this->storeRequest()));
            //$this->redirect('Login:default');
        }
		
    }

	/**
	 * Prepares common helpers and injects logged user object into template.
	 * 
	 * @param  string|NULL
	 * @return ITemplate
	 */
	protected function createTemplate($class = NULL)
    {
		$template = parent::createTemplate($class);
        $user = Environment::getUser();		
		
        if($user->getIdentity() != null)
            switch ($user->getIdentity()->userType)
            {
                case "LDAP_valid":
                    $person = new Person();
                    $person->ID = strtoupper($user->getId());
                    $person->Email = $user->getIdentity()->email;
                    $person->Firstname = $user->getIdentity()->firstname;
                    $person->Surname = $user->getIdentity()->surname;
                    $template->userType = "LDAP_valid";
                    $template->loggedUser = $person;
                    break;
                case "account_valid":
                    $template->userType = "account_valid";
                    $template->loggedUser = $this->context->personRepository->find($user->getId());
                    break;
            }
        //if ($user->isLoggedIn()) 
		//	$template->loggedUser = $this->context->personRepository->find($user->getId());
		return $template;
    }

	/**
	 * Tests whether logged user has selected roles.
	 * 
	 * @param mixed Insert role names as string or array of strings.
	 * @return true
     */
    protected function testUserRoles($roles)
    {
        $logged_user_roles = Environment::getUser()->getRoles();
		if (is_array($roles) || (is_string($roles)))
			return Environment::getUser()->isInRole($roles);
		else
			throw new InvalidArgumentException ('Roles parameter must be string or array.');
    }
	
	/**
	 * Tests whether logged user may perform action with resource.
	 * @param string $resource Should be in range of self::RESOURCE_*
	 * @param string|array $action Should be in range of self::ACTION_*
	 * @return bool TRUE in case user has role which can perform action with given resource. FALSE otherwise.
	 * @todo Modify way of decision for more action on input.
	 */
	public function isAllowed($resource, $action)
	{
		if (!$this->acl)
			$this->acl = $this->getPermission();
		
		// Get assigned roles
		$roles = $this->getUser()->getRoles();
		// Add default role to logged user
		if ($this->getUser()->isLoggedIn())
			array_push($roles, UserRole::ROLE_USER);
		
		$is_allowed = FALSE;
		// If user has role which can perform action with given resource, action will be allowed.
		if (is_array($action))
		{
			foreach ($roles as $role)
				foreach ($action as $act)
			        $is_allowed = $is_allowed || $this->acl->isAllowed($role, $resource, $act);
		}
		else
		{
		    foreach ($roles as $role)
			    $is_allowed = $is_allowed || $this->acl->isAllowed($role, $resource, $action);
		}
		
		return $is_allowed;
	}
	
	/**
	 * Initializes permissions
	 */
	private function getPermission()
	{
		$acl = new Permission();
		
		// ================================
		//     Roles
		// ================================
		// HON Planning Tool + STRAP Tool
		$acl->addRole(UserRole::ROLE_AUTHENTICATED);
		$acl->addRole(UserRole::ROLE_GUEST);
		$acl->addRole(UserRole::ROLE_USER, UserRole::ROLE_GUEST);
		// HON Planning Tool
		$acl->addRole(UserRole::ROLE_PMO, UserRole::ROLE_USER);
		$acl->addRole(UserRole::ROLE_ADMIN, UserRole::ROLE_PMO);
		// STRAP Tool
		$acl->addRole(UserRole::ROLE_STRAP_CL);
		$acl->addRole(UserRole::ROLE_STRAP_ADMIN, UserRole::ROLE_STRAP_CL);
		
		// ================================
		//     Resources
		// ================================
		// HON Planning Tool
		$acl->addResource(self::RESOURCE_PROJECT);
		$acl->addResource(self::RESOURCE_PERSON);
		$acl->addResource(self::RESOURCE_ORGANIZATION);
		$acl->addResource(self::RESOURCE_WORK_CENTER);
		$acl->addResource(self::RESOURCE_LABOUR_RATE);
		$acl->addResource(self::RESOURCE_EXCHANGE_RATE);
		$acl->addResource(self::RESOURCE_IMPORT);
		$acl->addResource(self::RESOURCE_RESOURCE_LOCK);
		$acl->addResource(self::RESOURCE_PLACEHOLDER_TASK);
		$acl->addResource(self::RESOURCE_PROJECT_REAL_COST_SHARE);
		$acl->addResource(self::RESOURCE_TASK_ASSIGNMENT_LOCK);
		$acl->addResource(self::RESOURCE_PERSON_ASSIGNMENT_TRANSFER);
		$acl->addResource(self::RESOURCE_TASK_ASSIGNMENT_TRANSFER);
		// STRAP Tool
		$acl->addResource(self::RESOURCE_STRAP_ITEM);
		$acl->addResource(self::RESOURCE_STRAP_ACTIVITY);
		
		// ================================
		//     Binding
		// ================================
		// Resource Project
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_PROJECT, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_PROJECT, array(self::ACTION_CREATE ,self::ACTION_EDIT));
		$acl->allow(UserRole::ROLE_ADMIN, self::RESOURCE_PROJECT, self::ACTION_DELETE);
		// Resource Person
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_PERSON, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_PERSON, array(self::ACTION_EDIT ,self::ACTION_DELETE));
		$acl->allow(UserRole::ROLE_ADMIN, self::RESOURCE_PERSON, self::ACTION_CREATE);
		// Resource Person assignment transfer
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_PERSON_ASSIGNMENT_TRANSFER, array(self::ACTION_EDIT));
		// Resource Organization
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_ORGANIZATION, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_ADMIN, self::RESOURCE_ORGANIZATION, array(self::ACTION_CREATE ,self::ACTION_EDIT, self::ACTION_DELETE));
		// Resource Import
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_IMPORT, array(self::ACTION_VIEW, self::ACTION_RUN));
		// Work Center
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_WORK_CENTER, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_ADMIN, self::RESOURCE_WORK_CENTER, array(self::ACTION_CREATE ,self::ACTION_EDIT, self::ACTION_DELETE));
		// Resource Labour Rate
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_LABOUR_RATE, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_ADMIN, self::RESOURCE_LABOUR_RATE, array(self::ACTION_CREATE ,self::ACTION_EDIT, self::ACTION_DELETE));
		// Resource Exchange Rate
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_EXCHANGE_RATE, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_ADMIN, self::RESOURCE_EXCHANGE_RATE, array(self::ACTION_CREATE ,self::ACTION_EDIT, self::ACTION_DELETE));
		// Resource Assignment Lock
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_RESOURCE_LOCK, array(self::ACTION_RUN));
		// Resource Placeholder Task
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_PLACEHOLDER_TASK, array(self::ACTION_CREATE));
		// Resource Project Real Cost Share
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_PROJECT_REAL_COST_SHARE, array(self::ACTION_CREATE, self::ACTION_EDIT));
		// Resource STRAP Item
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_STRAP_ITEM, self::ACTION_VIEW);
		$acl->allow(UserRole::ROLE_STRAP_CL, self::RESOURCE_STRAP_ITEM, array(self::ACTION_CREATE ,self::ACTION_EDIT, self::ACTION_DELETE));
		// Resource STRAP Activity
		$acl->allow(UserRole::ROLE_USER, self::RESOURCE_STRAP_ACTIVITY, self::ACTION_VIEW, self::ACTION_EDIT, self::ACTION_DELETE);
		$acl->allow(UserRole::ROLE_STRAP_CL, self::RESOURCE_STRAP_ACTIVITY, array(self::ACTION_CREATE));
		// Resource Task assignment lock
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_TASK_ASSIGNMENT_LOCK, array(self::ACTION_EDIT));
		// Resource Task assignment transfer
		$acl->allow(UserRole::ROLE_PMO, self::RESOURCE_TASK_ASSIGNMENT_TRANSFER, array(self::ACTION_EDIT));
		
		
		return $acl;
	}
	
}
