<?php
/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * Locker is an interface for classes implementing locking mechanism
 * controlling concurrent access to resources.
 * 
 * Concrete implementation is to decide way of locking (actual)
 * key data.
 * 
 * Example of common usage:
 * <example>
 * $locker = new FileLock('LONG_RUNNING_PROCESS');
 * if ($lock->lock()) {
 *      // here you put code you want to run on the server in one instance only
 *      $lock->unlock();
 * }
 * else
 * {
 *      throw new LockException('Process is locked. Cannot run right now');
 * }
 * </example> 
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class FileLock implements Lock {

	/** string file name */
	private $lockName;

	/** string hash of the name (to be sure) */
	private $lockKey;
	
	/** int (timestamp) determines moment of death of lock */
	private $lockLifetime;
    
	/** string file name */
	private $lockDirectoryBase = '';

	/**
	 * Returns true if it was possible to lock (gain the key to resources).
	 * 
	 * @param string @name Name of lock.
	 * @param int @lifetime Lifetime of lock in seconds (In case of bad format, default lifetime is set to 3 hours).
	 * @return boolean
	 */
    public function FileLock($name, $lifetime)
    { 
		if (!is_int($lifetime)) // must be integer
			$lifetime = 10800;
		
        $this->lockName = $name;
        $this->lockKey = md5($name);
		$this->lockLifetime = mktime(date('H'), date('i'), $lifetime);
    }
    
    
	/**
	 * Returns true if it was possible to lock (gain the key to resources).
	 * 
	 * @return boolean
	 */
    private function createLockDir()
    {
        if (file_exists($this->lockDirectoryBase))
        {
            if (is_dir($this->lockDirectoryBase))
				return true;
            else
				throw new LockException('Specified path is not a writable directory: ' . $this->lockDirectoryBase);
        }
        else
        {
            if (!mkdir($this->lockDirectoryBase, 0700)) // create writable only for process
				throw new LockException('Failed to created base directory: ' . $this->lockDirectoryBase);
        }
    }

    
	/**
	 * Returns true if it was possible to lock (gain the key to resources).
	 * 
	 * @return boolean
     * @throws LockException in case of inability to lock.
	 */
    public function lock()
    {
        $this->createLockDir();
		if ($this->locked())
			return false;
		return (file_put_contents($this->getLockFilePath(), 'LOCK FILE FOR: ' . $this->lockName . "\r\n" . 'VALID UNTIL: ' . $this->lockLifetime) !== FALSE);
    }

    /**
	 * Returns true if it was possible to unlock (return the key to resources).
	 * 
	 * @return boolean
	 */
    public function unlock()
    {
		if ($this->locked())
	        return unlink($this->getLockFilePath());
		else
		    return true;			
    }

	/**
	 * Returns true if locker is locked.
	 * 
	 * @return boolean
	 */
    public function locked()
    {		
		if (!file_exists($this->getLockFilePath())) // lock does not exist
			return false;
		
		$lockContent = file_get_contents($this->getLockFilePath());
		if ($lockContent === FALSE) // cannot read content of lock
			return false;
		
		$lockLifetime = preg_replace('/^.*\r\n.* /', '', $lockContent); // regex depends on format generated by method $this->lock()
		if ($lockLifetime === NULL) // cannot parse content of lock
			return false;
		
		if (strtotime('now') > intval($lockLifetime)) // lock is invalid
			return false;
		
		return true;
    }
    
	/**
	 * Sets base directory for storing lock file.
	 * 
	 * @return boolean
	 */
    public function setLockDir($dir)
    {
		$this->lockDirectoryBase = $dir;
    }

	/**
	 * Gets base directory for storing lock file.
	 * 
	 * @return boolean
	 */
    public function getLockDir()
    {
        return $this->lockDirectoryBase;
    }
    
	/**
	 * Gets full path to lock file if base directory is setup.
	 * 
	 * @return string
	 */
    public function getLockFilePath()
    {
		if ($this->lockDirectoryBase != '')
			return $this->lockDirectoryBase . '/' . $this->lockKey;
		return '';
    }
    
}
