<?php

class EChargeWrapper
{
    /** @var Curl wrapper */
    private $curlResource;
    
    /** @var string New temporary filename */
    private $curlCookies;
    
    /** @var url eCharge URL */
    private $baseUrl;
    
    /** @var ILogger Logger */
    private $logger;
    
    /** @var boolean Is eChargeWrapper logged to eCharge? */
    private $loggedIn = FALSE;

    /** @var boolean If we are debugging eCharge */
    private $debugMode = FALSE;
    
	
    /**
     * Creates wrapper for reading actuals from eCharge
     * 
     * @param String $baseUrl Base URL of eCharge
     * @param ILogger $logger Logger
     */
    public function __construct($baseUrl, ILogger $logger = NULL)
    {
        
        $this->curlResource = new Curl;
        $this->baseUrl      = $baseUrl;
        $this->logger       = ($logger === NULL) ? (new NullLogger('')) : $logger;
        
    }
    
    /**
     * Logs into eCharge
     * 
     * @param String $user User EID
     * @param String $password User password
     * @throws EChargeWrapperException
     */
    public function login($user, $password)
    {
        $plain_password = EChargePasswordHandler::decrypt($password);
        $arguments = array(
            'hdneCharge'    => "Process",
            'changePWD'     => "",
            'badge'         => $user,
            'password'      => $plain_password,
            'btnSubmit'     => "Working..."
        );

        $this->curlCookies = tempnam ("/tmp", "CURLCOOKIE");
        $this->curlResource->options['CURLOPT_COOKIEJAR'] = $this->curlCookies;
        $this->curlResource->options['CURLOPT_COOKIEFILE'] = $this->curlCookies;
        $response = $this->curlResource->post($this->baseUrl . 'Login/LoginWin.asp', $arguments);
        
        if ($response)
        {
            //Debugger::log(var_export($response, true), Debugger::DEBUG);
            if ($response->headers['Status-Code'] == 200)
            {
                if ((strpos($response, 'Invalid Password') === FALSE) 
                        && (strpos($response, 'Invalid UserID') === FALSE)
                        && (strpos($response, 'Logon failure') === FALSE))
                {
                    Debugger::log('EChargeWrapper: Login Successful', Debugger::INFO);
                }
                else
                {
                    Debugger::log('EChargeWrapper: Invalid Credentials', Debugger::ERROR);
                    throw new EChargeWrapperException('Login to eCharge failed: invalid credentials');
                }
            }
            else
            {
                Debugger::log('EChargeWrapper: Wrong status code returned: ' . $response->headers['Status-Code'], Debugger::ERROR);
                throw new EChargeWrapperException('Login to eCharge failed: Wrong status code returned: ' . $response->headers['Status-Code']);
            }
        }
        else
        {
            Debugger::log('Curl: ' . $this->curlResource->error(), Debugger::ERROR);
            throw new EChargeWrapperException('Login to eCharge failed: ' . $this->curlResource->error());
        }
        
        //$this->validateLogin(); // for debugging only
        
        $this->loggedIn = TRUE;
        
    }
    
    /**
     * Validates if login was successful 
     * USED FOR DEBUGGING ONLY
     */
    private function validateLogin()
    {
        $url = $this->baseUrl . "/Login/ValidateLogin.asp";
        $response = $this->curlResource->get($url);
        Debugger::log(var_export($response, true), Debugger::DEBUG);
    }
    
    /**
     *
     * @param String $startDate Date in format j-n-Y (e.g. 1-31-2012)
     * @param String $endDate Date in format j-n-Y (e.g. 1-31-2012)
     * @return array Array of arrays representing rows where key is TH and value TD
     * @throws EChargeWrapperException
     */
    public function readActuals(DateTime $startDate, DateTime $endDate)
    {
        
        if (!$this->loggedIn)
            throw new EChargeWrapperException('You must log in first, use EChargeWrapper->login($username, $password) before reading actuals');
        
        $url = $this->baseUrl . "reports/Misc/AdvChrgForEmp.asp?begindate=".$startDate->format('n-j-Y')."&enddate=".$endDate->format('n-j-Y')."&empbadge=&empType=ALL&department=CZ&deptGroup=&chargenumber=&salesorder=&description=&workgroup=&workcenter=&activity=&tranType=&operation=&APPROVED=on&UPLOADED=on&SUBMITTED=on&ACTIVE=on&INCOMPLETE=on&ShowBadge=e.USERID&ShowChargeNumber=t.WORKORDER&ShowChargeDate=t.PAYDAY&ShowModifiedAt=t.MODIFIED_AT&ShowHours=t.HOURS&ShowTranDataLabel3=t.tranid&order1=e.USERID&order2=e.USERID&order3=e.USERID&groupby=&setSize=All&NEWWIN=on&ShowDescription=t.DESCRIPTION";
		$urlDS = $this->baseUrl . "reports/Misc/AdvChrgForEmp.asp?begindate=".$startDate->format('n-j-Y')."&enddate=".$endDate->format('n-j-Y')."&empbadge=&empType=ALL&department=3320&deptGroup=&chargenumber=&salesorder=&description=&workgroup=&workcenter=&activity=&tranType=&operation=&APPROVED=on&UPLOADED=on&SUBMITTED=on&ACTIVE=on&INCOMPLETE=on&ShowBadge=e.USERID&ShowChargeNumber=t.WORKORDER&ShowChargeDate=t.PAYDAY&ShowModifiedAt=t.MODIFIED_AT&ShowHours=t.HOURS&ShowTranDataLabel3=t.tranid&order1=e.USERID&order2=e.USERID&order3=e.USERID&groupby=&setSize=All&NEWWIN=on&ShowDescription=t.DESCRIPTION";
		if (!Debugger::$productionMode) // debug mode
			Debugger::log('URL used for reading from eCharge: ' . $url, Debugger::ERROR);
        $response = $this->curlResource->get($url);
		if (!Debugger::$productionMode) // debug mode
		{
			$path = TEMP_DIR . DIRECTORY_SEPARATOR . 'last-echarge-raw-html-data-'.$startDate->format('j-n-Y').'_to_'.$endDate->format('j-n-Y').'.html';
			if (file_put_contents($path, $response) === FALSE)
				Debugger::log('Failed writing raw output from eCharge to ' . $path, Debugger::ERROR);
			else
				Debugger::log('Written raw output from eCharge to ' . $path, Debugger::DEBUG);
		}
        $this->logger->log('Url for ECharge data reading (from '.$startDate->format('Y-n-j'). ' to '.$endDate->format('Y-n-j'). ') : '.$url, ILogger::DEBUG);
        if ($response)
        {
            if ($response->headers['Status-Code'] == 200)
            {
                // Get title for each row
                preg_match_all("/<th.*?>(.*?)<\/[\s]*th>/i", $response, $matches);
                $row_headers = $matches[1];
                
                // Iterate each row
                preg_match_all("/<tr.*?>(.*?)<\/[\s]*tr>/si", $response, $matches);

                $table = array();
                
                foreach ($matches[1] as $row_html)
                {
                    preg_match_all("/<td.*?>(.*?)<\/[\s]*td>/si", $row_html, $td_matches);
                    $row = array();
                    for ($i = 0; $i < count($td_matches[1]); $i++)
                    {
                        $td = strip_tags(html_entity_decode($td_matches[1][$i]));
                        $row[$row_headers[$i]] = $td;
                    }

                    if (count($row) > 0)
                        $table[] = $row;
                }
                
                array_pop($table); // last record contains totals info
                
                Debugger::log('EChargeWrapper: Actuals received', Debugger::DEBUG);
            }
            else
            {
                Debugger::log('EChargeWrapper: Wrong status code returned: ' . $response->headers['Status-Code'], Debugger::ERROR);
                throw new EChargeWrapperException('Reading actuals from eCharge failed: Wrong status code returned: ' . $response->headers['Status-Code']);
            }
        }
        else
        {
            Debugger::log('Curl: ' . $this->curlResource->error(), Debugger::ERROR);
            throw new EChargeWrapperException('Reading actuals from eCharge failed: ' . $this->curlResource->error());
        }
        
		// a workaround for D&S actuals with different DEPARTMENT than CZ* in ECharge query
		$responseDS = $this->curlResource->get($urlDS);
		if (!Debugger::$productionMode) // debug mode
		{
			$path = TEMP_DIR . DIRECTORY_SEPARATOR . 'last-echarge-raw-html-data-ds-'.$startDate->format('j-n-Y').'_to_'.$endDate->format('j-n-Y').'.html';
			if (file_put_contents($path, $responseDS) === FALSE)
				Debugger::log('Failed writing raw output (D&S) from eCharge to ' . $path, Debugger::ERROR);
			else
				Debugger::log('Written raw output (D&S) from eCharge to ' . $path, Debugger::DEBUG);
		}
        $this->logger->log('Url for ECharge data (D&S) reading (from '.$startDate->format('Y-n-j'). ' to '.$endDate->format('Y-n-j'). ') : '.$urlDS, ILogger::DEBUG);
        if ($responseDS)
        {
            if ($responseDS->headers['Status-Code'] == 200)
            {
                // Get title for each row
                preg_match_all("/<th.*?>(.*?)<\/[\s]*th>/i", $responseDS, $matches);
                $row_headers = $matches[1];
                
                // Iterate each row
                preg_match_all("/<tr.*?>(.*?)<\/[\s]*tr>/si", $responseDS, $matches);
                
                foreach ($matches[1] as $row_html)
                {
                    preg_match_all("/<td.*?>(.*?)<\/[\s]*td>/si", $row_html, $td_matches);
                    $row = array();
                    for ($i = 0; $i < count($td_matches[1]); $i++)
                    {
                        $td = strip_tags(html_entity_decode($td_matches[1][$i]));
                        $row[$row_headers[$i]] = $td;
                    }

                    if (count($row) > 0)
                        // append to the already existing $table
						array_push($table, $row);
                }
                
                array_pop($table); // last record contains totals info
                
                Debugger::log('EChargeWrapper: Actuals (D&S) received', Debugger::DEBUG);
                return $table;
            }
            else
            {
                Debugger::log('EChargeWrapper (D&S): Wrong status code returned: ' . $responseDS->headers['Status-Code'], Debugger::ERROR);
                throw new EChargeWrapperException('Reading actuals (D&S) from eCharge failed: Wrong status code returned: ' . $responseDS->headers['Status-Code']);
            }
        }
        else
        {
            Debugger::log('Curl: ' . $this->curlResource->error(), Debugger::ERROR);
            throw new EChargeWrapperException('Reading actuals (D&S) from eCharge failed: ' . $this->curlResource->error());
        }
    }
	
    /**
     *
     * @param String $startDate Date in format j-n-Y (e.g. 1-31-2012)
     * @param String $endDate Date in format j-n-Y (e.g. 1-31-2012)
     * @return array Array of arrays representing rows where key is TH and value TD
     * @throws EChargeWrapperException
     */
    public function readActualsForTask(DateTime $startDate, DateTime $endDate, Task $task)
    {
        
        if (!$this->loggedIn)
            throw new EChargeWrapperException('You must log in first, use EChargeWrapper->login($username, $password) before reading actuals');
        
        $url = $this->baseUrl . "reports/Misc/AdvChrgForEmp.asp?begindate=".$startDate->format('n-j-Y')."&enddate=".$endDate->format('n-j-Y')."&empbadge=&empType=ALL&department=CZ&deptGroup=&chargenumber=" . $task->getSAPIDWithDash() . "&salesorder=&description=&workgroup=&workcenter=&activity=&tranType=&operation=&APPROVED=on&UPLOADED=on&SUBMITTED=on&ACTIVE=on&INCOMPLETE=on&ShowBadge=e.USERID&ShowChargeNumber=t.WORKORDER&ShowChargeDate=t.PAYDAY&ShowModifiedAt=t.MODIFIED_AT&ShowHours=t.HOURS&ShowTranDataLabel3=t.tranid&order1=e.USERID&order2=e.USERID&order3=e.USERID&groupby=&setSize=All&NEWWIN=on";
		$urlDS = $this->baseUrl . "reports/Misc/AdvChrgForEmp.asp?begindate=".$startDate->format('n-j-Y')."&enddate=".$endDate->format('n-j-Y')."&empbadge=&empType=ALL&department=3320&deptGroup=&chargenumber=" . $task->getSAPIDWithDash() . "&salesorder=&description=&workgroup=&workcenter=&activity=&tranType=&operation=&APPROVED=on&UPLOADED=on&SUBMITTED=on&ACTIVE=on&INCOMPLETE=on&ShowBadge=e.USERID&ShowChargeNumber=t.WORKORDER&ShowChargeDate=t.PAYDAY&ShowModifiedAt=t.MODIFIED_AT&ShowHours=t.HOURS&ShowTranDataLabel3=t.tranid&order1=e.USERID&order2=e.USERID&order3=e.USERID&groupby=&setSize=All&NEWWIN=on";
		if (!Debugger::$productionMode) // debug mode
			Debugger::log('URL used for reading from eCharge: ' . $url, Debugger::ERROR);
        $response = $this->curlResource->get($url);
		if (!Debugger::$productionMode) // debug mode
		{
			$path = TEMP_DIR . DIRECTORY_SEPARATOR . 'last-echarge-raw-html-data-'.$startDate->format('j-n-Y').'_to_'.$endDate->format('j-n-Y').'.html';
			if (file_put_contents($path, $response) === FALSE)
				Debugger::log('Failed writing raw output from eCharge to ' . $path, Debugger::ERROR);
			else
				Debugger::log('Written raw output from eCharge to ' . $path, Debugger::DEBUG);
		}
        $this->logger->log('Url for ECharge data reading (from '.$startDate->format('Y-n-j'). ' to '.$endDate->format('Y-n-j'). ') : '.$url, ILogger::DEBUG);
        if ($response)
        {
            if ($response->headers['Status-Code'] == 200)
            {
                // Get title for each row
                preg_match_all("/<th.*?>(.*?)<\/[\s]*th>/i", $response, $matches);
                $row_headers = $matches[1];
                
                // Iterate each row
                preg_match_all("/<tr.*?>(.*?)<\/[\s]*tr>/si", $response, $matches);

                $table = array();
                
                foreach ($matches[1] as $row_html)
                {
                    preg_match_all("/<td.*?>(.*?)<\/[\s]*td>/i", $row_html, $td_matches);
                    $row = array();
                    for ($i = 0; $i < count($td_matches[1]); $i++)
                    {
                        $td = strip_tags(html_entity_decode($td_matches[1][$i]));
                        $row[$row_headers[$i]] = $td;
                    }

                    if (count($row) > 0)
                        $table[] = $row;
                }
                
                array_pop($table); // last record contains totals info
                
                Debugger::log('EChargeWrapper: Actuals received', Debugger::DEBUG);
            }
            else
            {
                Debugger::log('EChargeWrapper: Wrong status code returned: ' . $response->headers['Status-Code'], Debugger::ERROR);
                throw new EChargeWrapperException('Reading actuals from eCharge failed: Wrong status code returned: ' . $response->headers['Status-Code']);
            }
        }
        else
        {
            Debugger::log('Curl: ' . $this->curlResource->error(), Debugger::ERROR);
            throw new EChargeWrapperException('Reading actuals from eCharge failed: ' . $this->curlResource->error());
        }
		
		// a workaround for D&S actuals with different DEPARTMENT than CZ* in ECharge query
		$responseDS = $this->curlResource->get($urlDS);
		if (!Debugger::$productionMode) // debug mode
		{
			$path = TEMP_DIR . DIRECTORY_SEPARATOR . 'last-echarge-raw-html-data-ds-'.$startDate->format('j-n-Y').'_to_'.$endDate->format('j-n-Y').'.html';
			if (file_put_contents($path, $responseDS) === FALSE)
				Debugger::log('Failed writing raw output (D&S) from eCharge to ' . $path, Debugger::ERROR);
			else
				Debugger::log('Written raw output (D&S) from eCharge to ' . $path, Debugger::DEBUG);
		}
        $this->logger->log('Url for ECharge data (D&S) reading (from '.$startDate->format('Y-n-j'). ' to '.$endDate->format('Y-n-j'). ') : '.$urlDS, ILogger::DEBUG);
        if ($responseDS)
        {
            if ($responseDS->headers['Status-Code'] == 200)
            {
                // Get title for each row
                preg_match_all("/<th.*?>(.*?)<\/[\s]*th>/i", $responseDS, $matches);
                $row_headers = $matches[1];
                
                // Iterate each row
                preg_match_all("/<tr.*?>(.*?)<\/[\s]*tr>/si", $responseDS, $matches);
                
                foreach ($matches[1] as $row_html)
                {
                    preg_match_all("/<td.*?>(.*?)<\/[\s]*td>/i", $row_html, $td_matches);
                    $row = array();
                    for ($i = 0; $i < count($td_matches[1]); $i++)
                    {
                        $td = strip_tags(html_entity_decode($td_matches[1][$i]));
                        $row[$row_headers[$i]] = $td;
                    }

                    if (count($row) > 0)
                        // append to the already existing $table
						array_push($table, $row);
                }
                
                array_pop($table); // last record contains totals info
                
                Debugger::log('EChargeWrapper: Actuals (D&S) received', Debugger::DEBUG);
                return $table;
            }
            else
            {
                Debugger::log('EChargeWrapper (D&S): Wrong status code returned: ' . $responseDS->headers['Status-Code'], Debugger::ERROR);
                throw new EChargeWrapperException('Reading actuals (D&S) from eCharge failed: Wrong status code returned: ' . $responseDS->headers['Status-Code']);
            }
        }
        else
        {
            Debugger::log('Curl: ' . $this->curlResource->error(), Debugger::ERROR);
            throw new EChargeWrapperException('Reading actuals (D&S) from eCharge failed: ' . $this->curlResource->error());
        }
        
    }
    
    public function __destruct() 
	{
		if (file_exists($this->curlCookies))
			unlink($this->curlCookies);
    }
    
}

class EChargeWrapperException extends Exception {}
