<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * iASAP access class for Actual hours
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ActualsReaderSAPID extends IASAPWrapper
{
	/* possible HTS TRV task types */
	const HTS_TRV_UPLIFT = 'Uplift';
	const HTS_TRV_ACTUAL = 'Actual';
	
	/** @var array Array of tasks, task sap id used as mapping keys */
	private $tasks;
	
	/**
	 * @param array $tasks Array of PO numbers used as mapping keys
	 * @param ILogger $logger
	 */
	public function __construct($tasks, ILogger $logger = NULL)
	{
		parent::__construct($logger);
		
		$this->tasks = $tasks;
	}
	
	/**
	 * Gets value of tasks
	 * @return array
	 */
	public function getTasks()
	{
		return $this->tasks;
	}
	
	/**
	 * Reads actual hours from iASAP.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @return array Contains for each task SAP ID set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[PO_number][Person_EID][Fiscal_month<0..11>] = Sum_of_actual_hours
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readLabor(DateTime $start_date, DateTime $end_date)
	{
		$table = array();
		foreach ($this->tasks as $task)
		{
			$db_result = $this->conn
                ->query('
                	SELECT [ps_employee_charge.eId] AS [EID], UNIX_TIMESTAMP([ps_employee_charge.date]) AS [Date],
                	       [ps_employee_charge.hours] AS [Hours]
                	FROM [ps_activity], [ps_employee_charge]
                	WHERE [ps_activity.network] = %s AND
                	      [ps_activity.activityDefinition] = %s AND
                	      [ps_activity.activityId] = [ps_employee_charge.fkActivityId] AND
                	      %i <= UNIX_TIMESTAMP([ps_employee_charge.date]) AND UNIX_TIMESTAMP([ps_employee_charge.date]) <= %i
                	ORDER BY [ps_employee_charge.eId] ASC
                ', $task->Network, $task->Definition, $start_date->format('U'), $end_date->format('U'))
                ->fetchAll();

			if ($db_result)
			{
				foreach ($db_result as $row)
				{
					$eid = $this->repairEID($row->EID);
					$charge_date = new DateTime('@' . $row->Date);
					$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $charge_date->format('U'));				
					if (isset($table[$task->SAPID][$eid][$fiscal_month->Index]))
					{
						$table[$task->SAPID][$eid][$fiscal_month->Index] += $row->Hours;
					}
					else
					    $table[$task->SAPID][$eid][$fiscal_month->Index] = $row->Hours;
				}
			}
		}

		return $table;
	}
	
	/**
	 * Reads actual costs from iASAP.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @return array Contains for each task SAP ID set of costs.
	 *     And for each cost it contains sum of spended cost in defined fiscal months and Currency.
	 *     $table[SAP_ID][Fiscal_month<0..11>] = Sum_of_costs_hours
	 *     $table[SAP_ID][Currency] = Currency
	 *     For empty input set of tasks or in case of error returns empty array.
	 */
	public function readCost(DateTime $start_date, DateTime $end_date)
	{
		$table = array();
		foreach ($this->tasks as $task)
		{
			$sourceTable = 'ps_expense_detail';
			if($this->isHtsTrvUplift($task))
				$sourceTable = 'ps_expense';
			$db_result = $this->conn
                ->query('
                	SELECT [ps_activity.network] AS [Net], [ps_activity.activityDefinition] AS [Def],
                	       UNIX_TIMESTAMP([' . $sourceTable . '.expenseDate]) AS [ExpenseDate],
                	       UNIX_TIMESTAMP([' . $sourceTable . '.inputDate]) AS [InputDate], [' . $sourceTable . '.amount] AS [Amount],
                	       [' . $sourceTable . '.invoiceCurrency] AS [Currency]
                	FROM [ps_activity], [' . $sourceTable . ']
                	WHERE [ps_activity.network] = %s AND
                          [ps_activity.activityDefinition] = %s AND
                          [ps_activity.activityId] = [' . $sourceTable . '.fkActivityId] AND
                          %i <= UNIX_TIMESTAMP([' . $sourceTable . '.expenseDate]) AND UNIX_TIMESTAMP([' . $sourceTable . '.expenseDate]) <= %i
                ', $task->Network, $task->Definition, $start_date->format('U'), $end_date->format('U'))
                ->fetchAll();

			if ($db_result)
			{
				foreach ($db_result as $row)
				{
					$expance_date = new DateTime('@' . $row->ExpenseDate);
					$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $expance_date->format('U'));				
					if (isset($table[$task->SAPID]['Amount']))
					{
						$table[$task->SAPID]['Amount'] += $row->Amount;
					}
					else
					{
						$table[$task->SAPID]['Currency'] = $row->Currency;
						$table[$task->SAPID]['ExpenseDate'] = new DateTime('@' . $row->ExpenseDate);
						$table[$task->SAPID]['InputDate'] = new DateTime('@' . $row->InputDate);
					    $table[$task->SAPID]['FiscalMonth'] = $fiscal_month->Index;
						$table[$task->SAPID]['Amount'] = $row->Amount;
					}
				}
			}
		}
		return $table;
	}
	
	/**
	 * Finds out if given task is HTS TRV uplift
	 * 
	 * 
	 * @param Task $task
	 * @return boolean True if task is uplift otherwise false
	 */
	private function isHtsTrvUplift(Task $task)
	{
		$isUplift = false;
		try
		{
			$db_result = $this->conn
				->query('
					SELECT [ps_activity.l2Description]
					FROM [ps_activity]
					WHERE [ps_activity.network] = %s AND
						  [ps_activity.activityDefinition] = %s
					', $task->Network, $task->Definition)
				->fetch();
			if (strpos($db_result->l2Description, self::HTS_TRV_UPLIFT) !== false)
				$isUplift = true;
		}
        catch (DibiException $e)
        {
            Debugger::log("iASAP cost import cant recognize if task: " . $task->Name . '(' . $task->Network . ' ' . $task->Definition . ') is HTS TRV Uplift or not. ' . $e->getMessage(), Debugger::ERROR);
            return false;
        }
		return $isUplift;
	}
}
