<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils\Logging
 */

/**
 * HTMLTableLogger is providing logging functionality to HTML
 * table file output.
 * 
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Enable setting of styles and message categories (not only error, warning, info, ...)
 */
class HTMLTableLogger extends BatchLogger implements IFileLogger
{

	/** Handle to opened log file */
	private $logfile = NULL;
	/** Path to log file */
	private $logfilePath = NULL;
	
	/**
	 * Creates new logfile object.
	 * 
	 * @param string $filepath Path to file for logging.
	 * @param string $batch_id Identifier of batch.
	 */    
	public function __construct($filepath, $batch_id = NULL)
    {
		$this->logfilePath = $filepath;
		parent::__construct($batch_id);
	}

    
    /**
     * Gets logfilePath.
     * 
     * @return string
     */
    public function getLogfilePath()
    {
        return $this->logfilePath;
    }

	/**
	 * Starts logfile.
	 * 
	 * @param string $title Log title.
	 * @return boolean Returns TRUE if logfile was crated/opened successfully.
	 */
    public function open($title = NULL)
	{
		if ($this->logfile !== NULL) // already opened
			return true;
		$this->logfile = fopen($this->logfilePath, 'w');
		if ($this->logfile !== NULL)
		{
			fwrite($this->logfile, "<html><head><title>".$title."</title></head><body><table>\r\n");
			return true;
		}
		else
			return false;
	}	

	/**
	 * Ends logging.
	 */
    public function close()
	{
		if ($this->logfile !== NULL)
		{
			fwrite($this->logfile, "</table></body></html>\r\n");
			fclose($this->logfile);
			$this->logfile = NULL;
		}
	}
		
	/**
	 * Logs message into log file (if opened).
	 * 
	 * @param string $message Message text.
	 * @param mixed $level message level. Default INFO.
	 * @param string $domain domain of logged message (can be used for grouping or filtering later).
	 * @param mixed $context variable logging context (useful for special log parameters).
	 * @return boolean Returns TRUE if message was written.
	 */
    public function performLog($message, $level = self::INFO, $domain = NULL, $context = NULL)
	{
		$opened = ($this->logfile !== NULL);
		if (!$opened) // if not opened, try to open it
			$opened = $this->open();
		if ($opened)
		{
			switch ($level)
			{
				case self::DEBUG :
					$result = fwrite($this->logfile, '<tr><td style="color:#fff; background-color:#cccccc; font-weight: bold; text-align: center;">debug</td><td style="border-bottom: 1px Solid #dddddd;">' . htmlspecialchars($message) . "</td></tr>\r\n");
					break;
				case self::INFO :
					$result = fwrite($this->logfile, '<tr><td style="color:#fff; background-color:#000000; font-weight: bold; text-align: center;">info</td><td style="border-bottom: 1px Solid #dddddd;">' . htmlspecialchars($message) . "</td></tr>\r\n");
					break;
				case self::NOTICE :
					$result = fwrite($this->logfile, '<tr><td style="color:#fff; background-color:#00ff00; font-weight: bold; text-align: center;">notice</td><td style="border-bottom: 1px Solid #dddddd;">' . htmlspecialchars($message) . "</td></tr>\r\n");
					break;
				case self::WARNING :
					$result = fwrite($this->logfile, '<tr><td style="color:#fff; background-color:#ff9900; font-weight: bold; text-align: center;">Warning</td><td style="border-bottom: 1px Solid #dddddd;">' . htmlspecialchars($message) . "</td></tr>\r\n");
					break;
				case self::ERROR :
					$result = fwrite($this->logfile, '<tr><td style="color:#fff; background-color:#ff0000; font-weight: bold; text-align: center;">ERROR</td><td style="border-bottom: 1px Solid #dddddd;">' . htmlspecialchars($message) . "</td></tr>\r\n");
					break;
				case self::FATAL :
					$result = fwrite($this->logfile, '<tr><td style="color:#fff; background-color:#ff0000; font-weight: bold; text-align: center;">ERROR</td><td style="border-bottom: 1px Solid #dddddd;">' . htmlspecialchars($message) . "</td></tr>\r\n");
					break;
				default:
					$result = fwrite($this->logfile, '<tr><td colspan="2" style="color:#000; background-color:#ddd; font-weight: bold; text-align: center;">' . htmlspecialchars($message) . "</td></tr>\r\n");
			}
			return ($result !== false);
		}
		else
			return false;
	}
	
	public function __destruct()
	{
		$this->close();
	}
	
}

