<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * PlanningHelper provides us with shared methods related to the project charter
 * planning process such as Cost Share calculation.
 *
 * @author Leos Navratil <leos.navratil@honeywell.com>
 */

class PlanningHelper extends Object
{
	/** @var DIContainer DI context */
	private $context;
	/** @var int Current year */
	private $year;
	
	public function __construct(DIContainer $context, $year)
	{
		$this->context = $context;
		$this->year = $year;
	}
	
	/**
	 * Calculates labour rate solely for org units.
	 * 
	 * @param int $orgUnitID Organization Unit ID.
	 * @param int $projectID Project ID.
	 * @return array [LabourRate] => Labour Rate
	 *				 [WorkcenterID] => Work Center ID.
	 * 
	 */
	public function getOrgUnitLabourRate($orgUnitID, $projectID = NULL)
	{
		$result = array('LabourRate' => 0, 'WorkcenterID' => NULL);
		$peopleUnderOrgUnit = $this->context->personRepository->findByOrganizationUnitID($orgUnitID);
		// include TG's leader to the calculation (he or she may not be present in this TG)
		$orgUnit = $this->context->organizationUnitRepository->find($orgUnitID);
		$leader = $this->context->personRepository->find($orgUnit->Leader);
		if (!isset($peopleUnderOrgUnit[$leader->ID]))
			$peopleUnderOrgUnit[$leader->ID] = $leader;
		
		$workCenters = array();
		$workCenterPersonCount = array();

		foreach ($peopleUnderOrgUnit as $person)
		{
			// inactive person or a person shadowing TG
			if (!$person->Status || substr($person->ID, 0, 2) === 'TG')
				continue;

			$workCenters[$person->costCenterID][] = $person;

			if (isset($workCenterPersonCount[$person->costCenterID]))
				$workCenterPersonCount[$person->costCenterID] += 1;
			else
				$workCenterPersonCount[$person->costCenterID] = 1;

		}

		// OrgUnit without any people
		if (count($workCenterPersonCount) < 1)
			return $result;


		$WCswitTheHighestPeopleCount = array_keys($workCenterPersonCount, max($workCenterPersonCount));

		foreach ($WCswitTheHighestPeopleCount as $workCenterID)
		{
			$workCenter = $this->context->costCenterRepository->findCostCenter($workCenterID);
			$tempLaborRate = $workCenter->getYearLabourRate($this->year);
			$tempResult = 0;

			// if we have two or more WCs with the same person count,
			// then take the highest labour rate from them

			for ($i = 1; $i < 13; $i++)
			{
				$tempResult += $tempLaborRate->getMonthlyHourRate($i);
			}
			$tempResult /= 12;

			if ($tempResult > $result['LabourRate'])
			{
				$result['LabourRate'] = $tempResult;
				$result['WorkcenterID'] = $workCenterID;
			}
		}
		
		// If the Project ID is not NULL it means we are using prediction from
		// the planning DB.
		if (!is_null($projectID))
		{
			$projectPlanningCharter = $this->context->projectPlanningCharterRepository->findByProjectIDAndYearAndType($projectID, $this->year, 'HRS');
			
			$rate = $totalHours = 0;
			$planningCharterOrgUnits = $planningCharterPeople = $people = array();
			foreach ($projectPlanningCharter as $planningCharter)
			{
				$planningCharterPeople = array_merge($planningCharterPeople, $this->context->projectPlanningCharterItemRepository->findAllPeopleByProjectPlanningCharterID($planningCharter->ID));
				$planningCharterOrgUnits = array_merge($planningCharterOrgUnits, $this->context->projectPlanningCharterItemRepository->findAllOrgUnitsByProjectPlanningCharterIDByOrganizationUnit_ID($planningCharter->ID, $orgUnitID));
			}
			
			foreach ($planningCharterOrgUnits as $item)
			{
				$budgetResult = $this->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($item->ID);
				$sum = 0;
				foreach ($budgetResult as $budget)
				{
					$sum += $budget->Budget;
				}
				
				$rate += $sum * $result['LabourRate'];
				$totalHours += $sum;
			}
			
			foreach ($planningCharterPeople as $item)
			{
				$person = $this->context->personRepository->find($item->PersonID);
				if ($person->OrganizationUnitID == $orgUnitID)
					$people[$item->ID] = $person;
			}
			
			foreach ($people as $planningCharterItemID => $person)
			{
				$budgetResult = $this->context->projectPlanningCharterItemBudgetRepository->findByProjectPlanningCharterItemID($planningCharterItemID);
				$sum = 0;
				foreach ($budgetResult as $budget)
				{
					$sum += $budget->Budget;
				}
				
				$workCenter = $this->context->costCenterRepository->findCostCenter($person->CostCenterID);
				$laborRate = $workCenter->getYearLabourRate($this->year);
				$rateOfPerson = 0;

				for ($i = 1; $i < 13; $i++)
				{
					$rateOfPerson += $laborRate->getMonthlyHourRate($i);
				}

				$rateOfPerson /= 12;
				
				$rate += $sum * $rateOfPerson;
				$totalHours += $sum;
			}

			if ($totalHours != 0)
			{
				$rate = $rate / $totalHours;
				$result['LabourRate'] = $rate;
			}
		}
		
		return $result;
	}
	
	/**
	 * A helper class used for converting an array of 12 values to
	 * just 4 quartals.
	 * 
	 * @param array $values
	 * @param ProjectCharterBudget $budgetItem
	 */
	public function convertToQuartals($values, $budgetItem)
	{
		// init
		if (!isset($budgetItem->BudgetQ1))
			$budgetItem->BudgetQ1 = 0;
		if (!isset($budgetItem->BudgetQ2))
			$budgetItem->BudgetQ2 = 0;
		if (!isset($budgetItem->BudgetQ3))
			$budgetItem->BudgetQ3 = 0;
		if (!isset($budgetItem->BudgetQ4))
			$budgetItem->BudgetQ4 = 0;
		
		$i = 0;
		foreach ($values as $value)
		{
			$quartal = floor($i / 3) + 1;
			// just add this value up
			switch ($quartal)
			{
				case 1:
					$budgetItem->BudgetQ1 += $value;
					break;
				case 2:
					$budgetItem->BudgetQ2 += $value;
					break;
				case 3:
					$budgetItem->BudgetQ3 += $value;
					break;
				case 4:
					$budgetItem->BudgetQ4 += $value;
					break;
			}
			
			$i++;
		}
		return $budgetItem;
	}
	
	/**
	 * A helper class used for converting an array of 12 values to
	 * just 4 quartals.
	 * 
	 * @param array $values
	 * @param Form $formContainer
	 */
	public function convertToFormQuartals($values, $formContainer)
	{
		$i = 0;
		foreach ($values as $value)
		{
			$quartal = floor($i / 3) + 1;
			// just add this value up
			$formContainer['ownerQ' . $quartal]->setValue($value + $formContainer['ownerQ' . $quartal]->getValue());
			$i++;
		}
	}
}