<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for generating reports of projects to different
 * formats.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectReportPresenter extends Presenter
{
	
	/**
	 * Function translates numeric column index to string name
	 * of the column as is generated in spreadsheet tools.
	 * 
	 * Index is starting from 1. Max index is power of alphabet length (default: 26*26)
	 * 
	 * e.g. for english 26-letter alphabet:
	 * 1 => A
	 * 2 => B
	 * 3 => C
	 * ...
	 * 26 => Z
	 * 27 => AA
	 * 28 => AB
	 * ...
	 * 95 => CR
	 * 
	 * @param int $number Integer greater or equal to one.
	 * @return string
	 */
	private function numToApha($number)
	{
		$alphabet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$alphlen = strlen($alphabet);
		if (is_int($number) && ($number > 0) && ($number <= $alphlen * $alphlen))
		{
			if ($number > $alphlen)
			{
				$prefix_idx = intval($number / $alphlen);
				$idx = $number % $alphlen;
				if ($idx == 0)
					return ($alphabet[$prefix_idx - 2] . $alphabet[$alphlen - 1]);
				else
					return ($alphabet[$prefix_idx - 1] . $alphabet[$idx - 1]);
			}
			else
				return $alphabet[$number - 1];
		}
		else
			throw new InvalidArgumentException("Index must be integer in range <1,".$alphlen * $alphlen.">.");
		
	}

	/**
	 * Function transforms array of row indexes and column index
	 * into SUM expresion usable in spreadsheet processors like MS Excel.
	 * 
	 * Usage:
	 * rowColumnsSumToExpression(5, array(3,4,27))
	 * 
	 * returns string: =SUM(E3,E4,E27)
	 * 
	 * @param int $col_index
	 * @param array $row_index_array	Array of positive integers.
	 * @param string $cell_separator	Cell separator in the SUM expression. Default "+".
	 * @return string
	 */
	private function columnRowsSumExpression($col_index, $row_index_array, $cell_separator = '+')
	{
		if (count($row_index_array) > 0)
		{
			$first = true;
			$expression = '';
			$col = $this->numToApha($col_index);
			foreach ($row_index_array as $index)
			{
				if ($first)
					$first = false;
				else
					$expression .= $cell_separator;
				$expression .= $col . $index;
			}

			return ('=SUM(' . $expression . ')');
		}
		else
			return '';
	}

	/**
	 * Function transforms array of column indexes and row index
	 * into SUM expresion usable in spreadsheet processors like MS Excel.
	 * 
	 * Usage:
	 * rowColumnsSumToExpression(5, array(3,4,27))
	 * 
	 * returns string: =SUM(C5,D5,AA5)
	 * 
	 * @param int $row_index
	 * @param array $column_index_array	Array of positive integers.
	 * @param string $cell_separator	Cell separator in the SUM expression. Default "+".
	 * @return string
	 */
	private function rowColumnsSumExpression($row_index, $column_index_array, $cell_separator = '+')
	{
		if (count($column_index_array) > 0)
		{
			$first = true;
			$expression = '';
			foreach ($column_index_array as $index)
			{
				if ($first)
					$first = false;
				else
					$expression .= $cell_separator;
				$expression .= $this->numToApha($index). $row_index;
			}

			return ('=SUM(' . $expression . ')');
		}
		else
			return '';
	}
    
	/**
     * Set background color of cell
     * 
     * @param type $row Row number.
     * @param type $col Column letter.
     * @param type $changeRow Decide what color will be painted for each row
     * @param type $changeTask Decide what color will be painted for each task.
     * @param type $sheet Active excel sheet.
     */
    private function setColor($row, $col, $changeRow, $changeTask ,$sheet)
    {
        //colors
        $first1 = 'DBE5F1';
        $first2 = 'DBE5F1';
        $first3 = 'DBE5F1';
        $first4 = 'B8CCE4';
        $first5 = 'B8CCE4';
        $second1 = 'F2F2F2';
        $second2 = 'F2F2F2';
        $second3 = 'F2F2F2';
        $second4 = 'D8D8D8';
        $second5 = 'D8D8D8';
        
        if ($changeTask % 2 == 0)
        {
            if ($changeRow == 0)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($first1);
            elseif ($changeRow == 1)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($first2);
            elseif($changeRow == 2)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($first3);    
            elseif($changeRow == 3)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($first4);    
            else
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($first5);    
        }
        else
        {
            if ($changeRow == 0)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($second1);
            elseif ($changeRow == 1)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($second2);
            elseif($changeRow == 2)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($second3);
            elseif($changeRow == 3)
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($second4);    
            else
                $sheet->getStyle($row . $col)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($second5);    
        }
    }
    
    /**
     * Exports report of project worker assignments
	 * with assigned hours, planned hours and actual worked hours as a 
	 * XLSX (MS Excel).
     * 
     * @param int $project_id Id of the project to export.
     * @param int $year Year for actuals (plans).
     * @param string $token String value for freeze user interface during report generation.
     */
	public function renderProjectAssignmentsAndPlansWithActualsXLS($project_id, $year, $token) 
	{
		// checking of inputs
		if (!is_numeric($year) || ((0 + $year) != $year))
		{
			$this->flashMessage('Invalid year argument. Cannot render report.', 'error');
			$this->redirect('default');
		}
		$project = $this->context->projectRepository->findProject($project_id);
		if ($project === NULL)
		{
			$this->flashMessage('Project with ID='.$project_id.' not found. Cannot render report.', 'error');
			$this->redirect('default');
		}

		try
		{
			// MS Excel file as output
            $objPHPExcel = new PHPExcel();
			$this->generateProjectAssignmentsAndPlansWithActualsRowStyledXLS($project, $year, $objPHPExcel, $token);          
		}
		catch (IOException $e)
		{
			$this->flashMessage('File input error while generating report.', 'error');
			$this->redirect('default');
		}
	}

    /**
     * Exports report of project worker assignments
	 * with assigned hours, planned hours and EAC worked hours as a 
	 * XLSX (MS Excel).
     * 
     * @param int $project_id Id of the project to export.
     * @param int $year Year for actuals (plans).
     * @param string $token String value for freeze user interface during report generation.
     */
	public function renderProjectAssignmentsAndPlansWithEACXLS($project_id, $year, $token) 
	{
		// checking of inputs
		if (!is_numeric($year) || ((0 + $year) != $year))
		{
			$this->flashMessage('Invalid year argument. Cannot render report.', 'error');
			$this->redirect('default');
		}
		$project = $this->context->projectRepository->findProject($project_id);
		if ($project === NULL)
		{
			$this->flashMessage('Project with ID='.$project_id.' not found. Cannot render report.', 'error');
			$this->redirect('default');
		}

		try
		{
			// MS Excel file as output
            $objPHPExcel = new PHPExcel();
			$this->generateProjectAssignmentsAndPlansWithEACRowStyledXLS($project, $year, $objPHPExcel, $token);          
		}
		catch (IOException $e)
		{
			$this->flashMessage('File input error while generating report.', 'error');
			$this->redirect('default');
		}
	}
    
    /**
     * Exports report of project plan (from SAP)
	 * with planned hours and actual worked hours as a 
	 * XLSX (MS Excel).
     * 
     * @param int $project_id Id of the project to export.
     * @param int $year Year for actuals (plans).
     */
	public function renderProjectPlanWithActualsXLS($project_id, $year) 
	{
		// checking of inputs
		if (!is_numeric($year) || ((0 + $year) != $year))
		{
			$this->flashMessage('Invalid year argument. Cannot render report.', 'error');
			$this->redirect('default');
		}
		$project = $this->context->projectRepository->findProject($project_id);
		if ($project === NULL)
		{
			$this->flashMessage('Project with ID='.$project_id.' not found. Cannot render report.', 'error');
			$this->redirect('default');
		}
		try
		{
            // MS Excel file as output
            $objPHPExcel = new PHPExcel();
			$this->generateProjectPlanWithActualsRowStyledXLS($project, $year, $objPHPExcel);
		}
		catch (IOException $e)
		{
			$this->flashMessage('File input error while generating report.', 'error');
			$this->redirect('default');
		}
	}

    /**
     * Exports report with EAC for tasks in project in TSV (Tab-Separated-Values file).
     * 
     * @param int $project_id Id of the project to export.
     * @param int $year Year for EAC export.
     */
	public function renderProjectTaskEACTSV($project_id, $year) 
	{
		// checking of inputs
		if (!is_numeric($year) || ((0 + $year) != $year))
		{
			$this->flashMessage('Invalid year argument. Cannot render report.', 'error');
			$this->redirect('default');
		}
		// year has to be the same as current year
		$current_fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp(time());
		if ($current_fiscal_month->getYear() < $year)
		{
			$this->flashMessage('Cannot generate Task EACs report for finished years.', 'info');
			$this->redirect('default');
		}
		$project = $this->context->projectRepository->findProject($project_id);
		if ($project === NULL)
		{
			$this->flashMessage('Project with ID='.$project_id.' not found. Cannot render report.', 'error');
			$this->redirect('default');
		}
		$temp_file_name = tempnam(sys_get_temp_dir(), 'pea');
		try
		{
			// Tab-Separated-Values file with MS Windows style EOLs as output
			$tsv_writer = new DSVFileWriter($temp_file_name, TRUE, "\t", '', DSVFileWriter::WIN_EOL);
			//$this->generateProjectPlanWithActualsColStyledTSV($project, $year, $tsv_writer);
			$this->generateProjectTasksEACColStyledTSV($project, $current_fiscal_month, $tsv_writer);
			// we will send generated output file
			$temp_file_response = new TemporaryFileResponse($temp_file_name, $project->Name.' - Task EACs - '.$year.'.tsv', 'text/tab-separated-values');
			$this->sendResponse($temp_file_response);
		}
		catch (IOException $e)
		{
			$this->flashMessage('File input error while generating report.', 'error');
			$this->redirect('default');
		}
	}

     /**
     * Generates report of project plan (from SAP)
	 * with planned hours and actual worked hours as a 
	 * XLSX (MS-EXCEL file).
	 * 
	 * Actual worked hours and actual spendings are generated into additional rows.
     * 
     * @param Project $project Project to export.
     * @param int $year Year for actuals (plans).
     * @param PHHPExcel $objPHPExcel Output file.
	 * @throws IOException if there is problem with opening output file.
     */
    public function generateProjectPlanWithActualsRowStyledXLS (Project $project, $year, PHPExcel $objPHPExcel)
    {
        // SETUP PROPERTIES
        $project_id = $project->ID;
        $row_triplet = array('plan' => 'Plan', 'actuals' => 'Actuals', 'delta' => 'Delta');
        $months = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
        $currency = 'USD';
		$hours_short = 'hrs';        
        
        //SETUP PROPORTIES FOR $sheet - Assigned Resources and Actuals
        $row_index = 6;        
        $firstCol = "A";
        $hrsCol = "B";
        $firstMonthColIdx = 3;
        $totalCol = "O";
        $unitCol = "P"; // last column for efort, costs, hon investment
        $costShareCol = "Q"; 
        $sapIDCol = "R"; // last column for project plan    
        $q1Col = "C";
        $q1Co2 = "D";
        $q1Co3 = "E";
        $q1Co4 = "F"; 
        
        $sum_column_indexes = array();
		for ($i = 0; $i < 12; $i++)
			$sum_column_indexes[] = $firstMonthColIdx + $i;
        
        // GENERAL INFORMATIONS - hardcoded position
        $objPHPExcel->getActiveSheet()->setTitle("SAP Plan and Actuals");
        $sheet = $objPHPExcel->getActiveSheet();
        $sheet->setCellValue("A1", $project->Name . ' (SAP Plan and Actuals) ' . $year);
        $sheet->setCellValue("A3", 'SAP Project ID:');
        $sheet->setCellValue("B3", $project->SAP_ID);
        $sheet->setCellValue("A4", 'SAP Schedule export date:');
        $sheet->setCellValue("B4", date('m/d/Y h:i:s A'));
        
        // FORMAT SETTINGS
        $sheet->getStyle('A1')->getFont()->setBold(true);
        $sheet->getStyle('A3:B3')->getFont()->setBold(true);
        $sheet->getStyle('A4:B4')->getFont()->setBold(true);
        $sheet->getColumnDimension($firstCol)->setWidth(40);
        $sheet->getColumnDimension($totalCol)->setWidth(12);
        $sheet->getColumnDimension($costShareCol)->setWidth(12); 
        $sheet->getColumnDimension($sapIDCol)->setWidth(18);        
        $sheet->setShowSummaryBelow(false);    
        $borderTask = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderPerson = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
                
        // PROJECT PLAN - HEADER    
        $sheet->setCellValue($firstCol . $row_index, "--- " . $year . " PROJECT PLAN ---");
        $sheet->setCellValue($hrsCol . $row_index, "Hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->setCellValue($costShareCol . $row_index, "CostShare");
        $sheet->setCellValue($sapIDCol . $row_index, "SAP ID");
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getFont()->setBold(true); 
        $row_index++;
        
        // let's find planned tasks
		$tasks = $this->context->taskRepository->findPlannedProjectTasksInYear($project_id, $year);
		// let's find active tasks missing in plan
		$active_tasks = $this->context->taskRepository->findActiveProjectTasksInYear($project_id, $year);
		foreach ($active_tasks as $active_task_id => $active_task)
			if (!isset($tasks[$active_task_id]))
				$tasks[$active_task_id] = $active_task;
		$active_tasks = NULL; // useless data
		// sort tasks by name
		uasort($tasks, array($this, 'sortTasksByName'));
		// every non-labour task has its prefix with special meaning
		// non-labour tasks with same prefix are summarized separately
		$other_costs = array();
		foreach ($tasks as $task_id => $task)
			if (!$task->isLabourTask())
			{
				$non_labour_task_prefix = substr($task->Name, 0, 3); 
				if (!in_array($non_labour_task_prefix, $other_costs))
					$other_costs[] = $non_labour_task_prefix;
			}
                
        // PROCESS TASKS
		$task_row_indexes = array();
        $changeTask = 0;
		foreach ($tasks as $task_id => $task)
		{            
			$task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
			if ($task_plan === NULL) // task is not planned but has some work/costs for this year
				$task_plan = new TaskYearPlan($year);
			// work for labour tasks
			$task_work = $this->context->workRepository->findTaskYearWork($task_id, $year);
			$task_plan_values = $task_actuals = $task_deltas = array();
            if ($task->isLabourTask())
            {
			for ($i = 1; $i <= 12; $i++)
			{
				if ($task->isLabourTask())
				{
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Hours;
					$task_actuals[] = $task_work[$i];
					$task_deltas[] = $task_plan->getMonthPlan($i)->Hours - $task_work[$i];
				}
				else
				{
					$monthly_task_costs = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task_id, $i, $year, $currency);
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Budget;
					$task_actuals[] = $monthly_task_costs;
					$task_deltas[] = $task_plan->getMonthPlan($i)->Budget - $monthly_task_costs;
				}
			}
            
            $changeRow = 0;
            $sheet->setCellValue($firstCol . $row_index, $task->Name);
            $this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->setCellValue($hrsCol . $row_index, 'Plan');
            $this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);            
            for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_plan_values[$i]);
                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
            }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->setCellValue($unitCol . $row_index, ($task->isLabourTask() ? $hours_short : $currency));
            $this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
            $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);                     
            $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
            $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);                                   
            $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);                       
			$task_row_indexes[$task->ID]['plan'] = $row_index;
			$changeRow++;
            $row_index++;
            
            $sheet->setCellValue($firstCol . $row_index, $task->Name);
            $this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
            $sheet->setCellValue($hrsCol . $row_index, 'Actuals');
            $this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
            for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_actuals[$i]);
                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);                
            }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->setCellValue($unitCol . $row_index, ($task->isLabourTask() ? $hours_short : $currency));
            $this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
            $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);                     
            $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);                                                         
			$task_row_indexes[$task->ID]['actuals'] = $row_index;
            $changeRow++;
			$row_index++;
            
            $sheet->setCellValue($firstCol . $row_index, $task->Name);
            $this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
            $sheet->setCellValue($hrsCol . $row_index, 'Delta');
            $this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
            for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_deltas[$i]);
                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
                
            }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->setCellValue($unitCol . $row_index, ($task->isLabourTask() ? $hours_short : $currency));
            $this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet); 
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
            $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);                     
            $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);                                                         
			$task_row_indexes[$task->ID]['delta'] = $row_index;
			$changeRow++;
            $row_index++;
            $changeTask++;
            }
            else 
            {
			for ($i = 1; $i <= 12; $i++)
			{
				if ($task->isLabourTask())
				{
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Hours;
					$task_actuals[] = $task_work[$i];
					$task_deltas[] = $task_plan->getMonthPlan($i)->Hours - $task_work[$i];
				}
				else
				{
					$monthly_task_costs = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task_id, $i, $year, $currency);
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Budget;
					$task_actuals[] = $monthly_task_costs;
					$task_deltas[] = $task_plan->getMonthPlan($i)->Budget - $monthly_task_costs;
				}
			}
            
            $changeRow = 0;
            $sheet->setCellValue($firstCol . $row_index, $task->Name);
            $sheet->setCellValue($hrsCol . $row_index, 'Plan');           
            for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_plan_values[$i]);
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, ($task->isLabourTask() ? $hours_short : $currency));
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);                   
            $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
            $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);                                 
            $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);                       
			$task_row_indexes[$task->ID]['plan'] = $row_index;
            $row_index++;
            
            $sheet->setCellValue($firstCol . $row_index, $task->Name);
            $sheet->setCellValue($hrsCol . $row_index, 'Actuals');
            for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_actuals[$i]);               
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, ($task->isLabourTask() ? $hours_short : $currency));
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);                                                                              
			$task_row_indexes[$task->ID]['actuals'] = $row_index;
			$row_index++;
            
            $sheet->setCellValue($firstCol . $row_index, $task->Name);
            $sheet->setCellValue($hrsCol . $row_index, 'Delta');
            for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_deltas[$i]);
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, ($task->isLabourTask() ? $hours_short : $currency));
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);                                                                            
			$task_row_indexes[$task->ID]['delta'] = $row_index;
            $row_index++;
            }
		}
                          
        //SETUP PROPORTIES FOR $sheet2 - Labour rates        
        $row_index2 = 1;
        $firstCol2 = "A";
        $firstMonthColIdx2 = 2;
        $unitCol2 = "N"; // last column for labour rates 
        
        // COST CENTERS / LABOUR RATES
        $sheet2 = $objPHPExcel->createSheet(1);
        $sheet2->setTitle("Labour rates");        
        $sheet2->getColumnDimension($firstCol2)->setWidth(35);
        $sheet2->setCellValue($firstCol2 . $row_index2, "--- LABOUR RATES ---");
        $sheet2->setCellValue($unitCol2 . $row_index2, "Unit");
        for($i = 0; $i<12; $i++)
        {
            $sheet2->setCellValue($this->numToApha($i+$firstMonthColIdx2) . $row_index2, $months[$i]);
        }
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getFont()->setBold(true);
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index2++;
        
        $labour_rates_row_indexes = array();
        $labour_centers = $this->context->costCenterRepository->findAllCostCenters();      
        foreach ($labour_centers as $center)
		{
			$rate = $center->getYearLabourRate($year);
			if ($rate != NULL)
			{
                $sheet2->setCellValue($firstCol2 . $row_index2, $center->Name);				
                for ($i = 1; $i <= 12; $i++)
                    $sheet2->setCellValue($this->numToApha($i+1) . $row_index2, $rate->getMonthlyHourRate($i));
                $sheet2->setCellValue($unitCol2 . $row_index2, $currency . '/' . $hours_short);
               $labour_rates_row_indexes[$center->ID] = $row_index2;
                $row_index2++;                
            }              
		}        
        
        //EFFORT
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- EFFORT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
		$effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
						$effort_sum_indexes['delta'][] = $task_row_indexes[$task->ID]['delta'];
					}
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
                    $sheet->setCellValue($hrsCol . $row_index, $title);					
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);					
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					if (count($effort_sum_indexes[$key]) > 0)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $hours_short);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$effort_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, ">> TOTAL <<");
				$sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
				 $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $hours_short);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // COSTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- COSTS ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta'] = array();
			// remember indexes of row which belong to this labour center
            foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
						$effort_sum_indexes['delta'][] = $task_row_indexes[$task->ID]['delta'];
					}
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);					
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					if (count($effort_sum_indexes[$key]) > 0)                      
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]) . "*'Labour rates'!" . $this->numToApha ($firstMonthColIdx2 + $i) . $labour_rates_row_indexes[$center->ID]);                        
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $costs_row_indexes[$center->ID][$key] = $row_index;
            $effort_total_indexes[$key][] = $row_index;
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;            
            }
			$row_for_total_indexes[] = $row_index;            
		}
        foreach ($other_costs as $task_prefix)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)
				if (!$task->isLabourTask())
					// same type of non-labour tasks
					if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
						$effort_sum_indexes['delta'][] = $task_row_indexes[$task->ID]['delta'];
					}
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);	
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					if (count($effort_sum_indexes[$key]) > 0)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
				$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$effort_total_indexes[$key][] = $row_index;
				$costs_row_indexes[$task_prefix][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
			$row_for_total_indexes[] = $row_index;
		}
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);	
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // HONEYWELL INVESTMENTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- HON INVESTMENT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $row_for_total_indexes = array();
		$costs_hon_row_indexes = array();
		foreach ($labour_centers as $center)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					$cell_value = '';
					foreach ($tasks as $task_id => $task)
						if ($task->isLabourTask())
							if ($task->CostCenterID == $center->ID)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					if ($cell_value != '')                    
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, "=(".$cell_value.")*'Labour rates'!".$this->numToApha($firstMonthColIdx2 + $i).$labour_rates_row_indexes[$center->ID]);                                      
                    else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
				// totals for a center
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row_for_total_indexes[$key][] = $row_index;
				$costs_hon_row_indexes[$center->ID][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
        foreach ($other_costs as $task_prefix)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					$cell_value = '';
					foreach ($tasks as $task_id => $task)
					{
						if (!$task->isLabourTask())
							// same type of non-labour tasks
							if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 * 3 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					}
					if ($cell_value != '')
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '='.$cell_value);						
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
				// totals for a non-labor tasks type
				$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row_for_total_indexes[$key][] = $row_index;
				$costs_hon_row_indexes[$task_prefix][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);				
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // QUARTERLY SPEND PROFILE
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY SPEND PROFILE ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
		// prepare summary traversal array
		$summary = array();
		foreach ($labour_centers as $center)
			$summary[$center->ID] = $center->Name;
		foreach ($other_costs as $task_prefix)
			$summary[$task_prefix] = $task_prefix;
		$row_for_total_indexes = array();
		foreach ($summary as $cost_row_index => $summary_name)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
                    $sheet->setCellValue($firstCol . $row_index, $summary_name);
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				// 4 quarters of year, each has 3 months
				for ($i = 0; $i < 4; $i++)
				{
					$costs_col_sum_indexes = array();
					for ($j = 0; $j < 3; $j++)
						$costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
					$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                }
				$row_for_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;
		}
        
        // QUARTERLY HON INVEST
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY HON INVEST ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
		$row_for_total_indexes = array();
		foreach ($summary as $cost_row_index => $summary_name)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
                    $sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
                    $sheet->setCellValue($firstCol . $row_index, $summary_name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				// 4 quarters of year, each has 3 months
				for ($i = 0; $i < 4; $i++)
				{
					$costs_col_sum_indexes = array();
					for ($j = 0; $j < 3; $j++)
						$costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_hon_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
				}
				$row_for_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
				$sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
			$sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="' . $project->Name . ' - SAP Plan and Actuals - ' . $year . '.xlsx"');
        header('Cache-Control: max-age=0');
        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
        ob_end_clean();
        $objWriter->save('php://output');
        exit;
    }
    
     /**
     * Generates report of project worker assignments
	 * with assigned hours and actual worked hours as a 
	 * XLSX (MS-EXCEL file).
	 * 
	 * Actual worked hours and actual spendings are generated into additional rows.
     * 
     * @param Project $project Project to export.
     * @param int $year Year for actuals (plans).
     * @param PHHPExcel $objPHPExcel Output file.
	 * @throws IOException if there is problem with opening output file.
     */
    private function generateProjectAssignmentsWithActualsRowStyledXLS (Project $project, $year, PHPExcel $objPHPExcel)
    {
        // SETUP PROPERTIES
        $project_id = $project->ID;
        $row_triplet = array('plan' => 'Assigned', 'actuals' => 'Actuals', 'delta' => 'Delta');
        $months = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
        $currency = 'USD';
		$hours_short = 'hrs';        
        
        //SETUP PROPORTIES FOR $sheet - Assigned Resources and Actuals
        $row_index = 6;        
        $firstCol = "A";
        $hrsCol = "B";
        $firstMonthColIdx = 3;
        $totalCol = "O";
        $unitCol = "P"; // last column for efort, costs, hon investment
        $costShareCol = "Q"; 
        $sapIDCol = "R"; // last column for project plan    
        $q1Col = "C";
        $q1Co2 = "D";
        $q1Co3 = "E";
        $q1Co4 = "F";       
        
        $sum_column_indexes = array();
		for ($i = 0; $i < 12; $i++)
			$sum_column_indexes[] = $firstMonthColIdx + $i;
        
        // GENERAL INFORMATIONS - hardcoded position
        $objPHPExcel->getActiveSheet()->setTitle("Assigned Resources and Actuals");
        $sheet = $objPHPExcel->getActiveSheet();
        $sheet->setCellValue("A1", $project->Name . ' (Assigned Resources and Actuals) ' . $year);
        $sheet->setCellValue("A3", 'SAP Project ID:');
        $sheet->setCellValue("B3", $project->SAP_ID);
        $sheet->setCellValue("A4", 'SAP Schedule export date:');
        $sheet->setCellValue("B4", date('m/d/Y h:i:s A'));
        
        // FORMAT SETTINGS
        $sheet->getStyle('A1')->getFont()->setBold(true);
        $sheet->getStyle('A3:B3')->getFont()->setBold(true);
        $sheet->getStyle('A4:B4')->getFont()->setBold(true);
        $sheet->getColumnDimension($firstCol)->setWidth(40);
        $sheet->getColumnDimension($totalCol)->setWidth(12);
        $sheet->getColumnDimension($costShareCol)->setWidth(12); 
        $sheet->getColumnDimension($sapIDCol)->setWidth(18);        
        $sheet->setShowSummaryBelow(false);
        $borderTask = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderPerson = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
                
        // PROJECT PLAN - HEADER    
        $sheet->setCellValue($firstCol . $row_index, "--- " . $year . " PROJECT PLAN ---");
        $sheet->setCellValue($hrsCol . $row_index, "Hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->setCellValue($costShareCol . $row_index, "CostShare");
        $sheet->setCellValue($sapIDCol . $row_index, "SAP ID");
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getFont()->setBold(true); 
        $row_index++;

        // at first lets find assigned labour tasks
        $tasks = $this->context->taskRepository->findAssignedProjectTasksInYear($project_id, $year); // no non-labour task returned
        $planned_tasks = $this->context->taskRepository->findPlannedProjectTasksInYear($project_id, $year); // planned labour and non-labor
        
        // let's find active tasks missing in plan and task on which non-assigned people worked
        $active_tasks = $this->context->taskRepository->findActiveProjectTasksInYear($project_id, $year);
        foreach ($active_tasks as $active_task_id => $active_task)
        {
            if ($active_task->isLabourTask())
            {
                if (!isset($tasks[$active_task_id]))
                    $tasks[$active_task_id] = $active_task;
            }
            else
            {
                if (!isset($planned_tasks[$active_task_id]))
                    $planned_tasks[$active_task_id] = $active_task;
            }
        }
        $active_tasks = NULL; // useless data
        
        // sort tasks by name
        uasort($planned_tasks, array($this, 'sortTasksByName'));
    
        // then non-labour tasks
        // every non-labour task has its prefix with special meaning
        // non-labour tasks with same prefix are summarized separately
        $other_costs = array();
        foreach ($planned_tasks as $task_id => $task)
            if (!$task->isLabourTask())
            {
                $tasks[$task_id] = $task;
                $non_labour_task_prefix = substr($task->Name, 0, 3); 
                if (!in_array($non_labour_task_prefix, $other_costs))
    				$other_costs[] = $non_labour_task_prefix;
    		}
    	$planned_tasks = null; // useless now
        
        // PROCESS TASKS
        $task_row_indexes = array();
        $workers = array(); // array of Person - list of working labourers
        $changeTask = 0;
        foreach ($tasks as $task_id => $task)
        {                      
            if ($task->isLabourTask())
            {
                // work for labour tasks
				//$task_work = $this->context->workRepository->findTaskYearWork($task_id, $year);
				$worker_assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($task->ID, $year);
				// find people not assigned to the task but working on it
				$all_task_worker_ids = $this->context->workRepository->findPeopleWorkingOnTaskInYear($task->ID, $year);
				// let's create dummy/empty assignment for workers not assigned but working on a task
				for ($i = 0; $i < count($all_task_worker_ids); $i++)
					if (!isset($worker_assignments[$all_task_worker_ids[$i]]))
					{
						$year_assignment = array();
						for ($j = 1; $j <= 12; $j++)
						{
							$year_assignment[$j] = new PersonTaskMonthAssignment();
							$year_assignment[$j]->PersonID = $all_task_worker_ids[$i];
							$year_assignment[$j]->Month = $j;
							$year_assignment[$j]->Year = $year;
							$year_assignment[$j]->TaskID = $task->ID;
						}
						$worker_assignments[$all_task_worker_ids[$i]] = $year_assignment;                        
					}                
				$worker_assignments_count = count($worker_assignments);
				// we will prepare indexes of worker rows that will be summed on current task summary plan/actuals/delta rows
				$worker_row_indexes = array();
				$triplet_keys = array_keys($row_triplet);
				for ($i = 0; $i < count($triplet_keys); $i++)
					for ($j = 0; $j < $worker_assignments_count; $j++)
						$worker_row_indexes[$triplet_keys[$i]][] = $row_index + count($triplet_keys) + $j * count($triplet_keys) + $i;
				// task sum row triplet
				$task_name_written = true;                
                $changeRow = 0;      
				foreach ($row_triplet as $key => $title)
				{
                    $sheet->setCellValue($firstCol . $row_index, $task->Name);
                    $this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
					$sheet->setCellValue($hrsCol . $row_index, $title);
                    $this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                    for ($i = 0; $i < 12; $i++)
                    {
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $worker_row_indexes[$key]));
                        $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);                           
                    }					
                    // total
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
					if ($task_name_written)
                    {
                        $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
                        $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);                     
                        $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
                        $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                        $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);                       
                        $task_name_written = false;
                        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);                       
                    }
                    $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                    $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                    $changeRow++;
                    $task_row_indexes[$task->ID][$key] = $row_index;
                    $row_index++;
				}                
                foreach ($worker_assignments as $person_id => $assignments)
				{
					if (!isset($workers[$person_id]))
						$workers[$person_id] = $this->context->personRepository->find($person_id);
					$actuals = $this->context->workRepository->findPersonTaskYearWork($person_id, $task->ID, $year);
					// assignment/plan
					if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
                    $sheet->setCellValue($hrsCol . $row_index, $row_triplet['plan']);
                    for ($i = 1; $i <= 12; $i++)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $assignments[$i]->Hours);
                    // assignment/plan total for a worker
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $setBorder = true;
                    if ($setBorder)
                    {
                        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderPerson);
                        $setBorder = false;
                    }
                    $row_index++;
                    // actuals
                    if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
                   $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
					for ($i = 1; $i <= 12; $i++)
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $actuals[$i]);
                    // actuals total for a worker
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
                    //delta
                    if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
					$sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta']);
					for ($i = 1; $i <= 12; $i++)
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $assignments[$i]->Hours - $actuals[$i]);
					// delta total for a worker
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;            
                }
            }            
            else // non-labour task
            {          
                $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
				if ($task_plan === NULL) // task is not planned but has some costs for this year
					$task_plan = new TaskYearPlan($year);
				$task_plan_values = $task_actuals = $task_deltas = array();
				for ($i = 1; $i <= 12; $i++)
				{
					$monthly_task_costs = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task_id, $i, $year, $currency);
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Budget;
					$task_actuals[] = $monthly_task_costs;
					$task_deltas[] = $task_plan->getMonthPlan($i)->Budget - $monthly_task_costs;
				}
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['plan']);
                for ($i = 0; $i <= 11; $i++)
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_plan_values[$i]);
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
                $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
                $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);
                $task_row_indexes[$task->ID]['plan'] = $row_index;
                $row_index++;
                
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
                for ($i = 0; $i <= 11; $i++)
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_actuals[$i]);
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['actuals'] = $row_index;
                $row_index++;
                
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
                for ($i = 0; $i <= 11; $i++)
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_deltas[$i]);
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['delta'] = $row_index;
                $row_index++;
            }
            $changeTask++;
        }  
        
        //SETUP PROPORTIES FOR $sheet2 - Labour rates        
        $row_index2 = 1;
        $firstCol2 = "A";
        $firstMonthColIdx2 = 2;
        $unitCol2 = "N"; // last column for labour rates 
        
        // COST CENTERS / LABOUR RATES
        $sheet2 = $objPHPExcel->createSheet(1);
        $sheet2->setTitle("Labour rates");        
        $sheet2->getColumnDimension($firstCol2)->setWidth(35);
        $sheet2->setCellValue($firstCol2 . $row_index2, "--- LABOUR RATES ---");
        $sheet2->setCellValue($unitCol2 . $row_index2, "Unit");
        for($i = 0; $i<12; $i++)
        {
            $sheet2->setCellValue($this->numToApha($i+$firstMonthColIdx2) . $row_index2, $months[$i]);
        }
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getFont()->setBold(true);
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index2++;
        
        $labour_rates_row_indexes = array();
        $labour_centers = $this->context->costCenterRepository->findAllCostCenters();      
        foreach ($labour_centers as $center)
		{
			$rate = $center->getYearLabourRate($year);
			if ($rate != NULL)
			{
                $sheet2->setCellValue($firstCol2 . $row_index2, $center->Name);				
                for ($i = 1; $i <= 12; $i++)
                    $sheet2->setCellValue($this->numToApha($i+1) . $row_index2, $rate->getMonthlyHourRate($i));
                $sheet2->setCellValue($unitCol2 . $row_index2, $currency . '/' . $hours_short);
               $labour_rates_row_indexes[$center->ID] = $row_index2;
                $row_index2++;                
            }              
		}
        
        //EFFORT
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- EFFORT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);        
        $row_index++;                
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
		$effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
						$effort_sum_indexes['delta'][] = $task_row_indexes[$task->ID]['delta'];
					}
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
                    $sheet->setCellValue($hrsCol . $row_index, $title);					
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);					
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					if (count($effort_sum_indexes[$key]) > 0)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $hours_short);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$effort_total_indexes[$key][] = $row_index;                                
                $row_index++;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                
			}        
		}
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, ">> TOTAL <<");
				$sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
				 $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $hours_short);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // COSTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- COSTS ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta'] = array();
			// remember indexes of row which belong to this labour center
            foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
						$effort_sum_indexes['delta'][] = $task_row_indexes[$task->ID]['delta'];
					}
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);					
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					if (count($effort_sum_indexes[$key]) > 0)                      
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]) . "*'Labour rates'!" . $this->numToApha ($firstMonthColIdx2 + $i) . $labour_rates_row_indexes[$center->ID]);                        
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $costs_row_indexes[$center->ID][$key] = $row_index;
            $effort_total_indexes[$key][] = $row_index;
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;            
            }
			$row_for_total_indexes[] = $row_index; 
		}
        foreach ($other_costs as $task_prefix)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)
				if (!$task->isLabourTask())
					// same type of non-labour tasks
					if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
						$effort_sum_indexes['delta'][] = $task_row_indexes[$task->ID]['delta'];
					}
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);	
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					if (count($effort_sum_indexes[$key]) > 0)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
				$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$effort_total_indexes[$key][] = $row_index;
				$costs_row_indexes[$task_prefix][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
			$row_for_total_indexes[] = $row_index;            
		}

		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);	
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // HONEYWELL INVESTMENTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- HON INVESTMENT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);        
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $row_for_total_indexes = array();
		$costs_hon_row_indexes = array();
		foreach ($labour_centers as $center)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					$cell_value = '';
					foreach ($tasks as $task_id => $task)
						if ($task->isLabourTask())
							if ($task->CostCenterID == $center->ID)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					if ($cell_value != '')                    
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, "=(".$cell_value.")*'Labour rates'!".$this->numToApha($firstMonthColIdx2 + $i).$labour_rates_row_indexes[$center->ID]);                                      
                    else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
				// totals for a center
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row_for_total_indexes[$key][] = $row_index;
				$costs_hon_row_indexes[$center->ID][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
        foreach ($other_costs as $task_prefix)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					$cell_value = '';
					foreach ($tasks as $task_id => $task)
					{
						if (!$task->isLabourTask())
							// same type of non-labour tasks
							if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 * 3 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					}
					if ($cell_value != '')
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '='.$cell_value);						
					else
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
				}
				// totals for a non-labor tasks type
				$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row_for_total_indexes[$key][] = $row_index;
				$costs_hon_row_indexes[$task_prefix][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);				
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // QUARTERLY SPEND PROFILE
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY SPEND PROFILE ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
		// prepare summary traversal array
		$summary = array();
		foreach ($labour_centers as $center)
			$summary[$center->ID] = $center->Name;
		foreach ($other_costs as $task_prefix)
			$summary[$task_prefix] = $task_prefix;
		$row_for_total_indexes = array();
		foreach ($summary as $cost_row_index => $summary_name)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
                    $sheet->setCellValue($firstCol . $row_index, $summary_name);
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				// 4 quarters of year, each has 3 months
				for ($i = 0; $i < 4; $i++)
				{
					$costs_col_sum_indexes = array();
					for ($j = 0; $j < 3; $j++)
						$costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
					$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                }
				$row_for_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;
		}
        
        // QUARTERLY HON INVEST
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY HON INVEST ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
		$row_for_total_indexes = array();
		foreach ($summary as $cost_row_index => $summary_name)
		{
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
                    $sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
                    $sheet->setCellValue($firstCol . $row_index, $summary_name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				// 4 quarters of year, each has 3 months
				for ($i = 0; $i < 4; $i++)
				{
					$costs_col_sum_indexes = array();
					for ($j = 0; $j < 3; $j++)
						$costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_hon_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
				}
				$row_for_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
				$sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;
		}
        
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="' . $project->Name . ' - Assigned Resources and Actuals - ' . $year . '.xlsx"');
        header('Cache-Control: max-age=0');
        $objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
        ob_end_clean();
        $objWriter->save('php://output');
        exit;
    }
    
	/**
     * Generates report of project labour tasks EAC 
	 * as a TSV (Tab-Separated-Values file) starting from month
	 * in given (ending at the end of the month's year).
	 * 
     * @param Project $project Project to export.
     * @param FiscalMonth $month Fiscal month from which the report will be generated.
     * @param DSVFileWriter $writer Output file writer.
	 * @throws IOException if there is problem with opening output file.
     */
	private function generateProjectTasksEACColStyledTSV(Project $project, FiscalMonth $month, DSVFileWriter $tsv_writer) 
	{
		// setup
		$hours_short = 'hrs';
		// let's decide which decimal delimiter to use
		$decimal_point = '.';
		$setting = $this->context->userSettingRepository->actual();
		if ($setting !== NULL)
			$decimal_point = $setting->getEXPDecimalMark() ? '.' : ',';
		$project_id = $project->ID;
		$tsv_writer->open();

		// FILE INFO HEADER
		$tsv_writer->writeln($project->Name . ' (Task EAC) ' . $month->getYear())->writeln()
					->writeln('SAP Project ID:', $project->SAP_ID)
					->writeln('Export date:', date('m/d/Y h:i:s A'))
					->writeln('Current fiscal month:', $month->getMonthName())->writeln()
					->writeln('NOTICE: Tasks with "UPDATED EAC" state were updated in this fiscal month.')
					->writeln('NOTICE: EAC for tasks is exported only from current fiscal month.')->writeln();
		
		// let's prepare block headers
		$mon_start_index = $month->getIndex();
		$month_block_header = FiscalCalendar::getEnglishFiscalMonthNames();

		// let's find planned tasks and focus only on labour tasks
		$tasks = $this->context->taskRepository->findPlannedProjectTasksInYear($project_id, $month->getYear());
		// sort tasks by name
		uasort($tasks, array($this, 'sortTasksByName'));
		// every non-labour task has its prefix with special meaning
		// non-labour tasks with same prefix are summarized separately
		$labour_tasks = array();
		foreach ($tasks as $task_id => $task)
			if ($task->isLabourTask())
				$labour_tasks[$task_id] = $task;
		unset($tasks);
			
		$names = $sap_ids = $states = $eac_titles = $eac_arrays = array();
		// headers
		foreach ($labour_tasks as $task_id => $task) 
		{
			$names[$task_id] = $task->Name;
			$sap_ids[$task_id] = str_replace(' ', '-', $task->SAPID);
		}
		$tsv_writer->writeln('Task Name:', $names)->writeln('Task SAP ID:', $sap_ids);
		
		// values
		$ts_from = FiscalCalendar::dateToTimestamp($month->getFrom(), false);
		foreach ($labour_tasks as $task_id => $task) 
		{
			$states[$task_id] = ($this->context->personTaskAssignmentRepository->isTaskYearAssignmentChangedSince($task_id, $month->getYear(), $ts_from)) ? 'UPDATED EAC' : 'no change in EAC';
			$planner = new TaskResourceYearPlanner($this->context, $this->context->taskPlanRepository->findTaskYearPlan($task_id, $month->getYear()));
			$eac_arrays[$task_id] = $planner->getTaskYearEACHours();
		}
		$tsv_writer->writeln('State:', $states);
		if(count($labour_tasks) > 0)
		{
			$tsv_writer->writeln('', array_fill(0, count($labour_tasks), 'EAC('.$hours_short.')'));
		}
		for ($i = $mon_start_index + 1; $i <= 12; $i++)
		{
			$month_eac_values = array();
			foreach ($labour_tasks as $task_id => $task) 
				$month_eac_values[] = $eac_arrays[$task_id][$i];
			$tsv_writer->writeln($month_block_header[$i - 1], $month_eac_values);
		}
		$tsv_writer->writeln();
		$tsv_writer->close();
		setcookie('fileDownloadToken', time());
	}

    /**
     * Generates report of project worker assignments
	 * with assigned hours, planned hours and actual worked hours as a 
	 * XLSX (MS-EXCEL file).
	 * 
	 * Actual worked hours, planned hours and actual spendings are generated into additional rows.
     * 
     * @param Project $project Project to export.
     * @param int $year Year for actuals (plans).
     * @param PHHPExcel $objPHPExcel Output file.
     * @param string $token String value for freeze user interface during report generation.
	 * @throws IOException if there is problem with opening output file.
     */
    private function generateProjectAssignmentsAndPlansWithActualsRowStyledXLS (Project $project, $year, PHPExcel $objPHPExcel, $token)
    {
        // SETUP PROPERTIES
        $project_id = $project->ID;
        $row_triplet = array('plan' => 'Plan', 'assigned' => 'Assigned', 'actuals' => 'Actuals', 'delta1' => 'Delta (Plan - Actuals)', 'delta2' => 'Delta (Assigned - Actuals)');
        $months = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
        $currency = 'USD';
		$hours_short = 'hrs';        
        
        //SETUP PROPORTIES FOR $sheet and $sheet3 - Assigned Resources and Actuals
        $row_index = 6;
        $firstCol = "A";
        $hrsCol = "B";
        $firstMonthColIdx = 3;
        $totalCol = "O";
        $unitCol = "P"; // last column for efort, costs, hon investment
        $costShareCol = "Q"; 
        $sapIDCol = "R"; // last column for project plan    
        $q1Col = "C";
        $q1Co2 = "D";
        $q1Co3 = "E";
        $q1Co4 = "F";       
        
        $sum_column_indexes = array();
		for ($i = 0; $i < 12; $i++)
			$sum_column_indexes[] = $firstMonthColIdx + $i;
                
        // GENERAL INFORMATIONS - hardcoded position
        $objPHPExcel->getActiveSheet()->setTitle("Report Planned and Actuals");
        $sheet = $objPHPExcel->getActiveSheet();
        $sheet->setCellValue("A1", $project->Name . ' (planned and actuals) ' . $year);
        $sheet->setCellValue("A3", 'SAP Project ID:');
        $sheet->setCellValue("B3", $project->SAP_ID);
        $sheet->setCellValue("A4", 'SAP Schedule export date:');
        $sheet->setCellValue("B4", date('m/d/Y h:i:s A'));
        
        // LEGEND
        $sheet->setCellValue('E2', 'Legend:');
        $sheet->setCellValue('F2', 'Labour tasks are colored');
        $sheet->setCellValue('F3', 'Highlighted column by two lines indicates the last complete fiscal month');
        
        // FORMAT SETTINGS
        $sheet->getStyle('A1')->getFont()->setBold(true);
        $sheet->getStyle('A3:B3')->getFont()->setBold(true);
        $sheet->getStyle('A4:B4')->getFont()->setBold(true);
        $sheet->getColumnDimension($firstCol)->setWidth(40);
        $sheet->getColumnDimension($hrsCol)->setWidth(24);
        $sheet->getColumnDimension($totalCol)->setWidth(12);
        $sheet->getColumnDimension($costShareCol)->setWidth(12); 
        $sheet->getColumnDimension($sapIDCol)->setWidth(18);        
        $sheet->setShowSummaryBelow(false);   
        $sheet->freezePane('C7');
        $borderTask = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderPerson = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderHrs = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderTotal = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        //vertical border doesnt work!?
        $borderColumnR = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderColumnL = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $column = $this->numToApha((fiscalCalendar::currentFiscalMonth()-1) + $firstMonthColIdx);
        
        // PROJECT PLAN - HEADER    
        $sheet->setCellValue($firstCol . $row_index, "--- " . $year . " PROJECT PLAN ---");
        $sheet->setCellValue($hrsCol . $row_index, "Hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->setCellValue($costShareCol . $row_index, "CostShare");
        $sheet->setCellValue($sapIDCol . $row_index, "SAP ID");
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getFont()->setBold(true); 
        $row_index++;
        $start_row_index = $row_index;

        // at first lets find assigned labour tasks
        $tasks = $this->context->taskRepository->findAssignedProjectTasksInYear($project_id, $year); // no non-labour task returned
        $planned_tasks = $this->context->taskRepository->findPlannedProjectTasksInYear($project_id, $year); // planned labour and non-labor
        
        // let's find active tasks missing in plan and task on which non-assigned people worked
        $active_tasks = $this->context->taskRepository->findActiveProjectTasksInYear($project_id, $year);
        foreach ($active_tasks as $active_task_id => $active_task)
        {
            if ($active_task->isLabourTask())
            {
                if (!isset($tasks[$active_task_id]))
                    $tasks[$active_task_id] = $active_task;
            }
            else
            {
                if (!isset($planned_tasks[$active_task_id]))
                    $planned_tasks[$active_task_id] = $active_task;
            }
        }
        $active_tasks = NULL; // useless data
        
        // sort tasks by name
        uasort($planned_tasks, array($this, 'sortTasksByName'));
    
        // then non-labour tasks
        // every non-labour task has its prefix with special meaning
        // non-labour tasks with same prefix are summarized separately
        $other_costs = array();
        foreach ($planned_tasks as $task_id => $task)
            if ($task->isLabourTask())
                $tasks[$task_id] = $task;
        foreach ($planned_tasks as $task_id => $task)
            if (!$task->isLabourTask())
            {
                $tasks[$task_id] = $task;
                $non_labour_task_prefix = substr($task->Name, 0, 3); 
                if (!in_array($non_labour_task_prefix, $other_costs))
    				$other_costs[] = $non_labour_task_prefix;
    		}
    	$planned_tasks = null; // useless now
                
        $reporter = new ProjectEACReporter($this->context, $project, $year);
        
        // PROCESS TASKS
        $task_row_indexes = array();
        $workers = array(); // array of Person - list of working labourers
        $changeTask = 0;
        $agregateWorkers = array();
        foreach ($tasks as $task_id => $task)
        {
            $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
			if ($task_plan === NULL) // task is not planned but has some work/costs for this year
				$task_plan = new TaskYearPlan($year);
            
			$task_plan_values = $task_assigned = $task_actualRestAssigned = array();
            
            if ($task->isLabourTask())
            {
                $task_actualRestAssigned = $reporter->getTaskLabourEACHoursMonthTotals($task);
                $task_assigned = $this->context->personTaskAssignmentRepository->findTaskYearAssignmentTotals($task_id, $year);
                for ($i=1;$i<=12;$i++)
                {
                	$task_plan_values[] = $task_plan->getMonthPlan($i)->Hours;
                }                                    
                
                // work for labour tasks
				//$task_work = $this->context->workRepository->findTaskYearWork($task_id, $year);
				$worker_assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($task->ID, $year);
				// find people not assigned to the task but working on it
				$all_task_worker_ids = $this->context->workRepository->findPeopleWorkingOnTaskInYear($task->ID, $year);
				// let's create dummy/empty assignment for workers not assigned but working on a task
				for ($i = 0; $i < count($all_task_worker_ids); $i++)
					if (!isset($worker_assignments[strtolower($all_task_worker_ids[$i])]) && !isset($worker_assignments[strtoupper($all_task_worker_ids[$i])]))
					{
						$year_assignment = array();
						for ($j = 1; $j <= 12; $j++)
						{
							$year_assignment[$j] = new PersonTaskMonthAssignment();
							$year_assignment[$j]->PersonID = $all_task_worker_ids[$i];
							$year_assignment[$j]->Month = $j;
							$year_assignment[$j]->Year = $year;
							$year_assignment[$j]->TaskID = $task->ID;
						}
						$worker_assignments[$all_task_worker_ids[$i]] = $year_assignment;                        
					}                
				$worker_assignments_count = count($worker_assignments);
				// we will prepare indexes of worker rows that will be summed on current task summary plan/actuals/assigned/delta rows
				$worker_row_indexes = array();
                $pom = array('assigned' => 'Assigned', 'actuals' => 'Actuals', 'delta2' => 'Delta (Assigned - Actuals)');
				$triplet_keys = array_keys($pom);
				for ($i = 0; $i < count($triplet_keys); $i++)
					for ($j = 0; $j < $worker_assignments_count; $j++)
                        if($triplet_keys[0] == 'Assigned')                            
                            $worker_row_indexes[$triplet_keys[$i]][] = $row_index + 4 + $j * 3 + $i;
                        else
                            $worker_row_indexes[$triplet_keys[$i]][] = ($row_index + 4 + $j * 3 + $i)+1;
                // task sum row triplet
				$task_name_written = true;                
                $changeRow = 0;      
				foreach ($row_triplet as $key => $title)
				{
                    $sheet->setCellValue($firstCol . $row_index, $task->Name);
                    $this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
					$sheet->setCellValue($hrsCol . $row_index, $title);
                    $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                    $this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                    for ($i = 0; $i < 12; $i++)
                    {
                        if ($key == 'plan')
                        {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_plan_values[$i]);
                            $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet); 
                        }
                        elseif($key == 'delta1')
                        {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '=SUM(' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 3) . '-' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 1) . ')');
                            $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
                        }
                        elseif (!isset($worker_row_indexes[$key]))
                        {
                            for ($i = 0; $i < 12; $i++)
                            {
                                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
                                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);                           
                            }
                        }
                        else
                        {
                            for ($i = 0; $i < 12; $i++)
                            {
                                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $worker_row_indexes[$key]));
                                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);                           
                            }
                        }
                    }					
                    // total
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
					if ($task_name_written)
                    {
                        $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
                        $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);                     
                        $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
                        $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                        $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);                       
                        $task_name_written = false;
                        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);                       
                    }
                    $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                    $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                    $changeRow++;
                    $task_row_indexes[$task->ID][$key] = $row_index;
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }    
                    $row_index++;
				}                               
                
                foreach ($worker_assignments as $person_id => $assignments)
				{
					if (!isset($workers[$person_id]))
						$workers[$person_id] = $this->context->personRepository->find($person_id);
                    
                    $actuals = $this->context->workRepository->findPersonTaskYearWork($person_id, $task->ID, $year);

                    // assignment
                    if(array_key_exists($person_id, $agregateWorkers))
                    {
                        array_push($agregateWorkers[$person_id], $row_index);
                    }
                    else
                    {
                        $agregateWorkers[$person_id] = array();
                        array_push($agregateWorkers[$person_id], $row_index);
                    }
					if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
                    $sheet->setCellValue($hrsCol . $row_index, $row_triplet['assigned']);
                    $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                    for ($i = 1; $i <= 12; $i++)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $assignments[$i]->Hours);
                    // assignment total for a worker
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderPerson);
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $row_index++;
                    // actuals
                    if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
                   $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
                   $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
					for ($i = 1; $i <= 12; $i++)
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $actuals[$i]);
                    // actuals total for a worker
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $row_index++;
                    //delta
                    if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
					$sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta2']);
                    $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
					for ($i = 1; $i <= 12; $i++)
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, '=' . $this->numToApha($i+$firstMonthColIdx-1) . ($row_index-2) . '-' . $this->numToApha($i+$firstMonthColIdx-1) . ($row_index-1));
					// delta total for a worker
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $row_index++;            
                }
            }
            else // non-labour task
            {
				$changeRow = 0;
                $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
				if ($task_plan === NULL) // task is not planned but has some costs for this year
					$task_plan = new TaskYearPlan($year);
				$task_assignments = $task_plan_values = $task_actuals = array();
                $monthly_task_costs = $this->context->costCenterRepository->findActualCostsYearSummary($task_id, $year, $currency);
				$monthly_task_assignments = $this->context->assignmentsCostRepository->findTaskYearAssignments($task_id, $year);
				for ($i = 1; $i <= 12; $i++)
				{
                    //$monthly_task_costs = $this->context->costCenterRepository->findActualTaskCostsMonthSummary($task_id, $i, $year, $currency);
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Budget;
                    $task_actuals[] = $monthly_task_costs[$i];
					if(isset($monthly_task_assignments[$i-1]))
						$task_assignments[] = $monthly_task_assignments[$i-1]->Cost;
					else
						$task_assignments[] = $task_plan->getMonthPlan($i)->Budget;
				}
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['plan']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, round($task_plan_values[$i]));
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
				$this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);
                $task_row_indexes[$task->ID]['plan'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $row_index++;
				$changeRow++;
				
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['assigned']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, round($task_assignments[$i]));
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['assigned'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
                
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, round($task_actuals[$i]));
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['actuals'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
                
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta1']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
					$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '=SUM(' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 3) . '-' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 1) . ')');
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['delta1'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
				
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta2']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
					$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '=SUM(' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 3) . '-' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 2) . ')');
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['delta2'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
            }
            $start_row_index = $row_index;
            $changeTask++;
        }  

        // WORKERS AGREGATION
        $sheet3 = $objPHPExcel->createSheet(1);
        $sheet3->setTitle("Workers agregation");
        
        //SETUP PROPORTIES FOR $sheet3 - Workers agregation
        $sheet3->getStyle('A1')->getFont()->setBold(true);
        $sheet3->getColumnDimension($firstCol)->setWidth(40);
        $sheet3->getColumnDimension($hrsCol)->setWidth(28);
        $sheet3->getColumnDimension($totalCol)->setWidth(12);      
        $sheet3->setShowSummaryBelow(false); 
        $sheet3->freezePane('C2');
        $row_index3 = 1;
        $changeTask = 0;

        // WORKERS AGREGATION - HEADER    
        $sheet3->setCellValue($firstCol . $row_index3, "--- " . $year . " PROJECT PLAN ---");
        $sheet3->setCellValue($hrsCol . $row_index3, "Hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet3->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index3, $months[$i]);
        }
        $sheet3->setCellValue($totalCol . $row_index3, ">>TOTAL<<");
        $sheet3->setCellValue($unitCol . $row_index3, "Unit");

        $sheet3->getStyle($firstCol . $row_index3 . ':' . $sapIDCol . $row_index3)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $sheet3->getStyle($firstCol . $row_index3 . ':' . $sapIDCol . $row_index3)->getFont()->setBold(true); 
        $row_index3++;

        foreach($agregateWorkers as $rows)
        {
            $count = count($rows);
            $current = $current2 = current($rows);
            $actualsPattern1 = $assignedPattern1 = $deltaPattern1 = "=";
            $actualsPattern2 = $assignedPattern2 = $deltaPattern2 = "=";
            $actualsPattern3 = $assignedPattern3 = $deltaPattern3 = "=";
            $actualsPattern4 = $assignedPattern4 = $deltaPattern4 = "=";
            $actualsPattern5 = $assignedPattern5 = $deltaPattern5 = "=";
            $actualsPattern6 = $assignedPattern6 = $deltaPattern6 = "=";
            $actualsPattern7 = $assignedPattern7 = $deltaPattern7 = "=";
            $actualsPattern8 = $assignedPattern8 = $deltaPattern8 = "=";
            $actualsPattern9 = $assignedPattern9 = $deltaPattern9 = "=";
            $actualsPattern10 = $assignedPattern10 = $deltaPattern10 = "=";
            $actualsPattern11 = $assignedPattern11 = $deltaPattern11 = "=";
            $actualsPattern12 = $assignedPattern12 = $deltaPattern12 = "=";
            $actualsPattern13 = $assignedPattern13 = $deltaPattern13 = "=";
            
            for($i = 0; $i <= $count; $i++)
            {
                if($current != null)
                {
                    $actualsPattern1 .= "'Report Planned and Actuals'!C" . $current;
                    $actualsPattern2 .= "'Report Planned and Actuals'!D" . $current;
                    $actualsPattern3 .= "'Report Planned and Actuals'!E" . $current;
                    $actualsPattern4 .= "'Report Planned and Actuals'!F" . $current;
                    $actualsPattern5 .= "'Report Planned and Actuals'!G" . $current;
                    $actualsPattern6 .= "'Report Planned and Actuals'!H" . $current;
                    $actualsPattern7 .= "'Report Planned and Actuals'!I" . $current;
                    $actualsPattern8 .= "'Report Planned and Actuals'!J" . $current;
                    $actualsPattern9 .= "'Report Planned and Actuals'!K" . $current;
                    $actualsPattern10 .= "'Report Planned and Actuals'!L" . $current;
                    $actualsPattern11 .= "'Report Planned and Actuals'!M" . $current;
                    $actualsPattern12 .= "'Report Planned and Actuals'!N" . $current;
                    $actualsPattern13 .= "'Report Planned and Actuals'!O" . $current;
                    
                    $assignedPattern1 .= "'Report Planned and Actuals'!C" . ($current + 1);
                    $assignedPattern2 .= "'Report Planned and Actuals'!D" . ($current + 1);
                    $assignedPattern3 .= "'Report Planned and Actuals'!E" . ($current + 1);
                    $assignedPattern4 .= "'Report Planned and Actuals'!F" . ($current + 1);
                    $assignedPattern5 .= "'Report Planned and Actuals'!G" . ($current + 1);
                    $assignedPattern6 .= "'Report Planned and Actuals'!H" . ($current + 1);
                    $assignedPattern7 .= "'Report Planned and Actuals'!I" . ($current + 1);
                    $assignedPattern8 .= "'Report Planned and Actuals'!J" . ($current + 1);
                    $assignedPattern9 .= "'Report Planned and Actuals'!K" . ($current + 1);
                    $assignedPattern10 .= "'Report Planned and Actuals'!L" . ($current + 1);
                    $assignedPattern11 .= "'Report Planned and Actuals'!M" . ($current + 1);
                    $assignedPattern12 .= "'Report Planned and Actuals'!N" . ($current + 1);
                    $assignedPattern13 .= "'Report Planned and Actuals'!O" . ($current + 1);
                    
                    $deltaPattern1 .= "'Report Planned and Actuals'!C" . ($current + 2);
                    $deltaPattern2 .= "'Report Planned and Actuals'!D" . ($current + 2);
                    $deltaPattern3 .= "'Report Planned and Actuals'!E" . ($current + 2);
                    $deltaPattern4 .= "'Report Planned and Actuals'!F" . ($current + 2);
                    $deltaPattern5 .= "'Report Planned and Actuals'!G" . ($current + 2);
                    $deltaPattern6 .= "'Report Planned and Actuals'!H" . ($current + 2);
                    $deltaPattern7 .= "'Report Planned and Actuals'!I" . ($current + 2);
                    $deltaPattern8 .= "'Report Planned and Actuals'!J" . ($current + 2);
                    $deltaPattern9 .= "'Report Planned and Actuals'!K" . ($current + 2);
                    $deltaPattern10 .= "'Report Planned and Actuals'!L" . ($current + 2);
                    $deltaPattern11 .= "'Report Planned and Actuals'!M" . ($current + 2);
                    $deltaPattern12 .= "'Report Planned and Actuals'!N" . ($current + 2);
                    $deltaPattern13 .= "'Report Planned and Actuals'!O" . ($current + 2);
                }
                $current = next($rows);
                if($current != null)
                {
                    $actualsPattern1 .= "+";
                    $actualsPattern2 .= "+";
                    $actualsPattern3 .= "+";
                    $actualsPattern4 .= "+";
                    $actualsPattern5 .= "+";
                    $actualsPattern6 .= "+";
                    $actualsPattern7 .= "+";
                    $actualsPattern8 .= "+";
                    $actualsPattern9 .= "+";
                    $actualsPattern10 .= "+";
                    $actualsPattern11 .= "+";
                    $actualsPattern12 .= "+";
                    $actualsPattern13 .= "+";
                    
                    $assignedPattern1 .= "+";
                    $assignedPattern2 .= "+";
                    $assignedPattern3 .= "+";
                    $assignedPattern4 .= "+";
                    $assignedPattern5 .= "+";
                    $assignedPattern6 .= "+";
                    $assignedPattern7 .= "+";
                    $assignedPattern8 .= "+";
                    $assignedPattern9 .= "+";
                    $assignedPattern10 .= "+";
                    $assignedPattern11 .= "+";
                    $assignedPattern12 .= "+";
                    $assignedPattern13 .= "+";
                    
                    $deltaPattern1 .= "+";
                    $deltaPattern2 .= "+";
                    $deltaPattern3 .= "+";
                    $deltaPattern4 .= "+";
                    $deltaPattern5 .= "+";
                    $deltaPattern6 .= "+";
                    $deltaPattern7 .= "+";
                    $deltaPattern8 .= "+";
                    $deltaPattern9 .= "+";
                    $deltaPattern10 .= "+";
                    $deltaPattern11 .= "+";
                    $deltaPattern12 .= "+";
                    $deltaPattern13 .= "+";
                }
            }

            $sheet3->setCellValue("A". $row_index3, "='Report Planned and Actuals'!A" . $current2);
            $sheet3->getStyle($hrsCol . $row_index3)->applyFromArray($borderHrs);
            $sheet3->getStyle('O' . $row_index3)->applyFromArray($borderTotal);
            $this->setColor("A", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("B". $row_index3, "='Report Planned and Actuals'!B" . $current2);
            $this->setColor("B", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("C". $row_index3, $actualsPattern1);
            $this->setColor("C", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("D". $row_index3, $actualsPattern2);
            $this->setColor("D", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("E". $row_index3, $actualsPattern3);
            $this->setColor("E", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("F". $row_index3, $actualsPattern4);
            $this->setColor("F", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("G". $row_index3, $actualsPattern5);
            $this->setColor("G", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("H". $row_index3, $actualsPattern6);
            $this->setColor("H", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("I". $row_index3, $actualsPattern7);
            $this->setColor("I", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("J". $row_index3, $actualsPattern8);
            $this->setColor("J", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("K". $row_index3, $actualsPattern9);
            $this->setColor("K", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("L". $row_index3, $actualsPattern10);
            $this->setColor("L", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("M". $row_index3, $actualsPattern11);
            $this->setColor("M", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("N". $row_index3, $actualsPattern12);
            $this->setColor("N", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("O". $row_index3, $actualsPattern13);
            $this->setColor("O", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("P". $row_index3, "='Report Planned and Actuals'!P" . $current2);
            $this->setColor("P", $row_index3, 0, $changeTask, $sheet3);
            $row_index3++;
            $sheet3->setCellValue("A". $row_index3, "='Report Planned and Actuals'!A" . ($current2 + 1));
            $sheet3->getStyle($hrsCol . $row_index3)->applyFromArray($borderHrs);
            $sheet3->getStyle('O' . $row_index3)->applyFromArray($borderTotal);
            $this->setColor("A", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("B". $row_index3, "='Report Planned and Actuals'!B" . ($current2 + 1));
            $this->setColor("B", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("C". $row_index3, $assignedPattern1);
            $this->setColor("C", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("D". $row_index3, $assignedPattern2);
            $this->setColor("D", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("E". $row_index3, $assignedPattern3);
            $this->setColor("E", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("F". $row_index3, $assignedPattern4);
            $this->setColor("F", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("G". $row_index3, $assignedPattern5);
            $this->setColor("G", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("H". $row_index3, $assignedPattern6);
            $this->setColor("H", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("I". $row_index3, $assignedPattern7);
            $this->setColor("I", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("J". $row_index3, $assignedPattern8);
            $this->setColor("J", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("K". $row_index3, $assignedPattern9);
            $this->setColor("K", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("L". $row_index3, $assignedPattern10);
            $this->setColor("L", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("M". $row_index3, $assignedPattern11);
            $this->setColor("M", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("N". $row_index3, $assignedPattern12);
            $this->setColor("N", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("O". $row_index3, $assignedPattern13);
            $this->setColor("O", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("P". $row_index3, "='Report Planned and Actuals'!P" . ($current2 + 1));
            $this->setColor("P", $row_index3, 0, $changeTask, $sheet3);
            $row_index3++;
            $sheet3->setCellValue("A". $row_index3, "='Report Planned and Actuals'!A" . ($current2 + 2));
            $sheet3->getStyle($hrsCol . $row_index3)->applyFromArray($borderHrs);
            $sheet3->getStyle('O' . $row_index3)->applyFromArray($borderTotal);
            $this->setColor("A", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("B". $row_index3, "='Report Planned and Actuals'!B" . ($current2 + 2));
            $this->setColor("B", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("C". $row_index3, $deltaPattern1);
            $this->setColor("C", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("D". $row_index3, $deltaPattern2);
            $this->setColor("D", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("E". $row_index3, $deltaPattern3);
            $this->setColor("E", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("F". $row_index3, $deltaPattern4);
            $this->setColor("F", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("G". $row_index3, $deltaPattern5);
            $this->setColor("G", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("H". $row_index3, $deltaPattern6);
            $this->setColor("H", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("I". $row_index3, $deltaPattern7);
            $this->setColor("I", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("J". $row_index3, $deltaPattern8);
            $this->setColor("J", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("K". $row_index3, $deltaPattern9);
            $this->setColor("K", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("L". $row_index3, $deltaPattern10);
            $this->setColor("L", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("M". $row_index3, $deltaPattern11);
            $this->setColor("M", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("N". $row_index3, $deltaPattern12);
            $this->setColor("N", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("O". $row_index3, $deltaPattern13);
            $this->setColor("O", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("P". $row_index3, "='Report Planned and Actuals'!P" . ($current2 + 2));
            $this->setColor("P", $row_index3, 3, $changeTask, $sheet3);
            $row_index3++;
            $sheet3->getStyle($firstCol . $row_index3 . ':P' . $row_index3)->applyFromArray($borderTask);
            $changeTask++;
        }
        
        //SETUP PROPORTIES FOR $sheet2 - Labour rates        
        $row_index2 = 1;
        $firstCol2 = "A";
        $firstMonthColIdx2 = 2;
        $unitCol2 = "N"; // last column for labour rates 
        
        // COST CENTERS / LABOUR RATES
        $sheet2 = $objPHPExcel->createSheet(1);
        $sheet2->setTitle("Labour rates");        
        $sheet2->getColumnDimension($firstCol2)->setWidth(35);
        $sheet2->setCellValue($firstCol2 . $row_index2, "--- LABOUR RATES ---");
        $sheet2->setCellValue($unitCol2 . $row_index2, "Unit");
        for($i = 0; $i<12; $i++)
        {
            $sheet2->setCellValue($this->numToApha($i+$firstMonthColIdx2) . $row_index2, $months[$i]);
        }
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getFont()->setBold(true);
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index2++;
        
        $labour_rates_row_indexes = array();
        $labour_centers = $this->context->costCenterRepository->findAllCostCenters();
        foreach ($labour_centers as $center)
		{
			$rate = $center->getYearLabourRate($year);
			if ($rate != NULL)
			{
                $sheet2->setCellValue($firstCol2 . $row_index2, $center->Name);				
                for ($i = 1; $i <= 12; $i++)
                    $sheet2->setCellValue($this->numToApha($i+1) . $row_index2, $rate->getMonthlyHourRate($i));
                $sheet2->setCellValue($unitCol2 . $row_index2, $currency . '/' . $hours_short);
               $labour_rates_row_indexes[$center->ID] = $row_index2;
                $row_index2++;                
            }              
		}

        //EFFORT
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- EFFORT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;                
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
		$effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta1'] = $effort_total_indexes['delta2'] = $effort_total_indexes['assigned'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['assigned'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta1'] = $effort_sum_indexes['delta2'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['assigned'][] = $task_row_indexes[$task->ID]['assigned'];
                        $effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
                        $effort_sum_indexes['delta1'][] = $task_row_indexes[$task->ID]['delta1'];
						$effort_sum_indexes['delta2'][] = $task_row_indexes[$task->ID]['delta2'];
					}
			$center_name_written = false;
            if(count($effort_sum_indexes['plan']) > 0 AND count($effort_sum_indexes['assigned']) > 0 AND count($effort_sum_indexes['actuals']) > 0 AND count($effort_sum_indexes['delta1']) > 0 AND count($effort_sum_indexes['delta2']) > 0)
                foreach ($row_triplet as $key => $title)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);					
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center->Name);					
                        $sheet->setCellValue($hrsCol . $row_index, $title);					
                        $center_name_written = true;			
                    }
                    for ($i = 0; $i < 12; $i++)						                        
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));				
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $effort_total_indexes[$key][] = $row_index;                                
                    $row_index++;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);

                }        
		}
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, ">> TOTAL <<");
				$sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
				 $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
            if($year == date('Y'))
                if(fiscalCalendar::currentFiscalMonth() != 0)
                    {
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                    }
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $hours_short);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}

        // COSTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- COSTS ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $costs_row_indexes = array();
        $effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta1'] = $effort_total_indexes['delta2'] = $effort_total_indexes['assigned'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['assigned'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta1'] = $effort_sum_indexes['delta2'] = array();
			// remember indexes of row which belong to this labour center
            foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
                        $effort_sum_indexes['assigned'][] = $task_row_indexes[$task->ID]['assigned'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
                        $effort_sum_indexes['delta1'][] = $task_row_indexes[$task->ID]['delta1'];
						$effort_sum_indexes['delta2'][] = $task_row_indexes[$task->ID]['delta2'];
					}
			$center_name_written = false;
            if(count($effort_sum_indexes['plan']) > 0 AND count($effort_sum_indexes['assigned']) > 0 AND count($effort_sum_indexes['actuals']) > 0 AND count($effort_sum_indexes['delta1']) > 0 AND count($effort_sum_indexes['delta2']) > 0 )
                foreach ($row_triplet as $key => $title)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center->Name);					
                        $sheet->setCellValue($hrsCol . $row_index, $title);					
                        $center_name_written = true;			
                    }
                    for ($i = 0; $i < 12; $i++)
                    {
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]) . "*'Labour rates'!" . $this->numToApha ($firstMonthColIdx2 + $i) . $labour_rates_row_indexes[$center->ID]);                        
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $costs_row_indexes[$center->ID][$key] = $row_index;
                $effort_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;            
                }
			$row_for_total_indexes[] = $row_index; 
		}
        foreach ($other_costs as $task_prefix)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['assigned'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta1'] = $effort_sum_indexes['delta2'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)                
				if (!$task->isLabourTask())
					// same type of non-labour tasks
					if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
                        $effort_sum_indexes['assigned'][] = $task_row_indexes[$task->ID]['assigned'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];                        
						$effort_sum_indexes['delta1'][] = $task_row_indexes[$task->ID]['delta1'];
                        $effort_sum_indexes['delta2'][] = $task_row_indexes[$task->ID]['delta2'];
					}
			$center_name_written = false;
            if(count($effort_sum_indexes['plan']) > 0 AND count($effort_sum_indexes['assigned']) > 0 AND count($effort_sum_indexes['actuals']) > 0 AND count($effort_sum_indexes['delta1']) > 0 AND count($effort_sum_indexes['delta2']) > 0)
                foreach ($row_triplet as $key => $title)
                {               
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                        $sheet->setCellValue($hrsCol . $row_index, $title);	
                        $center_name_written = true;			
                    }
                    for ($i = 0; $i < 12; $i++)
                    {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));
                            $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    $sheet->setCellValue($unitCol . $row_index, $currency);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $effort_total_indexes[$key][] = $row_index;
                    $costs_row_indexes[$task_prefix][$key] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
                }
			$row_for_total_indexes[] = $row_index;            
		}

		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);	
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            if($year == date('Y'))
                if(fiscalCalendar::currentFiscalMonth() != 0)
                    {
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                    }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // HONEYWELL INVESTMENTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- HON INVESTMENT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);        
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $row_for_total_indexes['plan'] = $row_for_total_indexes['actuals'] = $row_for_total_indexes['delta1'] = $row_for_total_indexes['delta2'] = $row_for_total_indexes['assigned'] = array();
		$costs_hon_row_indexes = array();
		foreach ($labour_centers as $center)
		{
			$center_name_written = false;
            $cell_value = '';
					foreach ($tasks as $task_id => $task)
						if ($task->isLabourTask())
							if ($task->CostCenterID == $center->ID)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					if ($cell_value != '')
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					$cell_value = '';
					foreach ($tasks as $task_id => $task)
						if ($task->isLabourTask())
							if ($task->CostCenterID == $center->ID)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, "=(".$cell_value.")*'Labour rates'!".$this->numToApha($firstMonthColIdx2 + $i).$labour_rates_row_indexes[$center->ID]);                                      
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
				}
				// totals for a center
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row_for_total_indexes[$key][] = $row_index;
				$costs_hon_row_indexes[$center->ID][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
        foreach ($other_costs as $task_prefix)
		{
			$center_name_written = false;
            					$cell_value = '';
					foreach ($tasks as $task_id => $task)
					{
						if (!$task->isLabourTask())
							// same type of non-labour tasks
							if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 * 3 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					}
					if ($cell_value != '')
			foreach ($row_triplet as $key => $title)
			{
                if ($center_name_written)
                    $sheet->setCellValue($hrsCol . $row_index, $title);
                else
                {
                    $sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
                    $center_name_written = true;			
                }
                for ($i = 0; $i < 12; $i++)
                {
                    $cell_value = '';
                    foreach ($tasks as $task_id => $task)
                    {
                        if (!$task->isLabourTask())
                            // same type of non-labour tasks
                            if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
                            {
                                $cell_value .= ($cell_value !== '') ? '+' : '';
                                // CostShare for a task is on column index: $firstMonthColIdx + 13 * 3 + 1
                                $cell_value .= $this->numToApha($firstMonthColIdx + $i) 
                                        . $task_row_indexes[$task->ID][$key]
                                        . '*(100-' 
                                        . $this->numToApha($firstMonthColIdx + 13 + 1) 
                                        . $task_row_indexes[$task->ID]['plan']
                                        . ')/100';
                            }
                    }
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '='.$cell_value);						
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                }
                // totals for a non-labor tasks type
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $row_for_total_indexes[$key][] = $row_index;
                $costs_hon_row_indexes[$task_prefix][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);				
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            if($year == date('Y'))
                if(fiscalCalendar::currentFiscalMonth() != 0)
                    {
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                    }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // QUARTERLY SPEND PROFILE
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY SPEND PROFILE ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
		// prepare summary traversal array
		$summary = array();
		foreach ($labour_centers as $center)
			$summary[$center->ID] = $center->Name;
		foreach ($other_costs as $task_prefix)
			$summary[$task_prefix] = $task_prefix;
        $row_for_total_indexes['plan'] = $row_for_total_indexes['actuals'] = $row_for_total_indexes['delta1'] = $row_for_total_indexes['delta2'] = $row_for_total_indexes['assigned'] = array();
		foreach ($costs_row_indexes as $cost_row_index => $summary_name)
		{
            $center_name = $summary[$cost_row_index];
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
                if(strlen($center_name) == 3)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
                }
                elseif(strlen($center_name) != 3 )
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
                }
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;
		}
        
        // QUARTERLY HON INVEST
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY HON INVEST ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        $row_for_total_indexes['plan'] = $row_for_total_indexes['actuals'] = $row_for_total_indexes['delta1'] = $row_for_total_indexes['delta2'] = $row_for_total_indexes['assigned'] = array();
		foreach ($costs_row_indexes as $cost_row_index => $summary_name)
		{
            $center_name = $summary[$cost_row_index];
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
                if(strlen($center_name) == 3)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);				
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_hon_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;                    
                }
                elseif(strlen($center_name) != 3 )
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);				
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_hon_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++; 
                }
            }
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{
			if ($total_text_written)
				$sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;                        
		}
        setcookie('fileDownloadToken', $token);
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="' . $project->Name . ' - Assigned Resources and Actuals - ' . $year . '.xlsx"');
        header('Cache-Control: max-age=0');
        $objWriter = new PHPExcel_Writer_Excel2007($objPHPExcel);
        ob_end_clean();
        $objWriter->save('php://output');
        exit;
    }

    /**
     * Generates report of project worker assignments
	 * with assigned hours, planned hours and EAC worked hours as a 
	 * XLSX (MS-EXCEL file).
	 * 
	 * Actual worked hours, planned hours and EAC spendings are generated into additional rows.
     * 
     * @param Project $project Project to export.
     * @param int $year Year for actuals (plans).
     * @param PHHPExcel $objPHPExcel Output file.
     * @param string $token String value for freeze user interface during report generation.
	 * @throws IOException if there is problem with opening output file.
     *
     * @param Project $project
     * @param int $year
     * @param PHPExcel $objPHPExcel 
     */
    private function generateProjectAssignmentsAndPlansWithEACRowStyledXLS(Project $project, $year, PHPExcel $objPHPExcel, $token)
    {
        // SETUP PROPERTIES
        $project_id = $project->ID;
        $row_triplet = array('plan' => 'Plan', 'assigned' => 'Assigned', 'actuals' => 'Actuals/EAC', 'delta1' => 'Delta (Plan - Actuals/EAC)', 'delta2' => 'Delta (Assigned - Actuals/EAC)');
        $months = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
        $currency = 'USD';
		$hours_short = 'hrs';        
        
        //SETUP PROPORTIES FOR $sheet and $sheet3 - Assigned Resources and Actuals
        $row_index = 6;
        $firstCol = "A";
        $hrsCol = "B";
        $firstMonthColIdx = 3;
        $totalCol = "O";
        $unitCol = "P"; // last column for efort, costs, hon investment
        $costShareCol = "Q"; 
        $sapIDCol = "R"; // last column for project plan    
        $q1Col = "C";
        $q1Co2 = "D";
        $q1Co3 = "E";
        $q1Co4 = "F";       
        
        $sum_column_indexes = array();
		for ($i = 0; $i < 12; $i++)
			$sum_column_indexes[] = $firstMonthColIdx + $i;
        
        // GENERAL INFORMATIONS - hardcoded position
        $objPHPExcel->getActiveSheet()->setTitle("Report EAC");
        $sheet = $objPHPExcel->getActiveSheet();
        $sheet->setCellValue("A1", $project->Name . ' (EAC) ' . $year);
        $sheet->setCellValue("A3", 'SAP Project ID:');
        $sheet->setCellValue("B3", $project->SAP_ID);
        $sheet->setCellValue("A4", 'SAP Schedule export date:');
        $sheet->setCellValue("B4", date('m/d/Y h:i:s A'));
        
        // LEGEND
        $sheet->setCellValue('E2', 'Legend:');
        $sheet->setCellValue('F2', 'Labour tasks are colored');
        $sheet->setCellValue('F3', 'Highlighted column by two lines indicates the last complete fiscal month');
        
        // FORMAT SETTINGS
        $sheet->getStyle('A1')->getFont()->setBold(true);
        $sheet->getStyle('A3:B3')->getFont()->setBold(true);
        $sheet->getStyle('A4:B4')->getFont()->setBold(true);
        $sheet->getColumnDimension($firstCol)->setWidth(40);
        $sheet->getColumnDimension($hrsCol)->setWidth(28);
        $sheet->getColumnDimension($totalCol)->setWidth(12);
        $sheet->getColumnDimension($costShareCol)->setWidth(12); 
        $sheet->getColumnDimension($sapIDCol)->setWidth(18);        
        $sheet->setShowSummaryBelow(false); 
        $sheet->freezePane('C7');
        $borderTask = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderPerson = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderHrs = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderTotal = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        //vertical border doesnt work!?
        $borderColumnR = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderColumnL = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $column = $this->numToApha((fiscalCalendar::currentFiscalMonth()-1) + $firstMonthColIdx);
        
        // PROJECT PLAN - HEADER    
        $sheet->setCellValue($firstCol . $row_index, "--- " . $year . " PROJECT PLAN ---");
        $sheet->setCellValue($hrsCol . $row_index, "Hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->setCellValue($costShareCol . $row_index, "CostShare");
        $sheet->setCellValue($sapIDCol . $row_index, "SAP ID");
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->getFont()->setBold(true); 
        $row_index++;
        $start_row_index = $row_index;

        // at first lets find assigned labour tasks
        $tasks = $this->context->taskRepository->findAssignedProjectTasksInYear($project_id, $year); // no non-labour task returned
        $planned_tasks = $this->context->taskRepository->findPlannedProjectTasksInYear($project_id, $year); // planned labour and non-labor
        
        // let's find active tasks missing in plan and task on which non-assigned people worked
        $active_tasks = $this->context->taskRepository->findActiveProjectTasksInYear($project_id, $year);
        foreach ($active_tasks as $active_task_id => $active_task)
        {
            if ($active_task->isLabourTask())
            {
                if (!isset($tasks[$active_task_id]))
                    $tasks[$active_task_id] = $active_task;
            }
            else
            {
                if (!isset($planned_tasks[$active_task_id]))
                    $planned_tasks[$active_task_id] = $active_task;
            }
        }
        $active_tasks = NULL; // useless data
        
        // sort tasks by name
        uasort($planned_tasks, array($this, 'sortTasksByName'));
    
        // then non-labour tasks
        // every non-labour task has its prefix with special meaning
        // non-labour tasks with same prefix are summarized separately
        $other_costs = array();
        foreach ($planned_tasks as $task_id => $task)
            if ($task->isLabourTask())
                $tasks[$task_id] = $task;
        foreach ($planned_tasks as $task_id => $task)
            if (!$task->isLabourTask())
            {
                $tasks[$task_id] = $task;
                $non_labour_task_prefix = substr($task->Name, 0, 3); 
                if (!in_array($non_labour_task_prefix, $other_costs))
    				$other_costs[] = $non_labour_task_prefix;
    		}
    	$planned_tasks = null; // useless now
                
        $reporter = new ProjectEACReporter($this->context, $project, $year);
        
        // PROCESS TASKS
        $task_row_indexes = array();
        $workers = array(); // array of Person - list of working labourers
        $changeTask = 0;
        $agregateWorkers = array();
        foreach ($tasks as $task_id => $task)
        {
            $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
			if ($task_plan === NULL) // task is not planned but has some work/costs for this year
				$task_plan = new TaskYearPlan($year);
            
			$task_plan_values = $task_assigned = $task_actualRestAssigned = array();
            
            if ($task->isLabourTask())
            {
                $task_actualRestAssigned = $reporter->getTaskLabourEACHoursMonthTotals($task);
                $task_assigned = $this->context->personTaskAssignmentRepository->findTaskYearAssignmentTotals($task_id, $year);
                for ($i=1;$i<=12;$i++)
                {
                	$task_plan_values[] = $task_plan->getMonthPlan($i)->Hours;
                }                                    
                    
                // work for labour tasks
				//$task_work = $this->context->workRepository->findTaskYearWork($task_id, $year);
				$worker_assignments = $this->context->personTaskAssignmentRepository->findTaskYearAssignments($task->ID, $year);
				// find people not assigned to the task but working on it
				$all_task_worker_ids = $this->context->workRepository->findPeopleWorkingOnTaskInYear($task->ID, $year);
				// let's create dummy/empty assignment for workers not assigned but working on a task
				for ($i = 0; $i < count($all_task_worker_ids); $i++)
					if (!isset($worker_assignments[$all_task_worker_ids[$i]]))
					{
						$year_assignment = array();
						for ($j = 1; $j <= 12; $j++)
						{
							$year_assignment[$j] = new PersonTaskMonthAssignment();
							$year_assignment[$j]->PersonID = $all_task_worker_ids[$i];
							$year_assignment[$j]->Month = $j;
							$year_assignment[$j]->Year = $year;
							$year_assignment[$j]->TaskID = $task->ID;
						}
						$worker_assignments[$all_task_worker_ids[$i]] = $year_assignment;                        
					}                
				$worker_assignments_count = count($worker_assignments);
				// we will prepare indexes of worker rows that will be summed on current task summary plan/actuals/assigned/delta rows
				$worker_row_indexes = array();
                $pom = array('assigned' => 'Assigned', 'actuals' => 'Actuals', 'delta2' => 'Delta (Assigned - Actuals/EAC)');
				$triplet_keys = array_keys($pom);
				for ($i = 0; $i < count($triplet_keys); $i++)
					for ($j = 0; $j < $worker_assignments_count; $j++)
                        if($triplet_keys[0] == 'Assigned')                            
                            $worker_row_indexes[$triplet_keys[$i]][] = $row_index + 4 + $j * 3 + $i;
                        else
                            $worker_row_indexes[$triplet_keys[$i]][] = ($row_index + 4 + $j * 3 + $i)+1;
                // task sum row triplet
				$task_name_written = true;                
                $changeRow = 0;      
				foreach ($row_triplet as $key => $title)
				{
                    $sheet->setCellValue($firstCol . $row_index, $task->Name);
                    $this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
					$sheet->setCellValue($hrsCol . $row_index, $title);
                    $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                    $this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                    for ($i = 0; $i < 12; $i++)
                    {
                        if ($key == 'plan')
                        {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $task_plan_values[$i]);
                            $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet); 
                        }
                        elseif($key == 'delta1')
                        {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '=SUM(' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 3) . '-' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 1) . ')');
                            $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
                        }
                        elseif (!isset($worker_row_indexes[$key]))
                        {
                            for ($i = 0; $i < 12; $i++)
                            {
                                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, 0);
                                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);                           
                            }
                        }
                        else
                        {
                            for ($i = 0; $i < 12; $i++)
                            {
                                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $worker_row_indexes[$key]));
                                $this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);                           
                            }
                        }
                    }					
                    // total
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
					if ($task_name_written)
                    {
                        $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
                        $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);                     
                        $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
                        $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                        $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);                       
                        $task_name_written = false;
                        $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);                       
                    }
                    $this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                    $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                    $changeRow++;
                    $task_row_indexes[$task->ID][$key] = $row_index;
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }   
                    $row_index++;
				}                               
                
                foreach ($worker_assignments as $person_id => $assignments)
				{
					if (!isset($workers[$person_id]))
						$workers[$person_id] = $this->context->personRepository->find($person_id);
                    
                    $person_actualRestAssigned = $reporter->getPersonLabourEacHoursMonthTotals($task, $person_id);
                    
                    // assignment
                    if(array_key_exists($person_id, $agregateWorkers))
                    {
                        array_push($agregateWorkers[$person_id], $row_index);
                    }
                    else
                    {
                        $agregateWorkers[$person_id] = array();
                        array_push($agregateWorkers[$person_id], $row_index);
                    }
					if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
                    $sheet->setCellValue($hrsCol . $row_index, $row_triplet['assigned']);
                    $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                    for ($i = 1; $i <= 12; $i++)
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $assignments[$i]->Hours);
                    // assignment total for a worker
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderPerson);
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            } 
                    $row_index++;
                    // actuals
                    if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
                   $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
                   $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
					for ($i = 1; $i <= 12; $i++)
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $person_actualRestAssigned[$i]);
                    // actuals total for a worker
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            } 
                    $row_index++;
                    //delta
                    if (isset($workers[$person_id]))
						$sheet->setCellValue($firstCol . $row_index, $workers[$person_id]->getReversedShortName() . ' ('.$person_id.')');
					else
						$sheet->setCellValue($firstCol . $row_index, 'UNKNOWN WORKER' . '(' . $person_id . ')');
					$sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta2']);
                    $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
					for ($i = 1; $i <= 12; $i++)
						$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx-1) . $row_index, $assignments[$i]->Hours - $person_actualRestAssigned[$i]);
					// delta total for a worker
					$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            } 
                    $row_index++;            
                }
            }            
            else // non-labour task
            {
				$changeRow = 0;
                $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($task_id, $year);
                $task_actualRestAssigned = $reporter->getTaskNonLabourEACCostsMonthTotals($task, $currency);
				$monthly_task_assignments = $this->context->assignmentsCostRepository->findTaskYearAssignments($task_id, $year);
				if ($task_plan === NULL) // task is not planned but has some costs for this year
					$task_plan = new TaskYearPlan($year);
				$task_assignments = $task_plan_values = $task_actuals = $task_deltas = array();
				for ($i = 1; $i <= 12; $i++)
				{
					$task_plan_values[] = $task_plan->getMonthPlan($i)->Budget;
					$task_deltas[] = $task_plan->getMonthPlan($i)->Budget - $task_actualRestAssigned[$i];
					if(isset($monthly_task_assignments[$i-1]))
						$task_assignments[] = $monthly_task_assignments[$i-1]->Cost;
					else
						$task_assignments[] = $task_plan->getMonthPlan($i)->Budget;
				}
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['plan']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, round($task_plan_values[$i]));
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $sheet->setCellValue($costShareCol . $row_index, $task->CostShare * 100);
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($sapIDCol . $row_index, $task->SAPID);
				$this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($sapIDCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $sheet->getStyle($firstCol . $row_index . ':' . $sapIDCol . $row_index)->applyFromArray($borderTask);
                $task_row_indexes[$task->ID]['plan'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        } 
                $row_index++;
				$changeRow++;
                
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['assigned']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, round($task_assignments[$i]));
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['assigned'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
				
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['actuals']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
                    $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, round($task_actualRestAssigned[$i+1]));
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['actuals'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
                
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta1']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
					$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '=SUM(' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 3) . '-' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 1) . ')');
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['delta1'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
				
                $sheet->setCellValue($firstCol . $row_index, $task->Name);
				$this->setColor($firstCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->setCellValue($hrsCol . $row_index, $row_triplet['delta2']);
				$this->setColor($hrsCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($hrsCol . $row_index)->applyFromArray($borderHrs);
                for ($i = 0; $i <= 11; $i++)
				{
					$sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '=SUM(' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 3) . '-' . $this->numToApha($i+$firstMonthColIdx) . ($row_index - 2) . ')');
					$this->setColor($this->numToApha($i+$firstMonthColIdx), $row_index, $changeRow, $changeTask, $sheet);
				}
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
				$this->setColor($totalCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($totalCol . $row_index)->applyFromArray($borderTotal);
                $sheet->setCellValue($unitCol . $row_index, $currency);
				$this->setColor($unitCol, $row_index, $changeRow, $changeTask, $sheet);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $task_row_indexes[$task->ID]['delta2'] = $row_index;
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
				$this->setColor($costShareCol, $row_index, $changeRow, $changeTask, $sheet);
                $this->setColor($sapIDCol, $row_index, $changeRow, $changeTask, $sheet);
                $row_index++;
				$changeRow++;
            }
            $start_row_index = $row_index;
            $changeTask++;
        }  
        
        // WORKERS AGREGATION
        
        $sheet3 = $objPHPExcel->createSheet(1);
        $sheet3->setTitle("Workers agregation");
        
        //SETUP PROPORTIES FOR $sheet3 - Workers agregation
        $sheet3->getStyle('A1')->getFont()->setBold(true);
        $sheet3->getColumnDimension($firstCol)->setWidth(40);
        $sheet3->getColumnDimension($hrsCol)->setWidth(28);
        $sheet3->getColumnDimension($totalCol)->setWidth(12);      
        $sheet3->setShowSummaryBelow(false); 
        $sheet3->freezePane('C2');
        $row_index3 = 1;
        $changeTask = 0;
        
        // PROJECT PLAN - HEADER    
        $sheet3->setCellValue($firstCol . $row_index3, "--- " . $year . " PROJECT PLAN ---");
        $sheet3->setCellValue($hrsCol . $row_index3, "Hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet3->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index3, $months[$i]);
        }
        $sheet3->setCellValue($totalCol . $row_index3, ">>TOTAL<<");
        $sheet3->setCellValue($unitCol . $row_index3, "Unit");

        $sheet3->getStyle($firstCol . $row_index3 . ':' . $sapIDCol . $row_index3)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $sheet3->getStyle($firstCol . $row_index3 . ':' . $sapIDCol . $row_index3)->getFont()->setBold(true); 
        $row_index3++;
        
        foreach($agregateWorkers as $rows)
        {
            $count = count($rows);
            $current = $current2 = current($rows);
            $actualsPattern1 = $assignedPattern1 = $deltaPattern1 = "=";
            $actualsPattern2 = $assignedPattern2 = $deltaPattern2 = "=";
            $actualsPattern3 = $assignedPattern3 = $deltaPattern3 = "=";
            $actualsPattern4 = $assignedPattern4 = $deltaPattern4 = "=";
            $actualsPattern5 = $assignedPattern5 = $deltaPattern5 = "=";
            $actualsPattern6 = $assignedPattern6 = $deltaPattern6 = "=";
            $actualsPattern7 = $assignedPattern7 = $deltaPattern7 = "=";
            $actualsPattern8 = $assignedPattern8 = $deltaPattern8 = "=";
            $actualsPattern9 = $assignedPattern9 = $deltaPattern9 = "=";
            $actualsPattern10 = $assignedPattern10 = $deltaPattern10 = "=";
            $actualsPattern11 = $assignedPattern11 = $deltaPattern11 = "=";
            $actualsPattern12 = $assignedPattern12 = $deltaPattern12 = "=";
            $actualsPattern13 = $assignedPattern13 = $deltaPattern13 = "=";
            
            for($i = 0; $i <= $count; $i++)
            {
                if($current != null)
                {
                    $actualsPattern1 .= "'Report EAC'!C" . $current;
                    $actualsPattern2 .= "'Report EAC'!D" . $current;
                    $actualsPattern3 .= "'Report EAC'!E" . $current;
                    $actualsPattern4 .= "'Report EAC'!F" . $current;
                    $actualsPattern5 .= "'Report EAC'!G" . $current;
                    $actualsPattern6 .= "'Report EAC'!H" . $current;
                    $actualsPattern7 .= "'Report EAC'!I" . $current;
                    $actualsPattern8 .= "'Report EAC'!J" . $current;
                    $actualsPattern9 .= "'Report EAC'!K" . $current;
                    $actualsPattern10 .= "'Report EAC'!L" . $current;
                    $actualsPattern11 .= "'Report EAC'!M" . $current;
                    $actualsPattern12 .= "'Report EAC'!N" . $current;
                    $actualsPattern13 .= "'Report EAC'!O" . $current;
                    
                    $assignedPattern1 .= "'Report EAC'!C" . ($current + 1);
                    $assignedPattern2 .= "'Report EAC'!D" . ($current + 1);
                    $assignedPattern3 .= "'Report EAC'!E" . ($current + 1);
                    $assignedPattern4 .= "'Report EAC'!F" . ($current + 1);
                    $assignedPattern5 .= "'Report EAC'!G" . ($current + 1);
                    $assignedPattern6 .= "'Report EAC'!H" . ($current + 1);
                    $assignedPattern7 .= "'Report EAC'!I" . ($current + 1);
                    $assignedPattern8 .= "'Report EAC'!J" . ($current + 1);
                    $assignedPattern9 .= "'Report EAC'!K" . ($current + 1);
                    $assignedPattern10 .= "'Report EAC'!L" . ($current + 1);
                    $assignedPattern11 .= "'Report EAC'!M" . ($current + 1);
                    $assignedPattern12 .= "'Report EAC'!N" . ($current + 1);
                    $assignedPattern13 .= "'Report EAC'!O" . ($current + 1);
                    
                    $deltaPattern1 .= "'Report EAC'!C" . ($current + 2);
                    $deltaPattern2 .= "'Report EAC'!D" . ($current + 2);
                    $deltaPattern3 .= "'Report EAC'!E" . ($current + 2);
                    $deltaPattern4 .= "'Report EAC'!F" . ($current + 2);
                    $deltaPattern5 .= "'Report EAC'!G" . ($current + 2);
                    $deltaPattern6 .= "'Report EAC'!H" . ($current + 2);
                    $deltaPattern7 .= "'Report EAC'!I" . ($current + 2);
                    $deltaPattern8 .= "'Report EAC'!J" . ($current + 2);
                    $deltaPattern9 .= "'Report EAC'!K" . ($current + 2);
                    $deltaPattern10 .= "'Report EAC'!L" . ($current + 2);
                    $deltaPattern11 .= "'Report EAC'!M" . ($current + 2);
                    $deltaPattern12 .= "'Report EAC'!N" . ($current + 2);
                    $deltaPattern13 .= "'Report EAC'!O" . ($current + 2);
                }
                $current = next($rows);
                if($current != null)
                {
                    $actualsPattern1 .= "+";
                    $actualsPattern2 .= "+";
                    $actualsPattern3 .= "+";
                    $actualsPattern4 .= "+";
                    $actualsPattern5 .= "+";
                    $actualsPattern6 .= "+";
                    $actualsPattern7 .= "+";
                    $actualsPattern8 .= "+";
                    $actualsPattern9 .= "+";
                    $actualsPattern10 .= "+";
                    $actualsPattern11 .= "+";
                    $actualsPattern12 .= "+";
                    $actualsPattern13 .= "+";
                    
                    $assignedPattern1 .= "+";
                    $assignedPattern2 .= "+";
                    $assignedPattern3 .= "+";
                    $assignedPattern4 .= "+";
                    $assignedPattern5 .= "+";
                    $assignedPattern6 .= "+";
                    $assignedPattern7 .= "+";
                    $assignedPattern8 .= "+";
                    $assignedPattern9 .= "+";
                    $assignedPattern10 .= "+";
                    $assignedPattern11 .= "+";
                    $assignedPattern12 .= "+";
                    $assignedPattern13 .= "+";
                    
                    $deltaPattern1 .= "+";
                    $deltaPattern2 .= "+";
                    $deltaPattern3 .= "+";
                    $deltaPattern4 .= "+";
                    $deltaPattern5 .= "+";
                    $deltaPattern6 .= "+";
                    $deltaPattern7 .= "+";
                    $deltaPattern8 .= "+";
                    $deltaPattern9 .= "+";
                    $deltaPattern10 .= "+";
                    $deltaPattern11 .= "+";
                    $deltaPattern12 .= "+";
                    $deltaPattern13 .= "+";
                }
            }

            $sheet3->setCellValue("A". $row_index3, "='Report EAC'!A" . $current2);
            $sheet3->getStyle($hrsCol . $row_index3)->applyFromArray($borderHrs);
            $sheet3->getStyle('O' . $row_index3)->applyFromArray($borderTotal);
            $this->setColor("A", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("B". $row_index3, "='Report EAC'!B" . $current2);
            $this->setColor("B", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("C". $row_index3, $actualsPattern1);
            $this->setColor("C", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("D". $row_index3, $actualsPattern2);
            $this->setColor("D", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("E". $row_index3, $actualsPattern3);
            $this->setColor("E", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("F". $row_index3, $actualsPattern4);
            $this->setColor("F", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("G". $row_index3, $actualsPattern5);
            $this->setColor("G", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("H". $row_index3, $actualsPattern6);
            $this->setColor("H", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("I". $row_index3, $actualsPattern7);
            $this->setColor("I", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("J". $row_index3, $actualsPattern8);
            $this->setColor("J", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("K". $row_index3, $actualsPattern9);
            $this->setColor("K", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("L". $row_index3, $actualsPattern10);
            $this->setColor("L", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("M". $row_index3, $actualsPattern11);
            $this->setColor("M", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("N". $row_index3, $actualsPattern12);
            $this->setColor("N", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("O". $row_index3, $actualsPattern13);
            $this->setColor("O", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("P". $row_index3, "='Report EAC'!P" . $current2);
            $this->setColor("P", $row_index3, 0, $changeTask, $sheet3);
            $row_index3++;
            $sheet3->setCellValue("A". $row_index3, "='Report EAC'!A" . ($current2 + 1));
            $sheet3->getStyle($hrsCol . $row_index3)->applyFromArray($borderHrs);
            $sheet3->getStyle('O' . $row_index3)->applyFromArray($borderTotal);
            $this->setColor("A", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("B". $row_index3, "='Report EAC'!B" . ($current2 + 1));
            $this->setColor("B", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("C". $row_index3, $assignedPattern1);
            $this->setColor("C", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("D". $row_index3, $assignedPattern2);
            $this->setColor("D", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("E". $row_index3, $assignedPattern3);
            $this->setColor("E", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("F". $row_index3, $assignedPattern4);
            $this->setColor("F", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("G". $row_index3, $assignedPattern5);
            $this->setColor("G", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("H". $row_index3, $assignedPattern6);
            $this->setColor("H", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("I". $row_index3, $assignedPattern7);
            $this->setColor("I", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("J". $row_index3, $assignedPattern8);
            $this->setColor("J", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("K". $row_index3, $assignedPattern9);
            $this->setColor("K", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("L". $row_index3, $assignedPattern10);
            $this->setColor("L", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("M". $row_index3, $assignedPattern11);
            $this->setColor("M", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("N". $row_index3, $assignedPattern12);
            $this->setColor("N", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("O". $row_index3, $assignedPattern13);
            $this->setColor("O", $row_index3, 0, $changeTask, $sheet3);
            $sheet3->setCellValue("P". $row_index3, "='Report EAC'!P" . ($current2 + 1));
            $this->setColor("P", $row_index3, 0, $changeTask, $sheet3);
            $row_index3++;
            $sheet3->setCellValue("A". $row_index3, "='Report EAC'!A" . ($current2 + 2));
            $sheet3->getStyle($hrsCol . $row_index3)->applyFromArray($borderHrs);
            $sheet3->getStyle('O' . $row_index3)->applyFromArray($borderTotal);
            $this->setColor("A", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("B". $row_index3, "='Report EAC'!B" . ($current2 + 2));
            $this->setColor("B", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("C". $row_index3, $deltaPattern1);
            $this->setColor("C", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("D". $row_index3, $deltaPattern2);
            $this->setColor("D", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("E". $row_index3, $deltaPattern3);
            $this->setColor("E", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("F". $row_index3, $deltaPattern4);
            $this->setColor("F", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("G". $row_index3, $deltaPattern5);
            $this->setColor("G", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("H". $row_index3, $deltaPattern6);
            $this->setColor("H", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("I". $row_index3, $deltaPattern7);
            $this->setColor("I", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("J". $row_index3, $deltaPattern8);
            $this->setColor("J", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("K". $row_index3, $deltaPattern9);
            $this->setColor("K", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("L". $row_index3, $deltaPattern10);
            $this->setColor("L", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("M". $row_index3, $deltaPattern11);
            $this->setColor("M", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("N". $row_index3, $deltaPattern12);
            $this->setColor("N", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("O". $row_index3, $deltaPattern13);
            $this->setColor("O", $row_index3, 3, $changeTask, $sheet3);
            $sheet3->setCellValue("P". $row_index3, "='Report EAC'!P" . ($current2 + 2));
            $this->setColor("P", $row_index3, 3, $changeTask, $sheet3);
            $row_index3++;
            $sheet3->getStyle($firstCol . $row_index3 . ':P' . $row_index3)->applyFromArray($borderTask);
            $changeTask++;
        }
        
        //SETUP PROPORTIES FOR $sheet2 - Labour rates        
        $row_index2 = 1;
        $firstCol2 = "A";
        $firstMonthColIdx2 = 2;
        $unitCol2 = "N"; // last column for labour rates 
        
        // COST CENTERS / LABOUR RATES
        $sheet2 = $objPHPExcel->createSheet(1);
        $sheet2->setTitle("Labour rates");        
        $sheet2->getColumnDimension($firstCol2)->setWidth(35);
        $sheet2->setCellValue($firstCol2 . $row_index2, "--- LABOUR RATES ---");
        $sheet2->setCellValue($unitCol2 . $row_index2, "Unit");
        for($i = 0; $i<12; $i++)
        {
            $sheet2->setCellValue($this->numToApha($i+$firstMonthColIdx2) . $row_index2, $months[$i]);
        }
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getFont()->setBold(true);
        $sheet2->getStyle($firstCol2 . $row_index2 . ':' . $unitCol2 . $row_index2)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index2++;
        
        $labour_rates_row_indexes = array();
        $labour_centers = $this->context->costCenterRepository->findAllCostCenters();      
        foreach ($labour_centers as $center)
		{
			$rate = $center->getYearLabourRate($year);
			if ($rate != NULL)
			{
                $sheet2->setCellValue($firstCol2 . $row_index2, $center->Name);				
                for ($i = 1; $i <= 12; $i++)
                    $sheet2->setCellValue($this->numToApha($i+1) . $row_index2, $rate->getMonthlyHourRate($i));
                $sheet2->setCellValue($unitCol2 . $row_index2, $currency . '/' . $hours_short);
               $labour_rates_row_indexes[$center->ID] = $row_index2;
                $row_index2++;                
            }              
		}
        
        //EFFORT
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- EFFORT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;                
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
		$effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta1'] = $effort_total_indexes['delta2'] = $effort_total_indexes['assigned'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['assigned'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta1'] = $effort_sum_indexes['delta2'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
						$effort_sum_indexes['assigned'][] = $task_row_indexes[$task->ID]['assigned'];
                        $effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
                        $effort_sum_indexes['delta1'][] = $task_row_indexes[$task->ID]['delta1'];
						$effort_sum_indexes['delta2'][] = $task_row_indexes[$task->ID]['delta2'];
					}
			$center_name_written = false;
            if(count($effort_sum_indexes['plan']) > 0 AND count($effort_sum_indexes['assigned']) > 0 AND count($effort_sum_indexes['actuals']) > 0 AND count($effort_sum_indexes['delta1']) > 0 AND count($effort_sum_indexes['delta2']) > 0)
                foreach ($row_triplet as $key => $title)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);					
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center->Name);					
                        $sheet->setCellValue($hrsCol . $row_index, $title);					
                        $center_name_written = true;			
                    }
                    for ($i = 0; $i < 12; $i++)						                        
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));				
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->setCellValue($unitCol . $row_index, $hours_short);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $effort_total_indexes[$key][] = $row_index;                                
                    $row_index++;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);

                }        
		}
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, ">> TOTAL <<");
				$sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
				 $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
            if($year == date('Y'))
                if(fiscalCalendar::currentFiscalMonth() != 0)
                    {
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                    }
			$sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->setCellValue($unitCol . $row_index, $hours_short);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}

        // COSTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- COSTS ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $costs_row_indexes = array();
        $effort_total_indexes['plan'] = $effort_total_indexes['actuals'] = $effort_total_indexes['delta1'] = $effort_total_indexes['delta2'] = $effort_total_indexes['assigned'] = array();
		foreach ($labour_centers as $center)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['assigned'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta1'] = $effort_sum_indexes['delta2'] = array();
			// remember indexes of row which belong to this labour center
            foreach ($tasks as $task_id => $task)
				if ($task->isLabourTask())
					if ($task->CostCenterID == $center->ID)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
                        $effort_sum_indexes['assigned'][] = $task_row_indexes[$task->ID]['assigned'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];
                        $effort_sum_indexes['delta1'][] = $task_row_indexes[$task->ID]['delta1'];
						$effort_sum_indexes['delta2'][] = $task_row_indexes[$task->ID]['delta2'];
					}
			$center_name_written = false;
            if(count($effort_sum_indexes['plan']) > 0 AND count($effort_sum_indexes['assigned']) > 0 AND count($effort_sum_indexes['actuals']) > 0 AND count($effort_sum_indexes['delta1']) > 0 AND count($effort_sum_indexes['delta2']) > 0 )
                foreach ($row_triplet as $key => $title)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center->Name);					
                        $sheet->setCellValue($hrsCol . $row_index, $title);					
                        $center_name_written = true;			
                    }
                    for ($i = 0; $i < 12; $i++)
                    {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]) . "*'Labour rates'!" . $this->numToApha ($firstMonthColIdx2 + $i) . $labour_rates_row_indexes[$center->ID]);                        
                            $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $costs_row_indexes[$center->ID][$key] = $row_index;
                $effort_total_indexes[$key][] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;            
                }
			$row_for_total_indexes[] = $row_index; 
		}
        foreach ($other_costs as $task_prefix)
		{
			$effort_sum_indexes['plan'] = $effort_sum_indexes['assigned'] = $effort_sum_indexes['actuals'] = $effort_sum_indexes['delta1'] = $effort_sum_indexes['delta2'] = array();
			// remember indexes of row which belong to this labour center
			foreach ($tasks as $task_id => $task)                
				if (!$task->isLabourTask())
					// same type of non-labour tasks
					if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
					{
						$effort_sum_indexes['plan'][] = $task_row_indexes[$task->ID]['plan'];
                        $effort_sum_indexes['assigned'][] = $task_row_indexes[$task->ID]['assigned'];
						$effort_sum_indexes['actuals'][] = $task_row_indexes[$task->ID]['actuals'];                        
						$effort_sum_indexes['delta1'][] = $task_row_indexes[$task->ID]['delta1'];
                        $effort_sum_indexes['delta2'][] = $task_row_indexes[$task->ID]['delta2'];
					}
			$center_name_written = false;
            if(count($effort_sum_indexes['plan']) > 0 AND count($effort_sum_indexes['assigned']) > 0 AND count($effort_sum_indexes['actuals']) > 0 AND count($effort_sum_indexes['delta1']) > 0 AND count($effort_sum_indexes['delta2']) > 0)
                foreach ($row_triplet as $key => $title)
                {               
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                        $sheet->setCellValue($hrsCol . $row_index, $title);	
                        $center_name_written = true;			
                    }
                    for ($i = 0; $i < 12; $i++)
                    {
                            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_sum_indexes[$key]));
                            $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    if($year == date('Y'))
                        if(fiscalCalendar::currentFiscalMonth() != 0)
                            {
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                                $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                            }
                    $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                    $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    $sheet->setCellValue($unitCol . $row_index, $currency);
                    $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                    $effort_total_indexes[$key][] = $row_index;
                    $costs_row_indexes[$task_prefix][$key] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
            }
			$row_for_total_indexes[] = $row_index;            
		}

		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);	
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $effort_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            if($year == date('Y'))
                if(fiscalCalendar::currentFiscalMonth() != 0)
                    {
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                    }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // HONEYWELL INVESTMENTS
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- HON INVESTMENT ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        for($i = 0; $i<12; $i++)
        {
            $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $months[$i]);
        }
        $sheet->setCellValue($totalCol . $row_index, ">>TOTAL<<");
        $sheet->setCellValue($unitCol . $row_index, "Unit");
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);        
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
        $row_for_total_indexes['plan'] = $row_for_total_indexes['actuals'] = $row_for_total_indexes['delta1'] = $row_for_total_indexes['delta2'] = $row_for_total_indexes['assigned'] = array();
		$costs_hon_row_indexes = array();
		foreach ($labour_centers as $center)
		{
			$center_name_written = false;
            $cell_value = '';
					foreach ($tasks as $task_id => $task)
						if ($task->isLabourTask())
							if ($task->CostCenterID == $center->ID)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					if ($cell_value != '')
			foreach ($row_triplet as $key => $title)
			{
				if ($center_name_written)
					$sheet->setCellValue($hrsCol . $row_index, $title);
				else
				{
					$sheet->setCellValue($firstCol . $row_index, $center->Name);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
					$center_name_written = true;			
				}
				for ($i = 0; $i < 12; $i++)
				{
					$cell_value = '';
					foreach ($tasks as $task_id => $task)
						if ($task->isLabourTask())
							if ($task->CostCenterID == $center->ID)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, "=(".$cell_value.")*'Labour rates'!".$this->numToApha($firstMonthColIdx2 + $i).$labour_rates_row_indexes[$center->ID]);                                      
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
				}
				// totals for a center
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
				$row_for_total_indexes[$key][] = $row_index;
				$costs_hon_row_indexes[$center->ID][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
			}
		}
        foreach ($other_costs as $task_prefix)
		{
			$center_name_written = false;
            					$cell_value = '';
					foreach ($tasks as $task_id => $task)
					{
						if (!$task->isLabourTask())
							// same type of non-labour tasks
							if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
							{
								$cell_value .= ($cell_value !== '') ? '+' : '';
								// CostShare for a task is on column index: $firstMonthColIdx + 13 * 3 + 1
								$cell_value .= $this->numToApha($firstMonthColIdx + $i) 
										. $task_row_indexes[$task->ID][$key]
										. '*(100-' 
										. $this->numToApha($firstMonthColIdx + 13 + 1) 
										. $task_row_indexes[$task->ID]['plan']
										. ')/100';
							}
					}
					if ($cell_value != '')
			foreach ($row_triplet as $key => $title)
			{
                if ($center_name_written)
                    $sheet->setCellValue($hrsCol . $row_index, $title);
                else
                {
                    $sheet->setCellValue($firstCol . $row_index, $task_prefix);					
                    $sheet->setCellValue($hrsCol . $row_index, $title);
                    $center_name_written = true;			
                }
                for ($i = 0; $i < 12; $i++)
                {
                    $cell_value = '';
                    foreach ($tasks as $task_id => $task)
                    {
                        if (!$task->isLabourTask())
                            // same type of non-labour tasks
                            if (substr($task->Name, 0, strlen($task_prefix)) == $task_prefix)
                            {
                                $cell_value .= ($cell_value !== '') ? '+' : '';
                                // CostShare for a task is on column index: $firstMonthColIdx + 13 * 3 + 1
                                $cell_value .= $this->numToApha($firstMonthColIdx + $i) 
                                        . $task_row_indexes[$task->ID][$key]
                                        . '*(100-' 
                                        . $this->numToApha($firstMonthColIdx + 13 + 1) 
                                        . $task_row_indexes[$task->ID]['plan']
                                        . ')/100';
                            }
                    }
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, '='.$cell_value);						
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                }
                // totals for a non-labor tasks type
                if($year == date('Y'))
                    if(fiscalCalendar::currentFiscalMonth() != 0)
                        {
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                            $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                        }
                $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
                $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                $sheet->setCellValue($unitCol . $row_index, $currency);
                $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
                $row_for_total_indexes[$key][] = $row_index;
                $costs_hon_row_indexes[$task_prefix][$key] = $row_index;
                $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                $sheet->getRowDimension($row_index)->setVisible(false);
                $row_index++;
            }
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);				
				$total_text_written = true;			
			}
			for ($i = 0; $i < 12; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            if($year == date('Y'))
                if(fiscalCalendar::currentFiscalMonth() != 0)
                    {
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnR);
                        $sheet->getStyle($column . $row_index . ":" . $column . $row_index)->applyFromArray($borderColumnL);
                    }
            $sheet->setCellValue($totalCol . $row_index, $this->rowColumnsSumExpression($row_index, $sum_column_indexes));
            $sheet->getStyle($totalCol . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            $sheet->setCellValue($unitCol . $row_index, $currency);
            $sheet->getStyle($unitCol . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
            $row_index++;
		}
        
        // QUARTERLY SPEND PROFILE
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY SPEND PROFILE ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        
		// prepare summary traversal array
		$summary = array();
		foreach ($labour_centers as $center)
			$summary[$center->ID] = $center->Name;
		foreach ($other_costs as $task_prefix)
			$summary[$task_prefix] = $task_prefix;
        $row_for_total_indexes['plan'] = $row_for_total_indexes['actuals'] = $row_for_total_indexes['delta1'] = $row_for_total_indexes['delta2'] = $row_for_total_indexes['assigned'] = array();
		foreach ($costs_row_indexes as $cost_row_index => $summary_name)
		{
            $center_name = $summary[$cost_row_index];
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
                if(strlen($center_name) == 3)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
                }
                elseif(strlen($center_name) != 3 )
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;
                }
			}
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{	
			if ($total_text_written)
                $sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;
		}
        
        // QUARTERLY HON INVEST
        $row_index++;
        $sheet->setCellValue($firstCol . $row_index, "--- QUARTERLY HON INVEST ---");    
        $sheet->setCellValue($hrsCol . $row_index, "hrs");
        $sheet->setCellValue($q1Col . $row_index, "Q1");
        $sheet->setCellValue($q1Co2 . $row_index, "Q2");
        $sheet->setCellValue($q1Co3 . $row_index, "Q3");
        $sheet->setCellValue($q1Co4 . $row_index, "Q4");        
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getFont()->setBold(true);
        $sheet->getStyle($firstCol . $row_index . ':' . $q1Co4 . $row_index)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
        $row_index++;
        $sheet->getRowDimension($row_index)->setOutlineLevel(1);
        $sheet->getRowDimension($row_index)->setVisible(false);
        $row_for_total_indexes['plan'] = $row_for_total_indexes['actuals'] = $row_for_total_indexes['delta1'] = $row_for_total_indexes['delta2'] = $row_for_total_indexes['assigned'] = array();
		foreach ($costs_row_indexes as $cost_row_index => $summary_name)
		{
            $center_name = $summary[$cost_row_index];
			$center_name_written = false;
			foreach ($row_triplet as $key => $title)
			{
                if(strlen($center_name) == 3)
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);				
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_hon_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++;                    
                }
                elseif(strlen($center_name) != 3 )
                {
                    if ($center_name_written)
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                    else
                    {
                        $sheet->setCellValue($firstCol . $row_index, $center_name);				
                        $sheet->setCellValue($hrsCol . $row_index, $title);
                        $center_name_written = true;			
                    }
                    // 4 quarters of year, each has 3 months
                    for ($i = 0; $i < 4; $i++)
                    {
                        $costs_col_sum_indexes = array();
                        for ($j = 0; $j < 3; $j++)
                            $costs_col_sum_indexes[] = $firstMonthColIdx + $i * 3 + $j;
                        $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->rowColumnsSumExpression($costs_hon_row_indexes[$cost_row_index][$key], $costs_col_sum_indexes));
                        $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
                    }
                    $row_for_total_indexes[$key][] = $row_index;
                    $sheet->getRowDimension($row_index)->setOutlineLevel(1);
                    $sheet->getRowDimension($row_index)->setVisible(false);
                    $row_index++; 
                }
            }
		}
		// total effort for cost centers
		$total_text_written = false;
		foreach ($row_triplet as $key => $title)
		{
			if ($total_text_written)
				$sheet->setCellValue($hrsCol . $row_index, $title);
			else
			{
                $sheet->setCellValue($firstCol . $row_index, '>> TOTAL <<');					
                $sheet->setCellValue($hrsCol . $row_index, $title);
				$total_text_written = true;			
			}
			for ($i = 0; $i < 4; $i++)
            {
                $sheet->setCellValue($this->numToApha($i+$firstMonthColIdx) . $row_index, $this->columnRowsSumExpression($firstMonthColIdx + $i, $row_for_total_indexes[$key]));
                $sheet->getStyle($this->numToApha($i+$firstMonthColIdx) . $row_index)->getNumberFormat()->setFormatCode('### ### ##0');
            }
            $sheet->getRowDimension($row_index)->setOutlineLevel(1);
            $sheet->getRowDimension($row_index)->setVisible(false);
			$row_index++;                        
		}
        setcookie('fileDownloadToken', $token);
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment; filename="' . $project->Name . ' - Assigned Resources and Actuals/EAC - ' . $year . '.xlsx"');
        header('Cache-Control: max-age=0');
        $objWriter = new PHPExcel_Writer_Excel2007($objPHPExcel);
        ob_end_clean();
        $objWriter->save('php://output');
        exit;
    }
    
	/**
	 * Compares two tasks by name alphabetically.
	 * 
	 * @param Task $task1
	 * @param Task $task2
	 * @return int
	 */
	private function sortTasksByName($task1, $task2)
	{
		if (($task1 instanceof Task) && ($task2 instanceof Task))
			return strcmp($task1->Name, $task2->Name);
		else
			throw new InvalidArgumentException('Sort function accepts only Task objects as arguments.');
	}
	
}